package captcha.c02;

import drjava.util.*;
import prophecy.common.Var;
import prophecy.common.gui.Form;

import javax.swing.*;
import java.awt.*;

import static drjava.util.ObjectUtil.nice;

public class Main implements Updatable {
  private JFrame frame;
  private Form form;
  private Font formFont;
  private JLabel lblCaptchas, lblCaptchasSolved, lblSolver, lblStatus;
  private int updateFreq = 500;
  private Blackboard blackboard = new Blackboard();
  private Var<String> status = new Var<String>("");
  private MS1 solver = new MS1();

  public static void main(String[] args) {
    new Main().showFrame();
  }

  private void showFrame() {
    form = new Form();
    formFont = form.getFont().deriveFont(14.0f);
    form.setFont(formFont);

    lblCaptchas = makeLabel("0");
    lblCaptchasSolved = makeLabel("0");
    lblSolver = makeLabel(nice(solver));
    lblStatus = makeLabel("");

    form.addRow("Solver:" , lblSolver);
    form.addRow("Captchas:", lblCaptchas);
    form.addRow("Captchas solved:", lblCaptchasSolved);
    form.addRow("Status:", lblStatus);

    JPanel buttons = new JPanel(LetterLayout.stalactite());
    /*JButton btnStart = makeButton("Start node", this, "start");
    buttons.add(btnStart);*/

    frame = new JFrame("Captcha solver");
    frame.getContentPane().setLayout(new LetterLayout("FFB").setBorder(10));
    frame.getContentPane().add("F", form);
    frame.getContentPane().add("B", buttons);
    frame.pack();
    frame.setSize(300, frame.getHeight());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    GUIUtil.showFrame(frame);

    new SwingTimerHelper(this, updateFreq).installOn(form);

    start();
  }

  private void start() {
    new Thread() {
      public void run() {
        try {
          status.set("loading captchas");
          blackboard.load();
          status.set("solving");
          solver.go(blackboard);
          status.set("done");
        } catch (Throwable e) {
          status.set("Error (" + nice(e) + ")");
          Errors.add(e);
        }
      }
    }.start();
  }

  public void update() {
    lblCaptchas.setText("" + blackboard.numCaptchas.get());
    lblCaptchasSolved.setText("" + blackboard.numCaptchasSolved.get());
    lblStatus.setText(status.get());
  }

  private JLabel makeLabel(String text) {
    JLabel label = new JLabel(text);
    if (formFont != null)
      label.setFont(formFont);
    return label;
  }

}
