package captcha.c04;

import captcha.c01.RiddleToSolve;
import captcha.c02.Blackboard;
import drjava.util.Randomizer;
import drjava.util.RealRandomizer;
import drjava.util.StringUtil;
import drjava.util.Tree;
import prophecy.common.ClassData;
import prophecy.common.PersistentTree;

import java.io.DataInputStream;
import java.io.IOException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class DetectQuestionType {
  private Blackboard blackboard;
  private List<RiddleToSolve> list;
  private Map<RiddleToSolve, String> tagMap = new HashMap<RiddleToSolve, String>();

  public static void main(String[] args) throws IOException {
    new DetectQuestionType();
  }

  DetectQuestionType() throws IOException {
    blackboard = new Blackboard();
    blackboard.load();
    System.out.println(blackboard.riddlesToSolve.size()+ " captchas loaded");
    loadTags();
    inputLoop();
  }

  private void inputLoop() throws IOException {
    DataInputStream input = new DataInputStream(System.in);
    while (true) {
      System.out.print("> ");
      String line = input.readLine();
      if (line == null)
        break;
      processInput(line);
    }
  }

  private void processInput(String line) {
    line = line.trim();

    if (line.equals("exit") || line.equals("bye")) {
      System.exit(0);
    }

    if (StringUtil.isInteger(line)) {
      int n = Integer.parseInt(line);
      List<RiddleToSolve> baseList = blackboard.riddlesToSolve;
      chooseRandomSet(n, baseList);
      return;
    }

    Matcher matcher = Pattern.compile("(\\d+) tagged").matcher(line);
    if (matcher.matches()) {
      int n = Integer.parseInt(matcher.group(1));
      List<RiddleToSolve> baseList = getTaggedRiddles();
      chooseRandomSet(n, baseList);
    }

    matcher = Pattern.compile("(\\d+) untagged").matcher(line);
    if (matcher.matches()) {
      int n = Integer.parseInt(matcher.group(1));
      List<RiddleToSolve> baseList = getUntaggedRiddles();
      chooseRandomSet(n, baseList);
    }

    matcher = Pattern.compile("(\\d+)=(.*)").matcher(line);
    if (matcher.matches()) {
      int idx = Integer.parseInt(matcher.group(1));
      String val = matcher.group(2);
      if (list == null) {
        System.out.println("No list");
        return;
      }
      if (idx < 1 || idx > list.size()) {
        System.out.println("Index " + idx + " is not in range 1-" + list.size());
        return;
      }
      RiddleToSolve riddle = list.get(idx - 1);
      if (val.length() == 0)
        tagMap.remove(riddle);
      else
        tagMap.put(riddle, val);
      printListEntry(idx);
      return;
    }

    if (line.equals("list")) {
      printList();
      return;
    }

    if (line.equals("save")) {
      saveTags();
      return;
    }

    if (line.equals("tags")) {
      printAllTags();
      return;
    }

    if (line.equals("stats")) {
      int total = blackboard.riddlesToSolve.size();
      int tagged = getTaggedRiddles().size();
      int untagged = total-tagged;
      int listSize = list == null ? 0 : list.size();
      System.out.println("Captchas: " + total);
      System.out.println(tagged + " tagged, " + untagged + " untagged");
      if (listSize != 0)
        System.out.println("In list: " + listSize + " captchas");
    }

    // input = an existing tag
    if (tagMap.containsValue(line)) {
      List<RiddleToSolve> list = new ArrayList<RiddleToSolve>();
      for (RiddleToSolve riddle : blackboard.riddlesToSolve)
        if (line.equals(tagMap.get(riddle)))
          list.add(riddle);
      setList(list);
    }
  }

  private List<RiddleToSolve> getUntaggedRiddles() {
    List<RiddleToSolve> baseList = new ArrayList<RiddleToSolve>();
    for (RiddleToSolve riddle : blackboard.riddlesToSolve)
      if (!tagMap.containsKey(riddle))
        baseList.add(riddle);
    return baseList;
  }

  private List<RiddleToSolve> getTaggedRiddles() {
    List<RiddleToSolve> baseList = new ArrayList<RiddleToSolve>();
    for (RiddleToSolve riddle : blackboard.riddlesToSolve)
      if (tagMap.containsKey(riddle))
        baseList.add(riddle);
    return baseList;
  }

  private void chooseRandomSet(int n, List<RiddleToSolve> baseList) {
    int total = baseList.size();
    if (n > total) n = total;
    List<RiddleToSolve> base = new ArrayList<RiddleToSolve>(baseList);
    List<RiddleToSolve> list = new ArrayList<RiddleToSolve>();
    Randomizer randomizer = new RealRandomizer();
    for (int i = 0; i < n; i++) {
      int idx = randomizer.random(base.size());
      list.add(base.get(idx));
      base.remove(idx);
    }
    setList(list);
  }

  private void setList(List<RiddleToSolve> list) {
    this.list = list;
    printList();
  }

  private void printAllTags() {
    TreeSet<String> tags = new TreeSet<String>(tagMap.values());
    for (String tag : tags) {
      System.out.println("  " + tag);
    }
  }

  private void loadTags() {
    PersistentTree tree = ClassData.get(this).subTree("tags");
    tagMap.clear();
    for (Map.Entry<String, Tree> entry : tree.children().entrySet()) {
      RiddleToSolve riddle = findRiddle(entry.getKey());
      if (riddle != null)
        tagMap.put(riddle, entry.getValue().name());
    }
    System.out.println(tagMap.size() + " tags loaded");
  }

  private RiddleToSolve findRiddle(String questionHash) {
    for (RiddleToSolve riddle : blackboard.riddlesToSolve) {
      if (riddle.riddle.getQuestionHash().equals(questionHash))
        return riddle;
    }
    return null;
  }

  private void saveTags() {
    PersistentTree tree = ClassData.get(this).subTree("tags");
    tree.clear();
    for (Map.Entry<RiddleToSolve, String> entry : tagMap.entrySet()) {
      tree.put(entry.getKey().riddle.getQuestionHash(), entry.getValue());
    }
    tree.save();
    System.out.println(tagMap.size() + " tags saved");
  }

  private void printList() {
    if (list == null) return;
    for (int i = 1; i <= list.size(); i++) {
      printListEntry(i);
    }
  }

  private void printListEntry(int i) {
    String s = i + ": ";
    RiddleToSolve riddle = list.get(i - 1);
    String tag = tagMap.get(riddle);
    if (tag != null && tag.length() != 0)
      s += "[" + tag + "] ";
    System.out.println(s + riddle.question);
  }
}
