package captcha.c04;

import captcha.c01.RiddleToSolve;
import captcha.c02.Blackboard;
import prophecy.common.ClassData;
import prophecy.common.PersistentTree;

import java.util.ArrayList;
import java.util.List;

import static drjava.util.StringUtil.n;

/** looks for ways to automate question type detection */
public class MakeDetectors {
  private TagMap tagMap;
  private Blackboard blackboard;

  public static void main(String[] args) {
    new MakeDetectors();
  }

  MakeDetectors() {
    blackboard = new Blackboard();
    blackboard.load();

    PersistentTree tree = ClassData.get(DetectQuestionType.class).subTree("tags");
    tagMap = new TagMap();
    tagMap.fromTree(tree);

    List<String> tags = tagMap.tags();
    for (String tag : tags) {
      solveTag(tag);
    }
  }

  private void solveTag(String tag) {
    List<RiddleToSolve> list = riddlesForTag(tag);
    System.out.println("Solving tag " + tag + " (" + n(list.size(), "example") + ")");

    if (list.size() == 1) {
      RiddleToSolve riddle = list.get(0);
      String[] words = riddle.question.split("\\W+");
      Best<KeywordDetector> best = new Best<KeywordDetector>();
      for (String word : words) {
        KeywordDetector detector = new KeywordDetector(word, tag);

        int positives = numPositives(detector, tag);
        int falsePositives = numFalsePositives(detector, tag);

        //System.out.println("Keyword: " + word + ", positives: " + positives + ", false positives: " + falsePositives);
        best.tryCandidate(detector, positives-falsePositives);
      }

      if (best.getBest() != null)
        System.out.println("  Best keyword: " + best.getBest().keyword);
    }
  }

  private int numPositives(KeywordDetector detector, String tag) {
    List<RiddleToSolve> list = riddlesForTag(tag);
    int count = 0;
    for (RiddleToSolve riddle : list) {
      String answer = detector.detect(riddle.question);
      if (tag.equals(answer))
        ++count;
    }
    return count;
  }

  private int numFalsePositives(KeywordDetector detector, String mainTag) {
    int count = 0;
    for (String tag : tagMap.tags()) {
      if (tag.equals(mainTag)) continue;
      List<RiddleToSolve> list = riddlesForTag(tag);
      for (RiddleToSolve riddle : list) {
        String answer = detector.detect(riddle.question);
        if (answer != null)
          ++count;
      }
    }
    return count;
  }

  private List<RiddleToSolve> riddlesForTag(String tag) {
    List<RiddleToSolve> list = new ArrayList<RiddleToSolve>();
    for (String questionHash : tagMap.riddlesForTag(tag))
      list.add(blackboard.get(questionHash));
    return list;
  }
}
