/*
Data fields: bytes, ref, remoteref
Data states:
-frozen: bytes
-local: ref
-remote: remote
Transitions:
remote => local
frozen => local
*/

package drjava.superversion;

import drjava.superversion.DownloadFailedException;
import drjava.superversion.SuperversionException;
import drjava.superversion.api.Data;
import drjava.superversion.tools.FingerprintMaker;

public class DataImpl extends Data {
  byte[] bytes;
  private boolean binary;

  public DataImpl(byte[] bytes, long fingerprint, boolean binary) {
    super(fingerprint);
    this.bytes = bytes;
    this.binary = binary;
  }

  public DataImpl(byte[] bytes, boolean binary) {
    this(bytes, 0, binary);
  }

  /** @deprecated */
  public byte[] getBytes() throws DownloadFailedException {
    return bytes;
  }

  // TODO: do we need this?
  public long getSize() throws DownloadFailedException {
    return getBytes().length;
  }

  public void ensureAvailability() throws DownloadFailedException {
  }

  public Data freeze() {
    return this;
  }

  public long getFingerprint() {
    if (fingerprint != 0)
      return fingerprint;
    else {
      try {
        fingerprint = FingerprintMaker.getFingerprint(getBytes(), isBinary());
      } catch (DownloadFailedException e) {
        throw new SuperversionException(e);
      }
      return fingerprint;
    }
  }

  public boolean isLinkable() {
    return true;
  }

  void resetFingerprint() {
    throw new RuntimeException("not implemented");
  }

  public boolean isBinary() {
    return binary;
  }
}
