package drjava.superversion.api;

import drjava.superversion.DownloadFailedException;
import drjava.superversion.SuperversionException;
import drjava.vats.Promise;
import drjava.vats.PromiseMaker;
import drjava.vats.Vat;

public abstract class Data {
  protected long fingerprint;

  public Data(long fingerprint) {
    this.fingerprint = fingerprint;
  }

  public abstract byte[] getBytes() throws DownloadFailedException;

  public Promise<byte[]> getBytes_promise() {
    PromiseMaker<byte[]> promiseMaker = Vat.currentVat().<byte[]>newPromiseMaker();
    try {
      promiseMaker.fulfill(getBytes());
    } catch (DownloadFailedException e) {
      promiseMaker.brEak(e);
    }
    return promiseMaker.getPromise();
  }

  public long getFingerprint() {
    if (fingerprint == 0)
      throw new SuperversionException("Missing fingerprint");
    return fingerprint;
  }

  public abstract boolean isBinary();

  public abstract long getSize() throws DownloadFailedException;

  public abstract Data freeze() throws DownloadFailedException;

  public int getRef() {
    return 0;
  }

  /** not very clearly defined, it's just for display */
  public Data getOriginal() {
    return this;
  }

  public boolean isLinkable() {
    return false;
  }

  public abstract void ensureAvailability() throws DownloadFailedException;
}
