package drjava.superversion.filesystem;

import drjava.util.Logger;
import drjava.util.ProgressControl;
import drjava.vats.Promise;

public class CachedDirectory implements MutableDirectory {
  private CachedFileSystem fileSystem;
  private MutableDirectory dir;

  CachedDirectory(CachedFileSystem fileSystem, MutableDirectory dir) {
    this.fileSystem = fileSystem;
    this.dir = dir;
  }

  public void deleteFile(String path) {
    dir.deleteFile(path);
  }

  public void create() {
    dir.create();
  }

  public void saveFile(String path, byte[] contents) {
    dir.saveFile(path, contents);
  }

  public FileSystem getFileSystem() {
    return dir.getFileSystem();
  }

  public DirectoryContents getContents(Logger logger) {
    return fileSystem.getContents(dir, logger);
  }

  public Promise<DirectoryContents> getContents_promise(Logger logger, ProgressControl progressControl) {
    return dir.getContents_promise(logger, progressControl); // TODO
  }

  public String getName() {
    return dir.getName();
  }

  public String getPath() {
    return dir.getPath();
  }

  public File getFile(String path) {
    return fileSystem.getFile(dir, path);
  }

  public boolean isLocal() {
    return dir.isLocal();
  }

  public boolean exists() {
    return dir.exists();
  }

  public MutableDirectory getDirectory(String name) {
    return dir.getDirectory(name);
  }
}
