package drjava.superversion.filesystem;

import drjava.util.Logger;
import drjava.util.StringUtil;

import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;

public class CachedFileSystem implements FileSystem {
  private FileSystem fileSystem;
  private Map<String,CachedDirectory> dirs = new HashMap<String,CachedDirectory>();
  private Map<String,DirectoryContents> contents = new HashMap<String,DirectoryContents>();

  public CachedFileSystem(FileSystem fileSystem) {
    this.fileSystem = fileSystem;
  }

  public MutableDirectory getDirectory(String absolutePath) {
    if (dirs.containsKey(absolutePath))
      return dirs.get(absolutePath);
    CachedDirectory dir = new CachedDirectory(this, fileSystem.getDirectory(absolutePath));
    dirs.put(absolutePath, dir);
    return dir;
  }

  public boolean isLocal(String absolutePath) {
    return fileSystem.isLocal(absolutePath);
  }

  public void clearCache() {
    dirs = new HashMap<String, CachedDirectory>();
    contents = new HashMap<String, DirectoryContents>();
  }

  DirectoryContents getContents(MutableDirectory dir, Logger logger) {
    if (contents.containsKey(dir.getPath()))
      return contents.get(dir.getPath());
    DirectoryContents cont = dir.getContents(logger);
    List<Directory> subdirs = new ArrayList<Directory>();
    for (Directory subdir : cont.getDirectories())
      subdirs.add(getDirectory(subdir.getPath()));
    cont = new DirectoryContents(subdirs, new ArrayList<File>(cont.getFiles()));
    contents.put(dir.getPath(), cont);
    return cont;
  }

  public File getFile(MutableDirectory dir, String path) {
    String fullPath = StringUtil.addSlash(dir.getPath()) + path;
    int idx = fullPath.lastIndexOf('/');
    String dirPath = fullPath.substring(0, idx), fileName = fullPath.substring(idx+1);
    if (contents.containsKey(dirPath)) {
      List<File> files = contents.get(dirPath).getFiles();
      for (File file : files)
        if (file.getName().equals(fileName))
          return file;
      return null;
    } else
      return dir.getFile(path);
  }

}
