package drjava.superversion.filesystem;

import drjava.superversion.DataImpl;
import drjava.superversion.SuperversionException;
import drjava.superversion.api.Data;
import drjava.superversion.gui.Actions;
import drjava.superversion.tools.FileUtil;
import drjava.superversion.tools.FingerprintMaker;
import drjava.superversion.tools.IOManagerImpl;
import drjava.util.Logger;
import drjava.util.ProgressControl;
import drjava.vats.FulfilledPromise;
import drjava.vats.Promise;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

public class RealDirectory implements MutableDirectory {
  private java.io.File dir;

  public RealDirectory(String path) {
    dir = new java.io.File(path);
  }

  public FileSystem getFileSystem() {
    return new RealFileSystem();
  }

  public DirectoryContents getContents(Logger logger) {
    List<File> list = new ArrayList<File>();
    List<Directory> dirs = new ArrayList<Directory>();
    java.io.File[] files = dir.listFiles();
    if (files == null)
      logger.logWarning("Directory unreadable: " + dir.getAbsolutePath());
    else try {
      String canonicalPath = dir.getCanonicalPath() + java.io.File.separator;
      for (final java.io.File f : files) {
        if (f.isDirectory() && f.getCanonicalPath().equals(canonicalPath + f.getName()))
          dirs.add(new RealDirectory(f.getPath()));
      }

      for (final java.io.File f : files) {
        if (!f.isDirectory())
          list.add(makeFile(f));
      }
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    return new DirectoryContents(dirs, list);
  }

  public Promise<DirectoryContents> getContents_promise(Logger logger, ProgressControl progressControl) {
    return new FulfilledPromise<DirectoryContents>(getContents(logger)); // TODO
  }

  public String getPath() {
    return dir.getPath();
  }

  public String getName() {
    return dir.getName();
  }

  public File getFile(String path) {
    java.io.File f = new java.io.File(dir, path);
    return f.isFile() ? makeFile(f) : null;
  }

  public boolean isLocal() {
    return true;
  }

  private static File makeFile(final java.io.File f) {
    return new RealFile() {
      public String getName() {
        return f.getName();
      }

      public Data getContents() {
        return new Data(0) {
          public byte[] getBytes() {
            try {
              return FileUtil.loadFile(f);
            } catch (IOException e) {
              throw new SuperversionException(e);
            }
          }

          public Data freeze() {
            return new DataImpl(getBytes(), isBinary());
          }

          public void ensureAvailability() {
          }

          public boolean isBinary() {
            try {
              return FileUtil.isBinary(f);
            } catch (IOException e) {
              throw new SuperversionException(e);
            }
          }

          public long getSize() {
            return f.length();
          }

          public long getFingerprint() {
            return FingerprintMaker.getFingerprint(f, isBinary());
          }

          public Promise<byte[]> getBytes_promise() {
            return IOManagerImpl.getInstance().loadFile(f);
          }
        };
      }

      public long getDate() {
        return f.lastModified();
      }

      public int[] getRevision() {
        return null;
      }

      public java.io.File getNativeFile() {
        return f;
      }
    };
  }

  public void deleteFile(String path) {
    new java.io.File(dir, path).delete();
  }

  public void create() {
    dir.mkdirs();
  }

  public boolean exists() {
    return dir.exists();
  }

  public MutableDirectory getDirectory(String name) {
    return getFileSystem().getDirectory(new java.io.File(dir, name).getPath());
  }

  public void saveFile(String path, byte[] contents) {
    try {
      java.io.File file = new java.io.File(dir, path);
      file.getParentFile().mkdirs();
      Actions.saveFile(file, contents);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }
}
