package drjava.superversion.filesystem;

import drjava.superversion.DataImpl;
import drjava.superversion.DownloadFailedException;
import drjava.superversion.SuperversionException;
import drjava.superversion.api.Data;
import drjava.superversion.tools.FingerprintMaker;
import drjava.vats.Function;
import drjava.vats.Promise;

/**
 * assumption: binary flag is preserved; binary files are not modified
 */
public abstract class WrappedFile implements File {
  private File file;

  /**
   * assumption: file.getContents() never throws DownloadFailedException
   */
  public WrappedFile(File file) {
    this.file = file;
  }

  public String getName() {
    return file.getName();
  }

  public Data getContents() {
    final Data contents = file.getContents();
    if (contents.isBinary()) return contents;
    return new WrappedData(contents) {
      private boolean isBinary, isBinaryCached;

      private boolean isBinaryCached() {
        if (!isBinaryCached) {
          isBinary = isBinary();
          isBinaryCached = true;
        }
        return isBinary;
      }

      /** @deprecated */
      @SuppressWarnings({"deprecation"})
      public byte[] getBytes() throws DownloadFailedException {
        if (!isBinaryCached())
          return wrapContents(contents.getBytes());
        else
          return contents.getBytes();
      }

      public Promise<byte[]> getBytes_promise() {
        return contents.getBytes_promise().applyQ(new Function<byte[], byte[]>() {
          public byte[] get(byte[] bytes) {
            if (!isBinaryCached())
              return wrapContents(bytes);
            else
              return bytes;
          }
        });
      }

      public long getFingerprint() {
        try {
          return FingerprintMaker.getFingerprint(getBytes(), false);
        } catch (DownloadFailedException e) {
          throw new SuperversionException(e);
        }
      }

      public Data freeze() throws DownloadFailedException {
        return new DataImpl(getBytes(), 0, false);
      }
    };
  }

  protected abstract byte[] wrapContents(byte[] contents);

  public long getDate() {
    return file.getDate();
  }

  public int[] getRevision() {
    return file.getRevision();
  }

  public File getFile() {
    return file;
  }
}
