package drjava.superversion.tools;

import drjava.util.StringUtil;

import java.util.*;
import java.io.*;
import java.nio.charset.*;
import java.nio.*;

public class FileUtil {

  public static boolean isBinary(byte[] bytes) {
    return isBinaryRaw(bytes) && !isUTF8(bytes);
  }

  public static boolean isBinaryRaw(byte[] bytes) {
   for (int i = 0; i < bytes.length; i++) {
      int b = ((int) bytes[i]) & 0xFF;
      if (isBinaryRaw(b)) {
        //System.out.println("char " + b + " at " + i + " is not printable");
        return true;
      }
    }
    return false;
  }

  private static boolean isBinaryRaw(int b) {
    return !((b >= 0x20 && b <= 0x7E) || b == '\t' || b == '\n' || b == '\r' || b >= 0xA0);
  }

  private static boolean isUTF8(byte[] bytes) {
    //CharBuffer charBuffer = Charset.forName("UTF-8").decode(ByteBuffer.wrap(bytes));
    CharsetDecoder decoder = Charset.forName("UTF-8").newDecoder();
    decoder.onMalformedInput(CodingErrorAction.REPORT);
    decoder.onUnmappableCharacter(CodingErrorAction.REPORT);
    try {
      CharBuffer charBuffer = decoder.decode(ByteBuffer.wrap(bytes));
      return !isBinaryRaw(charBuffer.array(), charBuffer.length());
    } catch (CharacterCodingException e) {
      return false;
    }
  }

  private static boolean isBinaryRaw(char[] chars, int length) {
    for (int i = 0; i < length; i++) {
      int c = ((int) chars[i]) & 0xFFFF;
      if (c <= 0xFF && isBinaryRaw(c))
        return true;
    }
    return false;
  }

  /*private static boolean isUTF8(byte[] bytes) {
    try {
      String s = new String(bytes, "UTF-8");
      for (int i = 0; i < s.length(); i++)
        if (!Character.isDefined(s.charAt(i)))
          return false;
      return true;
    } catch (UnsupportedEncodingException e) {
      throw new RuntimeException(e);
    }
  }*/

  public static boolean isBinary(File f) throws IOException {
    DataInputStream dis = new DataInputStream(new FileInputStream(f));
    byte[] buf = new byte[(int) Math.min(f.length(), 1024)];
    dis.readFully(buf);
    dis.close();
    return isBinary(buf);
  }

  public static byte[] loadFile(File f) throws IOException {
    int len = (int) f.length();
    byte[] buf = new byte[len];
    DataInputStream in = new DataInputStream(new FileInputStream(f));
    in.readFully(buf);
    in.close();
    return buf;
  }
}
