package drjava.superversion.tools;

import java.security.*;
import java.io.*;

import drjava.superversion.*;

public class FingerprintMaker {
  public static int BUFLEN = 128 * 1024;

  public static long getFingerprint(byte[] data, boolean binary) {
    if (!binary)
      data = normalizeNewlines(data);
    try {
      MessageDigest md = MessageDigest.getInstance("MD5");
      byte[] digest = md.digest(data);
      return digestToLong(digest);
    } catch (NoSuchAlgorithmException e) {
      throw new SuperversionException(e);
    }
  }

  public static long getFingerprint_debug(byte[] data, boolean binary) {
    if (!binary)
      data = normalizeNewlines(data);
    try {
      byte[] digest = MessageDigest.getInstance("MD5").digest();
      for (int i = 1; i <= data.length; i++) {
        byte[] dataPart = new byte[i];
        System.arraycopy(data, 0, dataPart, 0, i);
        MessageDigest md = MessageDigest.getInstance("MD5");
        digest = md.digest(dataPart);
        System.out.print("char " + i + " (" + Integer.toHexString((int) data[i-1] & 0xFF) + "): ");
        for (int j = 0; j < 4; j++)
          System.out.print(Integer.toHexString((int) digest[j] & 0xFF) + " ");
        System.out.println();
      }
      return digestToLong(digest);
    } catch (NoSuchAlgorithmException e) {
      throw new SuperversionException(e);
    }
  }

  private static long digestToLong(byte[] digest) {
    long l = 0;
    for (int i = 0; i < 8; i++)
      l = (l << 8) | (((long) digest[i]) & 0xFF);
    return l;
  }

  /** \r\n or \r => \n */
  private static byte[] normalizeNewlines(byte[] data) {
    int len = data.length;
    for (int i = 0; i < data.length - 1; i++)
      if (data[i] == '\r' && data[i + 1] == '\n')
        --len;

    byte[] data2 = new byte[len];
    int j = 0;
    for (int i = 0; i < data.length; i++) {
      if (data[i] == '\r' && i < data.length - 1 && data[i + 1] == '\n') {
        data2[j++] = '\n';
        i++;
      } else if (data[i] == '\n' || data[i] == '\r')
        data2[j++] = '\n';
      else
        data2[j++] = data[i];
    }
    return data2;
  }

  public static long getFingerprint(File f, boolean binary) {
    try {
      MessageDigest md = MessageDigest.getInstance("MD5");
      byte[] carry = new byte[0];
      BufferedInputStream in = new BufferedInputStream(new FileInputStream(f), BUFLEN);
      try {
        byte[] buf = new byte[BUFLEN];
        while (true) {
          int bytes = in.read(buf);
          if (bytes <= 0) break;
          int l = bytes;
          while (l > 0 && (buf[l - 1] == '\r' || buf[l - 1] == '\n'))
            --l;
          byte[] buf2 = new byte[carry.length + l];
          System.arraycopy(carry, 0, buf2, 0, carry.length);
          System.arraycopy(buf, 0, buf2, carry.length, l);
          carry = new byte[bytes - l];
          System.arraycopy(buf, l, carry, 0, carry.length);
          if (!binary)
            buf2 = normalizeNewlines(buf2);
          md.update(buf2);
        }
      } finally {
        in.close();
      }
      if (!binary)
        carry = normalizeNewlines(carry);
      return digestToLong(md.digest(carry));
    } catch (NoSuchAlgorithmException e) {
      throw new SuperversionException(e);
    } catch (IOException e) {
      throw new SuperversionException(e);
    }
  }
}