/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package drjava.tools;

import drjava.util.FileUtil;

import java.io.*;
import java.util.*;

public class AddLicenseHeader {
  private String header;
  private boolean execute;
  private List<String> oldHeaders = new ArrayList<String>();

  public AddLicenseHeader(String header) {
    this.header = header;
  }

  public void scan(File dir) throws IOException {
    for (File f : dir.listFiles()) {
      if (f.isDirectory() && !f.getName().startsWith("."))
        scan(f);
      else if (f.isFile() && f.getName().endsWith(".java"))
        checkFile(f);
    }
  }

  private void checkFile(File file) throws IOException {
    String contents = FileUtil.loadTextFile(file.getPath());
    String newContents = header + stripHeader(contents);
    if (!contents.equals(newContents))
      if (execute)
        FileUtil.saveTextFile(file.getPath(), newContents);
      else
        System.out.println("Needs license update: " + file.getPath());
  }

  private String stripHeader(String contents) {
    if (contents.startsWith(header)) return contents.substring(header.length());
    for (String oldHeader : oldHeaders) {
      if (contents.startsWith(oldHeader)) {
        System.out.println("Old header found");
        return contents.substring(oldHeader.length());
      }
    }
    return contents;
  }

  public boolean isExecute() {
    return execute;
  }

  public void setExecute(boolean execute) {
    this.execute = execute;
  }

  public void addOldHeader(String header) {
    oldHeaders.add(header);
  }
}