package eye.eye01;

import prophecy.common.image.RGBImage;

import java.awt.*;
import java.awt.image.BufferedImage;

public class TextPainter2 {
  private FontMetrics metrics;
  private Font font;
  private static final int defaultInset = 5;

  public TextPainter2(Font font) {
    this.font = font;

    // Make 1x1 image to get the font metrics...
    BufferedImage dummyImage = new BufferedImage(1, 1, BufferedImage.TYPE_BYTE_BINARY);
    Graphics g = dummyImage.getGraphics();
    metrics = g.getFontMetrics(font);
  }

  public RGBImage makeImage(String text) {
    return makeImage(text, defaultInset);
  }

  public RGBImage makeImage(String text, int inset) {
    String[] lines = text.split("\r?\n");
    Dimension size = sizeNeeded(lines);
    BufferedImage image = new BufferedImage(size.width+inset*2, size.height+inset*2, BufferedImage.TYPE_BYTE_BINARY);
    Graphics g = image.getGraphics();
    g.setColor(Color.white);
    g.fillRect(0, 0, image.getWidth(), image.getHeight());
    g.setColor(Color.black);
    g.setFont(font);

    int y = metrics.getAscent()+inset;
    for (String line : lines) {
      g.drawString(line, inset, y);
      y += metrics.getHeight();
    }

    return new RGBImage(image);
  }

  private Dimension sizeNeeded(String[] lines) {
    int w = 0;
    for (String line : lines) {
      w = Math.max(w, metrics.stringWidth(line));
    }
    return new Dimension(w, metrics.getHeight()*lines.length);
  }
}
