package eye.eye02;

import drjava.util.*;
import eye.eye01.EyeGuiUtil;
import eye.eye01.ScrollableImage;
import eye.eye01.TextPainter2;
import eye.eye03.RecognizerInputType;
import eye.eye03.Recognizers;
import eyedev._01.OCRUtil;
import eyedev._08.FontLearner;
import eyedev._08.TopList;
import eyedev._09.WithFlexibleSegmenter;
import prophecy.common.ClassData;
import prophecy.common.gui.CancelButton;
import prophecy.common.gui.Sheet;
import prophecy.common.image.RGBImage;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

public class MakeRecognizerDialog extends JFrame {
  private ScrollableImage scrollableImage;
  private JTextArea taResults;
  private JRadioButton rbAllSizes;
  private JButton btnTest;
  private String recognizer, recognizerFontName;
  private FontSelector fontSelector;
  private static final String previewText = "ABCDEFGH";
  private ClassData classData;
  private JButton btnSave;
  private Recognizers recognizers;
  private JRadioButton rbAllChars;

  public MakeRecognizerDialog(Recognizers recognizers) {
    this.recognizers = recognizers;
    setTitle("Learn a font");
    setSize(600, 500);
    GUIUtil.centerOnScreen(this);

    classData = ClassData.get(this);

    JPanel buttons = new JPanel(LetterLayout.stalactite());

    JButton btnLearn = new JButton("Learn font");
    btnLearn.addActionListener(EyeGuiUtil.actionListener(this, "start"));
    buttons.add(btnLearn);

    btnTest = new JButton("Test recognizer");
    btnTest.addActionListener(EyeGuiUtil.actionListener(this, "test"));
    btnTest.setEnabled(false);
    buttons.add(btnTest);

    btnSave = new JButton("Save recognizer");
    btnSave.addActionListener(EyeGuiUtil.actionListener(this, "save"));
    btnSave.setEnabled(false);
    buttons.add(btnSave);

    buttons.add(new CancelButton("Done"));

    Sheet sheet = new Sheet();
    sheet.getSheetLayout().setBorder(0);

    fontSelector = new FontSelector();
    fontSelector.selectFont(classData.getString("fontName"));
    fontSelector.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent itemEvent) {
        classData.setString("fontName", fontSelector.getSelectedFontName());
        classData.save();
        updatePreviewImage();
      }
    });

    //sheet.addLeftAlignedLabel("Font to be learned:");
    sheet.addComponent(GUIUtil.withLabel("Font to be learned:", fontSelector));

    // character set selection

    ButtonGroup group = new ButtonGroup();
    JRadioButton rbUppercase = new JRadioButton("Uppercase only");
    rbAllChars = new JRadioButton("Upper + lower + punctuation", true);
    group.add(rbUppercase);
    group.add(rbAllChars);

    sheet.addComponent(new Line(rbUppercase, rbAllChars));

    // sizes selection

    group = new ButtonGroup();
    JRadioButton rbOneSize = new JRadioButton("Learn only size 30", true);
    rbAllSizes = new JRadioButton("Learn 8 sizes (30-100)");
    group.add(rbOneSize);
    group.add(rbAllSizes);

    sheet.addComponent(new Line(rbOneSize, rbAllSizes));

    sheet.addBlueHeading("Font preview (size 30)");
    scrollableImage = new ScrollableImage();
    scrollableImage.setMinimumSize(new Dimension(100, 65));
    sheet.addComponent(scrollableImage);

    sheet.addBlueHeading("Results");

    taResults = new JTextArea();
    taResults.setLineWrap(true);
    taResults.setWrapStyleWord(true);

    getContentPane().setLayout(new LetterLayout("SSB", "RRB", "RRB").setBorder(10));
    getContentPane().add("B", buttons);
    getContentPane().add("S", sheet.getPanel());
    getContentPane().add("R", new JScrollPane(taResults));

    updatePreviewImage();
  }

  public void updatePreviewImage() {
    try {
      int size = 30;
      Font font = getFont(size);

      RGBImage image = new TextPainter2(font).makeImage(previewText);
      scrollableImage.setImage(image);
    } catch (Throwable e) {
      Errors.report(e);
    }
  }

  private Font getFont(int size) throws Exception {
    Font font = fontSelector.getSelectedFont();
    if (font == null) return null;
    font = font.deriveFont((float) size);
    return font;
  }

  @SuppressWarnings({"UnusedDeclaration"})
  public void start() {
    String result;
    try {
      FontLearner fontLearner = new FontLearner();
      if (rbAllSizes.isSelected()) {
        for (int size = 30; size <= 100; size += 10)
          fontLearner.addFont(getFont(size));
      } else
        fontLearner.addFont(getFont(30));
      fontLearner.setAllChars(rbAllChars.isSelected());
      fontLearner.go();

      TopList toplist = fontLearner.getToplist();
      if (toplist.isEmpty())
        result = "Font could not be learned";
      else {
        result = "Font has been learned. Achieved accuracy: " +
          StringUtil.formatDouble(toplist.getTopScore()*100, 1) + "%\n\n" +
          "Recognizer code (" + toplist.get(0).length() + " chars):\n\n" + toplist.get(0);
        recognizer = toplist.get(0);
        recognizer = OCRUtil.getImageReaderDescription(
          new WithFlexibleSegmenter(recognizer));
        recognizerFontName = fontSelector.getSelectedFontName();
        btnTest.setEnabled(true);
        btnSave.setEnabled(true);
      }
    } catch (Throwable e) {
      e.printStackTrace();
      result = e.toString();
    }

    taResults.setText(result);
    taResults.setCaretPosition(0);
  }

  @SuppressWarnings({"UnusedDeclaration"})
  public void test() {
    if (recognizer != null)
      new RecognitionDialog(recognizer, recognizerFontName + " recognizer (user-made)",
        recognizerFontName).setVisible(true);
  }

  @SuppressWarnings({"UnusedDeclaration"})
  public void save() {
    try {
      if (recognizer != null) {
        EyeDialogs.saveRecognizer(this, recognizers, recognizer, recognizerFontName,
          RecognizerInputType.character, null, true);
      }
    } catch (Throwable e) {
      Errors.report(e);
    }
  }
}
