package eye.eye03;

import drjava.util.*;
import eye.eye01.EyeGuiUtil;
import eye.eye01.MakeImageDialog;
import eye.eye02.MakeRecognizerDialog;
import eye.eye04.ChallengesDialog;
import eye.eye05.DebugInfoWindow;
import eye.eye05.RecognizableImage;
import eyedev._01.DebugItem;
import eyedev._01.ImageReader;
import eyedev._01.OCRUtil;
import eyedev._13.CompareImages;
import eyedev._13.StartIRDialog;
import eyedev._13.WithTextFinder1;
import eyedev._14.Fratboy;
import prophecy.common.image.RGBImage;
import prophecy.m.m28.CenteredLine;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.List;

public class Main {
  static final String version = "Alpha 7";

  protected EyeData eyeData;
  protected Recognizers recognizers;

  private RGBImage image;
  private List<DebugItem> debugInfo;

  protected JFrame frame;
  private JMenuBar menuBar;
  protected JPanel buttons;
  //private JButton btnRecognize;
  private RecognizableImage recognizableImage;
  private ActionListener openImage;
  private ActionListener makeImage;
  protected JMenu extrasMenu;
  private JButton btnDebug;
  private RecognizerSelector recognizerSelector;
  private JTextArea textArea;

  public static void main(String[] args) {
    //Prophecy.systemLookAndFeel();
    new Main();
  }

  public Main() {
    miscInit();
    makeFrame();
    showFrame();
  }

  public void miscInit() {
    //UIManager.getDefaults().setDefaultLocale(Locale.ENGLISH);
    EyeEnv.setUp();
    eyeData = new EyeData();
    recognizers = eyeData.recognizers;
  }

  protected void makeFrame() {
    frame = new JFrame("Eye (" + version + ")");
    frame.setSize(700, 500);

    ScalingSplitPane splitPane = new ScalingSplitPane(ScalingSplitPane.VERTICAL_SPLIT, 0.7f);
    recognizableImage = new RecognizableImage();

    String imagePath = eyeData.getLastImagePath();
    if (imagePath != null) {
      JButton btnLoadLastImage = new JButton("Use same image as last time");
      btnLoadLastImage.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent actionEvent) {
          try {
            String path = eyeData.getLastImagePath();
            if (path != null)
              openImage(new File(path));
          } catch (Throwable e) {
            Errors.report(e);
          }
        }
      });
      recognizableImage.setComponent(new MiddleComponent(new CenteredLine(btnLoadLastImage)));
    }

    recognizerSelector = new RecognizerSelector(recognizers);
    recognizerSelector.selectRecognizer(eyeData.getDefaultRecognizerName());
    recognizerSelector.addTriggerListener(frame);
    recognizerSelector.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent itemEvent) {
        if (itemEvent.getStateChange() == ItemEvent.SELECTED)
          recognize();
      }
    });

    JPanel topPanel = new JPanel(new LetterLayout("N", "I", "I").setSpacing(10));
    topPanel.add("I", GUIUtil.withTitle("Input image", recognizableImage));
    /*JMultilineLabel note = new JMultilineLabel("Note: This window uses the Eye standard recognizer trained on the Arial font. " +
      "To recognize other fonts, please select \"Extras/Learn a font...\" or \"Recognizers\".") {
      public Dimension getMinimumSize() {
        return new Dimension(100, 50);
      }
    };
    topPanel.add("N", note);*/
    topPanel.add("N", GUIUtil.withLabel("Recognizer to use:", recognizerSelector));
    splitPane.setTopComponent(/*GUIUtil.withTitle(
      note,
      GUIUtil.withTitle("Input image", scrollableImage))*/ topPanel);

    textArea = new JTextArea();
    /*JPanel bottomPanel = new JPanel(new LetterLayout("R", "T", "T"));
    bottomPanel.add("R", GUIUtil.withLabel("Recognizer:", cbRecognizer));
    bottomPanel.add("T", GUIUtil.withTitle("Recognized text", new JScrollPane(textArea)));*/
    splitPane.setBottomComponent(GUIUtil.withTitle("Recognized text", new JScrollPane(textArea)));

    ActionListener openImage = EyeGuiUtil.actionListener(this, "openImage");
    ActionListener makeImage = EyeGuiUtil.actionListener(this, "makeImage");

    buttons = new JPanel(LetterLayout.stalactite());
    JButton btnOpenImage = new JButton("Load image...");
    this.openImage = openImage;
    btnOpenImage.addActionListener(this.openImage);
    buttons.add(btnOpenImage);

    JButton btnMakeImage = new JButton("Make image...");
    this.makeImage = makeImage;
    btnMakeImage.addActionListener(this.makeImage);
    buttons.add(btnMakeImage);

    /*btnRecognize = new JButton("Recognize text");
    btnRecognize.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        try {
          recognize();
        } catch (Throwable t) {
          showError(t);
        }
      }
    });
    btnRecognize.setEnabled(false);
    buttons.add(btnRecognize);*/

    JPanel mainPanel = new JPanel(new LetterLayout("CCB").setBorder(10));
    mainPanel.add("C", splitPane);
    mainPanel.add("B", buttons);
    frame.getContentPane().add(mainPanel);

    menuBar = new JMenuBar();

    makeMenus();
    frame.setJMenuBar(menuBar);

    buttons.add(new JPanel());

    JButton button = new JButton("Manage recognizers");
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        manageRecognizers();
      }
    });
    buttons.add(button);

    buttons.add(new JPanel());

    /*button = new JButton("Challenges");
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        challenges();
      }
    });
    buttons.add(button);*/

    buttons.add(new JPanel());

    btnDebug = new JButton("Debug");
    btnDebug.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        debug();
      }
    });
    btnDebug.setEnabled(false);
    buttons.add(btnDebug);
  }

  protected void showFrame() {
    GUIUtil.centerOnScreen(frame);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible(true);
  }

  protected void makeMenus() {
    JMenuItem mi;

    JMenu infoMenu = new JMenu("Info");
    mi = new JMenuItem("About Eye...");
    mi.addActionListener(EyeGuiUtil.actionListener(this, "info"));
    infoMenu.add(mi);

    JMenu imageMenu = new JMenu("Image");
    mi = new JMenuItem("Load image...");
    mi.addActionListener(openImage);
    imageMenu.add(mi);
    mi = new JMenuItem("Make image...");
    mi.addActionListener(makeImage);
    imageMenu.add(mi);

    extrasMenu = new JMenu("Extras");
    mi = new JMenuItem("Learn a font...");
    mi.addActionListener(EyeGuiUtil.actionListener(this, "makeRecognizer"));
    extrasMenu.add(mi);

    menuBar.add(infoMenu);
    menuBar.add(imageMenu);
    menuBar.add(extrasMenu);

    mi = new JMenuItem("Manage recognizers");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        manageRecognizers();
      }
    });
    extrasMenu.add(mi);

    mi = new JMenuItem("Challenges");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        challenges();
      }
    });
    extrasMenu.add(mi);

    JMenu newStuff = new JMenu("New stuff");
    mi = new JMenuItem("Interactive recognition...");
    mi.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        interactiveRecognition();
      }
    });
    newStuff.add(mi);

    menuBar.add(newStuff);
  }

  private void interactiveRecognition() {
    new StartIRDialog(recognizers).setVisible(true);
  }

  /*private void showError(Throwable t) {
    t.printStackTrace();
    JOptionPane.showMessageDialog(frame, t.toString());
  }*/

  @SuppressWarnings({"UnusedDeclaration"})
  public void openImage() throws IOException {
    JFileChooser fileChooser = new JFileChooser();
    fileChooser.setCurrentDirectory(new File("examples"));
    if (fileChooser.showDialog(frame, "Open image") == JFileChooser.APPROVE_OPTION) {
      File file = fileChooser.getSelectedFile();
      if (file != null)
        openImage(file);
    }
  }

  @SuppressWarnings({"UnusedDeclaration"})
  public void makeImage() {
    MakeImageDialog dialog = new MakeImageDialog(frame);
    dialog.setModal(true);
    dialog.setVisible(true);
    if (dialog.image != null)
      setImage(dialog.image);
  }

  private void openImage(File file) throws IOException {
    eyeData.setLastImagePath(file.getPath());
    BufferedImage image = ImageIO.read(file);
    setImage(new RGBImage(image));
  }

  private void setImage(RGBImage image) {
    this.image = image;
    recognizableImage.setImage(image);

    //btnRecognize.setEnabled(true);
    recognize();
  }

  private void recognize() {
    try {
      RecognizerInfo recognizerInfo = recognizerSelector.getSelectedRecognizer();
      if (recognizerInfo == null) return;
      eyeData.setDefaultRecognizerName(recognizerInfo.getName());

      if (image == null) return;
      
      Tree code = recognizerInfo.getCode();
      System.out.println(code);
      ImageReader recognizer = OCRUtil.makeImageReader(code);

      // If it's a character recognizer of the right type => add fratboy segmenter
      if (recognizer instanceof CompareImages) {
        recognizer = new Fratboy((CompareImages) recognizer);
      } else if (recognizerInfo.getInputType() == RecognizerInputType.character) {
        // If it's a character recognizer => add default segmenter
        recognizer = new WithTextFinder1(recognizer.getDescription());
      }

      recognizer.setCollectDebugInfo(true);
      String text = recognizer.readImage(image.toBW());
      debugInfo = recognizer.getDebugInfo();
      btnDebug.setEnabled(debugInfo != null);
      recognizableImage.setDebugInfo(debugInfo);

      if (text == null) text = "";
      textArea.setText(text);
    } catch (Throwable e) {
      Errors.report(e);
    }
  }

  @SuppressWarnings({"UnusedDeclaration"})
  public void makeRecognizer() {
    new MakeRecognizerDialog(recognizers).setVisible(true);
  }

  public void manageRecognizers() {
    try {
      new RecognizersDialog(recognizers).setVisible(true);
    } catch (Throwable e) {
      Errors.report(e);
    }
  }

  private void challenges() {
    try {
      new ChallengesDialog(ChallengesDialog.standardChallenges(), recognizers).setVisible(true);
    } catch (Throwable e) {
      Errors.report(e);
    }
  }

  private void debug() {
    try {
      new DebugInfoWindow(debugInfo).setVisible(true);
    } catch (Throwable e) {
      Errors.report(e);
    }
  }

  @SuppressWarnings({"UnusedDeclaration"})
  public void info() {
    JOptionPane.showMessageDialog(frame, "Eye " + version + "\n" +
      "by Stefan Reich\n\n" +
      "http//eyeocr.sf.net", "About Eye", JOptionPane.INFORMATION_MESSAGE);
  }
}
