package eye.eye05;

import drjava.util.ObjectUtil;
import drjava.util.Trigger;
import eyedev._01.DebugItem;
import eyedev._09.Subrecognition;
import prophecy.common.gui.SingleComponentPanel;
import prophecy.common.image.ImageSurface;
import prophecy.common.image.RGBImage;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.List;

public class RecognizableImage extends SingleComponentPanel {
  private RGBImage image;
  private double zoom = 1.0;
  private Subrecognition markedSubrecognition;
  protected java.util.List<DebugItem> debugInfo;
  private ImageSurface imageSurface;
  private boolean selectableBoxes;
  private Subrecognition selectedSubrecognition;
  public final Trigger markChangeTrigger = new Trigger();
  public final Trigger selectionChangeTrigger = new Trigger();
  private boolean doubleBuffering;

  public RecognizableImage() {
    setImage(null);
  }

  public RecognizableImage(RGBImage image) {
    setImage(image);
  }

  public void setImage(RGBImage image) {
    this.image = image;
    if (image == null) {
      setComponent(new JScrollPane());
      imageSurface = null;
    } else {
      imageSurface = new ImageSurface(image) {
        @Override
        protected void fillPopupMenu(JPopupMenu menu) {
          super.fillPopupMenu(menu);
          RecognizableImage.this.fillPopupMenu(menu);
        }

        public void render(int w, int h, Graphics2D g) {
          super.render(w, h, g);
          g.scale(imageSurface.getZoomX(), imageSurface.getZoomY());
          renderOverlays(g);
        }
      };

      imageSurface.addMouseMotionListener(new MouseMotionListener() {
        public void mouseDragged(MouseEvent e) {
        }

        public void mouseMoved(MouseEvent e) {
          surfaceMouseMove(e);
        }
      });

      imageSurface.addMouseListener(new MouseAdapter() {
        public void mouseEntered(MouseEvent e) {
          surfaceMouseMove(e);
        }

        public void mouseExited(MouseEvent e) {
          setMarkedSubrecognition(null);
        }

        @Override
        public void mousePressed(MouseEvent e) {
          surfaceMouseClick(e);
        }
      });

      imageSurface.setZoom(zoom);
      if (doubleBuffering)
        // This makes Surface create a buffer image
        imageSurface.setImageType(2);
      JScrollPane scrollPane = new JScrollPane(imageSurface);
      setComponent(scrollPane);
    }
  }

  private Subrecognition findSubrecognition(MouseEvent e) {
    Point p = new Point((int) (e.getX()/imageSurface.getZoomX()), (int) (e.getY()/imageSurface.getZoomY()));
    if (debugInfo != null)
      for (DebugItem item : debugInfo)
        if (item.data instanceof Subrecognition) {
          Subrecognition s = (Subrecognition) item.data;
          Rectangle r = new Rectangle(s.clip);
          r.grow(1, 1);
          if (r.contains(p))
            return s;
        }
    return null;
  }

  private void surfaceMouseMove(MouseEvent e) {
    setMarkedSubrecognition(findSubrecognition(e));
  }

  private void surfaceMouseClick(MouseEvent e) {
    if (selectableBoxes)
      setSelectedSubrecognition(findSubrecognition(e));
  }

  private void fillPopupMenu(JPopupMenu menu) {
    /*if (markedSubrecognition != null) {
      JMenuItem mi = new JMenuItem("Correct letter");
      mi.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent actionEvent) {
          new AddCorrectionDialog(markedSubrecognition).setVisible(true);
        }
      });
      menu.insert(mi, 0);
      menu.insert(new JSeparator(), 1);
    }*/
  }

  public RGBImage getImage() {
    return image;
  }

  public void setZoom(double zoom) {
    if (zoom == this.zoom) return;
    this.zoom = zoom;
    if (image != null)
      setImage(image);
  }

  protected void renderOverlays(Graphics2D g) {
    if (markedSubrecognition != null)
      drawBox(g, markedSubrecognition, Color.green, 2);
    if (selectedSubrecognition != null)
      drawBox(g, selectedSubrecognition, Color.blue, 2);
  }

  public void drawBox(Graphics2D g, Subrecognition subrecognition, Color color, int strokeWidth) {
    g.setColor(color);
    g.setStroke(new BasicStroke((float) strokeWidth));
    Rectangle r = subrecognition.clip;
    g.drawRect(r.x-1, r.y-1, r.width+2, r.height+2);
  }

  private void setMarkedSubrecognition(Subrecognition subrecognition) {
    if (!ObjectUtil.equal(subrecognition, markedSubrecognition)) {
      markedSubrecognition = subrecognition;
      imageSurface.repaint();
      markChangeTrigger.trigger();
    }
  }

  public void setSelectedSubrecognition(Subrecognition subrecognition) {
    if (!ObjectUtil.equal(subrecognition, selectedSubrecognition)) {
      selectedSubrecognition = subrecognition;
      imageSurface.repaint();
      selectionChangeTrigger.trigger();
    }
  }

  public void setDebugInfo(List<DebugItem> debugInfo) {
    this.debugInfo = debugInfo;
    selectedSubrecognition = null;
  }

  public void setSelectableBoxes(boolean selectableBoxes) {
    this.selectableBoxes = selectableBoxes;
  }

  public Subrecognition getSelectedSubrecognition() {
    return selectedSubrecognition;
  }

  public Subrecognition getMarkedSubrecognition() {
    return markedSubrecognition;
  }

  public void setDoubleBuffering(boolean b) {
    doubleBuffering = b;
    setImage(image);
  }
}
