package eyedev._01;

import drjava.util.Tree;
import prophecy.common.image.BWImage;

import java.util.ArrayList;
import java.util.List;

public abstract class ImageReader implements Describable {
  protected boolean collectDebugInfo;
  protected List<DebugItem> debugInfo;

  public String getDescription() {
    return OCRUtil.getImageReaderDescription(this);
  }

  public void setCollectDebugInfo(boolean collectDebugInfo) {
    this.collectDebugInfo = collectDebugInfo;
    debugInfo = collectDebugInfo ? new ArrayList<DebugItem>() : null;
  }

  public void addDebugItem(DebugItem item) {
    if (collectDebugInfo)
      debugInfo.add(item);
  }

  public List<DebugItem> getDebugInfo() {
    return debugInfo;
  }

  // override this to customize persistence
  public void fromTree(Tree tree) {
  }

  // override this to customize persistence
  public Tree toTree() {
    return OCRUtil.treeFor(this);
  }

  // subclasses override either readImage or extendedReadImage
  public abstract String readImage(BWImage image);

  // subclasses override either readImage or extendedReadImage
  public RecognizedText extendedReadImage(BWImage image) {
    return new RecognizedText(readImage(image));
  }

  protected String delegateRecognition(ImageReader imageReader, BWImage image) {
    imageReader.setCollectDebugInfo(collectDebugInfo);
    String result = imageReader.readImage(image);
    if (collectDebugInfo)
      debugInfo = imageReader.getDebugInfo();
    return result;
  }
}
