package eyedev._08;

import drjava.util.MultiMap;
import eyedev._01.Example;
import eyedev._01.ExampleSet;
import eyedev._01.ImageReader;
import eyedev._01.OCRUtil;
import eyedev._05.Alphabet;
import eyedev._06.SegmentSignature;
import prophecy.common.image.BWImage;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class FontLearner {
  private TopList toplist = new TLBestScore();
  private List<Font> fonts = new ArrayList<Font>();
  private MultiMap<String, BWImage> exampleMap;
  private SimpleExperiment experiment;
  private boolean allChars;
  private Runnable progressListener;

  public FontLearner(Font font) {
    fonts.add(font);
  }

  public FontLearner() {
  }

  public void addFont(Font font) {
    fonts.add(font);
  }

  public void go() {
    ExampleSet exampleSet = new ExampleSet();
    for (Font font : fonts) {
      if (allChars)
        exampleSet.add(Alphabet.makeAllChars(font));
      else
        exampleSet.add(Alphabet.makeAlphabet(font));
    }

    exampleSet.crop();

    // make exampleMap

    exampleMap = new MultiMap<String, BWImage>();
    for (Example example : exampleSet.examples) {
      exampleMap.put(example.text, example.image);
    }

    experiment = new SimpleExperiment(exampleSet);
    experiment.setProgressListener(progressListener);
    experiment.addTopList(toplist);
    experiment.tryRecognizer(new SegmentSignature(exampleSet));

    float lastTopScore = toplist.getTopScore();
    while (lastTopScore < 1f && toplist.size() != 0) {
      tryToFixRecognizer(toplist.get(0), experiment);
      float newScore = toplist.getTopScore();
      if (newScore == lastTopScore)
        break;
      System.out.println("Score improved to " + newScore);
      lastTopScore = newScore;
    }

    toplist.print();
  }

  private void tryToFixRecognizer(String desc, SimpleExperiment experiment) {
    ImageReader recognizer = OCRUtil.makeImageReader(desc);
    for (Example example : experiment.exampleSet.examples) {
      String answer = recognizer.readImage(example.image);
      if (!example.text.equals(answer)) {
        //System.out.println("Recognizer fails on " + example.text + " (gives " + answer + ")");
        if (tryToFix(recognizer, example.text, answer))
          return;
      }
    }
  }

  private boolean tryToFix(ImageReader recognizer, String char1, String char2) {
    List<BWImage> images1 = exampleMap.get(char1);
    List<BWImage> images2 = exampleMap.get(char2);

    boolean fixFound = false;
    for (int featureNr = 0; featureNr < StandardFeatures.getNumberOfFeatures(); featureNr++) {
      Fix fix = FixMaker.makeFix(featureNr, images1, images2, char1, char2, recognizer);
      if (fix != null) {
        experiment.tryRecognizer(fix);
        fixFound = true;
      }
    }
    return fixFound;
  }

  public TopList getToplist() {
    return toplist;
  }

  public void setAllChars(boolean b) {
    allChars = b;
  }

  public String getBestRecognizer() {
    return toplist.getTopEntry();
  }

  public void setProgressListener(Runnable runnable) {
    progressListener = runnable;
  }
}
