package eyedev._09;

import drjava.util.Tree;
import eyedev._01.OCRImageUtil;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class FlexibleSegmenter extends Segmenter {
  private BWImage baseImage;
  private float threshold = 0.5f;

  public FlexibleSegmenter() {
  }

  public List<Segment> segment(BWImage img) {
    baseImage = ImageProcessing.threshold(img, threshold);
    Rectangle box = OCRImageUtil.getBoundingBox(baseImage);
    Point translation = new Point(box.x - 1, box.y - 1);
    baseImage = ImageProcessing.addBorder(OCRImageUtil.trim(baseImage), 1);

    List<Segment> letters = new ArrayList<Segment>();
    int w = baseImage.getWidth(), h = baseImage.getHeight();
    int lastSep = 0;
    int[] lastTrail = null;
    xloop: for (int x1 = 0; x1 < w; x1++) {
      int x = x1;
      int[] trail = new int[h];
      for (int y = 0; y < h-1; y++) {
        //markedImage.setPixel(x, y, new RGB(Color.red));
        trail[y] = x;

        // The parts right of the && are so we don't step diagonally over 1-pixel wide lines
        boolean l = baseImage.getPixel(x-1, y+1) >= threshold && baseImage.getPixel(x-1, y) >= threshold;
        boolean m = baseImage.getPixel(x, y+1) >= threshold;
        boolean r = baseImage.getPixel(x+1, y+1) >= threshold && baseImage.getPixel(x+1, y) >= threshold;
        
        if (m) {
          // ok, proceed downwards
        } else if (l) {
          --x;
        } else if (r) {
          ++x;
        } else
          continue xloop;
      }
      trail[h-1] = x;
      if (x > lastSep+1) {
        Segment letter = cutOutLetter(lastTrail, trail);
        letter.boundingBox.translate(translation.x, translation.y);
        if (letter.image.getWidth() != 0 && letter.image.getHeight() != 0)
          letters.add(letter);
        lastTrail = trail;
      }
      lastSep = x;
    }
    //System.out.println("letters: " + letters.size());
    return letters;
  }

  private Segment cutOutLetter(int[] trail1, int[] trail2) {
    BWImage letter = new BWImage(baseImage.getWidth(), baseImage.getHeight(), 1f);
    for (int y = 0; y < baseImage.getHeight(); y++) {
      int x1 = trail1 == null ? 0 : trail1[y];
      int x2 = trail2[y];
      ImageProcessing.copy(baseImage, x1, y, letter, x1, y, x2-x1, 1);
    }
    return new Segment(SegmentLevel.character, OCRImageUtil.getBoundingBox(letter), OCRImageUtil.trim(letter));
  }

  @Override
  public void fromTree(Tree tree) {
    threshold = tree.getFloat("threshold", threshold);
  }

  @Override
  public Tree toTree() {
    return super.toTree().setFloat("threshold", threshold);
  }

  public float getThreshold() {
    return threshold;
  }

  public void setThreshold(float threshold) {
    this.threshold = threshold;
  }
}
