package eyedev._09;

import drjava.util.Tree;
import eyedev._01.DebugItem;
import eyedev._01.ImageReader;
import eyedev._01.OCRUtil;
import eyedev._01.RecognizedText;
import eyedev._13.CompareImages;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;

import java.awt.*;
import java.util.List;

/** Segment and Recognize - splits image in lines, then in characters,
 *  then applies character recognizer */
public class SaR extends ImageReader {
  private Tree lineFinder, lineSegmenter, charRecognizer;

  public SaR() {
  }

  public SaR(String lineFinderDesc, String lineSegmenterDesc, String charRecognizerDesc) {
    lineFinder = Tree.parse(lineFinderDesc);
    lineSegmenter = Tree.parse(lineSegmenterDesc);
    charRecognizer = Tree.parse(charRecognizerDesc);
  }

  public SaR(Segmenter lineFinder, Segmenter lineSegmenter, ImageReader charRecognizer) {
    this.lineFinder = lineFinder.toTree();
    this.lineSegmenter = lineSegmenter.toTree();
    this.charRecognizer = charRecognizer.toTree();
  }

  public String readImage(BWImage image) {
    //image = ImageProcessing.threshold(image, .5f);

    Segmenter lineFinder = OCRUtil.makeSegmenter(this.lineFinder);
    Segmenter lineSegmenter = OCRUtil.makeSegmenter(this.lineSegmenter);
    ImageReader recognizer = OCRUtil.makeImageReader(charRecognizer);

    List<Segment> lines = lineFinder.segment(image);
    StringBuffer buf = new StringBuffer();

    for (int iLine = 0, linesSize = lines.size(); iLine < linesSize; iLine++) {
      Segment line = lines.get(iLine);
      if (buf.length() != 0) buf.append("\n");

      List<Segment> letters = lineSegmenter.segment(line.image);

      for (int i = 0; i < letters.size(); i++) {
        if (i > 0 && isSpace(letters.get(i - 1).boundingBox, letters.get(i).boundingBox))
          buf.append(" ");

        Segment letter = letters.get(i);
        RecognizedText text = recognizer.extendedReadImage(letter.image);

        if (collectDebugInfo) {
          Rectangle box = new Rectangle(letter.boundingBox);
          box.translate(line.boundingBox.x, line.boundingBox.y);
          addDebugItem(new DebugItem("Line " + (iLine + 1) + " letter " + (i + 1),
            new Subrecognition(box, letter.image, charRecognizer.toString(), text)));
        }

        buf.append(text == null || text.text == null ? "?" : text.text);
      }
    }
    return buf.toString();
  }

  private boolean isSpace(Rectangle leftChar, Rectangle rightChar) {
    int h = Math.max(leftChar.height, rightChar.height);
    int distance = rightChar.x-(leftChar.x+leftChar.width);
    float threshold = h/3;
    //System.out.println("threshold: " + threshold + ", distance: " + distance);
    return distance >= threshold;
  }

  public void fromTree(Tree tree) {
    lineFinder = tree.get(0);
    lineSegmenter = tree.get(1);
    charRecognizer = tree.get(1);
  }

  public Tree toTree() {
    return OCRUtil.treeFor(this)
      .add(lineFinder)
      .add(lineSegmenter)
      .add(charRecognizer);
  }
}
