package eyedev._12;

import eyedev._01.OCRImageUtil;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGB;
import prophecy.common.image.RGBImage;

import java.awt.*;
import java.util.List;

public class Textfinder1 {
  private static final int maxBlueGapToFill = 3;

  private RGBImage baseImage;
  int rows, cols;
  MarkedTile[][] markedTiles;

  /** Warning: baseImage is changed in the process */
  public Textfinder1(RGBImage baseImage) {
    LineRemover.removeLines(baseImage);
    this.baseImage = baseImage;
    makeTiles();
    fillBlueGaps();
  }

  private void makeTiles() {
    int tileSize = 4;
    Tiles tiles = new Tiles(baseImage, tileSize);
    rows = tiles.getRows();
    cols = tiles.getCols();
    markedTiles = new MarkedTile[rows][cols];
    for (int row = 0; row < rows; row++)
      for (int col = 0; col < cols; col++) {
        Tile tile = tiles.getTile(col, row);
        BWImage img = tile.getImage().toBW();
        TileType type = TileType.white;

        double avg = img.averageBrightness();
        //System.out.println(avg);

        if (avg >= 0.99)
          type = TileType.yellow;
        else {
          float min = OCRImageUtil.minBrightness(img);
          float max = OCRImageUtil.maxBrightness(img);
          if (min <= 0.1 && max >= 0.9)
            type = TileType.blue;
        }

        MarkedTile markedTile = new MarkedTile(tile, type);
        markedTiles[row][col] = markedTile;
      }
  }

  public RGBImage getMarkedImage() {
    RGBImage image = new RGBImage(baseImage);
    for (int row = 0; row < rows; row++)
      for (int col = 0; col < cols; col++) {
        MarkedTile markedTile = markedTiles[row][col];
        Tile tile = markedTile.tile;
        Rectangle r = tile.getRect();
        Color color;
        switch (markedTile.type) {
          case white:
            color = null;
            break;
          case yellow:
            color = Color.yellow;
            break;
          case blue:
            color = Color.blue;
            break;
          case lightblue:
            color = Color.blue; // just render this as blue
            break;
          default:
            color = Color.black;
        }

        if (color != null)
          ImageProcessing.drawRect(image, r.x, r.y, r.width - 1, r.height - 1, new RGB(color));
      }
    return image;
  }

  private void fillBlueGaps() {
    for (int row = 0; row < rows; row++)
      loopCol:for (int col = 0; col < cols - 2;) {
        MarkedTile markedTile = markedTiles[row][col];
        if (markedTile.type == TileType.blue) {
          ++col;
          for (int col2 = col; col2 < cols - 1; col2++) {
            MarkedTile markedTile2 = markedTiles[row][col2];
            if (markedTile2.type == TileType.blue) {
              int numYellow = col2 - col;
              if (numYellow > 0 && numYellow <= maxBlueGapToFill) {
                //System.out.println("Filling blue gap at " + row + "/" + col + ": " + numYellow);
                for (int col3 = col; col3 < col2; col3++)
                  markedTiles[row][col3].type = TileType.lightblue;
              }
            } else // if (markedTile2.type == TileType.yellow)
              continue;

            col = col2;
            continue loopCol;
          }
        }

        ++col;
      }
  }

  public List<TileCluster> getClusters() {
    return new TileClusterer(rows, cols, markedTiles).getClusters();
  }

  public RGBImage getClusterImage(TileCluster cluster) {
    return baseImage.clip(cluster.getBoundingRect());
  }
}
