package eyedev._13;

import drjava.util.Tree;
import eyedev._01.DebugItem;
import eyedev._01.ImageReader;
import eyedev._01.OCRUtil;
import eyedev._09.WithFlexibleSegmenter;
import eyedev._12.Textfinder1;
import eyedev._12.TileCluster;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;

import java.awt.*;
import java.util.List;

public class WithTextFinder1 extends ImageReader implements IRSegmenter {
  private Tree characterRecognizer;
  private Textfinder1 textFinder;
  private List<TileCluster> clusters;

  public WithTextFinder1() {
  }

  public WithTextFinder1(String characterRecognizerDesc) {
    characterRecognizer = Tree.parse(characterRecognizerDesc);
  }

  public String readImage(BWImage image) {
    image = ImageProcessing.threshold(image, .5f);

    textFinder = new Textfinder1(image.toRGB());
    clusters = textFinder.getClusters();

    return recognize();
  }

  /** use this to re-recognize the same image with another character recognizer */
  public String rerecognize() {
    return recognize();
  }

  private String recognize() {
    WithFlexibleSegmenter lineRecognizer = new WithFlexibleSegmenter(characterRecognizer.toString());

    StringBuffer buf = new StringBuffer();
    for (TileCluster cluster : clusters) {
      Rectangle r = cluster.getBoundingRect();
      BWImage clip = textFinder.getClusterImage(cluster).toBW();
      lineRecognizer.setCollectDebugInfo(collectDebugInfo);
      String text = lineRecognizer.readImage(clip);
      if (text != null && text.length() != 0)
        buf.append(text).append("\n");

      if (collectDebugInfo) {
        for (DebugItem debugItem : lineRecognizer.getDebugInfo()) {
          debugInfo.add(debugItem.translate(r.x, r.y));
        }
      }
    }
    return buf.toString();
  }

  public void fromTree(Tree tree) {
    characterRecognizer = tree.get(0);
  }

  public Tree toTree() {
    return OCRUtil.treeFor(this)
      .add(characterRecognizer);
  }

  public void setCharRecognizer(String description) {
    characterRecognizer = Tree.parse(description);
  }
}
