/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package jazz.retroui;

import java.awt.*;

public class FancyScreen implements Screen {
  private Screen screen;

  public FancyScreen(Screen screen) {
    this.screen = screen;
  }

  public void print(int x, int y, String text) {
    screen.print(x, y, text);
  }

  public int cols() {
    return screen.cols();
  }

  public int rows() {
    return screen.rows();
  }

  public void setCursor(Cursor cursor) {
    screen.setCursor(cursor);
  }

  public void drawBox(int x1, int y1, int w, int h) {
    drawRow(x1, y1, w, '\u250C', '\u2500', '\u2510');
    for (int y = y1+1; y < y1+h-1; y++)
      drawRow(x1, y, w, '\u2502', ' ', '\u2502');
    drawRow(x1, y1+h-1, w, '\u2514', '\u2500', '\u2518');
  }

  private void drawRow(int x1, int y1, int w, char l, char m, char r) {
    StringBuffer buf = new StringBuffer();
    buf.append(l);
    for (int x = 2; x < w; x++)
      buf.append(m);
    buf.append(r);
    print(x1, y1, buf.toString());
  }

  private void drawRow(int x1, int y1, int w, char c) {
    drawRow(x1, y1, w, c, c, c);
  }

  public FancyScreen subscreen(final int x1, final int y1, final int w, final int h) {
    return new FancyScreen(new Screen() {
      public void print(int x, int y, String text) {
        if (y < 0 || y >= h || x >= w) return;
        if (x < 0) {
          if (text.length() < -x) return;
          text = text.substring(-x);
        }
        if (x+text.length() > w)
          text = text.substring(0, w-x);
        FancyScreen.this.print(x1+x, y1+y, text);
      }

      public int cols() {
        return w;
      }

      public int rows() {
        return h;
      }

      public void setCursor(Cursor cursor) {
        throw new RuntimeException("TODO");
      }

      public Cursor getCursor() {
        throw new RuntimeException("TODO");
      }
    });
  }

  public Rectangle centeredRect(int w, int h) {
    return new Rectangle((cols()-w)/2, (rows()-h)/2, w, h);
  }

  public void clear() {
    for (int y = 0; y < rows(); y++)
      drawRow(0, y, cols(), ' ', ' ', ' ');
  }

  public void print(int x, int y, int width, String text) {
    if (width < 0) return;
    if (text.length() > width) text = text.substring(0, width);
    drawRow(x+text.length(), y, width-text.length(), ' ');
    print(x, y, text);
  }

  public void printCentered(int x1, int y, int width, String text) {
    clearRow(x1, y, width);
    print(x1+(width-text.length())/2, y, text);
  }

  private void clearRow(int x1, int y, int width) {
    drawRow(x1, y, width, ' ');
  }

  public int getWidth() {
    return cols();
  }

  public int getHeight() {
    return rows();
  }

  public Cursor getCursor() {
    return screen.getCursor();
  }
}