/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package jazz.retroui;

import drjava.util.SwingTimerHelper;

import javax.swing.*;
import java.awt.*;

public class RetroUIPanel extends JComponent {
  private int cols, rows;
  private char[][] chars;
  private Font theFont;
  private Cursor cursor;
  private SwingTimerHelper timerHelper;

  public RetroUIPanel(int cols, int rows) {
    this.cols = cols;
    this.rows = rows;
    chars = new char[rows][cols];
    for (int y = 0; y < rows; y++)
      for (int x = 0; x < cols; x++)
        chars[y][x] = ' ';
  }

  public void paint(Graphics g) {
    int width = getWidth(), height = getHeight();
    g.setColor(Color.black);
    g.fillRect(0, 0, width, height);
    Font font = theFont();
    g.setFont(font);
    FontMetrics metrics = getFontMetrics(font);
    int ascent = metrics.getMaxAscent(), lineHeight = lineHeight(metrics), y = 0;
    g.setColor(Color.cyan);
    for (int i = 0; i < rows; i++) {
      g.drawChars(chars[i], 0, cols, 0, y + ascent);
      y += lineHeight;
    }
    if (cursor != null)
      cursor.paint(this, g);
  }

  public Font theFont() {
    if (theFont == null)
      theFont = new Font("Courier New", Font.BOLD, 16);
    return theFont;
  }

  public int lineHeight(FontMetrics metrics) {
    //return metrics.getHeight();
    return metrics.getMaxAscent();
  }

  public Dimension getMinimumSize() {
    return getPreferredSize();
  }

  public Dimension getPreferredSize() {
    FontMetrics metrics = getFontMetrics(theFont());
    return new Dimension(metrics.getMaxAdvance() * cols, lineHeight(metrics) * rows);
  }

  public FancyScreen getScreen() {
    return new FancyScreen(new Screen() {
      public void print(int x, int y, String text) {
        for (int i = 0; i < text.length(); i++)
          putChar(x + i, y, text.charAt(i));
        repaint();
      }

      public int cols() {
        return cols;
      }

      public int rows() {
        return rows;
      }

      public void setCursor(Cursor cursor) {
        RetroUIPanel.this.setCursor(cursor);
      }

      public Cursor getCursor() {
        return cursor;
      }
    });
  }

  public void setCursor(Cursor cursor) {
    if (timerHelper == null && cursor != null && cursor.getBlinkRate() != 0) {
      timerHelper = new SwingTimerHelper(new Runnable() {
        public void run() {
          repaint();
        }
      }, cursor.getBlinkRate());
      timerHelper.installOn(this);
    }
    if (cursor == null && timerHelper != null) {
      timerHelper.uninstall();
      timerHelper = null;
    }
    this.cursor = cursor;
    repaint();
  }

  private void putChar(int x, int y, char c) {
    if (x >= 0 && x < cols && y >= 0 && y < rows)
      chars[y][x] = c;
  }

}