package newinternet.ni01;

import prophecy.common.socket.SocketHandler;

import java.util.regex.Matcher;

public class CmdFileTransfer extends Cmd {
  FileTransferHandler transferHandler;

  public CmdFileTransfer(FileTransferHandler transferHandler) {
    super("begin file transfer");
    this.transferHandler = transferHandler;
  }

  @Override
  public void run(String input, Matcher matcher, SocketHandler handler) {
    // make a sub-cli for the file transfer negotiation
    FreeFormCLI cli = new FreeFormCLI();
    cli.setReadyMsg("[ft] ready");
    cli.addStandardCmds();
    cli.addCmd(new CmdCancel());
    cli.addCmd(new CmdFileName());
    cli.addCmd(new CmdFileSize());
    cli.addCmd(new CmdHexData());
    cli.addCmd(new CmdDone());
    cli.loop(handler);
  }

  class CmdCancel extends Cmd {
    CmdCancel() {
      super("cancel");
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      transferHandler.cancel();
      handler.println("file transfer canceled");
      FreeFormCLI.exitLoop();
    }
  }

  class CmdFileName extends Cmd {
    CmdFileName() {
      super("file name (.*)");
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      transferHandler.setFileName(matcher.group(1));
    }
  }

  class CmdFileSize extends Cmd {
    CmdFileSize() {
      super("file size (.*)( bytes?)?");
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      transferHandler.setFileSize(Long.parseLong(matcher.group(1)));
    }
  }

  class CmdHexData extends Cmd {
    CmdHexData() {
      super("hex data ([a-f0-9]+)");
    }

    byte[] hexToBytes(String s) {
      byte[] bytes = new byte[s.length()/2];
      for (int i = 0; i < bytes.length; i++)
        bytes[i] = (byte) Integer.parseInt(s.substring(i*2, i*2+2), 16);
      return bytes;
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      String hex = matcher.group(1);
      byte[] bytes = hexToBytes(hex);
      transferHandler.data(bytes);
    }
  }

  class CmdDone extends Cmd {
    CmdDone() {
      super("done");
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      transferHandler.done();
      handler.println("file transfer complete");
      FreeFormCLI.exitLoop();
    }
  }

}
