/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package newinternet.ni01;

import drjava.util.Errors;
import prophecy.common.socket.SocketHandler;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;

/** Command Line Interface with regexp-based command parsing */
public class FreeFormCLI {
  List<Cmd> cmds = new ArrayList<Cmd>();
  private String readyMsg;

  public void addCmd(Cmd cmd) {
    cmds.add(cmd);
  }

  public void loop(SocketHandler handler) {
    while (true) {
      // print ready message

      if (readyMsg != null)
        handler.println(readyMsg);

      // read line

      String line = handler.readLine();

      // leave loop if conn closed

      if (line == null) break;

      // lookup and handle cmd

      Cmd cmd = lookupCmd(line);
      if (cmd == null)
        unknownCommand(handler, line);
      else
        try {
          int charactersSent = handler.getCharactersSent();
          cmd.run(line, handler);
          if (charactersSent == handler.getCharactersSent())
            handler.println("OK");
        } catch (ExitLoop e) {
          break;
        } catch (Throwable e) {
          Errors.add(e);
          handler.println("Sorry, an internal error occured.");
        }
    }
  }

  /** overridable */
  protected void unknownCommand(SocketHandler handler, String input) {
    handler.println("??? " + input);
  }

  public Cmd lookupCmd(String input) {
    for (Cmd cmd : cmds) {
      if (cmd.matches(input))
        return cmd;
    }
    return null;
  }

  public void addStandardCmds() {
    //addCmd(new CmdHelp("\\?"));
    addCmd(new CmdHelp("Help"));
    addCmd(new CmdBye("Bye"));
  }

  public static FreeFormCLI withStandardCmds() {
    FreeFormCLI cli = new FreeFormCLI();
    cli.addStandardCmds();
    return cli;
  }

  public class CmdHelp extends Cmd {
    public CmdHelp(String cmd) {
      super(cmd);
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      for (Cmd cmd : cmds) {
        handler.println("cmd: " + cmd.getPattern());
      }
    }
  }

  public static class CmdBye extends Cmd {
    public CmdBye(String name) {
      super(name);
    }

    public void run(String input, Matcher matcher, SocketHandler handler) {
      handler.println("Bye");
      exitLoop();
    }
  }

  private static class ExitLoop extends RuntimeException {
  }

  public static void exitLoop() {
    throw new ExitLoop();
  }

  public String getReadyMsg() {
    return readyMsg;
  }

  public void setReadyMsg(String readyMsg) {
    this.readyMsg = readyMsg;
  }
}