
package newinternet.ni01;

import drjava.util.*;
import prophecy.common.Prophecy;
import prophecy.common.ClassData;
import drjava.util.ErrorListener;
import drjava.util.Errors;
import prophecy.common.LogView;
import prophecy.common.gui.Form;
import prophecy.common.gui.AutoVMExit;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class Main implements Updatable {
  private JFrame frame;
  private JLabel lblNodeStatus;
  private JLabel lblNodeAddress;
  //private JLabel lblOnInternet;
  //private JTextField lblNodeAddress;
  private JLabel lblIncomingCount, lblIncomingOpen;
  private int updateFreq = 500;
  private Node node;
  private Form form;
  private Font formFont;
  private LogView log;
  private FileTransferHistory fileTransferHistory = new FileTransferHistory();
  private FileTransferSettings fileTransferSettings;

  public Main() {
  }

  public static void main(String[] args) {
    AutoVMExit.install();
    Prophecy.initUI();
    new Main().showFrame();
  }

  private void showFrame() {
    form = new Form();
    formFont = form.getFont().deriveFont(14.0f);
    form.setFont(formFont);
    lblNodeStatus = makeLabel("not running");
    lblNodeAddress = makeLabel("");
    //lblOnInternet = makeLabel("");
    lblIncomingCount = makeLabel("0");
    lblIncomingOpen = makeLabel("0");
    /*lblNodeAddress = new JTextField();
    lblNodeAddress.setEditable(false);
    lblNodeAddress.setBorder(null);*/

    log = new LogView();
    log.setMinimumSize(new Dimension(100, 100));
    Errors.addListener(new ErrorListener() {
      public void errorReported(final Throwable error) {
        final String text = error.toString();
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            log.logText("[ERROR] " + text);
          }
        });
      }
    });

    ClassData classData = ClassData.get(this);
    fileTransferSettings = new FileTransferSettings(classData.subTree("fileTransfer"));
    FileTransferPanel fileTransferPanel = new FileTransferPanel(fileTransferSettings, fileTransferHistory);

    JTabbedPane tabbedPane = new JTabbedPane();
    tabbedPane.addTab("Log", log);
    tabbedPane.addTab("File transfer", LetterLayout.withBorder(fileTransferPanel, 10));

    form.addRow("Node status:", lblNodeStatus);
    form.addRow("Node address:", lblNodeAddress);
    //form.addRow("Node on Internet:", lblOnInternet);
    form.addRow("Total incoming connections:", lblIncomingCount);
    form.addRow("Open incoming connections:", lblIncomingOpen);
    form.addRow(tabbedPane);

    new PopupMenuHelper() {
      @Override
      protected void fillMenu(JPopupMenu menu, Point point) {
        JMenuItem menuItem = new JMenuItem("Copy address to clipboard");
        menuItem.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
            SwingUtil.copyTextToClipboard(lblNodeAddress.getText());
          }
        });
        menu.add(menuItem);
      }
    }.install(lblNodeAddress);

    JPanel buttons = new JPanel(LetterLayout.stalactite());
    /*JButton btnStart = makeButton("Start node", this, "start");
    buttons.add(btnStart);*/

    frame = new JFrame("New Internet");
    frame.getContentPane().setLayout(new LetterLayout("FFB").setBorder(10));
    frame.getContentPane().add("F", form);
    frame.getContentPane().add("B", buttons);
    frame.pack();
    frame.setSize(500, frame.getHeight());
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    GUIUtil.showFrame(frame);

    node = new NodeImpl();
    new SwingTimerHelper(this, updateFreq).installOn(form);

    start();
  }

  private JLabel makeLabel(String text) {
    JLabel label = new JLabel(text);
    if (formFont != null)
      label.setFont(formFont);
    return label;
  }

  private JButton makeButton(String text, final Object object, final String methodName) {
    JButton button = new JButton(text);
    button.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          object.getClass().getMethod(methodName).invoke(object);
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    });
    return button;
  }

  public void start() {
    new Thread() {
      public void run() {
        try {
          node.start();
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    }.start();
  }

  private void setText(final JLabel label, final String text) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        label.setText(text);
      }
    });
  }

  private void setText(final JTextField textField, final String text) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        textField.setText(text);
      }
    });
  }

  public void update() {
    String status = node.getStatus();
    String address = "";
    if (node.isRunning()) {
      address = node.getIP() + ":" + node.getPort();
      status += node.isInternetIP() ? " (Internet)" : " (LAN)";
    }
    lblNodeStatus.setText(status);
    lblNodeAddress.setText(address);
    //lblOnInternet.setText(!node.isRunning() ? "" : (node.isInternetIP() ? "Yes" : "No"));
    lblIncomingCount.setText("" + node.getIncomingCount());
    lblIncomingOpen.setText("" + node.getIncomingOpenCount());
  }

  class NodeImpl extends Node {
    public FileTransferHandler makeFileTransferHandler() {
      return new RealFileTransferHandler(fileTransferSettings, fileTransferHistory);
    }
  }
}
