package newinternet.ni01;

import drjava.util.Errors;
import prophecy.common.Var;
import prophecy.common.socket.DialogServer;
import prophecy.common.socket.SocketCallable;
import prophecy.common.socket.SocketHandler;

import java.io.IOException;
import java.net.InetAddress;

public abstract class Node implements SocketCallable {
  private int port = 50000;
  private boolean firewallOpened;
  private String ip;
  private boolean running;
  private Var<String> status = new Var<String>("");
  private Var<Integer> incomingCount = new Var<Integer>(0);
  private Var<Integer> incomingOpenCount = new Var<Integer>(0);

  public void start() throws IOException {
    openFirewall();
    startServer();
  }

  private void startServer() throws IOException {
    status.set("Starting node");
    try {
      new DialogServer(port, this).start();
      running = true;
      status.set("Up and running");
    } finally {
      if (!running)
        status.set("Error, could not start");
    }
  }

  private void openFirewall() {
    status.set("Contacting firewall");
    FirewallOpener fo = new FirewallOpener();
    try {
      fo.openPort(port);
      if (fo.isPortOpened()) {
        ip = fo.getExternalIP();
        firewallOpened = true;
      } else {
        ip = InetAddress.getLocalHost().getHostAddress();
      }
    } catch (Exception e) {
      Errors.add(e);
    }
  }

  public int getPort() {
    return port;
  }

  public boolean isFirewallOpened() {
    return firewallOpened;
  }

  public String getIP() {
    return ip;
  }

  public boolean isInternetIP() {
    return ip != null
      && !ip.startsWith("10.")
      && !ip.startsWith("192.168.") 
      && !ip.startsWith("127.");
  }

  public void takeCall(SocketHandler handler) {
    // TODO: these inc/dec statements are actually not 100% thread-safe... (but it's just stats)
    incomingCount.set(incomingCount.get()+1);
    incomingOpenCount.set(incomingOpenCount.get()+1);

    try {
      new CallHandler(this).takeCall(handler);
    } finally {
      incomingOpenCount.set(incomingOpenCount.get()-1);
    }
  }

  public int getIncomingCount() {
    return incomingCount.get();
  }

  public int getIncomingOpenCount() {
    return incomingOpenCount.get();
  }

  public String getStatus() {
    return status.get();
  }

  public boolean isRunning() {
    return running;
  }

  public abstract FileTransferHandler makeFileTransferHandler();
}
