package newinternet.ni01;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.regex.Pattern;

public class RealFileTransferHandler extends FileTransferHandler {
  private FileTransferSettings settings;
  private String fileName;
  private long fileSize;
  private FileOutputStream fileOutputStream;
  private File file;
  private FileTransferHistory history;

  public RealFileTransferHandler(FileTransferSettings settings, FileTransferHistory history) {
    this.settings = settings;
    this.history = history;
  }

  public void setFileName(String fileName) {
    if (!settings.isAllowReceive())
      throw new RuntimeException("File transfer is disabled");
    if (this.fileName != null)
      throw new RuntimeException("File name already set");
    if (fileName.contains("/") || fileName.contains("\\") || fileName.contains(":"))
      throw new RuntimeException("Illegal file name");
    else {
      this.fileName = fileName;
      try {
        file = new File(settings.getDir(), fileName);
        int count = 2;
        while (file.exists()) {
          file = new File(settings.getDir(), fileName + "(" + count + ")");
          ++count;
        }
        history.writing(file.getPath());
        file.getParentFile().mkdirs();
        fileOutputStream = new FileOutputStream(file);
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
    }
  }

  public void setFileSize(long fileSize) {
    this.fileSize = fileSize;
  }

  public void data(byte[] data) {
    if (fileName == null)
      throw new RuntimeException("No file name set");
    try {
      fileOutputStream.write(data);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public void done() {
    history.doneWriting(file.getPath());
    close();
  }

  private void close() {
    try {
      fileOutputStream.close();
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public void cancel() {
    history.canceledWriting(file.getPath());
    close();
    file.delete();
  }
}
