/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy;

import drjava.util.Errors;
import prophecy.audio.a08.A08;
import prophecy.common.Application;
import prophecy.common.gui.Sheet;
import prophecy.joe.joe06.Joe06Server;
import prophecy.joe.joe07.Joe07Server;
import prophecy.joe.joe09.Joe09Server;
import prophecy.video.v01.ImageBrowser;
import prophecy.video.v03.Eyes;
import prophecy.video.v06.ContestsGUI;
import prophecy.video.v09.V09BlurryVision;
import prophecy.video.v10.V10NervousAlphabet;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class OldStartMenu extends Sheet {
  private DefaultListModel serverListModel;
  private JList serverList;
  private Application application = new Application("Start menu");

  public static void main(String[] args) {
    OldStartMenu startMenu = new OldStartMenu();
    startMenu.run();
  }

  private void run() {
    application.asMainApplication();
    JFrame frame = asFrame();
    application.showFinalFrame(frame);
    frame.setLocation(0, frame.getY());
  }

  public OldStartMenu() {
    super("Project Prophecy R3");
    setSize(300, 750);

    serverListModel = new DefaultListModel();
    serverList = new JList(serverListModel);

    addLabel("Experiments<br>[for everyone]");
    addButton("V10: Nervous Alphabet", V10NervousAlphabet.class);
    addButton("V09: Blurry Vision", V09BlurryVision.class);
    //addButton("V08", V08ThresholdExperiment.class);
    addAppButton("V03: Eyes", Eyes.class);
    addSpacer();

    addLabel("Half-Finished Tools<br>[for nosy geeks]");
    addButton("Contests", ContestsGUI.class);
    addButton("Image Browser", ImageBrowser.class);
    addButton("Audio Clip Browser", A08.class);
    addSpacer();

    addLabel("Aibo Software<br>[for Aibo owners]");
    addAppButton("Joe09: Parrot", Joe09Server.class);
    addAppButton("Joe06: Ball tracker", Joe06Server.class);
    addAppButton("Joe07: Pet me", Joe07Server.class);

    /*addSpacer();
    addLabel("Applications you started");
    JScrollPane scrollPane = new JScrollPane(serverList);
    scrollPane.setMinimumSize(new Dimension(50, 50));
    addComponent(scrollPane);*/

    JLabel aibo = new JLabel(new ImageIcon("img/aibo3bit.jpg"));
    aibo.setMinimumSize(new Dimension(50, aibo.getMinimumSize().height));
    panel.add(aibo);
  }

  private void addAppButton(String text, final Class<? extends Application> appClass) {
    addButton(text, new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        Application app;
        try {
          app = appClass.newInstance();
        } catch (InstantiationException e) {
          Errors.add(e);
          return;
        } catch (IllegalAccessException e) {
          Errors.add(e);
          return;
        }
        addApplication(app);
      }
    });
  }

  private void addApplication(final Application application) {
    final Object element = new Object() {
      public String toString() {
        return drjava.util.ObjectUtil.toNiceString(application) + " [" + application.getStateDescription() + "]";
      }
    };
    serverListModel.insertElementAt(element, 0);
    application.addListener(new Runnable() {
      public void run() {
        if (application.isRetired())
          serverListModel.removeElement(element);
        serverList.repaint();
      }
    });
    application.start();
  }
}
