/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy;

import static drjava.util.GUIUtil.setMinimumHeight;

import drjava.util.ErrorListener;
import drjava.util.Errors;
import prophecy.audio.a08.A08;
import prophecy.common.*;
import prophecy.common.gui.Sheet;
import prophecy.common.comments.Shoutbox;
import prophecy.joe.joe06.Joe06Server;
import prophecy.joe.joe07.Joe07Server;
import prophecy.joe.joe09.Joe09Server;
import prophecy.m.m01.ProgramWatch;
import prophecy.m.m14.M14Calc;
import prophecy.video.v01.ImageBrowser;
import prophecy.video.v03.Eyes;
import prophecy.video.v06.ContestsGUI;
import prophecy.video.v09.V09BlurryVision;
import prophecy.video.v10.V10NervousAlphabet;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.PrintWriter;
import java.io.StringWriter;

public class StartMenu extends Sheet {
  private DefaultListModel serverListModel;
  private JList serverList;
  private Application application = new Application("Start menu");
  private Sheet currentSheet;

  public static void main(String[] args) {
    Prophecy.init();
    StartMenu startMenu = new StartMenu();
    startMenu.run();
  }

  private void run() {
    ProgramWatch.applicationStarted(application);
    application.asMainApplication();
    JFrame frame = asFrame();
    application.showFinalFrame(frame);
    frame.setLocation(0, frame.getY());
  }

  public StartMenu() {
    super("Project Prophecy R6");
    setSize(350, 750);

    serverListModel = new DefaultListModel();
    serverList = new JList(serverListModel);

    JTabbedPane tabs = new JTabbedPane();
    setMinimumHeight(tabs, 350);
    addComponent(tabs);

    Sheet toolsSheet = newSheet();
    tabs.addTab("Tools", toolsSheet.getScrollPane());
    currentSheet.addPair(
      makeAppButton("M14: Loyal Calculator", "img/m12.jpg", M14Calc.class),
      null);

    Sheet experimentsSheet = newSheet();
    tabs.addTab("Fun", experimentsSheet.getScrollPane());

    currentSheet.addPair(
      null/*makeAppButton("M01: Restart", null, Restart.class)*/,
      makeAppButton("V10: Nervous Alphabet", "img/v10.jpg", V10NervousAlphabet.class));
    currentSheet.addPair(
      makeAppButton("V09: Blurry Vision", "img/v09.jpg", V09BlurryVision.class),
      makeAppButton("V03: Eyes", "img/v03.jpg", Eyes.class));

    currentSheet = this;

    Sheet aiboSheet = newSheet();
    tabs.addTab("Aibo", aiboSheet.getScrollPane());
    currentSheet.addPair(null, makeAppButton("Joe09: Parrot", Joe09Server.class));
    currentSheet.addPair(
      makeAppButton("Joe06: Ball tracker", "img/joe06.jpg", Joe06Server.class),
      makeAppButton("Joe07: Pet me", "img/joe07.jpg", Joe07Server.class));

    Sheet geekSheet = newSheet();
    tabs.addTab("Geeky", geekSheet.getScrollPane());

    geekSheet.addButton("Source Browser", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        SourceBrowser sourceBrowser = new SourceBrowser();
        sourceBrowser.scanDir("src");
        sourceBrowser.scanZip("prophecy-src.zip", "src");
        sourceBrowser.selectPackage("prophecy");
        sourceBrowser.show();
      }
    });
    geekSheet.addButton("Contests", ContestsGUI.class);
    geekSheet.addButton("Image Browser", ImageBrowser.class);
    geekSheet.addButton("Audio Clip Browser", A08.class);

    tabs.addTab("Tasks", new JScrollPane(serverList));

    final LogView errorsView = new LogView();
    Errors.addListener(new ErrorListener() {
      public void errorReported(final Throwable error) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            StringWriter writer = new StringWriter();
            PrintWriter printWriter = new PrintWriter(writer);
            error.printStackTrace(printWriter);
            errorsView.logText(writer.toString());
          }
        });
      }
    });
    tabs.addTab("Errors", errorsView);

    /*addSpacer();
    addLabel("Applications you started");
    JScrollPane scrollPane = new JScrollPane(serverList);
    scrollPane.setMinimumSize(new Dimension(50, 50));
    addComponent(scrollPane);*/

    addComponent(new Shoutbox("fluesterstern.de", getTitle()).getSheet().getPanel());

    JLabel aibo = new JLabel(new ImageIcon("img/aibo3bit.jpg"));
    aibo.setMinimumSize(new Dimension(50, aibo.getMinimumSize().height));
    addComponent(aibo);
  }

  private JButton makeAppButton(String text, String img, Class<? extends Application> appClass) {
    JButton button = makeAppButton(text, appClass);
    if (img != null)
      button.setIcon(new ImageIcon(img));
    button.setVerticalTextPosition(SwingConstants.BOTTOM);
    button.setHorizontalTextPosition(SwingConstants.CENTER);
    button.setMargin(new Insets(1, 1, 1, 1));
    return button;
  }

  private JButton makeAppButton(String text, Class<? extends Application> appClass) {
    JButton button = new JButton(text);
    button.addActionListener(appActionListener(appClass));
    return button;
  }

  private Sheet newSheet() {
    return currentSheet = new Sheet();
  }

  private JButton addAppButton(String text, final Class<? extends Application> appClass) {
    return currentSheet.addButton(text, appActionListener(appClass));
  }

  private ActionListener appActionListener(final Class<? extends Application> appClass) {
    return new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        Application app;
        try {
          app = appClass.newInstance();
        } catch (InstantiationException e) {
          Errors.add(e);
          return;
        } catch (IllegalAccessException e) {
          Errors.add(e);
          return;
        }
        addApplication(app);
      }
    };
  }

  private void addApplication(final Application application) {
    final Object element = new Object() {
      public String toString() {
        return drjava.util.ObjectUtil.toNiceString(application) + " [" + application.getStateDescription() + "]";
      }
    };
    serverListModel.insertElementAt(element, 0);
    application.addListener(new Runnable() {
      public void run() {
        if (application.isRetired())
          serverListModel.removeElement(element);
        serverList.repaint();
      }
    });
    application.start();
  }
}