/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a01;

import prophecy.common.Prophecy;
import prophecy.common.*;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundSurface;
import prophecy.common.sound.SoundUtil;

import java.io.File;
import java.io.IOException;

public class A1 {
  public static void main(String[] args) throws IOException {
    File file = new File(Prophecy.dataDir() + "audio/aibo.wav");
    Sound sound = SoundUtil.load(file);
    System.out.println("Seconds: " + sound.getSeconds());
    Marker speechStart = new Marker(1.59);
    Marker speechEnd = new Marker(2.21);
    DetectSpeechFragments contest = new DetectSpeechFragments(sound, speechStart, speechEnd);
    contest.setTrainer(new IgnoreUntrainables<Subset<Sound>>(new SubsetAutoTrainer<Sound, SubsetContestant<Sound>>(contest)));

    makeContestants(contest);
    contest.run();
    ContestUtil.saveResults(contest, "a1");
    SoundSurface surface = new SoundDiscriminationSurface(sound, contest, contest.getClipSize(), contest.getWinner());
    surface.addMarker(speechStart);
    surface.addMarker(speechEnd);
    SurfaceUtil.show("A1", surface);
  }

  private static void makeContestants(DetectSpeechFragments contest) {
    for (int sections = 2; sections <= 15; sections++)
      contest.addContestant(new BayesSubsetContestant<Sound>(
        new FeatureVerbalizer<Sound>(new AutoScaleFeature<Sound>(new Loudness()), sections)));
  }
}