/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a01;

import drjava.util.FromTree;
import drjava.util.ToTree;
import drjava.util.Tree;
import drjava.util.TreeUtil;

import java.util.List;

public class AutoScaleFeature<A> extends Feature<A> implements ToTree, FromTree {
  double[] range = {0, 1};
  private Feature<A> feature;

  /** default constructor for fromTree  */
  public AutoScaleFeature() {
    super("");
  }

  public AutoScaleFeature(Feature<A> feature) {
    super(makeName(feature));
    this.feature = feature;
  }

  public AutoScaleFeature(Feature<A> feature, double min, double max) {
    super(feature.getName());
    this.feature = feature;
    range = new double[] {min, max};
  }

  private static String makeName(Feature feature) {
    return "as:" + feature.getName();
  }

  public void train(List<A> data) {
    range = getMinMax(data);
    //System.out.println(getName() + " range: " + range[0] + " - " + range[1]);
  }

  private double[] getMinMax(List<A> data) {
    boolean first = true;
    double min = 0, max = 1;

    for (A a : data) {
      double f = feature.extract(a);
      if (first || f < min) min = f;
      if (first || f > max) max = f;
      first = false;
    }

    if (max == min) max = min+1;
    return new double[] {min, max};
  }

  public double extract(A a) {
    //return Math.max(0, Math.min(1, (feature.extract(a)-range[0])/(range[1]-range[0])));
    return (feature.extract(a)-range[0])/(range[1]-range[0]);
  }

  public Tree toTree() {
    return super.toTree()
      .add("feature", feature)
      .add("min", range[0])
      .add("max", range[1]);
  }

  public void fromTree(Tree tree) {
    feature = (Feature<A>) TreeUtil.treeToObject(tree.get("feature"));
    range = new double[] { tree.getDouble("min"), tree.getDouble("max") };
    setName(makeName(feature));
  }
}