/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a01;

import drjava.util.*;
import prophecy.e.e10.VerbalizerUtil;
import prophecy.common.Verbalizer;

import java.util.Collection;
import java.util.List;

public class FeatureVerbalizer<A> implements Verbalizer<A>, ToTree, FromTree {
  private Feature<A> feature;
  private int sections;

  /** default constructor for fromTree  */
  public FeatureVerbalizer() {
  }

  public FeatureVerbalizer(Feature<A> feature, int sections) {
    this.feature = feature;
    this.sections = sections;
  }

  public FeatureVerbalizer(Feature<A> feature) {
    this(feature, 10);
  }

  public Collection<String> getWords(A a) {
    return VerbalizerUtil.sections(feature.getName(), feature.extract(a), sections);
  }

  public void train(List<A> trainingData) {
    feature.train(trainingData);
  }

  public String toString() {
    return sections + "sec:" + ObjectUtil.toNiceString(feature);
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("sections", sections)
      .add("feature", feature);
  }

  public void fromTree(Tree tree) {
    sections = tree.getInt("sections");
    feature = (Feature) TreeUtil.treeToObject(tree.get("feature"));
  }
}