/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a02;

import drjava.util.StdOutDisplay;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.common.Prophecy;
import prophecy.audio.a01.AutoScaleFeature;
import prophecy.audio.a01.FeatureVerbalizer;
import prophecy.audio.a01.Loudness;
import prophecy.common.*;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundUtil;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class A2 {
  public static void main(String[] args) throws IOException {
    SoundDiscriminationContest contest = makeContest();

    // run subcontest
    Subset<Sound> subWinner = ContestUtil.runAndGetWinner(contest, makeContestants());

    // run main contest
    ContestRun<Subset<Sound>> contestRun = new ContestRun<Subset<Sound>>(contest);
    contestRun.addContestants(experimentWithRepeatedTraining(subWinner));
    contestRun.run();
    ContestUtil.saveResults(contestRun, "a1");
  }

  public static SoundDiscriminationContest makeContest() {
    File file1 = new File(Prophecy.dataDir() + "audio/beat.wav");
    File file2 = new File(Prophecy.dataDir() + "audio/background.wav");

    SoundDiscriminationContest contest = new SoundDiscriminationContest();
    contest.addExample(SoundUtil.load(file1), true);
    contest.addExample(SoundUtil.load(file2), false);
    contest.setTrainer(new IgnoreUntrainables<Subset<Sound>>(new SubsetAutoTrainer<Sound, SubsetContestant<Sound>>(contest)));
    return contest;
  }

  private static List<Subset<Sound>> experimentWithRepeatedTraining(Subset<Sound> baseContestant) {
    Tree tree = TreeUtil.objectToTree(baseContestant);
    new StdOutDisplay().putnl(tree);
    List<Subset<Sound>> result = new ArrayList<Subset<Sound>>();
    for (int trainings = 0; trainings <= 5; trainings++) {
      Subset<Sound> clone = (Subset<Sound>) TreeUtil.treeToObject(tree);
      result.add(new RepeatedTrainingContestant<Sound>(clone, trainings));
    }
    return result;
  }

  private static List<Subset<Sound>> makeContestants() {
    List<Subset<Sound>> result = new ArrayList<Subset<Sound>>();
    for (int sections = 2; sections <= 15; sections++)
      result.add(new BayesSubsetContestant<Sound>(
        new FeatureVerbalizer<Sound>(new AutoScaleFeature<Sound>(new Loudness()), sections)));
    return result;
  }
}