/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a02;

import drjava.util.ObjectUtil;
import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.common.Subset;
import prophecy.common.SubsetTI;
import prophecy.common.Trainable;

import java.util.ArrayList;
import java.util.List;

public class RepeatedTrainingContestant<A> implements Subset<A>, Trainable<SubsetTI<A>>, ToTree {
  private Subset<A> contestant;
  private int trainings;

  public RepeatedTrainingContestant(Subset<A> contestant, int trainings) {
    this.contestant = contestant;
    this.trainings = trainings;
  }

  public boolean contains(A a) {
    return contestant.contains(a);
  }

  public void train(List<SubsetTI<A>> trainingData) {
    List<SubsetTI<A>> newData = new ArrayList<SubsetTI<A>>();
    for (int i = 0; i < trainings; i++)
      newData.addAll(trainingData);
    ((Trainable<SubsetTI<A>>) contestant).train(newData);
  }

  public String toString() {
    return "train " + trainings + "x " + ObjectUtil.toNiceString(contestant);
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("contestant", contestant)
      .add("trainings", trainings);
  }
}