/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a03;

import prophecy.audio.a01.*;
import prophecy.audio.a02.A2;
import prophecy.audio.a02.SoundDiscriminationContest;
import prophecy.common.*;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundSurface;

import java.util.ArrayList;
import java.util.List;

public class A3 {
  public static void main(String[] args) {
    SoundDiscriminationContest contest = A2.makeContest();
    //test1(contest);
    double[] freqs = { 200, 400, 800, 1600 };
    List<Feature<Sound>> features = makeFreqFeatures(freqs);
    features.add(new AutoScaleFeature<Sound>(new Loudness()));
    ContestRun<Subset<Sound>> contestRun = new ExperimentWithFeatures<Sound>(contest, features).run(25);
    ContestUtil.saveResults(contestRun, "a3_ExperimentWithFeatures");
  }

  private static void test1(SoundDiscriminationContest contest) {
    double[] freqs = { 100, 200, 400, 800, 1600 };
    /*for (double freq : freqs) {
      contest.addContestants(makeStandardFeatureContestants(new FreqFeature(freq), 3, 10));
    }*/
    /*List<Verbalizer<Sound>> verbalizers = makeVerbalizers(freqs);
    contest.addContestant(new BayesSubsetContestant<Sound>(new BayesParams(), verbalizers));*/

    List<Feature<Sound>> features = makeFreqFeatures(freqs);
    //features.add(new Loudness());

    Verbalizer<Sound> verbalizer = new MultiFeatureVerbalizer<Sound>("freqs", features);
    List<Verbalizer<Sound>> verbalizers = makeVerbalizers(freqs);
    verbalizers.add(verbalizer);

    ContestRun<Subset<Sound>> contestRun = new ContestRun<Subset<Sound>>(contest);
    contestRun.addContestant(new BayesSubsetContestant<Sound>(verbalizers));
    contestRun.run();
    ContestUtil.saveResults(contestRun, "a3");
  }

  public static List<Feature<Sound>> makeFreqFeatures(double... freqs) {
    List<Feature<Sound>> features = new ArrayList<Feature<Sound>>();
    for (double freq : freqs) {
      features.add(new AutoScaleFeature<Sound>(new FreqFeature(freq)));
    }
    return features;
  }

  private static List<Verbalizer<Sound>> makeVerbalizers(double[] freqs) {
    List<Verbalizer<Sound>> verbalizers = new ArrayList<Verbalizer<Sound>>();
    for (double freq : freqs) {
      verbalizers.add(new FeatureVerbalizer<Sound>(new AutoScaleFeature<Sound>(new FreqFeature(freq))));
    }
    return verbalizers;
  }

  private static void showSine() {
    SurfaceUtil.show(new SoundSurface(new Sine(48000, 1, 440)));
  }

  private static List<Subset<Sound>> experimentWithSections(Feature<Sound> feature, int... sectionsList) {
    List<Subset<Sound>> result = new ArrayList<Subset<Sound>>();
    for (int sections : sectionsList)
      result.add(new BayesSubsetContestant<Sound>(
        new FeatureVerbalizer<Sound>(new AutoScaleFeature<Sound>(feature), sections)));
    return result;
  }

  public static Subset<Sound> loadEWFWinner() {
    SoundDiscriminationContest contest = A2.makeContest();
    Subset<Sound> contestant = (Subset<Sound>) ContestUtil.loadWinner("a3_ExperimentWithFeatures");
    if (contestant != null) {
      contest.addContestant(contestant);
      contest.run();
    }
    return contestant;
  }
}