/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a03;

import drjava.util.Randomizer;
import drjava.util.RealRandomizer;
import prophecy.audio.a01.Feature;
import prophecy.audio.a01.FeatureVerbalizer;
import prophecy.common.Verbalizer;
import prophecy.common.*;

import java.util.ArrayList;
import java.util.List;

public class ExperimentWithFeatures<A> {
  private Contest<Subset<A>> contest;
  private List<Feature<A>> features;
  private Randomizer randomizer = new RealRandomizer();

  public ExperimentWithFeatures(Contest<Subset<A>> contest, List<Feature<A>> features) {
    this.contest = contest;
    this.features = features;
  }

  public ContestRun<Subset<A>> run(int contestants) {
    ContestRun<Subset<A>> contestRun = new ContestRun<Subset<A>>(contest);
    contestRun.addContestant(new EmptySet<A>());
    contestRun.addContestant(new FullSet<A>());
    for (int i = 0; i < contestants; i++)
      contestRun.addContestant(makeRandomContestant());
    contestRun.run();
    return contestRun;
  }

  private Subset<A> makeRandomContestant() {
    return new BayesSubsetContestant<A>(makeRandomVerbalizers2());
  }

  private List<Verbalizer<A>> makeRandomVerbalizers1() {
    List<Feature<A>> remainingFeatures = new ArrayList<Feature<A>>(features);
    List<Verbalizer<A>> verbalizers = new ArrayList<Verbalizer<A>>();
    while (!remainingFeatures.isEmpty()) {
      if (randomizer.oneIn(2))
        verbalizers.add(new FeatureVerbalizer<A>(remainingFeatures.get(0)));
      remainingFeatures.remove(0);
    }
    return verbalizers;
  }

  private List<Verbalizer<A>> makeRandomVerbalizers2() {
    List<Feature<A>> remainingFeatures = new ArrayList<Feature<A>>(features);
    List<Verbalizer<A>> verbalizers = new ArrayList<Verbalizer<A>>();
    int mfCount = 0;
    while (!remainingFeatures.isEmpty()) {
      int n = randomizer.random(remainingFeatures.size())+1;
      List<Feature<A>> features = new ArrayList<Feature<A>>();
      for (int i = 0; i < n; i++) {
        int idx = randomizer.random(remainingFeatures.size());
        features.add(remainingFeatures.get(idx));
        remainingFeatures.remove(idx);
      }
      verbalizers.add(new MultiFeatureVerbalizer<A>("mf" + (++mfCount), features));
    }
    return verbalizers;
  }
}