/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a03;

import drjava.util.FromTree;
import drjava.util.Tree;
import prophecy.audio.a01.Feature;
import prophecy.common.sound.Sound;

public class FreqFeature extends Feature<Sound> implements FromTree {
  private double frequency;

  /** for fromTree */
  public FreqFeature() {
    super("");
  }

  public FreqFeature(double frequency) {
    super("");
    setFrequency(frequency);
  }

  public double extract(Sound sound) {
    double length = vectorLength(AudioAnalysis.sineSum(sound, frequency), AudioAnalysis.cosineSum(sound, frequency));
    return length;
  }

  private double vectorLength(double x, double y) {
    return Math.sqrt(x*x+y*y);
  }

  public Tree toTree() {
    return super.toTree().add("frequency", frequency);
  }

  public void fromTree(Tree tree) {
    setFrequency(tree.getDouble("frequency"));
  }

  public void setFrequency(double frequency) {
    this.frequency = frequency;
    setName("Freq(" + frequency + ")");
  }
}