/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a03;

import drjava.util.FromTree;
import drjava.util.ObjectUtil;
import drjava.util.ToTree;
import drjava.util.Tree;
import static drjava.util.TreeUtil.listToTree;
import static drjava.util.TreeUtil.treeToList;
import prophecy.audio.a01.Feature;
import prophecy.common.Verbalizer;
import prophecy.e.e10.VerbalizerUtil;

import java.util.Collection;
import static java.util.Collections.singleton;
import java.util.List;

public class MultiFeatureVerbalizer<A> implements Verbalizer<A>, ToTree, FromTree {
  private String name;
  private List<Feature<A>> features;

  /** for fromTree */
  public MultiFeatureVerbalizer() {
  }

  public MultiFeatureVerbalizer(String name, List<Feature<A>> features) {
    this.name = name;
    this.features = features;
  }

  public MultiFeatureVerbalizer(List<Feature<A>> features) {
    this.name = makeName(features);
    this.features = features;
  }

  private String makeName(List<Feature<A>> features) {
    return ObjectUtil.toNiceString(features);
  }

  public Collection<String> getWords(A a) {
    String word = name;
    for (Feature<A> feature : features) {
      word += VerbalizerUtil.tenSections("", feature.extract(a));
    }
    return singleton(word);
  }

  public void train(List<A> a) {
    for (Feature<A> feature : features) {
      feature.train(a);
    }
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("name", name)
      .add("features", listToTree(features));
  }

  public void fromTree(Tree tree) {
    name = tree.getString("name");
    features = treeToList(tree.get("features"));
  }

  public String toString() {
    return "multifeature" + ObjectUtil.toNiceString(features);
  }
}