/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a06;

import drjava.util.FromTree;
import drjava.util.ToTree;
import drjava.util.Tree;
import static drjava.util.TreeUtil.treeToList;
import prophecy.common.Contest;
import prophecy.common.Marker;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundWithMarkers;

import java.util.ArrayList;
import java.util.List;

public class BaseDrumContest extends Contest<AudioEventFinder> implements ToTree, FromTree {
  private Sound sound;
  private List<Marker> realEvents;
  private double tolerance = 0.01;
  private double maxDistance = 0.3;

  /** for fromTree */
  public BaseDrumContest() {
    super(AudioEventFinder.class);
  }

  public BaseDrumContest(SoundWithMarkers soundWithMarkers) {
    this();
    this.sound = soundWithMarkers.sound;
    this.realEvents = soundWithMarkers.markers;
  }

  public double evaluate(AudioEventFinder contestant) {
    List<Marker> events = contestant.findEvents(sound);
    List<Marker> remainingEvents = new ArrayList<Marker>(realEvents);
    double penalty = 0;
    for (Marker event : events) {
      Marker closest = findClosest(remainingEvents, event);
      if (closest == null)
        penalty += 1.0;
      else {
        penalty += Math.max(0, Math.min(1, (closest.distanceTo(event)-tolerance)/(maxDistance-tolerance)));
        remainingEvents.remove(closest);
      }
    }
    penalty += remainingEvents.size();
    return -penalty;
  }

  private Marker findClosest(List<Marker> markers, Marker marker) {
    double bestDistance = 0;
    Marker bestMarker = null;

    for (Marker m : markers) {
      double distance = m.distanceTo(marker);
      if (bestMarker == null || distance < bestDistance) {
        bestMarker = m;
        bestDistance = distance;
      }
    }

    return bestMarker;
  }

  public Tree toTree() {
    return super.toTree()
      .add("sound", sound)
      .add("realEvents", realEvents)
      .add("tolerance", tolerance)
      .add("maxDistance", maxDistance);
  }

  public void fromTree(Tree tree) {
    sound = tree.getObject("sound");
    realEvents = treeToList(tree.get("realEvents"));
    tolerance = tree.getDouble("tolerance");
    maxDistance = tree.getDouble("maxDistance");
  }
}