/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a06;

import prophecy.common.Marker;
import prophecy.common.SubsetTI;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundWithMarkers;

import java.util.ArrayList;
import java.util.List;

public class Clipifier {
  private final double clipSize;

  public Clipifier(double clipSize) {
    this.clipSize = clipSize;
  }

  public List<SubsetTI<Sound>> makeTrainingItems(SoundWithMarkers soundWithMarkers) {
    List<SubsetTI<Sound>> trainingItems = new ArrayList<SubsetTI<Sound>>();
    List<Section> sections = new ArrayList<Section>();
    for (Marker marker : soundWithMarkers.markers) {
      sections.add(new Section(marker, marker.plus(clipSize)));
      trainingItems.add(new SubsetTI<Sound>(soundWithMarkers.sound.clip(marker, marker.plus(clipSize)), true));
    }

    loop: for (double s = 0; s < soundWithMarkers.sound.getSeconds(); s += clipSize) {
      Marker marker = new Marker(s);
      for (Section section : sections) {
        if (section.contains(marker))
          continue loop;
      }
      trainingItems.add(new SubsetTI<Sound>(soundWithMarkers.sound.clip(marker, marker.plus(clipSize)), false));
    }

    return trainingItems;
  }

  public List<Sound> makeClips(Sound sound) {
    List<Sound> clips = new ArrayList<Sound>();
    for (double s = 0; s < sound.getSeconds(); s += clipSize) {
      Marker marker = new Marker(s);
      clips.add(sound.clip(marker, marker.plus(clipSize)));
    }
    return clips;
  }

  public double getClipSize() {
    return clipSize;
  }
}