/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a07;

import prophecy.audio.a01.Loudness;
import prophecy.audio.a06.A06;
import prophecy.audio.a06.AudioEventFinder;
import prophecy.audio.a06.DeafContestant;
import prophecy.common.*;
import prophecy.common.Generator;

import java.io.PrintStream;
import java.util.ArrayList;
import static java.util.Arrays.asList;
import java.util.List;

public class A07 {
  public static void main(String[] args) {
    System.setOut(new PrintStream(new SilentOutputStream()));
    OpenContestRun run = ContestUtil.getOpenContestRun(A06.CONTEST_STAGE0);
    run.setUI(new OpenContestRetroUI("Prophecy A07"));
    for (Generator<? extends AudioEventFinder> generator : makeA06Generators(run)) {
      run.addGenerator(generator);
    }
    run.run();
  }

  private static List<Generator<? extends AudioEventFinder>> makeA06Generators(final OpenContestRun<AudioEventFinder> run) {
    List<Generator<? extends AudioEventFinder>> generators = new ArrayList<Generator<? extends AudioEventFinder>>();
    LocalMaximum localMaximum = new LocalMaximum(new Loudness());
    localMaximum.setNoiseThreshold(-10.0);
    generators.add(new GeneratorFromIterator<AudioEventFinder>(
      new DeafContestant(),
      localMaximum));
    generators.add(new Tuner<LocalMaximum>(new Tunable<LocalMaximum>() {
      double scale = 0.1;
      public double getScore(LocalMaximum localMaximum) {
        return ContestUtil.getScore(run.getContest(), localMaximum);
      }

      public List<Parameter> getParameters(LocalMaximum localMaximum) {
        return asList(new Parameter("minValue", localMaximum.getNoiseThreshold()*scale));
      }

      public LocalMaximum getTunedClone(LocalMaximum localMaximum, Parameter parameter, double newValue) {
        LocalMaximum clone = new LocalMaximum(localMaximum.getFeature());
        clone.setNoiseThreshold(newValue/scale);
        return clone;
      }
    }, localMaximum));
    return generators;
  }
}