/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a07;

import static drjava.util.StringUtil.formatDouble;
import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.audio.a01.Feature;
import prophecy.audio.a01.Loudness;
import prophecy.audio.a03.AudioAnalysis;
import prophecy.audio.a06.A06;
import prophecy.audio.a06.AudioEventFinder;
import prophecy.common.Marker;
import static prophecy.common.SurfaceUtil.show;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundSurface;
import prophecy.common.sound.SoundWithMarkers;

import java.util.ArrayList;
import java.util.List;

public class LocalMaximum implements AudioEventFinder, ToTree {
  private Feature<Sound> feature;
  private Double noiseThreshold;

  public LocalMaximum(Feature<Sound> feature) {
    this.feature = feature;
  }

  public List<Marker> findEvents(Sound sound) {
    double clipSize = 0.05;
    List<Sound> clips = AudioAnalysis.clips(sound, clipSize);
    double[] values = new double[clips.size()];
    for (int i = 0; i < clips.size(); i++) {
      Sound clip = clips.get(i);
      values[i] = feature.extract(clip);
    }

    List<Marker> events = new ArrayList<Marker>();
    for (int i = 1; i < values.length-1; i++)
      if (values[i] > values[i-1] && values[i] > values[i+1]) {
        if (noiseThreshold == null || values[i] >= noiseThreshold)
          events.add(new Marker(i*clipSize));
      }
    //System.out.println(events.size() + " events found in " + values.length + " values");
    return events;
  }

  public void train(List<SoundWithMarkers> data) {
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("feature", feature)
      .add("minValue", noiseThreshold);
  }

  public LocalMaximum(Tree tree) {
    feature = tree.getObject("feature");
    noiseThreshold = tree.getDouble("minValue");
  }

  public static void main(String[] args) {
    A06 a6 = new A06();
    Sound sound = a6.getExample1().sound;
    LocalMaximum localMaximum = new LocalMaximum(new Loudness());
    localMaximum.setNoiseThreshold(-1.6);
    show(new SoundSurface(new SoundWithMarkers(sound, localMaximum.findEvents(sound))));
  }

  public void setNoiseThreshold(Double noiseThreshold) {
    this.noiseThreshold = noiseThreshold;
  }

  public Feature<Sound> getFeature() {
    return feature;
  }

  public double getNoiseThreshold() {
    return noiseThreshold;
  }

  public String toString() {
    return "LocalMaximum noiseThreshold=" + formatDouble(noiseThreshold, 4);
  }
}