/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a07;

import drjava.util.Randomizer;
import drjava.util.RepeatableRandomizer;
import prophecy.common.Generator;

import java.util.List;

public class Tuner<A> implements Generator<A> {
  private Tunable<A> tunable;
  private A currentObject;
  private Randomizer randomizer = new RepeatableRandomizer();

  public Tuner(Tunable<A> tunable, A seed) {
    this.tunable = tunable;
    currentObject = seed;
  }

  public A next() {
    List<Parameter> parameters = tunable.getParameters(currentObject);
    if (parameters.isEmpty()) return null;

    Parameter p = parameters.get(randomizer.random(parameters.size()));
    double newValue = p.getValue()+randomizer.randomProb()*2-1;
    A newObject = tunable.getTunedClone(currentObject, p, newValue);
    if (tunable.getScore(newObject) > tunable.getScore(currentObject))
      currentObject = newObject;
    return newObject;
  }
}