/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a08;

import drjava.util.StringUtil;
import drjava.vats.FulfilledPromise;
import drjava.vats.Promise;
import prophecy.common.Marker;
import prophecy.common.PromiseBasedTableModel;

import java.util.List;

public class MarkersTableModel extends PromiseBasedTableModel<Marker> {
  private static final String NAME = "Name", POSITION = "Position";
  private static final String[] COLUMNS = { NAME, POSITION };

  public MarkersTableModel(List<Marker> markers) {
    super(COLUMNS, markers);
  }

  protected int compareRows(Marker marker, Marker b) {
    return 0;
  }

  protected Promise<String> getCell(Marker marker, int row, int col) {
    if (col == 0)
      return new FulfilledPromise<String>(marker.getName());
    if (col == 1)
      return new FulfilledPromise<String>(StringUtil.formatDouble(marker.getSecond(), 3) + "s");
    return new FulfilledPromise<String>("");
  }

  public boolean isCellEditable(int row, int col) {
    return true;
  }

  public void setValueAt(Object value, int rowIndex, int columnIndex) {
    if (columnIndex == 0) {
      getList().get(rowIndex).setName((String) value);
      fireTableCellUpdated(rowIndex, columnIndex);
    } else if (columnIndex == 1) {
      String s = (String) value;
      if (s.endsWith("s")) s = s.substring(0, s.length()-1);
      getList().get(rowIndex).setSecond(Double.valueOf(s.trim()));
      fireTableCellUpdated(rowIndex, columnIndex);
    }
  }
}