/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a08;

import drjava.util.FileTreePersistence;
import drjava.util.ToTree;
import drjava.util.Tree;
import static drjava.util.TreeUtil.listToTree;
import static drjava.util.TreeUtil.treeToList;
import prophecy.common.Marker;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class SampleInfo implements ToTree {
  private List<Marker> markers = new ArrayList<Marker>();

  public SampleInfo(Tree tree) {
    markers.addAll(treeToList(tree.get("markers")));
  }

  public SampleInfo() {
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("markers", listToTree(markers));
  }

  public List<Marker> getMarkers() {
    return markers;
  }

  private static File stripExtension(File file) {
    int i = file.getName().lastIndexOf('.');
    return i < 0 ? file : new File(file.getParentFile(), file.getName().substring(0, i));
  }

  public static SampleInfo loadSampleInfo(File file) throws IOException {
    FileTreePersistence persistence = getPersistence(file);
    SampleInfo info = (SampleInfo) persistence.loadObject();
    return info == null ? new SampleInfo() : info;
  }

  private static FileTreePersistence getPersistence(File file) {
    String fileName = stripExtension(file).getPath() + ".tree";
    return new FileTreePersistence(fileName);
  }

  public static void saveSampleInfo(File file, SampleInfo sampleInfo) throws IOException {
    getPersistence(file).store(sampleInfo);
  }

  public void setMarkers(List<Marker> markers) {
    this.markers = new ArrayList<Marker>(markers);
  }

  public double getBPMFromMarkers() {
    Marker first = null, last = null;
    for (Marker marker : markers) {
      if (marker.getName().startsWith("beat")) {
        if (first == null) first = marker;
        last = marker;
      }
    }
    if (first != null && last != null) {
      int beat1 = Integer.parseInt(first.getName().substring(4));
      int beat2 = Integer.parseInt(last.getName().substring(4));
      return 60*(beat2-beat1)/(last.getSecond()-first.getSecond());
    }
    return 0;
  }
}