/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a08;

import drjava.util.GUIUtil;
import drjava.util.LetterLayout;
import drjava.util.ScalingSplitPane;
import static drjava.util.StringUtil.formatDouble;
import prophecy.common.Marker;
import prophecy.common.sound.SoundSurface;
import prophecy.common.sound.SoundWithMarkers;

import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

public class SamplePanel extends ScalingSplitPane {
  private SoundSurface soundSurface;

  public SamplePanel(final SampleOnDisk sample) {
    super(JSplitPane.VERTICAL_SPLIT);
    setDividerLocationLater(0.5);
    final List<Marker> markers = sample.getInfo().getMarkers();
    soundSurface = new SoundSurface(new SoundWithMarkers(sample.load(), markers));
    setTopComponent(soundSurface);
    final MarkersTableModel tableModel = new MarkersTableModel(markers);
    final JTable markersTable = new JTable(tableModel);

    JPanel buttonPanel = new JPanel(LetterLayout.stalactite().setSpacing(10));
    JButton btnNew = new JButton("New");
    btnNew.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        tableModel.addItem(new Marker(0));
      }
    });
    buttonPanel.add(btnNew);

    JButton btnDel = new JButton("Delete");
    btnDel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        int row = markersTable.getSelectedRow();
        if (row >= 0 && row < markersTable.getRowCount())
          tableModel.removeItem(row);
      }
    });
    buttonPanel.add(btnDel);

    final JLabel bpmDisplay = new JLabel(" ");
    double bpm = sample.getInfo().getBPMFromMarkers();
    if (bpm != 0) bpmDisplay.setText(formatDouble(bpm, 1) + " bpm");
    buttonPanel.add(bpmDisplay);

    JPanel bottomPanel = new JPanel(new LetterLayout("TTB").setSpacing(10));
    bottomPanel.add("T", markersTable);
    bottomPanel.add("B", buttonPanel);
    setBottomComponent(GUIUtil.withTitle("Markers", bottomPanel));

    tableModel.addTableModelListener(new TableModelListener() {
      public void tableChanged(TableModelEvent e) {
        sample.getInfo().setMarkers(tableModel.getList());
        sample.saveInfo();
        soundSurface.repaint();
      }
    });

  }
}