/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a09;

import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.common.Contest;
import prophecy.common.sound.Sound;

import static java.lang.Math.abs;
import static java.lang.Math.max;

public class BPMContest extends Contest<BPMDetector> implements ToTree {
  protected Sound sound;
  protected double realBPM;
  private double tolerance = 1.0;

  public BPMContest(Sound sound, double realBPM) {
    super(BPMDetector.class);
    this.sound = sound;
    this.realBPM = realBPM;
  }

  public double evaluate(BPMDetector contestant) {
    double answer = contestant.detectBPM(sound);
    double score = -max(abs(realBPM - answer) - tolerance, 0);
    System.out.println("real: " + realBPM + ", answer: " + answer + ", score: " + score);
    return score;
  }

  public BPMContest(Tree tree) {
    super(tree);
    sound = tree.getObject("sound");
    realBPM = tree.getDouble("realBPM");
  }

  public Tree toTree() {
    return super.toTree()
      .setObject("sound", sound)
      .setDouble("realBPM", realBPM);
  }

  public Double getBestPossibleScore() {
    return 0.0;
  }
}