/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a09;

import static drjava.util.ObjectUtil.toNiceString;
import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.audio.a06.AudioEventFinder;
import static prophecy.audio.a10.Debug.debugging;
import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.common.Marker;
import prophecy.common.Visualizable;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundSurface;

import javax.swing.*;
import static java.lang.Math.abs;
import static java.util.Arrays.asList;
import java.util.List;

public class BeatGrid implements BPMDetector, ToTree, InspectableParent, Visualizable {
  private GridFinder gridFinder = new GridFinder1();
  private AudioEventFinder beatFinder;
  private Sound debug_sound;
  private List<Double> debug_gridSizes;
  private List<Marker> debug_markers;

  public BeatGrid(AudioEventFinder beatFinder) {
    this.beatFinder = beatFinder;
  }

  public BeatGrid(AudioEventFinder beatFinder, GridFinder gridFinder) {
    this.beatFinder = beatFinder;
    this.gridFinder = gridFinder;
  }

  public double detectBPM(Sound sound) {
    List<Marker> markers = beatFinder.findEvents(sound);
    if (debugging) {
      debug_sound = sound;
      debug_markers = markers;
    }
    double[] events = new double[markers.size()];
    for (int i = 0; i < events.length; i++)
      events[i] = markers.get(i).getSecond();

    List<Double> gridSizes = gridFinder.getPossibleGridSizes(events);
    if (debugging)
      debug_gridSizes = gridSizes;
    MinMax<Double> minMax = new MinMax<Double>();
    for (Double gridSize : gridSizes) {
      //System.out.println("bpm candidate: " + StringUtil.formatDouble(bpm, 1));
      if (gridSize == 0)
        add(minMax, 0);
      else {
        add(minMax, 60/gridSize);
        add(minMax, 120/gridSize);
        add(minMax, 240/gridSize);
      }
    }

    return minMax.getMin(0.0);
  }

  private void add(MinMax<Double> minMax, double bpm) {
    minMax.add(bpm, abs(120-bpm));
  }

  public BeatGrid(Tree tree) {
    beatFinder = tree.getObject("beatFinder");
    gridFinder = tree.getObject("gridFinder", gridFinder);
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("beatFinder", beatFinder)
      .add("gridFinder", gridFinder);
  }

  public String toString() {
    return "BeatGrid (" + toNiceString(beatFinder) + ") (" + toNiceString(gridFinder) + ")";
  }

  public List<InspectionNode> getInspectionChildren() {
    return asList(
      new InspectionNode("beatFinder", beatFinder),
      new InspectionNode("gridFinder", gridFinder),
      new InspectionNode("debug_markers", debug_markers),
      new InspectionNode("debug_gridSizes", debug_gridSizes));
  }

  public JComponent visualize() {
    return debug_sound == null ? null : new SoundSurface(debug_sound, debug_markers);
  }

  public AudioEventFinder getBeatFinder() {
    return beatFinder;
  }
}