/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a10;

import drjava.util.Factory;
import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.audio.a09.GridFinder;
import prophecy.audio.a09.RelativeDistributionMetric;
import prophecy.common.InspectionProxy;
import prophecy.e.e01.Analyzer;
import prophecy.e.e01.WatcherPool;
import prophecy.e.e02.Distribution;
import prophecy.e.e02.DistributionMaker;

import java.util.ArrayList;
import java.util.List;

public class GridFinder2 implements GridFinder, InspectionProxy, ToTree {
  WatcherPool<Double, Distribution> watcherPool;
  private int maxStep = 1;

  public GridFinder2(int maxStep) {
    this.maxStep = maxStep;
  }

  public List<Double> getPossibleGridSizes(double[] points) {
    watcherPool = new WatcherPool<Double, Distribution>(new Factory<Analyzer<Double, Distribution>>() {
      public Analyzer<Double, Distribution> make() {
        return new DistributionMaker();
      }
    }, new RelativeDistributionMetric());
    watcherPool.setRadius(0.1);

    for (int i = 1; i < points.length; i++)
      for (int step = 1; step <= maxStep; step++)
        if (i-step >= 0)
          watcherPool.processData(points[i]-points[i-step]);

    //watcherPool.display(new StdOutDisplay());

    List<Double> result = new ArrayList<Double>();
    for (Distribution distribution : watcherPool.getResults()) {
      result.add(distribution.getAverage());
    }

    return result;
  }

  public Object getObject() {
    return watcherPool;
  }

  public int getMaxStep() {
    return maxStep;
  }

  public void setMaxStep(int maxStep) {
    this.maxStep = maxStep;
  }

  public GridFinder2(Tree tree) {
    maxStep = tree.getInt("maxStep");
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("maxStep", maxStep);
  }
}