/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a10;

import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.audio.a06.AudioEventFinder;
import prophecy.common.Marker;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundWithMarkers;

import java.util.List;

public abstract class PreprocessedAudioEventFinder implements AudioEventFinder, ToTree {
  private AudioEventFinder audioEventFinder;

  public PreprocessedAudioEventFinder(AudioEventFinder audioEventFinder) {
    this.audioEventFinder = audioEventFinder;
  }

  public abstract Sound preprocess(Sound sound);

  public List<Marker> findEvents(Sound sound) {
    return audioEventFinder.findEvents(preprocess(sound));
  }

  public void train(List<SoundWithMarkers> data) {
    throw new RuntimeException("TODO");
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("audioEventFinder", audioEventFinder);
  }

  public PreprocessedAudioEventFinder(Tree tree) {
    audioEventFinder = tree.getObject("audioEventFinder");
  }
}