/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a11;

import drjava.util.LetterLayout;
import prophecy.common.Marker;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import static prophecy.common.image.ImageProcessing.resize;
import prophecy.common.image.ImageSurface;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundSurface;
import prophecy.common.sound.SoundUtil;
import prophecy.video.v01.ImageMeta;
import prophecy.video.v01.ImageMetaSurface;
import prophecy.common.gui.SheetApp;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class A11 extends SheetApp {
  public A11() {
    super("A11");

    getSheetLayout().setBorder(20);

    final Sound sound = SoundUtil.load("data/audio/gutenmorgenaibo.wav");
    SoundSurface originalSurface = makeSoundSurface(sound);
    addBlueHeading("Original sample");
    addComponent(originalSurface);

    SoundChunker chunker = new SoundChunker(sound);

    addBlueHeading("Loudness");
    BWImage image = chunker.makeLoudnessImage();
    addComponent(loudnessImageSurface(image));

    addBlueHeading("Blurred loudness");
    BWImage blurredImage = chunker.blur(image);
    addComponent(loudnessImageSurface(blurredImage));

    /*Histogram histogram = new Histogram(blurredImage);
    double threshold = histogram.getThresholdBrightness(0.5);*/
    BWImage bwImage = chunker.makeBWImage(blurredImage);
    bwImage = ImageProcessing.resize(bwImage, bwImage.getWidth(), 15);
    bwImage = ImageProcessing.addBorder(bwImage, 0, 5, 0, 5);
    final java.util.List<Rectangle> chunks = chunker.findChunks(bwImage);
    for (Rectangle chunk : chunks) {
      System.out.println(chunk);
    }

    addBlueHeading(chunks.size() + " chunk(s) found");
    ImageMetaSurface bwSurface = new ImageMetaSurface(bwImage.toRGB(),
      new ImageMeta("", chunks));
    //setZoom(bwSurface);
    addComponent(bwSurface);

    JPanel playPanel = new JPanel(LetterLayout.leftAlignedRow(10));
    Object[] cbItems = new Object[chunks.size()];
    for (int i = 0; i < cbItems.length; i++)
      cbItems[i] = "Chunk " + (i+1);
    final JComboBox cbChunk = new JComboBox(cbItems);
    playPanel.add(cbChunk);

    JButton btnPlay = new JButton("Play");
    final BWImage bwImage1 = bwImage;
    btnPlay.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        Rectangle chunk = chunks.get(cbChunk.getSelectedIndex());
        double second1 = (chunk.x/(double) bwImage1.getWidth()*sound.getSeconds());
        double second2 = (chunk.x+chunk.width)/(double) bwImage1.getWidth()*sound.getSeconds();
        Sound clip = sound.clip(new Marker(second1), new Marker(second2));
        SoundUtil.playInExtraThread(clip);
      }
    });
    playPanel.add(btnPlay);

    addComponent(playPanel);

    showFrame();
  }

  private ImageSurface loudnessImageSurface(BWImage image) {
    ImageSurface imageSurface = new ImageSurface(resize(image, 300, 25));
    //setZoom(imageSurface);
    return imageSurface;
  }

  private void setZoom(ImageSurface imageSurface) {
    imageSurface.setZoom(300.0/imageSurface.getImage().getWidth(), 50.0/imageSurface.getImage().getHeight());
  }

  /*private Sound blurSound(Sound sound, int blur) {
    float[] data = new float[sound.getNumSamples()/blur];
    for (int i = 0; i < data.length; i++) {
      PixelMixer
    }
    return
  }*/

  private SoundSurface makeSoundSurface(Sound sound) {
    SoundSurface originalSurface = new SoundSurface(sound);
    originalSurface.setMinimumSize(new Dimension(300, 150));
    originalSurface.setAmplification(3.0);
    return originalSurface;
  }

  public static void main(String[] args) {
    new A11().asMainApplication();
  }
}