/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.audio.a11;

import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import prophecy.common.sound.Sound;
import prophecy.video.v09.BlobFinder;
import prophecy.video.v09.BlockyBlur;

import java.awt.*;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

public class SoundChunker {
  private Sound sound;
  private static final int imageWidth = 300;

  public SoundChunker(Sound sound) {
    this.sound = sound;
  }

  public BWImage makeLoudnessImage() {
    float amplification = 5;
    BWImage image = new BWImage(sound.getNumSamples(), 1, 1);
    for (int x = 0; x < image.getWidth(); x++) {
      image.setPixel(x, 0, 1-Math.abs(sound.getSample(x))*amplification);
    }
    return image;
  }

  public BWImage blur(BWImage image) {
    int blur = (int) (0.1*sound.getSampleRate());
    double brightness = 0.1;
    return BlockyBlur.blur(image, blur, 1, brightness);
  }

  public BWImage makeBWImage(BWImage blurredImage) {
    double threshold = 0.9;
    BWImage bwImage = ImageProcessing.threshold(blurredImage, threshold);
    bwImage = ImageProcessing.resize(bwImage, imageWidth, 1);
    return bwImage;
  }

  public List<Rectangle> findChunks(BWImage bwImage) {
    return sortChunks(new BlobFinder().findBlobs(bwImage));
  }

  private List<Rectangle> sortChunks(List<Rectangle> chunks) {
    chunks = new ArrayList<Rectangle>(chunks);
    Collections.sort(chunks, new Comparator<Rectangle>() {
      public int compare(Rectangle r1, Rectangle r2) {
        return r1.x-r2.x;
      }
    });
    return chunks;
  }

  public List<Rectangle> findChunks() {
    return findChunks(makeBWImage(blur(makeLoudnessImage())));
  }

  public static int getImageWidth() {
    return imageWidth;
  }

  public double toSeconds(int x) {
    return x*sound.getSeconds()/getImageWidth();
  }
}