/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.ce.ce01;

import drjava.util.XTestCase;
import prophecy.common.Timestamp;

public class SoupTest extends XTestCase {
  private String nick = "joe";
  private Soup soup;

  public void setUp() {
    soup = new Soup();
  }

  public void testQuery() {
    UserStateQuery query = new UserStateQuery(nick);
    soup.add(query);
    assertNull(query.getReplacement());

    UserState userState = new UserState(nick, true, "here", new Evidence(this));
    soup.add(userState);
    soup.stir();

    assertNotNull("query.getReplacement()", query.getReplacement());
    CurrentUserState currentUserState = query.getReplacement();
    assertEquals(userState, currentUserState.getUserState());
  }

  public void testStateUpdate() {
    UserState userState1 = new UserState(nick, true, "here",
      new Evidence(this, new Timestamp(1000)));
    UserState userState2 = new UserState(nick, true, "somewhere else",
      new Evidence(this, new Timestamp(2000)));
    CurrentUserState currentUserState1 = new CurrentUserState(userState1);
    CurrentUserState currentUserState2 = new CurrentUserState(userState2);
    soup.add(currentUserState1);
    soup.add(currentUserState2);
    soup.stir();

    // Only the older state gets a replacement, the other one remains as it is
    // already up-to-date

    assertNotNull("currentUserState1.getReplacement()", currentUserState1.getReplacement());
    assertEquals(userState2, currentUserState1.getReplacement().getUserState());
    assertNull("currentUserState2.getReplacement()", currentUserState2.getReplacement());
  }

  public void testStirredFlag() {
    assertEquals(true, soup.isStirred());
    soup.add(new Object());
    assertEquals(false, soup.isStirred());
    soup.stir();
    assertEquals(true, soup.isStirred());
  }

  public void testSoupReference() {
    ActiveIngredient ingredient = new ActiveIngredient();
    soup.add(ingredient);
    assertEquals(soup, ingredient.getSoup());
    soup.remove(ingredient);
    assertEquals(null, ingredient.getSoup());
  }

  public void testAutoAddReplacement() {
    ActiveIngredient ingredient = new ActiveIngredient();
    Object replacement = new Object();
    soup.add(ingredient);
    ingredient.replaceWith(replacement);
    soup.stir();
    assertTrue(soup.contains(replacement));
  }
}