/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.ce.ce02;

import drjava.util.Lizt;
import static drjava.util.ObjectUtil.nice;
import prophecy.ce.ce01.ActiveIngredient;
import prophecy.ce.ce01.CurrentUserState;
import prophecy.ce.ce01.PossibleReaction;
import prophecy.t.t03.CaseInsensitiveComparator;

import java.util.List;
import java.util.TreeMap;

public class NickList extends ActiveIngredient {
  private TreeMap<String, CurrentUserState> stateByNick
    = new TreeMap<String, CurrentUserState>(new CaseInsensitiveComparator());

  public NickList() {
  }

  public String toString() {
    return "NickList " + nice(stateByNick.keySet());
  }

  public List<PossibleReaction> getPossibleReactions() {
    return Lizt.of(new PossibleReaction(this, CurrentUserState.class) {
      public boolean runWith(Object b) {
        CurrentUserState currentUserState = (CurrentUserState) b;
        String nick = currentUserState.getUserState().getNick();
        if (currentUserState.getUserState().isLoggedIn())
          synchronized(NickList.this) {
            stateByNick.put(nick, currentUserState);
          }
        else
          synchronized(NickList.this) {
            stateByNick.remove(nick);
          }
        return true;
      }
    }, new PossibleReaction(this, RebuildNickList.class) {
      public boolean runWith(Object b) {
        RebuildNickList refreshNickList = (RebuildNickList) b;
        refreshNickList.replaceWith(new RebuiltNickList());
        replaceWith(new NickList());
        return true;
      }
    });
  }

  public NickList getMostRecentVersion() {
    return replacement == null ? this : ((NickList) replacement).getMostRecentVersion();
  }

  public synchronized TreeMap<String, CurrentUserState> getStateByNick() {
    return new TreeMap<String, CurrentUserState>(stateByNick);
  }
}