/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.FromTree;
import drjava.util.StringUtil;
import drjava.util.ToTree;
import drjava.util.Tree;

public class BayesParams implements ToTree, FromTree {
  private static double defaultMinInterestingness = 0.1;
  private double minInterestingness;

  /**
   *
   * @param minInterestingness between 0.0 and 0.5
   */
  public BayesParams(double minInterestingness) {
    this.minInterestingness = minInterestingness;
  }

  public BayesParams() {
    minInterestingness = defaultMinInterestingness;
  }

  public double getMinInterestingness() {
    return minInterestingness;
  }

  public Tree toTree() {
    return new Tree(getClass()).add("minInterestingness", minInterestingness);
  }

  public void fromTree(Tree tree) {
    minInterestingness = tree.getDouble("minInterestingness");
  }

  public String toString() {
    if (minInterestingness == defaultMinInterestingness)
      return "";
    else
      return "mi=" + StringUtil.formatProbability(getMinInterestingness());
  }
}