/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.IRunnable;
import drjava.util.StringUtil;
import prophecy.m.m03.Sandbox;
import prophecy.m.m09.ListAnalyzer;
import prophecy.common.gui.MenuProvider;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

public class BeanInspectionNode extends InspectionNode {
  public BeanInspectionNode(Object o) {
    super(o);
    init();
  }

  public BeanInspectionNode(String name, Object object) {
    super(name, object);
    init();
  }

  private void init() {
    setMenuProvider(new MenuProvider() {
      public List<JMenuItem> getMenuItems() {
        List<JMenuItem> items = new ArrayList<JMenuItem>();
        if (getObject() instanceof List) {
          JMenuItem miAnalyzeList = new JMenuItem("Analyze list");
          miAnalyzeList.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
              ListAnalyzer.analyze((List) getObject());
            }
          });
          items.add(miAnalyzeList);
        }
        return items;
      }
    });
  }

  protected ObjectAnalyzer getObjectAnalyzer() {
    return new BeanObjectAnalyzer();
  }

  protected void makeChildren() {
    super.makeChildren();
    if (object == null) return;
    Method[] methods = object.getClass().getMethods();
    for (final Method method : methods) {
      if ((method.getModifiers() & Modifier.STATIC) != 0) continue;
      String prefix = null;
      if (method.getName().startsWith("get")) prefix = "get";
      else if (method.getName().startsWith("is")) prefix = "is";
      if (prefix != null && method.getParameterTypes().length == 0
        /*&& !method.getName().equals("getClass")*/
        && !method.getName().equals("isEmpty")) {
        Sandbox sandbox = new Sandbox();
        Object result = sandbox.run(new IRunnable<Object>() {
          public Object run() throws Exception {
            return method.invoke(object);
          }
        });
        if (sandbox.getException() != null) result = sandbox.getException();
        String propertyName = StringUtil.firstToLower(method.getName().substring(prefix.length()));
        add(new BeanInspectionNode(propertyName, result));
      }
    }
  }
}