/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.ToTree;
import drjava.util.Tree;

public abstract class Contest<A> extends AbstractContestRun<A> implements IContest<A>, ToTree {
  private Class<A> contestantInterface;

  protected Contest(Class<A> contestantInterface) {
    this.contestantInterface = contestantInterface;
    init();
  }

  public Contest(Tree tree) {
    init();
    trainer = tree.getObject("trainer");
    String className = tree.getString("contestantInterface");
    if (className != null)
      try {
        contestantInterface = (Class<A>) Class.forName(className);
      } catch (ClassNotFoundException e) {
        throw new RuntimeException(e);
      }
  }

  protected void init() {
  }

  // overridable
  public Double getBestPossibleScore() {
    return null;
  }

  // overridable
  public Double getWorstPossibleScore() {
    return null;
  }

  public Tree toTree() {
    Tree tree = new Tree(getClass());
    if (persistTrainer)
      tree.add("trainer", trainer);
    return tree
      .add("contestantInterface", contestantInterface.getName());
  }
}