/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import prophecy.common.gui.MenuProvider;

import java.util.ArrayList;
import java.util.List;

public class InspectionNode implements InspectableParent {
  private String name;
  protected Object object;
  private List<InspectionNode> children;
  private Visualizable visualizable;
  private MenuProvider menuProvider;

  public InspectionNode(Object object) {
    this(InspectionUtil.objectToString(object), object, true);
  }

  public InspectionNode(String name, Object object) {
    this(name, object, false);
  }

  public InspectionNode(String name, Object object, boolean forceName) {
    this.name = forceName ? name : name + ": " + InspectionUtil.objectToString(object);
    this.object = object;
  }

  public String getName() {
    return name;
  }

  public List<InspectionNode> getChildren() {
    if (children == null) {
      children = new ArrayList<InspectionNode>();
      makeChildren();
    }
    return children;
  }

  /** override me */
  protected void makeChildren() {
    addChildren(getObjectAnalyzer().detectChildren(object));
  }

  protected ObjectAnalyzer getObjectAnalyzer() {
    return new ObjectAnalyzer();
  }

  protected void addChildren(List<InspectionNode> children) {
    this.children.addAll(children);
  }

  public List<InspectionNode> getInspectionChildren() {
    return getChildren();
  }

  public Object getObject() {
    return object;
  }

  public String toString() {
    return name;
  }

  public void add(Object child) {
    add(makeInspectionNode(child));
  }

  protected final InspectionNode makeInspectionNode(Object o) {
    return getObjectAnalyzer().makeInspectionNode("", o);
  }

  public void add(InspectionNode node) {
    getChildren().add(node);
  }

  public void setVisualizable(Visualizable visualizable) {
    this.visualizable = visualizable;
  }

  public Visualizable getVisualizable() {
    return visualizable;
  }

  public void setMenuProvider(MenuProvider menuProvider) {
    this.menuProvider = menuProvider;
  }

  public MenuProvider getMenuProvider() {
    return menuProvider;
  }
}
