/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.*;
import prophecy.common.Prophecy;
import prophecy.common.gui.SingleComponentPanel;

import javax.swing.*;
import javax.swing.event.TreeModelListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;

public class InspectionPanel {
  private JPanel mainPanel;
  private ScalingSplitPane splitPane;
  private JTree tree;
  private Object root;

  public InspectionPanel(Object root) {
    this.root = root;
    splitPane = new ScalingSplitPane(JSplitPane.HORIZONTAL_SPLIT);
    render();

    mainPanel = new JPanel(new LetterLayout("T", "S", "S").setBorder(10));
    JToolBar toolBar = new JToolBar();
    toolBar.setFloatable(false);
    JButton btnRefresh = new JButton(Prophecy.refreshIcon());
    btnRefresh.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        render();
        GUIUtil.revalidateAndRepaint(mainPanel);
      }
    });
    toolBar.add(btnRefresh);
    mainPanel.add("T", toolBar);
    mainPanel.add("S", splitPane);
  }

  private void render() {
    InspectionNode rootNode = asInspectionNode(root);
    tree = new JTree(makeTreeModel(rootNode));
    tree.setCellRenderer(new DefaultTreeCellRenderer() {
      public Component getTreeCellRendererComponent(JTree tree, Object value, boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        Component component = super.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
        setText((((InspectionNode) value).getName()));
        return component;
      }
    });
    splitPane.setLeftComponent(new JScrollPane(tree));
    final SingleComponentPanel detailPanel = new SingleComponentPanel();
    splitPane.setRightComponent(detailPanel);
    splitPane.setDividerLocationLater(0.4);

    tree.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
        TreePath path = e.getNewLeadSelectionPath();
        InspectionNode selectedNode = (InspectionNode) (path == null ? null : path.getLastPathComponent());

        // show detail
        detailPanel.setComponent(null);
        if (selectedNode != null) {
          Component component = makeDetailComponent(selectedNode);
          if (component != null) {
            if (!(component instanceof JScrollPane))
              component = new JScrollPane(component);
            detailPanel.setComponent(component);
          }
        }
      }
    });

    tree.addMouseListener(new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        TreePath path = tree.getLeadSelectionPath();
        InspectionNode selectedNode = (InspectionNode) (path == null ? null : path.getLastPathComponent());

        if (selectedNode != null && selectedNode.getMenuProvider() != null) {
          List<JMenuItem> menuItems = selectedNode.getMenuProvider().getMenuItems();
          for (JMenuItem menuItem : menuItems) {
            menu.add(menuItem);
          }
        }
      }
    });

    tree.setSelectionRow(0); // select root node
  }

  public void selectRow(int row) {
    tree.setSelectionRow(row);
    tree.scrollRowToVisible(row+1);
  }

  private static InspectionNode asInspectionNode(Object o) {
    /*if (o instanceof InspectionNode) return (InspectionNode) o;
    return new InspectionNode(o);*/
    return BeanInspector.makeNode(o);
  }

  public static Component makeDetailComponent(InspectionNode node) {
    if (node.getVisualizable() != null)
      return node.getVisualizable().visualize();

    Object o = node.getObject();
    if (o instanceof InspectionProxy)
      return makeDetailComponent(new InspectionNode(((InspectionProxy) o).getObject()));
    Visualizable visualizable = Attachments.find(o, Visualizable.class);
    if (visualizable != null)
      return visualizable.visualize();
    if (o instanceof Component)
      return (Component) o;
    if (o instanceof InspectionNode)
      return makeDetailComponent((InspectionNode) o);
    String text = o == null ? "" : o.toString();
    return new TextView(text);
  }

  public static TreeModel makeTreeModel(final InspectionNode root) {
    return new TreeModel() {
      public Object getRoot() {
        return root;
      }

      public Object getChild(Object o, int i) {
        return children(o).get(i);
      }

      public int getChildCount(Object o) {
        return children(o).size();
      }

      public boolean isLeaf(Object o) {
        return getChildCount(o) == 0;
      }

      public void valueForPathChanged(TreePath treePath, Object o) {
      }

      public int getIndexOfChild(Object o, Object o1) {
        return children(o).indexOf(o1);
      }

      public void addTreeModelListener(TreeModelListener treeModelListener) {
      }

      public void removeTreeModelListener(TreeModelListener treeModelListener) {
      }
    };
  }

  private static List<InspectionNode> children(Object o) {
    return ((InspectionNode) o).getChildren();
  }

  public JPanel getPanel() {
    return mainPanel;
  }

  public static JComponent makeInspectionPanel(Object root) {
    return new InspectionPanel(root).getPanel();
  }
}