/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.GUIUtil;
import drjava.util.ObjectUtil;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class InspectionUtil {
  public static InspectionPanel inspect(Object root) {
    return inspect(ObjectUtil.toNiceString(root), root);
  }

  public static InspectionPanel inspect(String frameTitle, Object root) {
    Prophecy.init();

    JFrame frame = new JFrame(frameTitle);
    frame.setSize(800, 600);
    InspectionPanel inspectionPanel = new InspectionPanel(root);

    frame.getContentPane().add(inspectionPanel.getPanel());
    GUIUtil.showFrame(frame);
    return inspectionPanel;
  }

  public static String objectToString(Object object) {
    /*if (object == null) return "null";
    String name = object.getClass().getName();
    int i = name.lastIndexOf('.');
    if (i >= 0) name = name.substring(i+1);
    return name;*/
    return ObjectUtil.toNiceString(object);
  }

  public static List<InspectionNode> getNodeList(Collection collection) {
    List<InspectionNode> list = new ArrayList<InspectionNode>();
    for (Object o : collection) {
      list.add(new InspectionNode(String.valueOf(list.size()+1), o));
    }
    return list;
  }

  public static void visualizeInFrame(Visualizable visualizable) {
    JComponent component = visualizable.visualize();
    if (component != null)
      SurfaceUtil.show(nice(visualizable), component);
    else
      InspectionUtil.inspectBean(visualizable);
  }

  public static void visualizeInFrame(Visualizable visualizable, Dimension size) {
    JComponent component = visualizable.visualize();
    if (component != null) {
      JFrame frame = SurfaceUtil.makeFrame(nice(visualizable), component);
      frame.setSize(size);
      GUIUtil.showFrame(frame);
    } else
      InspectionUtil.inspectBean(visualizable);
  }

  private static void inspectBean(Object o) {
    inspect(BeanInspector.makeNode(o));
  }
}