/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.ToTree;
import drjava.util.Tree;

import static java.lang.Math.abs;

public class Marker implements ToTree {
  private double second;
  private String name;

  public Marker(double second) {
    this.second = second;
  }

  public Marker(double second, String name) {
    this.second = second;
    this.name = name;
  }

  public boolean after(Marker marker) {
    return second > marker.second;
  }

  public boolean before(Marker marker) {
    return second < marker.second;
  }

  public double getSecond() {
    return second;
  }

  /** returns seconds */
  public double distanceTo(Marker marker) {
    return abs(getSecond()-marker.getSecond());
  }

  public Marker plus(double seconds) {
    return new Marker(getSecond()+seconds);
  }

  public Tree toTree() {
    Tree tree = new Tree(getClass())
      .add(new Tree(second))
      .setString("name", name);
    return tree;
  }

  public Marker(Tree tree) {
    Integer sample = tree.getInt("sample");
    if (sample != null)
      second = sample/tree.getDouble("sampleRate");
    else
      second = tree.get(0).doubleValue();
    name = tree.getString("name");
  }

  public String getName() {
    return name;
  }

  public void setName(String name) {
    this.name = name;
  }

  public void setSecond(Double second) {
    this.second = second;
  }
}