/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class ObjectAnalyzer {
  public List<InspectionNode> detectChildren(Object object) {
    if (object instanceof InspectionProxy)
      return detectChildren(((InspectionProxy) object).getObject());
    if (object instanceof InspectableParent)
      return ((InspectableParent) object).getInspectionChildren();
    if (object instanceof SimpleInspectableParent)
      return detectChildren(((SimpleInspectableParent) object).getInspectionChildren());
    if (object instanceof Collection) {
      List<InspectionNode> result = new ArrayList<InspectionNode>();
      int i = 0;
      for (Object o : (Collection) object) {
        String name = String.valueOf(++i);
        result.add(makeInspectionNode(name, o));
      }
      return result;
    }
    if (object != null && object.getClass().isArray()) {
      // TODO: this doesn't work at all really
      /*
      if (object.getClass().getComponentType() == Double.TYPE)
        return detectChildren(Arrays.asList((double[]) object));
      else
        return detectChildren(Arrays.asList((Object[]) object));
      */
    }
    return Collections.emptyList();
  }

  protected InspectionNode makeInspectionNode(String name, Object o) {
    return new InspectionNode(name, o);
  }
}