/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.Errors;
import drjava.util.GUIUtil;
import drjava.util.SwingTimerHelper;
import drjava.vats.SwingVat;
import prophecy.ProphecyCustomGUI;
import prophecy.common.lifeform.CommunityServices;
import prophecy.common.lifeform.Environment;
import prophecy.common.lifeform.Lifeform;
import prophecy.m.m03.HousePanel;
import prophecy.common.gui.AutoVMExit;

import javax.swing.*;
import java.io.File;
import java.net.URL;

public class Prophecy {
  private static boolean inited;
  private static CommunityServices globalCommunityServices = new CommunityServices();
  private static boolean initedHeadless;

  public static String dataDir() {
    return "data/";
  }

  public static String memoryDir() {
    return "memory/";
  }

  public static SourceBrowser makeSourceBrowser() {
    SourceBrowser sourceBrowser = new SourceBrowser();
    sourceBrowser.scanDir("src");
    sourceBrowser.scanZip("prophecy-src.zip", "src");
    return sourceBrowser;
  }

  public static synchronized void init() {
    if (!inited) {
      inited = true;
      AutoVMExit.install();
      initUI();
      //System.out.println("Decorations supported: " + UIManager.getLookAndFeel().getSupportsWindowDecorations());
      GUIUtil.setCustomGUI(new ProphecyCustomGUI());
      SwingTimerHelper.setPreRunHookUp(new Runnable() {
        public void run() {
          SwingVat.initThisThread();
          /*System.out.println("Prophecy.init: SwingTimerHelper: initThread "
            + Vat.threadToString(Thread.currentThread()));*/
        }
      });
    }
  }

  public static synchronized void initHeadless() {
    if (!initedHeadless) {
      initedHeadless = true;
      try {
        StackTraceElement[] stackTrace = new Throwable().getStackTrace();
        if (stackTrace.length >= 2) {
          String callingClass = stackTrace[1].getClassName();
          StdOutRecorder.recordAndFileUnder(Class.forName(callingClass));
        }
      } catch (Throwable e) {
        Errors.add(e);
      }
    }
  }

  public static ImageIcon happyIcon() {
    return getIcon("happy.gif");
  }

  public static ImageIcon flatIcon() {
    return getIcon("flat.gif");
  }

  public static ImageIcon ghostIcon() {
    return getIcon("ghost.gif");
  }

  public static ImageIcon crazyIcon() {
    return getIcon("rcain.gif");
  }

  public static ImageIcon cupIcon() {
    return getIcon("cup_orange.gif");
  }

  public static Icon downIcon() {
    return getIcon("down.gif");
  }

  public static Icon closeIcon() {
    return getIcon("x.gif");
  }

  public static Icon refreshIcon() {
    return getIcon("refresh16.gif");
  }

  public static ImageIcon getIcon(String name) {
    String path = "img/" + name;
    if (new File(path).exists())
      return new ImageIcon(path);
    URL url = Prophecy.class.getClassLoader().getResource(path);
    if (url != null)
      return new ImageIcon(url);
    return null;
  }

  public static CommunityServices getGlobalCommunityServices() {
    return globalCommunityServices;
  }

  public static void debug(Object object) {
    if (object instanceof Lifeform) {
      Lifeform lifeform = (Lifeform) object;
      if (lifeform instanceof Environment) {
        new HousePanel((Environment) lifeform).showFrame();
        return;
      }
      if (lifeform.getEnvironment() != null) {
        new HousePanel(lifeform.getEnvironment()).showFrame();
        return;
      }
    }
    InspectionUtil.inspect(BeanInspector.makeNode(object));
  }

  public static File getMemoryDir() {
    return new File("memory");
  }

  public static void initUI() {
    systemLookAndFeel();
  }

  public static void systemLookAndFeel() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Throwable e) {
      //Errors.add(e);
    }
  }

  public static void disableAutoExit() {
    AutoVMExit.disable();
  }
}