/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.Errors;

import java.io.File;
import java.io.IOException;
import java.util.Enumeration;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class SourceBrowser {
  private InspectionNode root;
  private int selectedRow;

  public SourceBrowser() {
    root = new InspectionNode("Packages");
  }

  public void scanDir(String dir) {
    if (new File(dir).isDirectory())
      scanPackage(dir, "");
  }

  public void scanZip(String zipFile, String embeddedDir) {
    if (new File(zipFile).isFile())
      try {
        scanZipImpl(zipFile, embeddedDir);
      } catch (IOException e) {
        Errors.add(e);
      }
  }

  private void scanZipImpl(String zipFileName, String embeddedDir) throws IOException {
    Map<String, InspectionNode> packages = new TreeMap<String, InspectionNode>();
    if (embeddedDir.length() != 0) embeddedDir += "/";
    ZipFile zipFile = new ZipFile(zipFileName);
    Enumeration<? extends ZipEntry> entries = zipFile.entries();
    while (entries.hasMoreElements()) {
      ZipEntry entry = entries.nextElement();
      if (entry.getName().startsWith(embeddedDir) && entry.getName().endsWith(".java")) {
        String name = entry.getName().substring(embeddedDir.length());
        int idx = name.lastIndexOf('/');
        if (idx >= 0) {
          String pkg = name.substring(0, idx).replace('/', '.');
          InspectionNode pkgNode = packages.get(pkg);
          if (pkgNode == null) {
            packages.put(pkg, pkgNode = new InspectionNode(pkg));
          }
          pkgNode.add(new ZipSourceFile(zipFileName, entry.getName()));
        }
      }
    }

    for (InspectionNode node : packages.values()) {
      root.add(node);
    }

    zipFile.close();
  }

  public void selectPackage(String pkg) {
    List<InspectionNode> children = root.getChildren();
    for (int i = 0; i < children.size(); i++)
      if (children.get(i).getName().equals(pkg)) {
        selectedRow = i+1;
        break;
      }
  }

  public void show() {
    InspectionPanel inspectionPanel = InspectionUtil.inspect("Source Browser", root);
    inspectionPanel.selectRow(selectedRow);
  }

  private void scanPackage(String srcDir, String pkgName) {
    InspectionNode packageNode = new InspectionNode(pkgName);
    File packageFile = new File(srcDir + "/" + pkgName.replace('.', '/'));
    System.out.println(packageFile);
    if (packageFile.exists()) {
      File[] files = packageFile.listFiles();

      // find java files
      for (File file : files) {
        if (file.getName().endsWith(".java"))
          packageNode.add(new SourceFile(file));
      }
      if (!packageNode.getChildren().isEmpty())
        root.add(packageNode);

      // scan subdirs
      for (File file : files) {
        if (file.isDirectory() && !file.getName().startsWith(".")) {
          String subPkg = pkgName.length() == 0 ? "" : pkgName + ".";
          scanPackage(srcDir, subPkg + file.getName());
        }
      }
    }
  }

  public static void main(String[] args) {
    SourceBrowser browser = new SourceBrowser();
    browser.scanZip("web/download/prophecy-src.zip", "src");
    browser.selectPackage("prophecy");
    browser.show();
  }
}