/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common;

import drjava.util.Tree;

import java.util.ArrayList;
import java.util.List;

public abstract class SubsetContest<A> extends Contest<Subset<A>> {
  protected SubsetContest() {
    super((Class) Subset.class);
  }

  protected SubsetContest(Tree tree) {
    super(tree);
  }

  public double evaluate(Subset<A> contestant) {
    int score = 0;
    for (int i = 0; i < getPositiveExamples().size(); i++) {
      A example = getPositiveExamples().get(i);
      if (!contestant.contains(example)) {
        --score;
        //System.out.println("pfail " + i);
      }
    }
    for (int i = 0; i < getNegativeExamples().size(); i++) {
      A example = getNegativeExamples().get(i);
      if (contestant.contains(example)) {
        --score;
        //System.out.println("nfail " + i);
      }
    }
    return score;
  }

  protected abstract List<A> getPositiveExamples();
  protected abstract List<A> getNegativeExamples();

  public Double getBestPossibleScore() {
    return 0.0;
  }

  public Double getWorstPossibleScore() {
    return new Double(-(getPositiveExamples().size() + getNegativeExamples().size()));
  }

  public List<SubsetTI<A>> getContestData() {
    ArrayList<SubsetTI<A>> result = new ArrayList<SubsetTI<A>>();
    for (A example : getPositiveExamples())
      result.add(new SubsetTI<A>(example, true));
    for (A example : getNegativeExamples())
      result.add(new SubsetTI<A>(example, false));
    return result;
  }
}