/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.comments;

import drjava.util.StringUtil;
import drjava.util.ToTree;
import drjava.util.Tree;

import java.util.Date;

public class Comment implements ToTree {
  private Date date;
  private String author, text;
  public static final int authorMaxLength = 40;
  public static final int textMaxLength = 500;

  public Comment(String author, String text) {
    this.date = new Date();
    this.author = StringUtil.maxLength(authorMaxLength, author);
    this.text = StringUtil.maxLength(textMaxLength, text);
  }

  public Comment(Tree tree) {
    date = new Date(tree.getLong("date"));
    author = tree.getString("author");
    text = tree.getString("text");
  }

  public Date getDate() {
    return date;
  }

  public String getAuthorAsHtml() {
    return StringUtil.escapeHtml(author);
  }

  public String getTextAsHtml() {
    return StringUtil.escapeHtml(text);
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("date", date.getTime())
      .add("author", author)
      .add("text", text);
  }

  public void setDate(Date date) {
    this.date = date;
  }
}