/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.comments;

import drjava.util.GUIUtil;
import prophecy.common.gui.Sheet;
import drjava.util.Errors;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.List;

public class CommentPanel {
  private Sheet sheet;
  private CommentClient client;
  private List<Comment> comments;
  private JTextField tfText;
  private JTextField tfAuthor;
  private JButton btnShout;

  public CommentPanel(String server, String shoutboxOwner) {
    client = new CommentClient(server);
    sheet = new Sheet("Shoutbox for " + shoutboxOwner);
    sheet.setSize(300, 700);

    loadAndRender();
  }

  private void loadAndRender() {
    new Thread() {
      public void run() {
        try {
          loadData();
          SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              render();
            }
          });
        } catch (Throwable e) {
          Errors.add(e);
          sheet.addLabel("[Could not connect to server]");
        }
      }
    }.start();
  }

  private void render() {
    final Sheet.Snapshot snapshot = sheet.snapshot();

    if (comments.isEmpty()) {
      sheet.addLabel("No comments yet. Add yours:");
    } else {
      for (Comment comment : comments) {
        Shoutbox.addComment(sheet, comment);
      }

      sheet.addBlueHeading("Shout");
    }

    tfAuthor = new JTextField(ShoutboxDefaults.getUserName());
    sheet.addComponent(GUIUtil.withLabel("Your name:", tfAuthor));

    tfText = new JTextField();
    sheet.addComponent(GUIUtil.withLabel("Text:", tfText));

    btnShout = sheet.addButton("Shout!", new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          ShoutboxDefaults.setUserName(tfAuthor.getText());
          client.addComment(new Comment(
            tfAuthor.getText(),
            tfText.getText()));
          snapshot.restore();
          loadAndRender();
        } catch (IOException e) {
          Errors.add(e);
        }
      }
    });

    DocumentListener documentListener = new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        update();
      }

      public void removeUpdate(DocumentEvent e) {
        update();
      }

      public void changedUpdate(DocumentEvent e) {
        update();
      }
    };
    tfAuthor.getDocument().addDocumentListener(documentListener);
    tfText.getDocument().addDocumentListener(documentListener);
    update();
  }

  private void update() {
    btnShout.setEnabled(
      checkLength(tfText.getText(), Comment.textMaxLength)
        && checkLength(tfAuthor.getText(), Comment.authorMaxLength));
  }

  private boolean checkLength(String text, int maxLength) {
    return text.length() > 0 && text.length() <= maxLength;
  }

  private void loadData() throws IOException {
    comments = client.getComments();
  }

  public Sheet getSheet() {
    return sheet;
  }
}