/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.comments;

import drjava.util.*;

import java.io.IOException;
import java.util.Date;
import java.util.List;

public class CommentTreelet implements Treelet {
  private CommentList commentList;

  public CommentTreelet(TreePersistence persistence) {
    commentList = new CommentList(persistence);
  }

  public static void main(String[] args) throws IOException {
    CommentTreelet treelet = new CommentTreelet(new FileTreePersistence("temp/testcomments.tree"));
    new TreeServer(54005, treelet).start();
  }

  public void serve(Tree question, TreeRequest request) {
    if (question.nameIs("getCommentCount")) {
      request.answer(new Tree(commentList.getCommentCount()));
      return;
    }
    if (question.nameIs("getComment")) {
      request.answer(commentList.getComment(question.getInt(0)).toTree());
      return;
    }
    if (question.nameIs("addComment")) {
      Comment comment = new Comment(question.get(0));
      comment.setDate(new Date());
      commentList.addComment(comment);
      return;
    }
    if (question.nameIs("getComments")) {
      request.answer(TreeUtil.listToTree(commentList.getComments()));
      return;
    }

    if (question.nameIs("getComments_keepalive")) {
      List<Comment> prev = commentList.getComments();
      request.answer(TreeUtil.listToTree(prev));
      while (true) {
        MultiCoreUtil.sleep(1000);
        List<Comment> comments = commentList.getComments();
        if (comments.size() != prev.size()) {
          prev = comments;
          request.answer(TreeUtil.listToTree(prev));
        }
      }
    }

    if (question.nameIs("getTriggerStream")) {
      List<Comment> prev = commentList.getComments();
      while (true) {
        MultiCoreUtil.sleep(1000);
        List<Comment> comments = commentList.getComments();
        if (comments.size() != prev.size()) {
          prev = comments;
          request.answer(null);
        }
      }
    }
  }
}