/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.comments;

import drjava.util.GUIUtil;
import drjava.util.LetterLayout;
import drjava.util.TextView;
import prophecy.common.gui.Sheet;
import drjava.util.Errors;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class Shoutbox {
  private Sheet sheet;
  private int commentCount;
  private CommentClient client;
  private List<Comment> lastComments;
  private String shoutboxOwner;
  private Sheet.Snapshot snapshot;
  private String server;

  public Shoutbox(String server, String shoutboxOwner) {
    this.server = server;
    this.shoutboxOwner = shoutboxOwner;
    client = new CommentClient(server);
    sheet = new Sheet();
    sheet.addBlueHeading("Shoutbox");

    snapshot = sheet.snapshot();
    sheet.addLabel("[Connecting to server...]");
    loadAndRender();
    waitForChanges();
  }

  private void loadAndRender() {
    new Thread() {
      public void run() {
        try {
          System.out.println("Loading data");
          loadData();
          System.out.println("Data loaded");
          SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              render();
            }
          });
        } catch (Throwable e) {
          Errors.add(e);
          snapshot.restore();
          sheet.addLabel("[Could not connect to server]");
        }
      }
    }.start();
  }

  private void waitForChanges() {
    try {
      client.getTriggerStream().addStreamListener(new StreamListener<List<Comment>>() {
        public void dataReceived(List<Comment> item) {
          loadAndRender();
        }

        public void streamClosed() {
        }
      });
    } catch (Throwable e) {
      Errors.add(e);
    }
  }

  private void render() {
    snapshot.restore();

    if (commentCount == 0) {
      sheet.addLabel("No comments yet." /*" Add yours:"*/);
    } else {
      for (Comment comment : lastComments) {
        addComment(sheet, comment);
      }
    }

    JButton btnMore = new JButton(
      commentCount > lastComments.size() ? "More..." : "Shout...");
    btnMore.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        GUIUtil.showFrame(new CommentPanel(server, shoutboxOwner).getSheet().asFrame());
      }
    });
    sheet.addComponent(LetterLayout.rightAlignedRowPanel(btnMore));

    sheet.getSheetLayout().setLeftBorder(0);
    sheet.getSheetLayout().setRightBorder(0);
  }

  public static void addComment(Sheet sheet, Comment comment) {
    sheet.addComponent(new JLabel("<html>"
      + TextView.nlToBr(comment.getTextAsHtml()) + "</html>"));
    JLabel lAuthor = new JLabel("<html>-" + comment.getAuthorAsHtml() + "</html>", JLabel.RIGHT);
    sheet.addComponent(lAuthor);
  }

  private void loadData() throws IOException {
    commentCount = client.getCommentCount();
    lastComments = new ArrayList<Comment>();
    for (int i = commentCount-1; i >= 0 && i >= commentCount-2; i--)
      lastComments.add(client.getComment(i));
  }

  public Sheet getSheet() {
    return sheet;
  }
}