/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.comments;

import drjava.util.Tree;
import drjava.util.TreeUtil;
import drjava.util.Errors;
import prophecy.common.socket.SocketHandler;

import java.io.IOException;
import java.net.Socket;

public class TreeClient {
  private String host;
  private int port;

  public TreeClient(String host, int port) {
    this.host = host;
    this.port = port;
  }

  public TreeClient(int port) {
    this.host = "localhost";
    this.port = port;
  }

  public Tree ask(Tree question) throws IOException {
    System.out.println("? " + question);
    SocketHandler handler = open(question);
    Tree answer = getAnswer(handler);
    System.out.println("! " + answer);
    handler.close();
    return answer;
  }

  private Tree getAnswer(SocketHandler handler) throws IOException {
    String line = handler.readLine();
    Tree answer = line == null ? null : TreeUtil.textToTree(line);
    return answer;
  }

  private SocketHandler open(Tree question) throws IOException {
    Socket socket = new Socket(host, port);
    SocketHandler handler = new SocketHandler(socket);
    handler.println(question.toString());
    return handler;
  }

  public Stream<Tree> ask_keepalive(final Tree question) throws IOException {
    final Stream<Tree> stream = new Stream<Tree>();
    new Thread() {
      public void run() {
        try {
          final SocketHandler handler = open(question);
          while (true) {
            String line = null;
            line = handler.readLine();
            System.out.println("stream: line received");
            if (line == null) {
              break;
            }
            Tree answer = TreeUtil.textToTree(line);
            stream.putIn(answer);
          }
        } catch (IOException e) {
          Errors.add(e);
        } finally {
          stream.close();
        }
      }
    }.start();
    return stream;
  }
}