/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.comments;

import drjava.util.Errors;
import drjava.util.Log;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.common.Trigger;
import prophecy.common.socket.SocketHandler;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public class TreeServer {
  private int port;
  private Treelet treelet;
  private ServerSocket serverSocket;
  private Trigger onClientHandled = new Trigger();

  public TreeServer(int port, Treelet treelet) {
    this.port = port;
    this.treelet = treelet;
  }

  public TreeServer(ServerSocket serverSocket, Treelet treelet) {
    this.serverSocket = serverSocket;
    this.treelet = treelet;
  }

  public void start() throws IOException {
    if (serverSocket == null)
      serverSocket = new ServerSocket(port);
    Log.info("TreeServer listening on port " + serverSocket.getLocalPort());
    new Thread() {
      public void run() {
        try {
          while (true) {
            final Socket socket = serverSocket.accept();
            new Thread() {
              public void run() {
                try {
                  handleClient(socket);
                } catch (IOException e) {
                  Errors.add(e);
                }
              }
            }.start();
          }
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    }.start();
  }

  private void handleClient(Socket socket) throws IOException {
    Treelet treelet = freshTreelet();
    final SocketHandler handler = new SocketHandler(socket);
    String line = handler.readLine();
    if (line != null) {
      System.out.println("recvd: " + line);
      try {
        Tree question = TreeUtil.textToTree(line);
        Log.info("? " + question);
        treelet.serve(question, new TreeRequest() {
          public void answer(Tree answer) {
            String answerText = answer == null ? "" : answer.toString();
            Log.info("! " + answerText);
            handler.println(answerText);
          }
        });
      } catch (Throwable e) {
        Errors.add(e);
      }
    }
    if (!handler.lineSent())
      handler.println("");
    handler.close();
    onClientHandled.trigger();
  }

  /** override me */
  public Treelet freshTreelet() {
    return treelet;
  }

  public Trigger getOnClientHandled() {
    return onClientHandled;
  }
}