/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.gui;

import drjava.util.Errors;
import drjava.util.Lizt;
import static drjava.util.ObjectUtil.nice;
import drjava.util.PopupMenuHelper;
import drjava.util.Line;
import prophecy.m.m03.LifeformUIUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

public class JObject extends Line {
  public JObject(final Object object) {
    setSpacing(2);

    if (object != null) {
      add(new PopDownButton() {
        protected void fillMenu(JPopupMenu menu) {
          if (object instanceof Runnable) {
            JMenuItem miRun = new JMenuItem("Run");
            miRun.addActionListener(new ActionListener() {
              public void actionPerformed(ActionEvent e) {
                new Thread((Runnable) object).start();
              }
            });
            menu.add(miRun);
          }

          LifeformUIUtil.objectMenuEntries(menu, object);
        }
      });
    }

    if (object instanceof RenderToLine) {
      if (renderToLine(object)) return;
    }

    renderStandard(object);
  }

  private boolean renderToLine(Object object) {
    try {
      Lizt<Object> parts = ((RenderToLine) object).renderToLine();
      for (Object part : parts) {
        if (part instanceof String)
          add(new JBetterLabel((String) part));
        else
          add(new JObject(part));
      }
      return true;
    } catch (Throwable e) {
      Errors.add(e);
      removeAll();
      // process with normal rendering
    }
    return false;
  }

  private void renderStandard(final Object object) {
    JLabel label = new JBetterLabel(nice(object));
    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        LifeformUIUtil.objectMenuEntries(menu, object);
      }
    }.install(this);

    add(label);
  }
}