/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.image;

import drjava.util.Errors;
import drjava.util.PopupMenuHelper;
import prophecy.common.Prophecy;
import prophecy.common.Surface;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.IOException;

public class ImageSurface extends Surface {
  private BufferedImage image;
  private double zoomX = 1, zoomY = 1;

  public ImageSurface() {
    this(new RGBImage(1, 1, new int[] { 0xFFFFFF }));
  }

  public ImageSurface(RGBImage image) {
    clearSurface = false;
    this.image = image.getBufferedImage();
    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        fillPopupMenu(menu);
      }
    }.install(this);
  }

  protected void fillPopupMenu(JPopupMenu menu) {
    JMenuItem miSave = new JMenuItem("Save image...");
    miSave.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        saveImage();
      }
    });
    menu.add(miSave);
  }

  public void saveImage() {
    RGBImage image = new RGBImage(getImage(), null);
    JFileChooser fileChooser = new JFileChooser(Prophecy.dataDir());
    if (fileChooser.showSaveDialog(this) == JFileChooser.APPROVE_OPTION) {
      try {
        image.save(fileChooser.getSelectedFile());
      } catch (IOException e) {
        Errors.add(e);
      }
    }
  }

  public void loadImage() {
    JFileChooser fileChooser = new JFileChooser(Prophecy.dataDir());
    if (fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
      try {
        RGBImage image = RGBImage.load(fileChooser.getSelectedFile());
        setImage(image);
      } catch (Throwable e) {
        Errors.add(e);
      }
    }
  }

  public ImageSurface(BWImage image) {
    this(image.toRGB());
  }

  public void render(int w, int h, Graphics2D g) {
    if (image != null)
      g.drawImage(image, 0, 0, getZoomedWidth(), getZoomedHeight(), null);
  }

  public void setZoom(double zoom) {
    setZoom(zoom, zoom);
  }

  public void setZoom(double zoomX, double zoomY) {
    this.zoomX = zoomX;
    this.zoomY = zoomY;
    repaint();
  }

  public Dimension getMinimumSize() {
    int w = getZoomedWidth();
    int h = getZoomedHeight();
    Dimension min = super.getMinimumSize();
    return new Dimension(Math.max(w, min.width), Math.max(h, min.height));
  }

  private int getZoomedHeight() {
    return (int) (image.getHeight() * zoomY);
  }

  private int getZoomedWidth() {
    return (int) (image.getWidth() * zoomX);
  }

  public void setImage(RGBImage image) {
    this.image = image.getBufferedImage();
    repaint();
  }

  public BufferedImage getImage() {
    return image;
  }

  public void setImage(BWImage image) {
    setImage(image.toRGB());
  }

  public double getZoomX() {
    return zoomX;
  }

  public double getZoomY() {
    return zoomY;
  }

  public Dimension getPreferredSize() {
    return new Dimension(getZoomedWidth(), getZoomedHeight());
  }
}