/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.lifeform;

import drjava.util.ObjectUtil;

public class Activity {
  private boolean ranOnce;
  private Lifeform lifeform;

  public String getStatus() {
    return ObjectUtil.nice(this);
  }

  /** override me */
  protected void idleImpl() throws Exception {
  }

  public final void idle() {
    if (!ranOnce) {
      ranOnce = true;
      try {
        runOnce();
      } catch (Throwable e) {
        error(e);
      }
    }
    try {
      idleImpl();
    } catch (Throwable e) {
      error(e);
    }
  }

  /** override me */
  protected void runOnce() throws Exception {
  }

  public Lifeform getLifeform() {
    return lifeform;
  }

  public void setLifeform(Lifeform lifeform) {
    this.lifeform = lifeform;
  }

  public void error(Throwable throwable) {
    lifeform.error(throwable);
  }

  public void error(String text) {
    lifeform.error(text);
  }

  /** override me */
  public void introduce(Lifeform lifeform) {
  }

  public void done() {
    if (lifeform != null && lifeform.getActivity() == this) {
      lifeform.setStatus("done");
      lifeform.setActivity(null);
    }
  }
}