/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.lifeform;

import drjava.util.Errors;
import drjava.util.StringUtil;
import prophecy.m.m03.EnvironmentListener;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

public abstract class Environment extends Lifeform {
  private List<Lifeform> inhabitants = new ArrayList<Lifeform>();
  private List<EnvironmentListener> listeners = new ArrayList<EnvironmentListener>();
  private String name;
  private Lifeform leader;
  private TimeService timeService;

  protected void idleInhabitants() {
    List<Lifeform> objects = getInhabitants();
    for (Lifeform object : objects) {
      long startTime = System.currentTimeMillis();
      try {
        object.idle();
      } catch (Throwable e) {
        Errors.add(e);
      }
      long endTime = System.currentTimeMillis();
      object.addCPUTime(Math.max(0, endTime-startTime));
    }
  }

  public Environment() {
  }

  public Environment(String name) {
    setName(name);
  }

  public synchronized void addInhabitant(Lifeform lifeform) {
    inhabitants.add(lifeform);
    lifeform.setEnvironment(this);
    for (EnvironmentListener listener : listeners) {
      try {
        listener.newInhabitant(this, lifeform);
      } catch (Throwable e) {
        Errors.add(e);
      }
    }
  }

  public synchronized void addVisitor(Lifeform visitor) {
    addInhabitant(visitor);
  }

  public synchronized List<Lifeform> getInhabitants() {
    return new ArrayList<Lifeform>(inhabitants);
  }

  public void addListener(EnvironmentListener listener) {
    listeners.add(listener);
  }

  public void removeInhabitant(Lifeform lifeform) {
    if (lifeform.getEnvironment() != this)
      logSurprise("removeInhabitant: " + lifeform + " is part of different environment: " + lifeform.getEnvironment());
    else {
      lifeform.setEnvironment(null);
      logActivity("Removing inhabitant: " + lifeform);
    }
    inhabitants.remove(lifeform);
  }

  public void release(Lifeform lifeform) {
    logActivity("Releasing: " + lifeform);
    addInhabitant(lifeform);
  }

  public void setName(String name) {
    this.name = name;
  }

  public String getName() {
    return name;
  }

  public String toString() {
    return StringUtil.notEmpty(name) ? name : super.toString();
  }

  public Lifeform getLeader() {
    return leader;
  }

  public void setLeader(Lifeform lifeform) {
    logActivity("setLeader: " + lifeform);
    if (leader == null && lifeform != null) {
      if (!lifeform.wantsToLead()) {
        logSurprise(lifeform + " does not want to lead");
        return;
      }

      log("Welcoming as leader: " + lifeform);
      leader = lifeform;
      leader.setLeader(true);
    }
  }

  public Icon makeIcon() {
    return new ImageIcon("img/icon_house.gif");
  }

  public void add(Lifeform inhabitant) {
    addInhabitant(inhabitant);
  }

  protected void idleImpl() {
    super.idleImpl();
    idleInhabitants();
  }

  public TimeService getTimeService() {
    return timeService;
  }

  public void setTimeService(TimeService timeService) {
    this.timeService = timeService;
  }
}