/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.lifeform;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class Introducer extends Lifeform {
  private HashSet<ObjectID> knownPeers = new HashSet<ObjectID>();
  private boolean randomIntro;

  public Introducer() {
    this(true);
  }

  public Introducer(boolean randomIntro) {
    this.randomIntro = randomIntro;
  }

  public void idleImpl() {
    super.idleImpl();
    if (getEnvironment() != null) {
      List<Lifeform> inhabitants = getEnvironment().getInhabitants();
      introNew(inhabitants);
      if (randomIntro)
        randomIntro(inhabitants);
    }
  }

  private void introNew(List<Lifeform> inhabitants) {
    Set<ObjectID> previousPeers = knownPeers;
    knownPeers = new HashSet<ObjectID>();
    for (Lifeform inhabitant : inhabitants) {
      if (!previousPeers.contains(inhabitant.getObjectID()))
        introToAll(inhabitant, inhabitants);
      knownPeers.add(inhabitant.getObjectID());
    }
  }

  private void introToAll(Lifeform lifeform, List<Lifeform> inhabitants) {
    for (Lifeform inhabitant : inhabitants) {
      inhabitant.introduce(lifeform);
      lifeform.introduce(inhabitant);
    }
  }

  private void randomIntro(List<Lifeform> inhabitants) {
    Lifeform object1 = getRandomizer().oneOf(inhabitants);
    Lifeform object2 = getRandomizer().oneOf(inhabitants);
    if (object1 != null && object2 != null) {
      logActivity("Introducing " + drjava.util.ObjectUtil.nice(object2) + " to " + drjava.util.ObjectUtil.nice(object1));
      object1.introduce(object2);
    }
  }
}