/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.lifeform;

import static drjava.util.ObjectUtil.nice;
import drjava.util.GUIUtil;
import prophecy.common.Prophecy;
import prophecy.m.m03.HousePanel;

import javax.swing.*;

public class ProphecyEnv {
  private Environment environment;
  private JTabbedPane tabbedPane;

  public ProphecyEnv(Environment environment) {
    Prophecy.init();
    this.environment = environment;
  }

  public ProphecyEnv() {
    this(new LiveEnvironment());
  }

  public ProphecyEnv(String houseName) {
    this();
    environment.setName(houseName);
  }

  public ProphecyEnv(String houseName, Lifeform lf1, Lifeform... moreInhabitants) {
    this(houseName);
    addInhabitant(lf1);
    if (lf1.wantsToLead())
      environment.setLeader(lf1);
    for (Lifeform inhabitant : moreInhabitants) {
      add(inhabitant);
    }
  }

  public ProphecyEnv(Lifeform lf1, Lifeform... moreInhabitants) {
    this("House of " + nice(lf1), lf1, moreInhabitants);
  }

  public Environment getHouse() {
    return environment;
  }

  public JFrame showMainFrame() {
    return GUIUtil.showMainFrame(nice(environment),
      tabbedPane != null ? tabbedPane : new HousePanel(environment));
  }

  public JFrame showFrame() {
    return GUIUtil.showFrame(nice(environment),
      tabbedPane != null ? tabbedPane : new HousePanel(environment));
  }

  public void addInhabitant(Lifeform lifeform) {
    environment.addInhabitant(lifeform);
  }

  public void addTab(String name, JComponent component) {
    if (tabbedPane == null) {
      tabbedPane = new JTabbedPane();
      tabbedPane.addTab("House", new HousePanel(environment));
    }
    tabbedPane.addTab(name, component);
  }

  public void add(Lifeform lifeform) {
    addInhabitant(lifeform);
  }

  public static void test(Lifeform lf) {
    new ProphecyEnv(lf).showMainFrame();
  }

  public static void test(Object o) {
    test(new Box(o));
  }

  /** stop idle thread */
  public void retire() {
    environment.retire();
  }
}