/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.lifeform;

import drjava.util.Errors;
import drjava.util.ObjectUtil;

import java.util.ArrayList;
import java.util.List;

public class Reference {
  private Lifeform value = Nobody.get();
  private List<ReferenceListener> listeners = new ArrayList<ReferenceListener>();

  public Lifeform get() {
    return value;
  }

  public void set(Lifeform value) {
    Lifeform oldValue = this.value;
    Lifeform newValue = value == null ? Nobody.get() : value;
    if (oldValue != newValue) {
      this.value = newValue;
      for (ReferenceListener listener : listeners) {
        try {
          listener.referenceChanged(this, oldValue, newValue);
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    }
  }

  public String toString() {
    return ObjectUtil.nice(value);
  }

  public void addReferenceListener(ReferenceListener listener) {
    listeners.add(listener);
  }

  public boolean isNobody() {
    return value == Nobody.get();
  }
}