/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.socket;

import prophecy.m.m01.SocketUtil;
import prophecy.common.socket.ObjectURL;
import prophecy.common.socket.DialogServer;
import prophecy.t.t03.Receptionist;
import prophecy.t.t08.Receptionist2;
import prophecy.t.t11.SocketGuard;

import java.io.IOException;
import java.net.ServerSocket;

// from: t04
public class ObjectServer {
  private Receptionist receptionist;
  protected DialogServer dialogServer;

  public ObjectServer(ServerSocket socket) {
    receptionist = makeReceptionist();
    dialogServer = new DialogServer(socket, receptionist);
    setName("os" + socket.getLocalPort());
    setURLs();
  }

  /** use random port */
  public ObjectServer() {
    this(SocketUtil.serverSocketAtPortAbove(400));
  }

  public ObjectServer(int port) throws IOException {
    this(new ServerSocket(port));
  }

  public ObjectServer start() throws IOException {
    dialogServer.start();
    return this;
  }

  private void setURLs() {
    ObjectURL url = ObjectURL.get(dialogServer);
    ObjectURL.set(this, url);
    ObjectURL.set(receptionist, url);
  }

  public ObjectURL getURL() {
    return dialogServer.getURL();
  }

  public void addObject(Object object) {
    receptionist.addObject(object);
  }

  public ObjectURL addObject(String address, Object o) {
    return receptionist.addObject(address, o);
  }

  public void stop() throws IOException {
    dialogServer.stop();
  }

  public void setName(String name) {
    receptionist.setServerName(name);
  }

  protected Receptionist makeReceptionist() {
    return new Receptionist2();
  }

  public void addGuard(SocketGuard guard) {
    dialogServer.addGuard(guard);
  }
}