/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.socket;

import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.t.t03.HasURL;
import prophecy.t.t01.BadURLException;
import prophecy.t.t01.Owner;
import drjava.util.Attachments;

import java.net.URLEncoder;

public class ObjectURL implements ToTree {
  private String url;

  public ObjectURL(String url) {
    url = fixURL(url);
    this.url = url;
    checkURL();
  }

  private void checkURL() {
    if (!url.startsWith("obj:"))
      throw new BadURLException(url);
  }

  public static String fixURL(String url) {
    url = url.trim();
    if (!url.startsWith("obj:"))
      url = "obj:" + url;
    return url;
  }

  public ObjectURL(Tree tree) {
    url = tree.getName();
  }

  public String getURL() {
    return url;
  }

  public String toString() {
    return url;
  }

  public ObjectURL append(String address) {
    return new ObjectURL(url + "/" + URLEncoder.encode(address));
  }

  public static ObjectURL get(Object o) {
    if (o instanceof HasURL)
      return ((HasURL) o).getURL();
    ObjectURL url = Attachments.get(o, ObjectURL.class);
    if (url == null) {
      Owner owner = Attachments.get(o, Owner.class);
      if (owner != null)
        url = get(owner.getOwner());
    }
    return url;
  }

  public static void set(Object o, ObjectURL url) {
    Attachments.set(o, url, ObjectURL.class);
  }

  public String getHead() {
    String url2 = stripProtocol();
    int i = url2.indexOf('/');
    return i < 0 ? url2 : url2.substring(0, i);
  }

  private String stripProtocol() {
    String url2 = url.replaceFirst("^obj:", "");
    return url2;
  }

  public String getTail() {
    String url2 = stripProtocol();
    int i = url2.indexOf('/');
    return i < 0 ? "" : url2.substring(i+1);
  }

  public Tree toTree() {
    return new Tree(url);
  }
}