/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.sound;

import drjava.util.FromTree;
import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.common.Marker;
import prophecy.common.Visualizable;

import javax.swing.*;

public class Sound implements Visualizable, ToTree, FromTree {
  private double sampleRate;
  protected float[] samples;
  private String file;

  /** for fromTree */
  public Sound() {
  }

  public Sound(double sampleRate, float[] samples) {
    this.sampleRate = sampleRate;
    this.samples = samples;
  }

  public Sound(double sampleRate, double seconds) {
    this.sampleRate = sampleRate;
    samples = new float[secondsToSampleNo(seconds)];
  }

  private int secondsToSampleNo(double seconds) {
    return (int) (seconds*sampleRate);
  }

  public double getSampleRate() {
    return sampleRate;
  }

  public float[] getSamples() {
    return samples;
  }

  public double getSeconds() {
    return samples.length/sampleRate;
  }

  public Sound clip(Marker m1, Marker m2) {
    return clip(m1.getSecond(), m2.getSecond());
  }

  public Sound clip(double second1, double second2) {
    int s1 = Math.max(0, Math.min(samples.length, secondsToSampleNo(second1)));
    int s2 = Math.max(0, Math.min(samples.length, secondsToSampleNo(second2)));
    float[] newArray = new float[s2-s1];
    System.arraycopy(samples, s1, newArray, 0, s2-s1);
    return new Sound(sampleRate, newArray);
  }

  public int getNumSamples() {
    return samples.length;
  }

  public float getSample(int sampleNo) {
    return samples[sampleNo];
  }

  public JComponent visualize() {
    return new SoundSurface(this);
  }

  public Tree toTree() {
    /*Tree tSamples = new Tree();
    for (float sample : samples)
      tSamples.add(new Tree(sample));
    return new Tree(getClass())
      .add("sampleRate", sampleRate)
      .add("samples", tSamples);*/
    return new Tree(getClass())
      .add("file", file);
  }

  public void fromTree(Tree tree) {
    Sound sound = SoundUtil.loadSound(tree.getString("file"));
    sampleRate = sound.sampleRate;
    samples = sound.samples;
    file = sound.file;
  }

  public void setFile(String file) {
    this.file = file;
  }

  public String getFile() {
    return file;
  }
}