/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.common.sound;

import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.common.Marker;
import prophecy.common.Visualizable;

import javax.swing.*;
import static java.util.Arrays.asList;
import java.util.List;

public class SoundWithMarkers implements InspectableParent, Visualizable, ToTree {
  public final Sound sound;
  public final List<Marker> markers;

  public SoundWithMarkers(Sound sound, List<Marker> markers) {
    this.sound = sound;
    this.markers = markers;
  }

  public SoundWithMarkers(Sound sound, double... markers) {
    this.sound = sound;
    this.markers = SoundUtil.makeMarkers(markers);
  }

  public List<InspectionNode> getInspectionChildren() {
    return asList(
      new InspectionNode("sound", sound),
      new InspectionNode("markers", markers));
  }

  public JComponent visualize() {
    return new SoundSurface(this);
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("sound", sound)
      .add("markers", markers);
  }
}