/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e01;

import drjava.util.Factory;
import drjava.util.MultiSet;
import drjava.util.StdOutDisplay;

public class CharCategorizer implements Analyzer<String, Stats<CharCategory>> {
  MultiSet<CharCategory> count = new MultiSet<CharCategory>();

  public void processString(String s) {
    for (int i = 0; i < s.length(); i++) {
      processChar(s.charAt(i));
    }
  }

  private void processChar(char c) {
    CharCategory category = getCategory(c);
    count.add(category);
  }

  public static CharCategory getCategory(char c) {
    if (Character.isLetter(c)) return CharCategory.alpha;
    if (Character.isDigit(c)) return CharCategory.num;
    if (Character.isSpaceChar(c)) return CharCategory.space;
    return CharCategory.other;
  }

  public Stats<CharCategory> getStats() {
    return new Stats<CharCategory>(count);
  }

  public static void main(String[] args) {
    String[] data = {
      "hello world",
      "what a lovely day",
      "isn't it?",
      "!$&/@*",
      "hehehe"
    };

    WatcherPool<String, Stats<CharCategory>> pool = new WatcherPool<String, Stats<CharCategory>>(
      new Factory<Analyzer<String, Stats<CharCategory>>>() {
        public Analyzer<String, Stats<CharCategory>> make() {
          return new CharCategorizer();
        }
      }, new Metric<Stats<CharCategory>>() {
        public double getDistance(Stats<CharCategory> a, Stats<CharCategory> b) {
          return StatsUtil.getDistance(a, b);
        }
      });

    for (String s : data) {
      double distance = pool.processData(s);
      System.out.println("Distance: " + distance + ", pool size: " + pool.getSize() + " # " + s);
    }

    pool.display(new StdOutDisplay());
  }

  public void addData(String s) {
    processString(s);
  }

  public Stats<CharCategory> getResult() {
    return getStats();
  }
}