/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e01;

import drjava.util.Display;
import drjava.util.Displayable;
import drjava.util.MultiSet;
import drjava.util.StdOutDisplay;

import java.util.*;

public class Stats<A> implements Displayable {
  Map<A, Float> table = new HashMap<A, Float>();

  public Stats() {
  }

  public Stats(MultiSet<A> multiSet) {
    for (A a : multiSet.asSet()) {
      put(a, multiSet.get(a));
    }
  }

  public void put(A a, float freq) {
    table.put(a, freq);
  }

  public float get(A a) {
    return unbox(table.get(a));
  }

  private float unbox(Float f) {
    return f != null ? f : 0;
  }

  public void display(Display display) {
    for (A a : elementsByDescFrequency()) {
      display.put(a).put(": ").put(table.get(a)).nl();
    }
  }

  private List<A> elementsByDescFrequency() {
    ArrayList<A> list = new ArrayList<A>(table.keySet());
    Collections.sort(list, new Comparator<A>() {
      public int compare(A a, A b) {
        float x = table.get(a), y = table.get(b);
        return x < y ? 1 : x == y ? 0 : -1;
      }
    });
    return list;
  }

  public float getTotal() {
    float sum = 0;
    for (Float f : table.values()) {
      sum += f;
    }
    return sum;
  }

  public Stats<A> normalized() {
    float total = getTotal();
    if (total == 0) return this;
    Stats<A> stats = new Stats<A>();
    for (A a : table.keySet()) {
      stats.put(a, table.get(a) / total);
    }
    return stats;
  }

  public static void main(String[] args) {
    Stats<String> stats = new Stats<String>();
    stats.put("world", 1);
    stats.put("hello", 3);
    stats.normalized().display(new StdOutDisplay());
  }

  public Collection<A> getObjects() {
    return table.keySet();
  }
}