/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e01;

import drjava.util.Display;
import drjava.util.Displayable;
import drjava.util.Factory;
import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.common.Visualizable;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

public class Watcher<A, B> implements Displayable, InspectableParent, Visualizable {
  private int logLimit = 10;
  private final ArrayList<A> rollingLog = new ArrayList<A>();
  private Factory<Analyzer<A, B>> analyzerFactory;
  private Metric<B> metric;
  private int count;
  private B cachedLogResult;

  public Watcher(Factory<Analyzer<A, B>> analyzerFactory, Metric<B> metric) {
    this.analyzerFactory = analyzerFactory;
    this.metric = metric;
  }

  public void processData(A a) {
    ++count;
    cachedLogResult = null;
    rollingLog.add(a);
    if (rollingLog.size() > logLimit)
      rollingLog.remove(0);
  }

  /** returns distance of new data to recent data */
  public double getDistanceToLog(A a) {
    B logResult = getResult();

    Analyzer<A, B> analyzer = analyzerFactory.make();
    analyzer.addData(a);

    double distance = metric.getDistance(logResult, analyzer.getResult());
    //System.out.println("distance=" + distance);
    return distance;
  }

  public B getResult() {
    if (cachedLogResult == null)
      cachedLogResult = getLogAnalyzer().getResult();
    return cachedLogResult;
  }

  private Analyzer<A, B> getLogAnalyzer() {
    Analyzer<A, B> analyzer = analyzerFactory.make();
    for (A a : rollingLog) {
      analyzer.addData(a);
    }
    return analyzer;
  }

  public void display(Display display) {
    display.putnl("Rolling log:").indent();
    for (A a : rollingLog)
      display.put(a).nl();
    display.unindent();
    display.putnl("Analysis:").indent();
    display.putnl(getResult());
    display.unindent();
  }

  public List<InspectionNode> getInspectionChildren() {
    ArrayList<InspectionNode> children = new ArrayList<InspectionNode>();
    children.add(new InspectionNode("count", count));
    children.add(new InspectionNode("result", getResult()));
    for (int i = 0; i < rollingLog.size(); i++) {
      A a = rollingLog.get(i);
      children.add(new InspectionNode("log #" + (i+1), a));
    }
    return children;
  }

  public int getCount() {
    return count;
  }

  public JComponent visualize() {
    B result = getResult();
    if (result instanceof Visualizable)
      return ((Visualizable) result).visualize();
    return null;
  }
}