/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e03;

import drjava.util.MultiSet;
import static drjava.util.ObjectUtil.toNiceString;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import prophecy.common.Prophecy;
import prophecy.common.image.RGBImage;

import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.stream.ImageInputStream;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class E3 {
  private ArrayList<CuckooSpotter<RGBImage>> contestants = new ArrayList<CuckooSpotter<RGBImage>>();
  private ArrayList<RGBImage> images;
  private MultiSet<CuckooSpotter<RGBImage>> scores;

  public void addContestant(CuckooSpotter<RGBImage> contestant) {
    contestants.add(contestant);
  }

  public void run() throws IOException {
    prepare();
    for (int i = 0; i < 100; i++)
      oneRound();
    printScores();
  }

  private void printScores() {
    System.out.println();
    List<CuckooSpotter<RGBImage>> list = scores.getSortedListDescending();
    for (int i = 0; i < list.size(); i++) {
      CuckooSpotter<RGBImage> spotter = list.get(i);
      System.out.println("#" + (i + 1) + " [" + scores.get(spotter) + "] " + toNiceString(spotter));
    }
  }

  private void oneRound() {
    int correctAnswer = 3;
    ArrayList<RGBImage> shuffledImages = new ArrayList<RGBImage>(images);
    Collections.shuffle(shuffledImages);
    correctAnswer = shuffledImages.indexOf(images.get(correctAnswer));

    for (CuckooSpotter<RGBImage> spotter : contestants) {
      int guess = spotter.guess(shuffledImages);
      //System.out.println("Spotter " + spotter + " -- correct answer: " + correctAnswer + ", guess: " + guess);
      if (guess == correctAnswer)
        scores.add(spotter);
    }
  }

  private void prepare() throws IOException {
    String[] imgNames = {"draussen", "draussen2", "draussen3", "station"};
    images = new ArrayList<RGBImage>();
    for (String imgName : imgNames) {
      String fileName = Prophecy.dataDir() + "experiment3/" + imgName + ".jpg";
      images.add(new RGBImage(loadImage(fileName), new File(fileName)));
      System.out.println("comment: " + getJPEGComment(fileName));
    }
    System.out.println(images.size() + " images loaded");

    scores = new MultiSet<CuckooSpotter<RGBImage>>();
  }

  private String getJPEGComment(String fileName) {
    try {
      IIOMetadata metadata = getImageMetadata(new File(fileName));
      if (metadata != null) {
        Node root = metadata.getAsTree(metadata.getNativeMetadataFormatName());
        NodeList result = (NodeList) XPathFactory.newInstance().newXPath().compile("//com")
          .evaluate(root, XPathConstants.NODESET);
        //displayMetadata(root);
        if (result.getLength() != 0)
          return result.item(0).getAttributes().getNamedItem("comment").getNodeValue();
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return "";
  }

  public static void main(String[] args) throws IOException {
    E3 contest = new E3();
    contest.addContestant(new RandomGuess<RGBImage>());
    contest.addContestant(new ByBrightness());
    contest.run();
  }

  private BufferedImage loadImage(String fileName) throws IOException {
    File file = new File(fileName);
    BufferedImage image = ImageIO.read(file);
    ImageInputStream inputStream = ImageIO.createImageInputStream(file);
    System.out.println(fileName + ": " + image.getWidth() + "*" + image.getHeight());
    return image;
  }

  private IIOMetadata getImageMetadata(File file) throws IOException {
    ImageReader imageReader = ImageIO.getImageReadersByFormatName("jpeg").next();
    ImageInputStream iis = ImageIO.createImageInputStream(file);
    imageReader.setInput(iis);
    BufferedImage image = imageReader.read(0);
    return imageReader.getImageMetadata(0);
  }

  public void displayMetadata(Node root) {
    displayMetadata(root, 0);
  }

  void indent(int level) {
    for (int i = 0; i < level; i++) {
      System.out.print("~ ");
    }
  }

  void displayMetadata(Node node, int level) {
    indent(level); // emit open tag
    System.out.print("<" + node.getNodeName());
    NamedNodeMap map = node.getAttributes();
    if (map != null) { // print attribute values
      int length = map.getLength();
      for (int i = 0; i < length; i++) {
        Node attr = map.item(i);
        System.out.print(" " + attr.getNodeName() +
          "=\"" + attr.getNodeValue() + "\"");
      }
    }

    Node child = node.getFirstChild();
    if (child != null) {
      System.out.println(">"); // close current tag
      while (child != null) { // emit child tags recursively
        displayMetadata(child, level + 1);
        child = child.getNextSibling();
      }
      indent(level); // emit close tag
      System.out.println("</" + node.getNodeName() + ">");
    } else {
      System.out.println("/>");
    }
  }
}