/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e04;

import drjava.util.GUIUtil;
import prophecy.common.Prophecy;
import prophecy.common.Surface;
import prophecy.common.image.ImageSet;
import prophecy.common.image.RGBImage;

import javax.swing.*;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;

public class ByEdges implements StationSpotter {
  static class Edge {
    int x, y;
    float deviation, skew;

    public Edge(int x, int y, float deviation, float skew) {
      this.x = x;
      this.y = y;
      this.deviation = deviation;
      this.skew = skew;
    }
  }

  final int w = 3, h = 16;
  BufferedImage image;
  ArrayList<Edge> edges;

  public boolean stationSpotted(RGBImage image) {
    this.image = image.getBufferedImage();
    edges = new ArrayList<Edge>();
    int xStep = w;
    for (int y = 0; y < image.getHeight()-h; y += h)
      for (int x = w+h; x < image.getWidth()-w-h; x += xStep) {
        EdgeScanner scanner = new EdgeScanner();
        scanner.scan(image, x, y, h, w);
        edges.add(new Edge(x, y, scanner.getDeviation(), scanner.getSkew()));
      }
    return false;
  }

  public Surface getSurface() {
    return new Surface() {
      public void render(int width, int height, Graphics2D g2) {
        if (image != null)
          g2.drawImage(image, 0, 0, null);
        if (edges != null)
          for (Edge edge : edges) {
            int h2 = h/2;
            g2.setColor(Color.yellow);
            g2.drawLine(edge.x, edge.y, (int) (edge.x+edge.skew*h2), edge.y+h2-1);
          }
      }
    };
  }

  public static void main(String[] args) throws IOException {
    ImageSet images = new ImageSet(Prophecy.dataDir() + "aibopics", "bongo", "bongo_station", "bongo_station2");
    ByEdges byEdges = new ByEdges();
    byEdges.stationSpotted(images.getImage(1));

    JFrame frame = new JFrame("SpotTheStation");
    frame.setSize(500, 500);
    frame.getContentPane().add(byEdges.getSurface());
    GUIUtil.showMainFrame(frame);
  }
}