/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e04;

import prophecy.common.image.RGB;
import prophecy.common.image.RGBImage;

public class EdgeScanner {
  private RGBImage image;
  private float skew, deviation;

  public void scan(RGBImage image, int startx, int starty, int lines, int width) {
    this.image = image;
    int x = startx;
    double totalDiffs = 0;
    for (int y = starty; y < starty+lines; y++) {
      int bestD = 0;
      double bestValue = 1e8;
      for (int d = -1; d <= 1; d++) {
        if (x-width+d >= 0 && x+width+d < image.getWidth()) {
          double value = lineDiff(x-width, y, x-width+d, y+1, width*2+1);
          if (value < bestValue) {
            bestValue = value;
            bestD = d;
          }
        }
      }

      x += bestD;
      totalDiffs += bestValue;
    }

    skew = (x-startx)/(float) lines;
    deviation = (float) totalDiffs/lines;
  }

  private double lineDiff(int x1, int y1, int x2, int y2, int width) {
    double sum = 0;
    for (int x = 0; x < width; x++)
      sum += pixelDiff(image.getRGB(x1+x, y1), image.getRGB(x2+x, y2));
    return sum;
  }

  private static double pixelDiff(RGB a, RGB b) {
    return square(a.r-b.r) + square(a.g-b.g) + square(a.b-b.b);
  }

  private static double square(double r) {
    return r*r;
  }

  public float getSkew() {
    return skew;
  }

  public float getDeviation() {
    return deviation;
  }
}