/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e05;

import drjava.util.Display;

import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

/** distinguishes two groups of float values after seeing examples */
public class FloatSplitter {
  static class Count {
    int yin, yang;

    public boolean isYin() {
      return yin > yang;
    }

    public String toString() {
      return "<" + yin + "|" + yang + ">";
    }
  }

  TreeMap<Float, Count> map = new TreeMap<Float, Count>();

  public void train(float value, boolean isYin) {
    Count count = map.get(value);
    if (count == null) {
      count = new Count();
      map.put(value, count);
    }
    if (isYin) ++count.yin; else ++count.yang;
  }

  public boolean isYin(float value) {
    Count count = map.get(value);
    if (count == null) {
      SortedMap<Float,Count> tailMap = map.tailMap(value);
      SortedMap<Float,Count> headMap = map.headMap(value);
      if (tailMap.isEmpty())
        if (headMap.isEmpty())
          return false; // random
        else
          return map.get(headMap.lastKey()).isYin();
      else
        if (headMap.isEmpty())
          return map.get(tailMap.firstKey()).isYin();
        else {
          float d1 = Math.abs(tailMap.firstKey()-value);
          float d2 = Math.abs(headMap.lastKey()-value);
          return map.get(d1 < d2 ? tailMap.firstKey() : headMap.lastKey()).isYin();
        }
    }
    return count.isYin();
  }

  public void display(Display display) {
    for (Map.Entry<Float, Count> entry : map.entrySet()) {
      display.put(entry.getKey()).put(": ").put(entry.getValue()).nl();
    }
  }
}