/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e05;

import drjava.util.Randomizer;
import drjava.util.RepeatableRandomizer;
import prophecy.common.Prophecy;
import prophecy.common.Contest;
import prophecy.common.Surface;
import prophecy.common.Trainer;
import prophecy.common.image.BWImage;
import static prophecy.common.image.ImageProcessing.*;
import prophecy.common.image.ImageSet;
import prophecy.common.image.RGBImage;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.io.IOException;
import java.util.ArrayList;

public class SpotStationParts extends Contest<StationPartSpotter> {
  final ImageSet images;
  Rectangle station_innerMask = new Rectangle(160, 93, 211 - 160, 221 - 93);
  Rectangle station_outerMask = new Rectangle(154, 85, 219 - 154, 227 - 85);
  int clipSize = 23;
  DualImageSet trainingClips, contestClips;
  Randomizer randomizer = new RepeatableRandomizer();
  ArrayList<Rectangle> vRectangles = new ArrayList<Rectangle>();

  public SpotStationParts() throws IOException {
    super(StationPartSpotter.class);
    setTrainer(new Trainer<StationPartSpotter>() {
      public void train(StationPartSpotter contestant) {
        for (int i = 0; i < 2; i++) {
          for (ImageSet.Image image : trainingClips.get(i).getImages()) {
            boolean hasStation = i == 0;
            contestant.train(image.image, hasStation);
          }
        }
        contestant.trainingDone();
      }
    });

    images = new ImageSet(Prophecy.dataDir() + "aibopics", "bongo", "bongo_station", "bongo_station2");
  }

  protected void customPreparation() {
    trainingClips = makeClips(images.getImage(1), station_innerMask, station_outerMask);
    contestClips = makeClips(images.getImage(1), station_innerMask, station_outerMask);
  }

  private DualImageSet makeClips(RGBImage img, Rectangle innerMask, Rectangle outerMask) {
    DualImageSet clips = new DualImageSet();
    vRectangles.clear();
    for (int i = 0; i < 10; i++) {
      Rectangle r = randomClip(randomizer, innerMask, clipSize, clipSize);
      if (r != null) {
        //System.out.println("yes: " + r);
        clips.a.add(img.clip(r));
        vRectangles.add(r);
      }
    }
    while (clips.b.size() < 40) {
      Rectangle r = randomClip(randomizer, new Rectangle(0, 0, img.getWidth(), img.getHeight()), clipSize, clipSize);
      if (!outerMask.intersects(r)) {
        //System.out.println("no: " + r);
        clips.b.add(img.clip(r));
        vRectangles.add(r);
      }
    }

    return clips;
  }

  public Surface getSurface() {
    return new Surface() {
      public void render(int w, int h, Graphics2D g) {
        g.setTransform(AffineTransform.getScaleInstance(2, 2));
        RGBImage image = images.getImage(1);
        /*RGBImage vh = vdiff(hdiff(image));
        RGBImage hv = hdiff(vdiff(image));
        image = ImageProcessing.mul(vh, hv);
        image = brighten(image, 1000);*/
        BWImage vd = vdiff(image), hd = hdiff(image);
        //image = ImageProcessing.add(ImageProcessing.rgbAdjust(hd, 1f, 1f, 0f), vd);
        image = vd.toRGB();
        image = brighten(image, 10);
        g.drawImage(image.getBufferedImage(), 0, 0, null);
        g.setColor(Color.blue);
        for (Rectangle r : vRectangles) {
          g.drawRect(r.x, r.y, r.width, r.height);
        }
      }
    };
  }

  private Rectangle randomClip(Randomizer randomizer, Rectangle r, int w, int h) {
    if (w > r.width || h > r.height)
      return null;
    int x = randomizer.random(r.width - w);
    int y = randomizer.random(r.height - h);
    return new Rectangle(r.x + x, r.y + y, w, h);
  }

  public double evaluate(StationPartSpotter contestant) {
    double score = 0;
    for (int i = 0; i < 2; i++) {
      for (ImageSet.Image image : contestClips.get(i).getImages()) {
        boolean hasStation = i == 0;
        boolean correct = hasStation == contestant.isStationPart(image.image).asBoolean();
        if (correct)
          score += 1.0;
      }
    }
    return score;
  }

  public DualImageSet getContestClips() {
    if (contestClips == null) customPreparation();
    return contestClips;
  }

  public void setRounds(int rounds) {
    this.rounds = rounds;
  }

  public int getClipSize() {
    return clipSize;
  }
}