/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e06;

import drjava.util.Factory;
import drjava.util.MultiSet;
import drjava.util.StdOutDisplay;
import prophecy.e.e01.*;

import java.util.ArrayList;
import java.util.Collection;

public abstract class Categorizer<Raw, Cat> implements Analyzer<Raw, Stats<Cat>> {
  MultiSet<Cat> count = new MultiSet<Cat>();

  public void addData(Raw data) {
    for (Cat cat : getCategories(data)) {
      count.add(cat);
    }
  }

  protected abstract Collection<Cat> getCategories(Raw data);

  public Stats<Cat> getStats() {
    return new Stats<Cat>(count);
  }

  public static void main(String[] args) {
    String[] data = {
      "hello world",
      "what a lovely day",
      "isn't it?",
      "!$&/@*",
      "hehehe"
    };

    WatcherPool<String, Stats<CharCategory>> pool = new WatcherPool<String, Stats<CharCategory>>(
      new Factory<Analyzer<String, Stats<CharCategory>>>() {
        public Analyzer<String, Stats<CharCategory>> make() {
          return new Categorizer<String, CharCategory>() {
            protected Collection<CharCategory> getCategories(String data) {
              ArrayList<CharCategory> result = new ArrayList<CharCategory>();
              for (int i = 0; i < data.length(); i++)
                result.add(CharCategorizer.getCategory(data.charAt(i)));
              return result;
            }
          };
        }
      }, StatsUtil.<CharCategory> getMetric());

    for (String s : data) {
      double distance = pool.processData(s);
      System.out.println("Distance: " + distance + ", pool size: " + pool.getSize() + " # " + s);
    }

    pool.display(new StdOutDisplay());
  }

  public Stats<Cat> getResult() {
    return getStats();
  }
}