/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e06;

import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;

import java.util.Arrays;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

public class Dual<A> implements Iterable<A>, InspectableParent {
  public final A a, b;

  public Dual(A a, A b) {
    this.a = a;
    this.b = b;
  }

  public Dual() {
    a = b = null;
  }

  public A get(int index) {
    return index == 0 ? a : index == 1 ? b : null;
  }

  public A get(boolean snd) {
    return snd ? b : a;
  }

  public Iterator<A> iterator() {
    return Arrays.asList(a, b).iterator();
  }

  public Dual<A> plus(A x) {
    if (a == null) return new Dual<A>(x, null);
    else if (b == null) return new Dual<A>(a, x);
    else throw new NoSuchElementException("Dual already has two elements");
  }

  public List<InspectionNode> getInspectionChildren() {
    return Arrays.asList(
      new InspectionNode("a", a),
      new InspectionNode("b", b));
  }
}