/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e06;

import prophecy.common.image.BWImage;

public class PreciseBWImage {
  private int width, height;
  private double[][] pixels;

  public PreciseBWImage(double[][] pixels) {
    this.pixels = pixels;
    height = pixels.length;
    width = pixels[0].length;
  }

  public PreciseBWImage(BWImage image) {
    width = image.getWidth();
    height = image.getHeight();
    pixels = new double[height][width];
    for (int y = 0; y < height; y++)
      for (int x = 0; x < width; x++) {
        float p = image.getPixel(x, y);
        pixels[y][x] = p;
      }
  }

  public PreciseBWImage(int width, int height, double[][] pixels) {
    this.width = width;
    this.height = height;
    this.pixels = pixels;
  }

  public double averageBrightness() {
    double sum = 0;
    for (int y = 0; y < height; y++)
      for (int x = 0; x < width; x++)
        sum += pixels[y][x];
    return (float) (sum/(double) (height*width));
  }

  public double getPixel(int x, int y) {
    return inRange(x, y) ? pixels[y][x] : 0;
  }

  private boolean inRange(int x, int y) {
    return x >= 0 && x < width && y >= 0 && y < height;
  }

  public int getWidth() {
    return width;
  }

  public int getHeight() {
    return height;
  }

  public BWImage toBWImage(double factor) {
    float[] bwPixels = new float[height*width];
    for (int y = 0; y < height; y++)
      for (int x = 0; x < width; x++) {
        double p = pixels[y][x];
        bwPixels[y*width+x] = (float) (p * factor);
      }
    return new BWImage(width, height, bwPixels);
  }
}