/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e06;

import drjava.util.Factory;
import drjava.util.StdOutDisplay;
import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import static prophecy.common.image.ImageProcessing.brighten;
import prophecy.common.image.RGBImage;
import prophecy.e.e01.Analyzer;
import prophecy.e.e01.WatcherPool;
import prophecy.e.e05.FuzzyBool;
import prophecy.e.e05.StationPartSpotter;

import java.util.Arrays;
import java.util.List;

public class VDiff2 extends StationPartSpotter implements InspectableParent {
  Dual<WatcherPool<BWImage, BWImage>> watcherPools;
  int miniClipSize = 4, step = 2;

  public VDiff2() {
    watcherPools = new Dual<WatcherPool<BWImage, BWImage>>(makeWatcherPool(), makeWatcherPool());
  }

  private WatcherPool<BWImage, BWImage> makeWatcherPool() {
    WatcherPool<BWImage, BWImage> pool = new WatcherPool<BWImage, BWImage>(new Factory<Analyzer<BWImage, BWImage>>() {
      public Analyzer<BWImage, BWImage> make() {
        return new ClipAnalyzer();
      }
    }, new BWImageMetric());
    pool.setSpawnThreshold(0.25);
    //pool.setSpawnThreshold(0.1);
    return pool;
  }

  public void train(RGBImage clip, boolean isStationPart) {
    BWImage bw = preprocessImage(clip);
    List<BWImage> miniClips = getMiniClips(bw);
    for (BWImage miniClip : miniClips) {
      watcherPools.get(isStationPart).processData(miniClip);
    }
    //System.out.println("isStation: " + isStationPart + ", brightness: " + brightness);
  }

  private List<BWImage> getMiniClips(BWImage image) {
    return ImageProcessing.getMiniclips(image, miniClipSize, miniClipSize, step, step);
  }

  public void trainingDone() {
    for (WatcherPool<BWImage, BWImage> pool : watcherPools) {
      pool.display(new StdOutDisplay());
    }
  }

  private BWImage preprocessImage(RGBImage clip) {
    return brighten(ImageProcessing.vdiff(clip), 10);
  }

  public FuzzyBool isStationPart(RGBImage clip) {
    BWImage bw = preprocessImage(clip);
    List<BWImage> miniClips = getMiniClips(bw);
    double sumYin = 0, sumYang = 0;
    // wrong approach!
    // - get stats or
    // - compare miniclip patterns
    for (BWImage miniClip : miniClips) {
      double yin = watcherPools.a.getMinDistance(miniClip);
      double yang = watcherPools.b.getMinDistance(miniClip);
      sumYin += yin;
      sumYang += yang;
    }
    return FuzzyBool.distances(sumYin, sumYang);
  }

  public List<InspectionNode> getInspectionChildren() {
    return Arrays.asList(
      new InspectionNode("yin pool", watcherPools.a),
      new InspectionNode("yang pool", watcherPools.b));
  }
}