/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e07;

import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.e.e01.Metric;

import java.util.ArrayList;
import java.util.List;

public class Prototypes<A> implements InspectableParent {
  private List<A> prototypes;
  private Metric<A> metric;

  public Prototypes(List<A> prototypes, Metric<A> metric) {
    this.prototypes = prototypes;
    this.metric = metric;
  }

  public int getClosestMatch(A a) {
    double minDistance = 1e6f;
    int closestIndex = -1;

    for (int i = 0; i < prototypes.size(); i++) {
      A prototype = prototypes.get(i);

      double distance = metric.getDistance(prototype, a);
      if (closestIndex < 0 || distance < minDistance) {
        minDistance = distance;
        closestIndex = i;
      }
    }

    return closestIndex;
  }

  public List<InspectionNode> getInspectionChildren() {
    List<InspectionNode> result = new ArrayList<InspectionNode>();
    for (int i = 0; i < prototypes.size(); i++) {
      A a = prototypes.get(i);
      result.add(new InspectionNode(String.valueOf(i+1), a));
    }
    return result;
  }
}