/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e07;

import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.common.image.BWImage;
import prophecy.common.image.RGBImage;
import prophecy.e.e01.Stats;
import prophecy.e.e01.StatsUtil;
import prophecy.e.e05.FuzzyBool;
import prophecy.e.e05.StationPartSpotter;
import prophecy.e.e06.Dual;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class VDiff3 extends StationPartSpotter implements InspectableParent {
  ArrayList<PreprocessedItem> preprocessedData = new ArrayList<PreprocessedItem>();
  private Dual<Stage2> stage2;
  private Dual<Prototypes<Stats<Integer>>> distributions;
  protected RGBtoBWPreprocessing preprocessing = new Preprocessing();

  public VDiff3() {
  }

  public void train(RGBImage clip, boolean isStationPart) {
    preprocessedData.add(new PreprocessedItem(preprocessing.preprocessImage(clip), isStationPart));
  }

  public void trainingDone() {
    Stage1 stage1 = new Stage1();
    Dual<Prototypes<BWImage>> prototypes = new Dual<Prototypes<BWImage>>(
      stage1.getPrototypes(preprocessedData, true),
      stage1.getPrototypes(preprocessedData, false));
    distributions = new Dual<Prototypes<Stats<Integer>>>();
    stage2 = new Dual<Stage2>(new Stage2(prototypes.a), new Stage2(prototypes.b));
    for (int i = 0; i < 2; i++) {
      distributions = distributions.plus(
        new Prototypes<Stats<Integer>>(stage2.get(i).getDistributions(getGroup(preprocessedData, i == 0)),
          StatsUtil.<Integer> getMetric()));
    }
  }

  private List<PreprocessedItem> getGroup(List<PreprocessedItem> items, boolean stationPart) {
    List<PreprocessedItem> result = new ArrayList<PreprocessedItem>();
    for (PreprocessedItem item : items) {
      if (item.stationPart == stationPart)
        result.add(item);
    }
    return result;
  }

  public FuzzyBool isStationPart(RGBImage clip) {
    PreprocessedItem trainingItem = new PreprocessedItem(preprocessing.preprocessImage(clip), true /* dummy value */);

    Stats<Integer> distA = stage2.a.getDistribution(trainingItem);
    Stats<Integer> distB = stage2.b.getDistribution(trainingItem);
    double distanceA = distributions.a.getClosestMatch(distA);
    double distanceB = distributions.b.getClosestMatch(distB);

    return FuzzyBool.distances(distanceA, distanceB);
  }

  public List<InspectionNode> getInspectionChildren() {
    return Arrays.asList(new InspectionNode("distributions", distributions));
  }
}