/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e08;

import prophecy.common.Surface;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.PixelUtil;
import prophecy.common.image.RGBImage;
import prophecy.e.e04.StationSpotter;
import prophecy.e.e05.FuzzyBool;
import prophecy.e.e05.StationPartSpotter;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

public class ByClips implements StationSpotter {
  private StationPartSpotter partSpotter;
  private int clipSize;
  private List<Clip> clips;
  private BufferedImage image;

  static class Clip {
    Rectangle rect;
    FuzzyBool value;

    public Clip(Rectangle rect, FuzzyBool value) {
      this.rect = rect;
      this.value = value;
    }
  }

  public ByClips(StationPartSpotter partSpotter, int clipSize) {
    this.partSpotter = partSpotter;
    this.clipSize = clipSize;
  }

  public boolean stationSpotted(RGBImage image) {
    this.image = image.getBufferedImage();
    clips = new ArrayList<Clip>();
    List<Rectangle> rects = ImageProcessing.getMiniclipRects(image.getWidth(), image.getHeight(),
      clipSize, clipSize, clipSize, clipSize);
    for (Rectangle r : rects) {
      FuzzyBool value = partSpotter.isStationPart(image.clip(r));
      clips.add(new Clip(r, value));
    }
    return false;
  }

  public Surface getSurface() {
    return new Surface() {
      public void render(int width, int height, Graphics2D g2) {
        if (image != null)
          g2.drawImage(image, 0, 0, null);
        if (clips != null)
          for (Clip clip : clips) {
            g2.setColor(PixelUtil.blend(Color.blue, Color.yellow, clip.value.getValue()).getColor());
            g2.drawRect(clip.rect.x, clip.rect.y, clip.rect.width-1, clip.rect.height-1);
          }
      }
    };
  }
}