/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e09;

import prophecy.common.Prophecy;
import prophecy.common.image.ImageSet;

import java.awt.*;
import static java.lang.Integer.parseInt;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class BallTrainingImages {
  private List<TrainingImage> trainingImages, contestImages;
  private static BallTrainingImages instance;

  public static BallTrainingImages get() {
    if (instance == null)
      instance = new BallTrainingImages();
    return instance;
  }

  private BallTrainingImages() {
    contestImages = makeTrainingImages(new ImageSet(Prophecy.dataDir() + "ball",
      "ball1", "ball2", "ball3", "ball4", "ball5", "ball6", "ball7",
      "noball3", "noball5", "noball8", "ball9", "noball10"));
    trainingImages = makeTrainingImages(new ImageSet(Prophecy.dataDir() + "ball",
      "ball3", "noball3", "ball5", "noball5", "noball8", "ball9", "noball10"));
  }

  private List<TrainingImage> makeTrainingImages(ImageSet imageSet) {
    List<TrainingImage> result = new ArrayList<TrainingImage>();
    for (ImageSet.Image image : imageSet.getImages()) {
      Rectangle r = null;
      Matcher matcher = Pattern.compile("ball=(\\d+),(\\d+),(\\d+),(\\d+)").matcher(image.comment);
      if (matcher.find()) {
        int x1 = parseInt(matcher.group(1)), y1 = parseInt(matcher.group(2));
        int x2 = parseInt(matcher.group(3)), y2 = parseInt(matcher.group(4));
        r = new Rectangle(x1, y1, x2-x1, y2-y1);
      }
      result.add(new TrainingImage(image.image, !image.name.contains("no"), r));
    }
    return result;
  }

  public List<TrainingImage> getTrainingImages() {
    return trainingImages;
  }

  public List<TrainingImage> getContestImages() {
    return contestImages;
  }
}