/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e09;

import drjava.util.StdOutDisplay;
import prophecy.common.image.RGBImage;
import prophecy.e.e05.FuzzyBool;

import java.util.Arrays;
import java.util.List;

public class ByPalette extends ObjectSpotter {
  // assumption: 4 images, pairwise (ball/noball)
  public void train(List<TrainingImage> images) {
    ColorCube cube1 = trainPair(images.subList(0, 2));
    ColorCube cube2 = trainPair(images.subList(2, 4));
    ColorCube cube3 = cube1.merge(cube2, new Function2<Double, Double, Double>() {
      public Double get(Double a, Double b) {
        if (a < 0)
          return b < 0 ? Math.max(a, b) : 0;
        else
          return b > 0 ? Math.min(a, b) : 0;
      }
    });
    cube3.display(new StdOutDisplay());
  }

  private ColorCube trainPair(List<TrainingImage> images) {
    List<ColorCube> colorCubes = Arrays.asList(new ColorCube(), new ColorCube());
    for (TrainingImage image : images) {
      colorCubes.get(image.containsObject ? 1 : 0).addPixels(image.image);
    }
    ColorCube diff = colorCubes.get(1).minus(colorCubes.get(0));
    diff = diff.denoise();
    diff.display(new StdOutDisplay());
    return diff;
  }

  public FuzzyBool objectSpotted(RGBImage image) {
    return new FuzzyBool(false); // TODO
  }
}