/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e10;

import prophecy.common.*;
import prophecy.common.image.RGBImage;
import prophecy.e.e09.BallTrainingImages;
import prophecy.e.e09.TrainingImage;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class E10 {
  private List<TrainingImage> trainingImages;
  private List<TrainingImage> contestImages;
  private List<E10Contestant> contestants = new ArrayList<E10Contestant>();
  private E10Contestant winner;
  private List<ImageVerbalizer> standardVerbalizers = Arrays.<ImageVerbalizer> asList(new StandardVerbalizer());

  public static void main(String[] args) throws IOException {
    E10 e10 = new E10();
    e10.runContest();
    e10.inspect();
  }

  public E10() {
    makeImageSets();
    makeContestants();
    train();
    //runContest(trainingImages);
  }

  public void runContest() {
    runContest(contestImages);
  }

  protected void makeContestants() {
    /*makeContestants1();*/
    addContestant(makeChampion());
    List<ImageVerbalizer> verbalizers = new ArrayList<ImageVerbalizer>(standardVerbalizers);
    verbalizers.add(new HVVerbalizer());
    contestants.add(new BayesContestant(new BayesParams(0.1), 0, verbalizers));
  }

  public void inspect() {
    inspect(contestImages, winner, "E10");
  }

  public static void inspect(List<TrainingImage> contestImages, Subset<RGBImage> contestant, String title) {
    SpotBallParts contest = new SpotBallParts(contestImages);
    List<Surface> surfaces = new ArrayList<Surface>();
    for (int i = 0; i < contestImages.size(); i++) {
      surfaces.add(contest.getSurface(i, contestant));
    }
    InspectionUtil.inspect(title, surfaces);
  }

  private void runContest(List<TrainingImage> images) {
    SpotBallParts contest = new SpotBallParts(images);
    contest.addContestants(contestants);
    contest.run();
    winner = (E10Contestant) contest.getWinner();
  }

  private void train() {
    List<SubsetTI<RGBImage>> trainingData = getTrainingData();
    for (E10Contestant contestant : contestants) {
      contestant.train(trainingData);
    }
  }

  public List<SubsetTI<RGBImage>> getTrainingData() {
    List<SubsetTI<RGBImage>> trainingData = new SpotBallParts(trainingImages).getContestData();
    return trainingData;
  }

  private void makeContestants1() {
    contestants.add(new Contestant1());
    for (int i = 0; i < 5; i++)
      contestants.add(new BayesContestant(new BayesParams(i*0.1), 0, standardVerbalizers));
  }

  private void makeContestants2() {
    for (int i = 0; i <= 10; i++)
      contestants.add(new BayesContestant(new BayesParams(0.1), i*5, standardVerbalizers));
  }

  public BayesContestant makeChampion() {
    return new BayesContestant(new BayesParams(0.1), 0, standardVerbalizers);
  }

  private void makeImageSets() {
    trainingImages = BallTrainingImages.get().getTrainingImages();
    contestImages = BallTrainingImages.get().getContestImages();
  }

  public E10Contestant getWinner() {
    return winner;
  }

  public static E10Contestant getBestClipAnalyzer() {
    return new E10() {
      protected void makeContestants() {
        addContestant(makeChampion());
      }
    }.getContestants().get(0);
  }

  public List<E10Contestant> getContestants() {
    return contestants;
  }

  public void addContestant(E10Contestant contestant) {
    contestants.add(contestant);
  }

  public List<TrainingImage> getContestImages() {
    return contestImages;
  }
}