/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e10;

import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.common.Subset;
import prophecy.common.SubsetContest;
import prophecy.common.Surface;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGBImage;
import prophecy.e.e09.TrainingImage;

import java.awt.*;
import java.awt.geom.Ellipse2D;
import java.util.ArrayList;
import java.util.List;

public class SpotBallParts extends SubsetContest<RGBImage> implements ToTree {
  private final int clipSize = 8;
  private final int inset = 2;
  private List<RGBImage> positiveExamples = new ArrayList<RGBImage>();
  private List<RGBImage> negativeExamples = new ArrayList<RGBImage>();
  private List<List<Rectangle>> positiveClips, negativeClips;
  private List<TrainingImage> images;

  public SpotBallParts(List<TrainingImage> images) {
    this.images = images;
    positiveClips = new ArrayList<List<Rectangle>>();
    negativeClips = new ArrayList<List<Rectangle>>();
    for (TrainingImage t : images) {
      List<Rectangle> pC = new ArrayList<Rectangle>();
      List<Rectangle> nC = new ArrayList<Rectangle>();
      positiveClips.add(pC);
      negativeClips.add(nC);
      Rectangle el = t.rectangle;
      for (Rectangle clip : ImageProcessing.getMiniclipRects(
        t.image.getWidth(), t.image.getHeight(), clipSize, clipSize, clipSize, clipSize)) {
        boolean pos = false, neg = false;
        if (el == null)
          neg = true;
        else {
          Ellipse2D.Double ellipse = new Ellipse2D.Double(el.x + inset, el.y + inset, el.width - inset * 2, el.height - inset * 2);
          if (ellipse.contains(clip))
            pos = true;

          ellipse = new Ellipse2D.Double(el.x - inset, el.y - inset, el.width + inset * 2, el.height + inset * 2);
          if (!ellipse.intersects(clip))
            neg = true;
        }

        if (pos) {
          pC.add(clip);
          positiveExamples.add(t.image.clip(clip));
        }
        if (neg) {
          nC.add(clip);
          negativeExamples.add(t.image.clip(clip));
        }

      }
    }
  }

  protected List<RGBImage> getPositiveExamples() {
    return positiveExamples;
  }

  protected List<RGBImage> getNegativeExamples() {
    return negativeExamples;
  }

  public Surface getSurface(final int imageIndex, final Subset<RGBImage> contestant) {
    return new Surface() {
      public void render(int width, int height, Graphics2D g2) {
        g2.drawImage(images.get(imageIndex).image.getBufferedImage(), 0, 0, null);
        if (positiveClips != null) {
          showClips(g2, positiveClips.get(imageIndex), Color.yellow, contestant);
          showClips(g2, negativeClips.get(imageIndex), Color.blue, contestant);
        }
      }

      private void showClips(Graphics2D g2, List<Rectangle> clips, Color color, Subset<RGBImage> contestant) {
        for (Rectangle clip : clips) {
          if (contestant != null)
            g2.setColor(contestant.contains(images.get(imageIndex).image.clip(clip)) ? Color.white : Color.black);
          else
            g2.setColor(color);
          g2.drawRect(clip.x, clip.y, clip.width - 1, clip.height - 1);
        }
      }
    };
  }

  public String toString() {
    return "SpotBallParts clips: " + positiveExamples.size() + "/" + negativeExamples.size();
  }

  public Tree toTree() {
    return super.toTree()
      .add("images", images);
  }
}