/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e11;

import drjava.util.Display;
import drjava.util.Displayable;
import drjava.util.StringUtil;
import prophecy.common.Subset;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGBImage;

import java.awt.*;
import java.util.List;

public class BallLocator1 implements BallLocator, Displayable {
  final int clipSize = 8;
  private final double threshold;
  private Subset<RGBImage> clipAnalyzer;

  public BallLocator1(Subset<RGBImage> clipAnalyzer, double threshold) {
    this.threshold = threshold;
    this.clipAnalyzer = clipAnalyzer;
  }

  public Point locateBall(RGBImage image) {
    int pos = 0, sumX = 0, sumY = 0;
    List<Rectangle> clips = ImageProcessing.getMiniclipRects(
      image.getWidth(), image.getHeight(), clipSize, clipSize, clipSize, clipSize);
    for (Rectangle clip : clips) {
      if (clipAnalyzer.contains(image.clip(clip))) {
        ++pos;
        sumX += clip.x+clip.width/2;
        sumY += clip.y+clip.height/2;
      }
    }
    if (pos > clips.size() * threshold)
      return new Point(sumX/pos, sumY/pos);
    else
      return null;
  }

  public String toString() {
    return "BallLocator1 " + StringUtil.formatProbability(threshold, 1);
  }

  public void display(Display display) {
    display.putnl("BallLocator1 threshold=" + threshold);
    display.put("clip analyzer: ").putnl(clipAnalyzer);
  }
}