/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e11;

import prophecy.common.Contest;
import prophecy.common.Surface;
import prophecy.common.image.RGBImage;
import prophecy.e.e09.BallTrainingImages;
import prophecy.e.e09.TrainingImage;

import java.awt.*;
import java.util.List;

public class LocateTheBall extends Contest<BallLocator> {
  private List<TrainingImage> contestImages;
  private final int tolerance = 5;

  public LocateTheBall() {
    super(BallLocator.class);
    contestImages = BallTrainingImages.get().getContestImages();
  }

  public double evaluate(BallLocator contestant) {
    double score = 0;
    for (TrainingImage image : contestImages) {
      double width = image.image.getWidth();
      double height = image.image.getHeight();
      Point p = contestant.locateBall(image.image);
      if (image.rectangle != null) {
        if (p == null)
          score -= 3;
        else {
          double centerX = Math.max(0, Math.min(width-1, image.rectangle.x + image.rectangle.width / 2));
          double centerY = Math.max(0, Math.min(height-1, image.rectangle.y + image.rectangle.height / 2));
          score -= Math.max(0, Math.abs(p.x-centerX)-tolerance)/width;
          score -= Math.max(0, Math.abs(p.y-centerY)-tolerance)/height;
        }
      } else
        if (p != null) score -= 3;
    }
    return score;
  }

  public Double getBestPossibleScore() {
    return 0.0;
  }

  public Double getWorstPossibleScore() {
    return -3.0*contestImages.size();
  }

  public Surface getSurface(final int imageIndex, final BallLocator contestant) {
    final RGBImage image = contestImages.get(imageIndex).image;
    return new BallLocatorSurface(image, contestant);
  }

  public List<TrainingImage> getContestImages() {
    return contestImages;
  }

}