/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e12;

import drjava.util.FromTree;
import drjava.util.ToTree;
import drjava.util.Tree;
import prophecy.common.BayesParams;
import prophecy.common.SubsetTI;
import prophecy.common.image.RGBImage;
import prophecy.e.e10.BayesContestant;
import prophecy.e.e10.E10Contestant;
import prophecy.e.e10.ImageVerbalizer;

import java.util.ArrayList;
import java.util.List;

public class FeatureContestant implements E10Contestant, FromTree, ToTree {
  private ImageFeatureSet featureSet;
  private E10Contestant contestant;
  private List<ImageVerbalizer> verbalizers = new ArrayList<ImageVerbalizer>();

  public FeatureContestant() {
    featureSet = new ImageFeatureSet();
  }

  public FeatureContestant(ImageFeatureSet featureSet) {
    this.featureSet = featureSet;
  }

  public boolean contains(RGBImage image) {
    return contestant.contains(image);
  }

  public void train(List<SubsetTI<RGBImage>> subsetTrainingItems) {
    ImageFeatureSet adjustedFeatureSet = FeatureSetAdjuster.adjust(featureSet, subsetTrainingItems);
    ImageVerbalizer verbalizer = new FeatureSetVerbalizer(adjustedFeatureSet);
    List<ImageVerbalizer> verbalizers = new ArrayList<ImageVerbalizer>(this.verbalizers);
    verbalizers.add(verbalizer);
    contestant = new BayesContestant(new BayesParams(0.1), 0, verbalizers);
    contestant.train(subsetTrainingItems);
  }

  public String toString() {
    return "FeatureContestant " + featureSet;
  }

  public void addVerbalizer(ImageVerbalizer verbalizer) {
    verbalizers.add(verbalizer);
  }

  public Tree toTree() {
    // DUMMY
    return new Tree(getClass()).add("featureSet", featureSet.toTree());
  }

  public void fromTree(Tree tree) {
    featureSet.fromTree(tree.get("featureSet"));
  }
}