/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e12;

import drjava.util.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;

public class ImageFeatureSet implements FromTree, ToTree {
  private List<ImageFeature> features = new ArrayList<ImageFeature>();

  public ImageFeatureSet(ImageFeature... features) {
    this.features.addAll(Arrays.asList(features));
  }

  public ImageFeatureSet(Collection<ImageFeature> features) {
    this.features.addAll(features);
  }

  public void add(ImageFeature feature) {
    features.add(feature);
  }

  public List<ImageFeature> getFeatures() {
    return features;
  }

  public String toString() {
    List<String> names = new ArrayList<String>();
    for (ImageFeature feature : features) names.add(feature.getName());
    return StringUtil.join(" ", names);
  }

  public Tree toTree() {
    Tree tree = new Tree(this.getClass());
    for (ImageFeature feature : features) {
      tree.add(feature.baum());
    }
    return tree;
  }

  public void fromTree(Tree tree) {
    features.addAll(TreeUtil.treeToList(tree));
  }
}