/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e13;

import drjava.util.*;
import prophecy.common.Scores;
import prophecy.common.Generator;
import prophecy.e.e10.E10Contestant;
import prophecy.e.e10.SpotBallParts;
import prophecy.e.e12.*;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/** produce SpotBallParts contestants by randomly/systematically combining parts */
public class E13 {
  Scores<E10Contestant> hallOfFame = new Scores<E10Contestant>();
  Generator<E10Contestant> generator;
  private SBPWithTraining sbp = new SBPWithTraining();
  private GUI gui;

  public static void main(String[] args) throws IOException {
    //MultiCoreUtil.setNumCores(1);
    new E13();
  }

  public E13() throws IOException {
    gui = new GUI();

    TreePersistence persistence = new FileTreePersistence("halloffame/e13.tree");
    Tree tree = persistence.load();
    if (tree != null) hallOfFame = new Scores<E10Contestant>(tree);
    gui.showHallOfFame(hallOfFame);

    generator = makeG1();

    while (true) {
      List<E10Contestant> contestants = new ArrayList<E10Contestant>();
      for (int i = 0; i < MultiCoreUtil.getNumCores(); i++) {
        E10Contestant contestant = produceContestant();
        if (contestant != null) {
          contestants.add(contestant);
          gui.newContestant(i, ObjectUtil.toNiceString(contestant));
        } else
          break;
      }

      if (contestants.isEmpty())
        break;

      List<Double> scores = MultiCoreUtil.parallelMap(contestants, new Function<E10Contestant, Double>() {
        public Double get(E10Contestant contestant) {
          return evaluateContestant(contestant);
        }
      });

      for (int i = 0; i < contestants.size(); i++) {
        hallOfFame.add(contestants.get(i), scores.get(i));
      }
      gui.showHallOfFame(hallOfFame);
      persistence.store(hallOfFame.toTree());
    }

    gui.done();
  }

  private G1 makeG1() {
    G1 g1 = new G1();
    g1.addFeatures(new ImageFeatureSet(new Brightness(), new Red(), new Green(), new Blue(), new HDiff(), new VDiff()).getFeatures());
    return g1;
  }

  private void tryContestant(E10Contestant contestant) {
    double score = evaluateContestant(contestant);
    hallOfFame.add(contestant, score);
    gui.showHallOfFame(hallOfFame);
  }

  private double evaluateContestant(E10Contestant contestant) {
    SpotBallParts contest = sbp.trainAndRunContest(contestant);

    // this is not threadsafe, but what the heck
    hallOfFame.setBestPossibleScore(contest.getScores().getBestPossibleScore());
    hallOfFame.setWorstPossibleScore(contest.getScores().getWorstPossibleScore());

    return contest.getScores().getScore(contestant);
  }

  private E10Contestant produceContestant() {
    E10Contestant contestant;
    do {
      contestant = produceContestant2();
      if (contestant == null) return null;
    } while (alreadyProduced(contestant));
    return contestant;
  }

  private boolean alreadyProduced(E10Contestant contestant) {
    Tree tree = TreeUtil.objectToTree(contestant);
    for (E10Contestant c : hallOfFame.getContestants()) {
      if (TreeUtil.equal(TreeUtil.objectToTree(c), tree))
        return true;
    }
    return false;
  }

  private E10Contestant produceContestant2() {
    if (generator == null) return null;
    return generator.next();
  }
}