/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e13;

import drjava.util.GUIUtil;
import drjava.util.StringUtil;
import jazz.retroui.Box;
import jazz.retroui.FancyScreen;
import jazz.retroui.RetroUIFrame;
import prophecy.common.Scores;
import prophecy.e.e10.E10Contestant;

import javax.swing.*;
import java.awt.*;
import java.util.List;

public class GUI {
  private RetroUIFrame frame;
  private Box countBox, lastBox, fameBox;
  private int numContestants;
  private long startTime = System.currentTimeMillis();

  public GUI() {
    frame = new RetroUIFrame("Prophecy E13", 80, 25);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    GUIUtil.showFrame(frame);
    FancyScreen screen = new FancyScreen(frame.getScreen());

    countBox = new Box(screen, new Rectangle(1, 1, screen.cols()-2, 3));
    countBox.drawBorder();
    countBox.drawTitle("Contestants generated");

    lastBox = new Box(screen, new Rectangle(1, 5, screen.cols()-2, 4));
    lastBox.drawBorder();
    lastBox.drawTitle("Current contestants");

    fameBox = new Box(screen, new Rectangle(1, 10, screen.cols()-2, 10));
    fameBox.drawBorder();
    fameBox.drawTitle("Hall of fame");

    update();
  }

  private void update() {
    countBox.innerScreen().clear();
    countBox.innerScreen().print(0, 0, numContestants + " ("
      + StringUtil.formatDouble(numContestants*1000.0/(System.currentTimeMillis()-startTime), 1) + "/s)");
  }

  public void newContestant(int threadIdx, String desc) {
    ++numContestants;
    update();

    FancyScreen screen = lastBox.innerScreen().subscreen(0, threadIdx, lastBox.innerScreen().cols(), 1);
    screen.clear();
    screen.print(0, 0, desc);
  }

  public void showHallOfFame(Scores<E10Contestant> hallOfFame) {
    fameBox.innerScreen().clear();
    List<E10Contestant> list = hallOfFame.getTop(8);
    for (int i = 0; i < list.size(); i++)
      fameBox.innerScreen().print(0, i, hallOfFame.formatScoreLine(list.get(i)));
  }

  public void done() {
    lastBox.innerScreen().clear();
  }
}