/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.e.e14;

import drjava.util.FileTreePersistence;
import prophecy.common.BayesParams;
import prophecy.common.IgnoreUntrainables;
import prophecy.common.Subset;
import prophecy.common.SubsetAutoTrainer;
import prophecy.common.image.RGBImage;
import prophecy.e.e09.BallTrainingImages;
import prophecy.e.e10.*;
import prophecy.e.e12.ColorCubeVerbalizer;
import prophecy.e.e13.SBPWithTraining;

import java.io.IOException;
import java.util.Arrays;

public class E14 {
  public static void main(String[] args) throws IOException {
    E10Contestant baseAnalyzer = E10.getBestClipAnalyzer();
    SpotBallParts preContest = new SBPWithTraining().trainAndRunContest(baseAnalyzer);

    FalsePositives contest = new FalsePositives(baseAnalyzer, preContest.getContestData());
    contest.setTrainer(new IgnoreUntrainables<Subset<RGBImage>>(new SubsetAutoTrainer<RGBImage, E10Contestant>(contest)));

    for (int graininess = 2; graininess <= 10; graininess++)
      contest.addContestant(new BayesContestant(new BayesParams(), 0,
        Arrays.<ImageVerbalizer> asList(new ColorCubeVerbalizer(graininess))));
    contest.run();

    // construct improved analyzer

    CorrectFalsePositives corrected = new CorrectFalsePositives(baseAnalyzer, contest.getWinner());

    // persist to disk

    new FileTreePersistence("temp/e14.tree").store(corrected.toTree());

    SpotBallParts postContest = new SpotBallParts(BallTrainingImages.get().getContestImages());
    postContest.addContestant(baseAnalyzer);
    postContest.addContestant(corrected);
    postContest.run();

    E10.inspect(BallTrainingImages.get().getContestImages(), corrected, "E14");
    //E11.run(corrected);
  }
}