/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe01;

import liburbi.UClient;
import liburbi.call.UCallbackListener;
import prophecy.audio.a03.AudioAnalysis;
import prophecy.common.Flag;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundUtil;
import static prophecy.joe.joe01.UrbiUtil.connect;

import javax.sound.sampled.*;
import java.io.File;
import java.io.IOException;

public class Aibo {
  private UClient client;
  private boolean motorOn;
  private ConnectionCheck connectionCheck;

  public Aibo() {
    this(connect());
  }

  public Aibo(UClient client) {
    this.client = client;
    connectionCheck = new ConnectionCheck(this);
    connectionCheck.start();
    motorOn();
  }

  public UClient getClient() {
    return client;
  }

  public void motorOn() {
    if (!motorOn) {
      motorOn = true;
      send("motors on;");
    }
  }

  public synchronized void send(String cmd) {
    try {
      client.send(cmd);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public void setCallback(UCallbackListener listener, String tag) {
    client.setCallback(listener, tag);
  }

  public void deleteCallback(String tag) {
    client.deleteCallback(tag);
  }

  public int playSound(String filename) {
    File file = new File(filename);

    try
      {
        AudioFileFormat fformat = AudioSystem.getAudioFileFormat(file);
        AudioFormat		format = fformat.getFormat();
            AudioInputStream	ais = AudioSystem.getAudioInputStream(file);
            int numBytes = 512*1024;
            byte[] audioBytes = new byte[numBytes];

            int numBytesRead = 0;
        while ((numBytesRead = ais.read(audioBytes)) != -1)
          {
            send("speaker.val = BIN " + numBytesRead + " raw "
                       + format.getChannels() + " " + (int)format.getSampleRate() +
                       " " + format.getSampleSizeInBits() + " 1;");
            try
              {
                client.effectiveSend(audioBytes, numBytesRead);
              }
            catch (IOException e)
              {
                System.out.println("Send binary error");
              }
          }
      }
    catch (IOException e)
      {
        throw new RuntimeException(e);
      } catch (UnsupportedAudioFileException e) {
      e.printStackTrace();
    }
    return 0;
  }

  public void disconnect() {
    send("quit;");
    getClient().disconnect();
  }

  public Flag getConnectionLostFlag() {
    return connectionCheck.getConnectionLostFlag();
  }

  private Sound normalizeWave(String waveFile) {
    Sound sound = AudioAnalysis.normalize(SoundUtil.loadSound(waveFile));
    SoundUtil.save(sound, waveFile);
    return sound;
  }

  public synchronized void playSoundAndMoveMouth(Sound sound) {
    File file = new File("temp/speech.wav");
    file.getParentFile().mkdirs();
    SoundUtil.save(sound, file.getPath());
    playSoundAndMoveMouth(file);
  }

  public synchronized void playSoundAndMoveMouth(File waveFile) {
    double seconds = normalizeWave(waveFile.getPath()).getSeconds();
    send("moveMouth: { talk: mouth.val'n = 0.7 sin:0.3s ampli: 0.2, wait(" + seconds
      + "s); stop talk; mouth.val'n = 1 smooth:200; }, ");
    playSound(waveFile.getPath());
  }

}