/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe01;

import drjava.util.FileUtil;
import drjava.util.GUIUtil;
import liburbi.UClient;
import liburbi.call.UCallbackListener;
import liburbi.call.URBIEvent;
import prophecy.common.Flag;
import prophecy.joe.joe07.UrbiConfigEscape;
import prophecy.joe.joe08.UrbiConfigDialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

public class UrbiUtil {
  public static final String DEFAULT_AIBO_IP = "192.168.1.4";
  private static final String AIBO_IP_FILE = "aibo_ip.txt";

  /** this works a lot better than UClient.sendFile */
  public static void sendFile(UClient client, String fileName) throws IOException {
    int len = (int) new File(fileName).length();
    byte[] buf = new byte[len];
    DataInputStream in = new DataInputStream(new FileInputStream(fileName));
    in.readFully(buf);
    in.close();
    client.effectiveSend(buf, len);
  }

  public static UClient connect() {
    final Flag flag = new Flag();
    UClient client = connect(flag, getAiboIP());
    flag.waitUntilUp();
    return client;
  }

  public static UClient connect(final Flag flag, String ip) {
    UClient client = new UClient(ip);
    client.setCallback(new UCallbackListener() {
      public void actionPerformed(URBIEvent event) {
        flag.raise();
      }
    }, "power");
    return client;
  }

  public static void send(UClient client, String cmd) throws IOException {
    System.out.println(cmd);
    client.send(cmd);
  }

  public static void showConfigDialog(String okText, final UrbiConfigEscape escape) {
    final UrbiConfigDialog dialog = new UrbiConfigDialog(okText, getAiboIP());
    final Flag escaped = new Flag();
    dialog.getOKButton().addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (escaped.raise()) {
          String ip = dialog.getIP();
          setAiboIP(ip);
          escape.ok(ip);
        }
      }
    });
    dialog.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        if (escaped.raise())
          escape.cancel();
      }

      public void windowClosed(WindowEvent e) {
        if (escaped.raise())
          escape.cancel();
      }
    });
    GUIUtil.showFrame(dialog);
  }

  public static void setAiboIP(String ip) {
    try {
      FileUtil.saveTextFile(AIBO_IP_FILE, ip);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public static String getAiboIP() {
    String ip = null;
    try {
      ip = FileUtil.loadTextFile(AIBO_IP_FILE);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    return ip != null ? ip : DEFAULT_AIBO_IP;
  }
}