/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe01;

import liburbi.call.UCallbackListener;
import liburbi.call.URBIEvent;

import static java.lang.Math.max;
import static java.lang.Math.min;
import java.util.ArrayList;
import java.util.List;

public class WagTail {
  private Aibo aibo;
  private double speed;
  private boolean started;
  private int amplitude = 35;
  private List<TailListener> listeners = new ArrayList<TailListener>();

  public WagTail(Aibo aibo) {
    this.aibo = aibo;
    aibo.setCallback(new UCallbackListener() {
      public void actionPerformed(URBIEvent event) {
        for (TailListener listener : listeners)
          listener.wagging();
      }
    }, "wag");
  }

  public void start(double iterationsPerSecond) {
    setSpeed(iterationsPerSecond);
    aibo.send("stop wag; tailPhase=0; tailTilt = 0 & wag: { " + wagCmd("") + " " +
      "& every (1s) { wag: 1 } },");
    started = true;
  }

  private String wagCmd(String modifiers) {
    return
      "tailPan = 0 sin:" + (1 / speed) + "s ampli:" + amplitude + " getphase:tailPhase " + modifiers;
  }

  public void stop() {
    //aibo.send("tailPan = 0 time:2s;");
    started = false;
    aibo.send("stop wag; tailPan = 0 time:2s,");
  }

  public void setSpeed(double iterationsPerSecond) {
    double newSpeed = min(3, max(0.05, iterationsPerSecond));
    if (speed != newSpeed) {
      speed = newSpeed;
      if (started)
        aibo.send("wag: { tailPhase2 = tailPhase; " + wagCmd("phase:tailPhase2") + " },");
    }
  }

  public void addListener(TailListener listener) {
    listeners.add(listener);
  }

  public double getSpeed() {
    return speed;
  }
}