/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe02;

import de.dfki.lt.mary.client.MaryClient;
import drjava.util.Errors;
import prophecy.joe.joe01.Aibo;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

public class MarySpeech {
  private Aibo aibo;
  private MaryClient maryClient;

  public MarySpeech(Aibo aibo) {
    this.aibo = aibo;
    try {
      maryClient = new MaryClient("localhost", 59125);
    } catch (Throwable e) {
      Errors.add(e);
    }
  }

  public void say(String text) {
    say(text, "TEXT_DE");
  }

  public void saySable(String text) {
    say("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n" +
      "<!DOCTYPE SABLE SYSTEM \"http://mary.dfki.de/lib/Sable.v0_2.dtd\">\n" +
      "<SABLE xml:lang=\"de\">\n" +
      text +
      "</SABLE>", "SABLE");
  }

  public void say(String text, String inputType) {
    if (maryClient == null) {
      System.out.println("<Aibo> " + text);
      return;
    }

    try {
      File waveFile = new File("temp/mary.wav");
      new File("temp").mkdirs();

      makeWave(text, waveFile, inputType);

      aibo.playSoundAndMoveMouth(waveFile);
      //aibo.send("mouth.val'n = 0.5 smooth:2000;");
    } catch (IOException e) {
      //throw new RuntimeException(e);
      Errors.add(e);
    }
  }

  private void makeWave(String text, File waveFile, String inputType) throws IOException {
    String outputType = "AUDIO";
    String audioType = "WAVE";
    String defaultVoiceName = /*"de4"*//*"german"*/"german-hmm-bits2";
    FileOutputStream outputStream = new FileOutputStream(waveFile);
    maryClient.process(text, inputType, outputType, audioType, defaultVoiceName, outputStream);
    outputStream.close();
  }

  public boolean isConnected() {
    return maryClient != null;
  }

  public void collectIssues(List<String> issues) {
    if (!isConnected())
      issues.add("Speech disabled (no connection to MARY server)");
  }
}