/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe02;

import prophecy.audio.a09.BPMDetector;
import prophecy.audio.a10.A10;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundUtil;
import prophecy.common.Var;

import java.util.ArrayList;
import java.util.List;

public class MusicDetector {
  Var<Sound> chunk = new Var<Sound>();
  private RotatingSoundBuffer buffer = new RotatingSoundBuffer(5.0);
  private BPMDetector bpmDetector;
  private List<BPMListener> listeners = new ArrayList<BPMListener>();
  private boolean saveChunks;
  private long timestamp = System.currentTimeMillis();
  private int count = 0;

  public MusicDetector(SoundIn soundIn) {
    bpmDetector = makeBPMDetector();

    soundIn.addListener(buffer);
    buffer.addListener(new SoundListener() {
      public void soundReceived(Sound sound) {
        if (saveChunks) {
          SoundUtil.save(sound, "temp/capture" + timestamp + "_" + (++count) + ".wav");
        }
        chunk.set(sound);
      }
    });
  }

  private BPMDetector makeBPMDetector() {
    return (BPMDetector) A10.volumes.getWinner();
  }

  public void work() {
    Sound sound = chunk.swap(null);
    if (sound != null && bpmDetector != null) {
      double bpm = bpmDetector.detectBPM(sound);
      for (BPMListener listener : listeners) {
        listener.bpmDetected(bpm);
      }
    }
  }

  public void addListener(BPMListener listener) {
    listeners.add(listener);
  }

  public void setSaveChunks(boolean saveChunks) {
    this.saveChunks = saveChunks;
  }
}