/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe02;

import org.tritonus.share.sampled.FloatSampleBuffer;
import prophecy.common.sound.Sound;

import java.util.ArrayList;
import java.util.List;

public class RotatingSoundBuffer implements AudioListener {
  /** in seconds */
  private double chunkSize;

  Sound lastChunk;
  List<float[]> newData = new ArrayList<float[]>();
  int newDataSize = 0;
  float sampleRate;
  private List<SoundListener> listeners = new ArrayList<SoundListener>();

  public RotatingSoundBuffer(double chunkSize) {
    this.chunkSize = chunkSize;
  }

  public double getChunkSize() {
    return chunkSize;
  }

  public synchronized void audioReceived(FloatSampleBuffer buffer) {
    sampleRate = buffer.getSampleRate();
    newData.add(addChannels(buffer.getChannel(0), buffer.getChannel(1)));
    newDataSize += buffer.getSampleCount();
    if (newDataSize >= chunkSize*sampleRate) {
      lastChunk = makeChunk();
      for (SoundListener listener : listeners) listener.soundReceived(lastChunk);
    }
  }

  private Sound makeChunk() {
    float[] samples = new float[(int) (chunkSize*sampleRate)];
    int idx = 0;
    for (float[] s : new ArrayList<float[]>(newData)) {
      int count = Math.min(s.length, samples.length - idx);
      //System.out.println("needed=" + samples.length + ", idx=" + idx + ", count=" + count + ", s.length=" + s.length);
      System.arraycopy(s, 0, samples, idx, count);
      if (count == s.length)
        newData.remove(0);
      else {
        float[] s2 = new float[s.length-count];
        System.arraycopy(s, count, s2, 0, s2.length);
        newData.set(0, s2);
      }
      newDataSize -= count;
      idx += count;
      if (idx >= samples.length) {
        //System.out.println("break");
        break;
      }
    }
    return new Sound(sampleRate, samples);
  }

  private float[] addChannels(float[] l, float[] r) {
    float[] result = new float[l.length];
    for (int i = 0; i < l.length; i++)
      result[i] = l[i] + r[i];
    return result;
  }

  public synchronized Sound getLastChunk() {
    return lastChunk;
  }

  public synchronized void addListener(SoundListener listener) {
    listeners.add(listener);
  }

  public float getSampleRate() {
    return sampleRate;
  }
}