/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe02;

import liburbi.call.UCallbackListener;
import liburbi.call.URBIEvent;
import org.tritonus.share.sampled.FloatSampleBuffer;
import prophecy.joe.joe01.Aibo;

import javax.sound.sampled.AudioFormat;
import java.util.ArrayList;
import java.util.List;

public class SoundIn implements UCallbackListener {
  private float sampleRate = 8000;
  private int sampleSizeInBits = 8;
  private int channels = 1;
  private boolean signed = true;

  private boolean capturing = false;
  private Aibo aibo;
  private List<AudioListener> listeners = new ArrayList<AudioListener>();

  public synchronized void actionPerformed(URBIEvent event) {
    if (!capturing || event.getBinBuffer() == null)
      return;
    //System.out.println("binBuffer: " + event.getBinBuffer().length);
    sampleRate = event.getSampleRate();
    sampleSizeInBits = event.getNbOfBits();
    channels = event.getNbOfChannels();
    signed = event.getSigned();

    for (AudioListener listener : listeners) {
      listener.audioReceived(new FloatSampleBuffer(event.getBinBuffer(), 0, event.getBinBuffer().length,
        new AudioFormat(sampleRate, sampleSizeInBits, channels, signed, false)));
    }
  }

  public synchronized void startCapture() {
    if (!capturing) {
      capturing = true;
      aibo.send("loopsound:loop sound:micro.val, ");
    }
  }

  public synchronized void stopCapture() {
    if (capturing) {
      aibo.send(" stop loopsound; wait(1000); ");
      capturing = false;
    }
  }

  /*public synchronized void saveAudio(String filename) {
    File file = new File(filename);

    try {
      if (file.exists())
        file.delete();
      file.createNewFile();

      byte audio[] = out.toByteArray();
      InputStream input = new ByteArrayInputStream(audio);

      SoundUtilities.setSampleRate(sampleRate);
      SoundUtilities.setSampleSizeInBits(sampleSizeInBits);
      SoundUtilities.setChannels(channels);
      SoundUtilities.setSigned(signed);
      final AudioFormat format = SoundUtilities.getFormat();

      final AudioInputStream ais = new AudioInputStream(input, format,
        audio.length / format.getFrameSize());
      AudioSystem.write(ais, AudioFileFormat.Type.WAVE, file);
    }
    catch (IOException e) {
      throw new RuntimeException(e);
    }
  }*/

  public SoundIn(Aibo aibo) {
    this.aibo = aibo;
    aibo.setCallback(this, "sound");
  }

  public float getSampleRate() {
    return sampleRate;
  }

  public int getSampleSizeInBits() {
    return sampleSizeInBits;
  }

  public int getChannels() {
    return channels;
  }

  public boolean isSigned() {
    return signed;
  }

  public boolean isCapturing() {
    return capturing;
  }

  public void addListener(AudioListener listener) {
    listeners.add(listener);
  }

  public void removeListener(AudioListener listener) {
    listeners.remove(listener);
  }
}