/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe03;

import liburbi.call.UCallbackListener;
import liburbi.call.URBIEvent;
import prophecy.common.image.RGBImage;
import prophecy.joe.joe01.Aibo;
import prophecy.common.Var;
import prophecy.joe.joe04.ImageUtilitiesFixed;

import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

public class Camera implements UCallbackListener {
  private Aibo aibo;
  private Var<RGBImage> lastImage = new Var<RGBImage>();
  private List<CameraListener> listeners = new ArrayList<CameraListener>();
  private volatile int width, height;
  private volatile double speed;

  public Camera(Aibo aibo) {
    this.aibo = aibo;

    aibo.send("camera.resolution = 0;" +
      "camera.format = 1;" +
      "camera.jpegfactor = 75;");
  }

  public void startCapture(double picsPerSecond) {
    aibo.setCallback(this, "cam");
    //aibo.send("loop { wait(" + delay + "); cam: camera.val; }, ");
    aibo.send("camloop: every (" + (int) (1000/picsPerSecond) + ") cam: camera.val, ");
    setSpeed(picsPerSecond);
  }

  public void changeSpeed(double picsPerSecond) {
    aibo.send("stop camloop; camloop: every (" + (int) (1000/picsPerSecond) + ") cam: camera.val, ");
    setSpeed(picsPerSecond);
  }

  private void setSpeed(double picsPerSecond) {
    speed = picsPerSecond;
    for (CameraListener listener : listeners)
      listener.speedChanged();
  }

  public void actionPerformed(URBIEvent event) {
    if (event.getBinBuffer() == null)
      return;
    width = event.getWidth();
    height = event.getHeight();

    ImageUtilitiesFixed.setWidth(width);
    ImageUtilitiesFixed.setHeight(height);
    Image im = ImageUtilitiesFixed.blockingLoad(event.getBinBuffer());
    //Image im = Toolkit.getDefaultToolkit().createImage(event.getBinBuffer());
    if (im != null) {
      BufferedImage buffer = ImageUtilitiesFixed.makeBufferedImage(im);
      RGBImage image = new RGBImage(buffer, null);
      lastImage.set(image);
      for (CameraListener listener : listeners) {
        listener.imageReceived(image);
      }
    }
  }

  public Var<RGBImage> getLastImage() {
    return lastImage;
  }

  public void addCameraListener(CameraListener listener) {
    listeners.add(listener);
  }

  public void stopCapture() {
    aibo.send("stop camloop;");
    aibo.deleteCallback("cam");
  }

  public int getImageWidth() {
    return width;
  }

  public int getImageHeight() {
    return height;
  }

  public double getSpeed() {
    return speed;
  }
}