/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe04;

import drjava.util.Log;
import prophecy.common.image.RGBImage;
import prophecy.e.e11.BallLocator;
import prophecy.e.e11.E11;
import prophecy.joe.joe03.Camera;
import prophecy.joe.joe03.CameraListener;

import java.awt.*;
import java.awt.geom.Point2D;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class DetectBall extends Action {
  private Camera camera;
  private BallLocator ballLocator;
  private Point ballLocation;
  private List<BallListener> listeners = new ArrayList<BallListener>();

  public DetectBall() {
    Log.info("Initialising ball location module");
    try {
      ballLocator = E11.getBestBallLocator();
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
    Log.info("done");
  }

  public DetectBall(Camera camera) {
    this();
    setCamera(camera);
  }

  protected void onStart() {
    if (camera == null)
      camera = new Camera(aibo);
    camera.addCameraListener(new CameraListener() {
      public void imageReceived(RGBImage image) {
        setBallLocation(ballLocator.locateBall(image));
      }

      public void speedChanged() {
      }
    });
    camera.startCapture(1);
  }

  protected void onStop() {
    camera.stopCapture();
  }

  public void setCamera(Camera camera) {
    this.camera = camera;
  }

  private void setBallLocation(Point point) {
    Point oldLocation = ballLocation;
    ballLocation = point;
    if (oldLocation != null && point == null)
      for (BallListener listener : listeners) listener.ballDisappeared();
    else if (oldLocation != null && !point.equals(oldLocation))
      for (BallListener listener : listeners) listener.ballMoved();
    else if (oldLocation == null && point != null)
      for (BallListener listener : listeners) listener.ballAppeared();
  }

  public boolean ballLocated() {
    return ballLocation != null;
  }

  public Point getBallLocation() {
    return ballLocation;
  }

  public Point2D getRelativeBallLocation() {
    return ballLocation == null ? null :
      new Point2D.Double(ballLocation.x/(double) camera.getImageWidth(),
        ballLocation.y/(double) camera.getImageHeight());
  }

  public void addListener(BallListener listener) {
    listeners.add(listener);
  }
}