/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe06;

import liburbi.UClient;
import prophecy.common.image.RGBImage;
import prophecy.joe.joe01.Head;
import prophecy.joe.joe03.CameraListener;

public class Joe06 extends AbstractJoe {
  private Head head;
  private BallTracker ballTracker;

  Joe06(final JoeUI ui, UClient client) {
    super(ui);
    aibo = new Aibo06(client);
    aibo.getSpeech();
    head = aibo.getHead();
    ballTracker = new BallTracker(aibo, new BallTrackerEnvironment() {
      public void requestHeadControl() {
        stopLookingAround();
        ui.showAction("Tracking ball");
      }

      public void yieldHeadControl() {
        head.normalYPosition();
        lookAround();
      }

      public void requestCameraSpeed(int picsPerSecond) {
        aibo.getCamera().changeSpeed(picsPerSecond);
      }

      public void yieldCameraSpeed() {
        aibo.getCamera().changeSpeed(1);
      }
    });

    aibo.getCamera().addCameraListener(new CameraListener() {
      public void imageReceived(RGBImage image) {
        ui.showCameraImage(image);
      }

      public void speedChanged() {
        ui.showCameraSpeed((int) aibo.getCamera().getSpeed());
      }
    });

    ui.showHealth(aibo.getHealth());
    lookAhead();
    lookAround();
    ballTracker.start();
    //sleepForever();
  }

  private void stopLookingAround() {
    head.stopMovement();
  }

  private void lookAhead() {
    head.stopMovement();
    head.normalPosition();
  }

  private void lookAround() {
    head.randomMovement(10);
    ui.showAction("Looking around");
  }
}