/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe06;

import drjava.util.GUIUtil;
import drjava.util.LetterLayout;
import jazz.retroui.FancyScreen;
import jazz.retroui.RetroUIPanel;
import prophecy.common.image.ImageSurface;
import prophecy.common.image.RGBImage;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class RetroUI06 implements JoeUI {
  private FancyScreen screen;
  private Actions06 actions;
  private JFrame frame;
  private String title;
  private ImageSurface cameraView;

  public RetroUI06(String title) {
    this.title = title;

    frame = new JFrame(title);
    JMenuBar menuBar = new JMenuBar();
    frame.setJMenuBar(menuBar);
    frame.setResizable(false);
    RetroUIPanel panel = new RetroUIPanel(60, 5);
    frame.getContentPane().setLayout(new LetterLayout("RR", "RR", "CC", "CC").setSpacing(20));
    frame.getContentPane().add("R", panel);
    cameraView = new ImageSurface();
    cameraView.setZoom(2);
    cameraView.setMinimumSize(new Dimension(208*2, 160*2));
    int space = (panel.getMinimumSize().width-cameraView.getMinimumSize().width)/2;
    cameraView.setBorder(BorderFactory.createEmptyBorder(0, space, 0, space));
    JPanel cameraPanel = new JPanel(new LetterLayout("C").setBorder(0, space, 20, space));
    cameraPanel.add("C", cameraView);
    frame.getContentPane().add("C", cameraPanel);
    frame.pack();

    JMenuItem miCaptureImage = new JMenuItem("Capture image");
    miCaptureImage.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        actions.captureImage();
      }
    });
    menuBar.add(miCaptureImage);

    screen = panel.getScreen();
    screen.print(1, 1, "Aibo status");
    screen.print(1, 2, "Action");
    screen.print(1, 3, "Camera speed");

    frame.pack();
    frame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        actions.shutDown();
      }
    });
    GUIUtil.showFrame(frame);
  }

  public void showCameraSpeed(int picsPerSecond) {
    screen.print(31, 3, picsPerSecond + "/s     ");
  }

  public void showCameraImage(RGBImage image) {
    cameraView.setImage(image);
  }

  public void showAction(String action) {
    screen.print(31, 2, 100, action);
    frame.setTitle(title + " [" + action + "]");
  }

  public void showHealth(String text) {
    screen.print(31, 1, 100, text);
  }

  public void dispose() {
    frame.dispose();
  }

  public void setActions(Actions06 actions) {
    this.actions = actions;
  }
}