/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe07;

import liburbi.UClient;
import prophecy.common.Application;
import prophecy.joe.joe01.UrbiUtil;
import prophecy.joe.joe06.AbstractJoe;
import prophecy.joe.joe08.UrbiConnectionFrameEscape;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public abstract class JoeServer extends Application {
  private volatile State state = State.fresh;
  private String name;
  private boolean quickConnect;

  public JoeServer(String name) {
    this.name = name;
  }

  protected void config() {
    if (!setState(State.config)) return;

    UrbiUtil.showConfigDialog("Start " + name, new UrbiConfigEscape() {
      public void cancel() {
        retire();
      }

      public void ok(String ip) {
        //new Joe07(new RetroUI06("Joe07"), UrbiUtil.connect());
        connect();
      }
    });
  }

  public void retire() {
    setState(State.dead);
    super.retire();
  }

  public void connect() {
    if (!setState(State.connecting)) return;

    UrbiConnectionFrame connectionFrame = new UrbiConnectionFrame(new UrbiConnectionFrameEscape() {
      public void success(UClient client) {
        if (state == State.connecting)
          communicate(client);
      }

      public void timeout() {
        if (state == State.connecting)
          connect();
      }

      public void changeIP() {
        if (state == State.connecting)
          config();
      }

      public void cancel() {
        if (state == State.connecting)
          retire();
      }
    }, quickConnect);
    connectionFrame.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        off();
      }
    });
  }

  private void off() {
    setState(State.dead);
  }

  private void communicate(UClient client) {
    if (!setState(State.communicating)) {
      try {
        client.disconnect();
      } catch (Throwable e) {
      }
      return;
    }

    final AbstractJoe joe = makeJoe(client);
    joe.getConnectionLostFlag().onRaise(new Runnable() {
      public void run() {
        connect();
      }
    });
    joe.getShutdownFlag().onRaise(new Runnable() {
      public void run() {
        retire();
      }
    });
  }

  protected abstract AbstractJoe makeJoe(UClient client);

  public boolean setState(State state) {
    if (this.state == state)
      return false;
    this.state = state;
    trigger();
    return true;
  }

  public String getStateDescription() {
    return state.getDescription();
  }

  public void setQuickConnect(boolean quickConnect) {
    this.quickConnect = quickConnect;
  }

  public static enum State {
    fresh("fresh"), config("configuring"), connecting("connecting"),
    communicating("communicating"), dead("dead");

    private String description;

    State(String description) {
      this.description = description;
    }

    public String getDescription() {
      return description;
    }
  }

  public void start() {
    connect();
  }
}
