/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe07;

import drjava.util.Errors;
import drjava.util.GUIUtil;
import drjava.util.LetterLayout;
import drjava.util.MultiCoreUtil;
import jazz.retroui.RetroUIFrame;
import liburbi.UClient;
import prophecy.common.Flag;
import prophecy.joe.joe01.UrbiUtil;
import prophecy.joe.joe08.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.Socket;

public class UrbiConnectionFrame extends RetroUIFrame {
  private UrbiConnectionFrameEscape escape;
  private boolean quickConnect;
  private String ip = UrbiUtil.getAiboIP();
  private Flag doneFlag = new Flag();

  public UrbiConnectionFrame(final UrbiConnectionFrameEscape escape, boolean quickConnect) {
    super("Connecting to Aibo", 80, 4);
    this.escape = escape;
    this.quickConnect = quickConnect;
    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        dispose();
        escape.cancel();
      }
    });
    GUIUtil.showFrame(this);
    tryPing();
    getSouthPanel().setLayout(new LetterLayout("  B  ").setBorder(10));
    getSouthPanel().setBackground(Color.black);
    JButton btnChangeIP = new JButton("Change IP");
    btnChangeIP.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dispose();
        escape.changeIP();
      }
    });
    getSouthPanel().add("B", btnChangeIP);
    pack();
  }

  private void tryPing() {
    getScreen().printCentered(0, 2, getScreen().getWidth(), "Contacting " + ip + "...");
    SocketConnector connector = new SocketConnector(ip, 54000);
    connector.setListener(new SocketConnectListener() {
      public void success(Socket socket) {
        try {
          socket.close();
        } catch (Throwable e) {
          Errors.add(e);
        }

        if (!doneFlag.isUp()) {
          if (!quickConnect) {
            getScreen().printCentered(0, 2, getScreen().getWidth(), "Connected, waiting for system startup");
            MultiCoreUtil.sleep(10000);
          }
          tryConnect();
        }
      }

      public void timeout() {
        quickConnect = false;
        tryPing();
      }
    });
    connector.check();
  }

  private void tryConnect() {
    getScreen().printCentered(0, 2, getScreen().getWidth(), "Establishing communication");
    UrbiConnector connector = new UrbiConnector(ip);
    connector.setTimeoutSeconds(20);
    connector.setListener(new UrbiConnectEscape() {
      public void success(UClient client) {
        if (doneFlag.raise()) {
          dispose();
          escape.success(client);
        }
      }

      public void timeout() {
        if (doneFlag.raise()) {
          dispose();
          escape.timeout();
        }
      }
    });
    connector.check();
  }
}