/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe08;

import drjava.util.LetterLayout;
import liburbi.UClient;
import prophecy.common.gui.Form;
import prophecy.joe.joe01.UrbiUtil;
import prophecy.video.v03.Help;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class UrbiConfigDialog extends JFrame {
  private JTextField tfIP;
  private JLabel lConnection;
  private JButton btnOK;

  public UrbiConfigDialog(String okText, String defaultIP) {
    super("Aibo Configuration [Project Prophecy]");
    setSize(420, 240);

    Form form = new Form();
    tfIP = new JTextField(defaultIP);
    form.addRow("Aibo IP", tfIP);
    lConnection = new JLabel("Not tested yet");
    form.addRow("Connection", lConnection);

    JPanel buttonPanel = new JPanel(LetterLayout.leftAlignedRow(10));

    btnOK = new JButton(okText);
    btnOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dispose();
      }
    });
    buttonPanel.add(btnOK);

    JButton btnTest = new JButton("Test connection");
    btnTest.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        testConnection();
      }
    });
    buttonPanel.add(btnTest);

    JButton btnCancel = new JButton("Cancel");
    btnCancel.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        dispose();
      }
    });
    buttonPanel.add(btnCancel);

    tfIP.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        btnOK.doClick();
      }
    });

    form.addRow(buttonPanel);
    form.addSpacer();
    form.addRow(Help.makeHelpComponent("Aibo must be equipped with a pink memory stick " +
      "containing the URBI system. You can tell if URBI is running by " +
      "checking if Aibo's back lights light up in a circular fashion.", 0));

    getContentPane().setLayout(new LetterLayout("F").setBorder(10));
    getContentPane().add("F", form);
  }

  private void testConnection() {
    final String ip = tfIP.getText();
    lConnection.setText("Connecting to " + ip + "...");
    UrbiConnector connector = new UrbiConnector(ip);
    connector.setListener(new UrbiConnectEscape() {
      public void success(UClient client) {
        setConnectionTextLater("Success - connected to Aibo at " + ip + "!");
      }

      public void timeout() {
        setConnectionTextLater("Timeout - could not connect to " + ip);
      }
    });
    connector.check();
  }

  private void setConnectionTextLater(final String text) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        lConnection.setText(text);
      }
    });
  }

  public static void main(String[] args) {
    UrbiUtil.showConfigDialog("Prima prima", null);
  }

  public String getIP() {
    return tfIP.getText();
  }

  public JButton getOKButton() {
    return btnOK;
  }
}