/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe09;

import drjava.util.GUIUtil;
import prophecy.common.sound.Sound;
import prophecy.joe.joe02.RotatingSoundBuffer;
import prophecy.joe.joe02.SoundIn;
import prophecy.joe.joe02.SoundListener;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class AiboSoundSheet extends SoundSheet {
  private SoundIn soundIn;
  private SoundBuffer soundBuffer;

  public AiboSoundSheet(String title, SoundIn soundIn, double bufferSeconds) {
    super(title);
    this.soundIn = soundIn;

    final RotatingSoundBuffer buffer = new RotatingSoundBuffer(bufferSeconds);
    soundIn.addListener(buffer);
    buffer.addListener(new SoundListener() {
      public void soundReceived(Sound sound) {
        setSound(sound);
      }
    });

    final JButton btnRecord = new JButton("Record");
    addControl(btnRecord);
    btnRecord.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (soundBuffer != null) {
          stopRecording();
          btnRecord.setText("Record");
        } else {
          startRecording();
          btnRecord.setText("Stop");
        }
      }
    });
  }

  private void stopRecording() {
    soundIn.removeListener(soundBuffer);
    Sound sound = soundBuffer.getSound();
    GUIUtil.showFrame(new SoundSheet("Recorded", sound).asFrame());
    soundBuffer = null;
  }

  private void startRecording() {
    soundBuffer = new SoundBuffer();
    soundIn.addListener(soundBuffer);
  }
}