/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe09;

import drjava.util.GUIUtil;
import static drjava.util.StringUtil.formatDouble;
import drjava.util.TextView;
import prophecy.joe.joe02.SoundIn;
import prophecy.joe.joe06.Aibo06;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class Joe09Sheet extends AiboSoundSheet {
  private TextView textView;
  private Parrot parrot;

  public Joe09Sheet(SoundIn soundIn, final Aibo06 aibo) {
    super("Joe09", soundIn, 0.1);

    textView = new TextView();
    textView.setMinimumSize(new Dimension(100, 100));
    addComponent(textView);

    addComponent(GUIUtil.newButton("Say something", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        parrot.saySomething();
      }
    }));

    parrot = new Parrot(aibo, soundIn);
    parrot.getBufferProcessedTrigger().addListener(new Runnable() {
      public void run() {
        update();
      }
    });

    parrot.start();
  }

  protected void init() {
    setShowPlayButton(false);
    setShowSaveButton(false);
    super.init();
  }

  public void update() {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        String text = "";
        text += "total words: " + parrot.getWordCollector().size() + "<br>";
        text += "current word length: " + parrot.getWordRecorder().getWordInFlightStats() + "<br>";
        text += "chunks in current snippet: " + parrot.getWordRecorder().getChunksInSnippet() + "<br>";
        text += "last action: " + parrot.getWordRecorder().getLastAction() + "<br>";
        text += "stereo balance: " + formatDouble(parrot.getStereoBalance().getCurrentRawBalance(), 2) + "<br>";
        text += "stereo average: " + formatDouble(parrot.getStereoBalance().getAverage(), 2) + "<br>";
        textView.setText(text);
      }
    });
  }
}
