/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe09;

import drjava.util.RandomUtil;
import drjava.util.StringUtil;
import prophecy.common.Prophecy;
import prophecy.audio.a03.AudioAnalysis;
import prophecy.common.Trigger;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundUtil;
import prophecy.joe.joe02.SoundIn;
import prophecy.joe.joe06.Aibo06;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

public class Parrot {
  private WordCollector wordCollector;
  private WordRecorder wordRecorder;
  private static final double pitch = 1.15;
  private Aibo06 aibo;
  private SoundIn soundIn;
  private StereoBalance stereoBalance = new StereoBalance();
  private double immediateEchoChance = 0.5;
  private double echoChance = 0.5;

  public Parrot(Aibo06 aibo, SoundIn soundIn) {
    this.aibo = aibo;
    this.soundIn = soundIn;
    wordCollector = new WordCollector(Prophecy.memoryDir() + "joe09/words" + StringUtil.ymd());
    wordRecorder = new WordRecorder(soundIn, wordCollector);
  }

  public void saySomething() {
    say(wordCollector.randomWord());
  }

  private void say(File file) {
    if (file != null) {
      Sound sound = SoundUtil.load(file);
      say(sound);
    }
  }

  private void say(Sound sound) {
    wordRecorder.setIgnoreNextWordCountdown(4.0, "Aibo talking");
    sound = specialEffects(sound);
    aibo.playSoundAndMoveMouth(sound);
  }

  private Sound specialEffects(Sound sound) {
    return AudioAnalysis.normalize(AudioAnalysis.resample(sound, pitch));
  }

  public Trigger getBufferProcessedTrigger() {
    return wordRecorder.getBufferProcessedTrigger();
  }

  public void start() {
    stereoBalance.listenTo(soundIn);

    wordRecorder.addListener(new WordRecorderListener() {
      public void wordRecorded(final Sound sound) {
        double x = stereoBalance.getCurrentBalance();
        if (Math.abs(x) >= 0.01)
          aibo.getHead().panRelN(x, 1);

        Sound soundToPlay = null;

        if (RandomUtil.chance(immediateEchoChance))
          soundToPlay = append(soundToPlay, sound);
        int count = 0;
        while (RandomUtil.chance(echoChance) && count++ < 10) {
          soundToPlay = append(soundToPlay, SoundUtil.load(wordCollector.randomWord()));
          System.out.println("append => " + soundToPlay.getSeconds() + "s");
        }

        final Sound soundToPlay1 = soundToPlay;
        if (soundToPlay != null) doIn(1.0, new ActionListener() {
          public void actionPerformed(ActionEvent e) {
            say(soundToPlay1);
          }
        });
      }
    });
  }

  private Sound append(Sound a, Sound b) {
    if (a == null) return b;
    return SoundUtil.append(SoundUtil.append(a, SoundUtil.silence(b.getSampleRate(), 0.5)), b);
  }

  private void doIn(double seconds, ActionListener actionListener) {
    Timer timer = new Timer((int) (seconds*1000), actionListener);
    timer.setRepeats(false);
    timer.start();
  }

  public WordCollector getWordCollector() {
    return wordCollector;
  }

  public WordRecorder getWordRecorder() {
    return wordRecorder;
  }

  public StereoBalance getStereoBalance() {
    return stereoBalance;
  }
}