/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe09;

import org.tritonus.share.sampled.FloatSampleBuffer;
import prophecy.common.sound.Sound;
import prophecy.joe.joe02.AudioListener;

import java.util.ArrayList;
import java.util.List;

public class SoundBuffer implements AudioListener {
  List<float[]> data = new ArrayList<float[]>();
  int dataSize = 0;
  double sampleRate;

  public SoundBuffer() {
  }

  public synchronized void audioReceived(FloatSampleBuffer buffer) {
    float[] newData = addChannels(buffer.getChannel(0), buffer.getChannel(1));
    append(buffer.getSampleRate(), newData);
  }

  private void append(double sampleRate, float[] newData) {
    this.sampleRate = sampleRate;
    data.add(newData);
    dataSize += newData.length;
  }

  public void append(Sound sound) {
    append(sound.getSampleRate(), sound.getSamples());
  }

  public Sound getSound() {
    float[] samples = new float[dataSize];
    int idx = 0;
    for (float[] s : data) {
      int count = s.length;
      System.arraycopy(s, 0, samples, idx, count);
      idx += count;
    }
    return new Sound(sampleRate, samples);
  }

  private float[] addChannels(float[] l, float[] r) {
    float[] result = new float[l.length];
    for (int i = 0; i < l.length; i++)
      result[i] = l[i] + r[i];
    return result;
  }

  public double getSeconds() {
    return dataSize/sampleRate;
  }
}