/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe09;

import drjava.util.LetterLayout;
import prophecy.common.Prophecy;
import prophecy.common.gui.Sheet;
import prophecy.common.sound.Sound;
import prophecy.common.sound.SoundSurface;
import prophecy.common.sound.SoundUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class SoundSheet extends Sheet {
  private SoundSurface soundSurface;
  private JPanel controlPanel;
  private boolean showPlayButton = true;
  private boolean showSaveButton = true;

  public SoundSheet(String title) {
    super(title);

    init();
  }

  protected void init() {
    soundSurface = new SoundSurface();
    soundSurface.setMinimumSize(new Dimension(300, 300));
    addComponent(soundSurface);
    controlPanel = new JPanel(LetterLayout.leftAlignedRow(10));
    addComponent(controlPanel);

    if (showPlayButton) {
      JButton btnPlay = new JButton("Play");
      btnPlay.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          play();
        }
      });
      addControl(btnPlay);
    }

    if (showSaveButton) {
      JButton btnSave = new JButton("Save...");
      btnSave.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          save();
        }
      });
      addControl(btnSave);
    }
  }

  private void save() {
    Sound sound = getSound();
    JFileChooser fileChooser = new JFileChooser(Prophecy.dataDir());
    if (fileChooser.showSaveDialog(panel) == JFileChooser.APPROVE_OPTION) {
      SoundUtil.save(sound, fileChooser.getSelectedFile().getPath());
    }
  }

  private void play() {
    SoundUtil.playInExtraThread(getSound());
  }

  private Sound getSound() {
    return soundSurface.getSound();
  }

  public SoundSheet(String title, Sound sound) {
    this(title);
    setSound(sound);
  }

  public void setSound(Sound sound) {
    soundSurface.setSound(sound);
  }

  public void addControl(JComponent component) {
    controlPanel.add(component);
  }

  public void setShowPlayButton(boolean showPlayButton) {
    this.showPlayButton = showPlayButton;
  }

  public void setShowSaveButton(boolean showSaveButton) {
    this.showSaveButton = showSaveButton;
  }
}