/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.joe.joe09;

import org.tritonus.share.sampled.FloatSampleBuffer;
import prophecy.audio.a03.AudioAnalysis;
import prophecy.common.sound.Sound;
import prophecy.e.e02.DistributionMaker;
import prophecy.joe.joe02.AudioListener;
import prophecy.joe.joe02.SoundIn;

public class StereoBalance {
  private double currentBalance;
  private DistributionMaker stereoDistribution = new DistributionMaker();

  public void listenTo(SoundIn soundIn) {
    soundIn.addListener(new AudioListener() {
      public void audioReceived(FloatSampleBuffer buffer) {
        currentBalance = getStereoBalance(buffer);
        stereoDistribution.addData(currentBalance);
      }
    });
  }

  private double getStereoBalance(FloatSampleBuffer buffer) {
    double left = AudioAnalysis.absAverage(new Sound(buffer.getSampleRate(), buffer.getChannel(0)));
    double right = AudioAnalysis.absAverage(new Sound(buffer.getSampleRate(), buffer.getChannel(1)));
    return left+right == 0 ? 0 : (right-left)/(left+right);
  }

  public double getAverage() {
    return stereoDistribution.getResult().getAverage();
  }

  public double getCurrentRawBalance() {
    return currentBalance;
  }

  public double getCurrentBalance() {
    return currentBalance-stereoDistribution.getResult().getAverage();
  }
}