/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m01;

import drjava.util.*;
import static drjava.util.GUIUtil.setMinimumHeight;
import prophecy.common.gui.Sheet;
import drjava.util.Errors;
import prophecy.common.comments.TreeClient;
import prophecy.common.comments.TreeRequest;
import prophecy.common.comments.TreeServer;
import prophecy.common.comments.Treelet;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.util.List;

public class Restart {
  private List<String> commandLine;
  private TextView tvStatus;
  private Sheet sheet;
  private int fatherPort;
  private JFrame frame;

  public Restart() {
    sheet = new Sheet("M01: Restart");
    sheet.setSize(800, 400);
    sheet.addLabel("This application can start clones of itself in a new Virtual Machine.");
    sheet.addButton("Clone", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        startClone();
      }
    });

    sheet.addButton("Restart (clone and terminate if successful)", new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         restart();
       }
     });

    tvStatus = new TextView();
    setMinimumHeight(tvStatus, 200);
    sheet.addComponent(tvStatus);
  }

  private void startClone() {
    startClone(new Runnable() {
      public void run() {
        appendStatus("Received notice from son");
      }
    });
  }

  private void restart() {
    setAppState("restarting");
    startClone(new Runnable() {
      public void run() {
        appendStatus("Received notice from son, exiting");
        setAppState("terminating");
        MultiCoreUtil.sleep(1000);
        System.exit(0);
      }
    });
  }

  private void setAppState(String state) {
    frame.setTitle("M01: Restart [" + state + "]");
  }

  public void setStatus(final String status) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        tvStatus.setText(status, true);
      }
    });
  }

  public static void main(String[] args) {
    Errors.setPopup(true);
    Restart restart = new Restart();
    for (int i = 0; i < args.length; i++) {
      String arg = args[i];
      if (arg.startsWith("cmdline[")) {
        List<String> list = new ArrayList<String>();
        list.add(arg.substring("cmdline[".length()));
        while (++i < args.length) {
          if (args[i].endsWith("]")) {
            list.add(args[i].substring(0, args[i].length()-1));
            break;
          }
          list.add(args[i]);
        }
        restart.setCommandLine(list);
      } else if (arg.startsWith("port[")) {
        restart.setFatherPort(Integer.parseInt(arg.substring("port[".length(), arg.length()-1)));
      }
    }
    restart.run();
  }

  private void setCommandLine(List<String> args) {
    commandLine = args;
    setStatus(StringUtil.join(" # ", args));
  }

  private void run() {
    frame = GUIUtil.showMainFrame(sheet.asFrame());
    Randomizer randomizer = new RealRandomizer();
    frame.setLocation(randomizer.random(500), randomizer.random(300));
    contactFather();
  }

  private void contactFather() {
    if (fatherPort != 0) {
      appendStatus("Contacting father at port " + fatherPort);
      try {
        Tree answer = new TreeClient("localhost", fatherPort).ask(new Tree("hello"));
        appendStatus("Answer from father: " + answer);
      } catch (IOException e) {
        error(e.toString());
      }
    }
  }

  private void startClone(final Runnable onConnection) {
    new Thread() {
      public void run() {
        try {
          setStatus("Starting server");
          ServerSocket socket = SocketUtil.serverSocketAtRandomPort();
          TreeServer server = new TreeServer(socket, new Treelet() {
            public void serve(Tree question, TreeRequest request) {
              request.answer(new Tree("hello"));
            }
          });
          server.getOnClientHandled().addListener(onConnection);
          server.start();
          setStatus("Starting clone");
          if (commandLine == null)
            error("Need command line to start clone");
          //Log.info("Running: " + StringUtil.join(" ", commandLine));
          List<String> newCommandLine = new ArrayList<String>();
          newCommandLine.addAll(commandLine);
          newCommandLine.add("cmdline[" + commandLine.get(0));
          newCommandLine.addAll(commandLine.subList(1, commandLine.size()-1));
          newCommandLine.add(commandLine.get(commandLine.size()-1) + "]");
          newCommandLine.add("port[" + socket.getLocalPort() + "]");
          String[] cmdarray = newCommandLine.toArray(new String[newCommandLine.size()]);
          setStatus("[" + cmdarray.length + "] " + StringUtil.join(" # ", cmdarray));
          final Process process = Runtime.getRuntime().exec(cmdarray);
          new Thread() {
            public void run() {
              try {
                int exitValue = process.waitFor();
                appendStatus("exit value: " + exitValue);
              } catch (Throwable e) {
                Errors.add(e);
              }
            }
          }.start();
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    }.start();
  }

  private void appendStatus(String text) {
    setStatus(getStatus() + "<br>" + text);
  }

  private String getStatus() {
    return tvStatus.getText();
  }

  private void error(String text) {
    setStatus(text);
    throw new RuntimeException(text);
  }

  public void setFatherPort(int fatherPort) {
    this.fatherPort = fatherPort;
  }
}