/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

public class ChangeAwareWatcher<A> implements Watcher<A> {
  private Watcher<A> baseWatcher;
  private ChangeWatcher<A> changeWatcher = new ChangeWatcher<A>();
  private A lastValue;

  public ChangeAwareWatcher(Watcher<A> baseWatcher) {
    this.baseWatcher = baseWatcher;
  }

  public Watcher<A> getBaseWatcher() {
    return baseWatcher;
  }

  public ChangeWatcher<A> getChangeWatcher() {
    return changeWatcher;
  }

  public void feed(A value) {
    baseWatcher.feed(value);
    changeWatcher.feed(value);
    lastValue = value;
  }

  public void describeYourself(Typewriter typewriter, String watchedObject) {
    baseWatcher.describeYourself(typewriter, watchedObject);
    changeWatcher.describeYourself(typewriter, /*watchedObject*/"");
  }

  public Prediction<A> predict() {
    Prediction<Boolean> changePrediction = changeWatcher.predict();
    if (changePrediction != null && !changePrediction.getValue() && lastValue != null) {
      return new Prediction<A>(lastValue);
    }
    return baseWatcher.predict();
  }
}