/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

import java.util.ArrayList;
import java.util.List;

public class CombinedWatcher<A> implements Watcher<A> {
  private List<Watcher<A>> watchers = new ArrayList<Watcher<A>>();

  public void addWatcherFront(Watcher<A> watcher) {
    watchers.add(0, watcher);
  }

  public void addWatcherBack(Watcher<A> watcher) {
    watchers.add(watcher);
  }

  public void feed(A value) {
    for (Watcher<A> watcher : watchers) {
      watcher.feed(value);
    }
  }

  public void describeYourself(Typewriter typewriter, String watchedObject) {
    for (Watcher<A> watcher : watchers) {
      watcher.describeYourself(typewriter, watchedObject);
    }
  }

  public Prediction<A> predict() {
    for (Watcher<A> watcher : watchers) {
      Prediction<A> prediction = watcher.predict();
      if (prediction != null)
        return prediction;
    }
    return null;
  }
}