/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

import static drjava.util.ObjectUtil.nice;
import drjava.util.RandomUtil;
import drjava.util.StringUtil;

import java.util.*;

public class Multi<A> extends Model<A> {
  private static final int maxSize = 10;
  private Set<A> examples = new TreeSet<A>();

  public Multi(Set<A> examples) {
    this.examples = examples;
  }

  public Multi(A... examples) {
    this.examples.addAll(Arrays.asList(examples));
  }

  public Set<A> getExamples() {
    return examples;
  }

  public Prediction<A> predict() {
    return new Prediction<A>(RandomUtil.oneOf(new ArrayList<A>(examples)));
  }

  public Model<A> plusExample(A example) {
    Set<A> newExamples = new TreeSet<A>(examples);
    newExamples.add(example);
    if (newExamples.size() > maxSize)
      return new Tilt<A>();
    return new Multi<A>(newExamples);
  }

  public String toString() {
    List<String> list = new ArrayList<String>();
    for (A example : examples) {
      list.add("\"" + nice(example) + "\"");
    }
    return StringUtil.join(" ", list);
  }

  public void describeYourself(Typewriter typewriter, String watchedObject) {
    ModelUtil.describeSet(typewriter, watchedObject, examples);
  }

  public boolean isMature() {
    return false; // TODO?
  }
}