/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

public class RunlengthWatcher implements Watcher<Boolean> {
  private int counter;
  private Watcher<Integer> lengthWatcher = new IntWatcher();
  private boolean inverted = false;

  public RunlengthWatcher(boolean inverted) {
    this.inverted = inverted;
  }

  public void feed(Boolean value) {
    if (value != inverted)
      ++counter;
    else {
      if (counter > 0) {
        lengthWatcher.feed(counter);
        counter = 0;
      }
    }
  }

  public void describeYourself(Typewriter typewriter, String watchedObjectDesc) {
    lengthWatcher.describeYourself(typewriter,
      watchedObjectDesc + (inverted ? " pause length" : " duration"));
  }

  public Prediction<Boolean> predict() {
    Prediction<Integer> predictedLength = lengthWatcher.predict();
    if (predictedLength != null && counter > 0)
      return new Prediction<Boolean>((counter < predictedLength.getValue()) != inverted);
    return null;
  }

  public Watcher<Integer> getLengthWatcher() {
    return lengthWatcher;
  }
}