/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

import drjava.util.ObjectUtil;
import static drjava.util.ObjectUtil.nice;

import java.util.Collections;
import java.util.Set;

public class Stable<A> extends Model<A> {
  private A value;
  private int count;

  public Stable(A value, int count) {
    this.value = value;
    this.count = count;
  }

  public boolean isStable() {
    return true;
  }

  public A getValue() {
    return value;
  }

  public Model<A> plusExample(A example) {
    return ObjectUtil.equal(value, example)
      ? new Stable<A>(value, count+1)
      : new Multi<A>(value, example);
  }

  public Set<A> getExamples() {
    return Collections.singleton(value);
  }

  public Prediction<A> predict() {
    return new Prediction<A>(value);
  }

  public String toString() {
    return nice(value) + " (stable, "
      + (isMature() ? "mature" : "count=" + count) + ")";
  }

  public void describeYourself(Typewriter typewriter, String watchedObjectDesc) {
    typewriter.addSentence(watchedObjectDesc + " is " + Typewriter.quote(value));
  }

  public int getCount() {
    return count;
  }

  public boolean isMature() {
    return count >= 10;
  }
}