/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

import drjava.util.ObjectUtil;
import drjava.util.StringUtil;

import javax.swing.*;
import java.awt.*;

public class Typewriter implements Report {
  private StringBuilder stringBuilder = new StringBuilder();
  private boolean timestamps;
  private long startTime;
  private Font font;
  //private boolean centered;

  public static String plural(String word) {
    return word + "s";
  }

  public void addHeading(String heading) {
    stringBuilder.append("<h3>" + heading + "</h3>");
  }

  public void addSentence(String s) {
    if (timestamps) {
      int secs = (int) ((System.currentTimeMillis()-startTime)/1000);
      int mins = secs / 60;
      secs = secs % 60;
      stringBuilder.append("[T+" + mins + ":" + (secs < 10 ? "0" : "") + secs + "] ");
    }

    if (!s.endsWith(".") && !s.endsWith("!") && !s.endsWith("?") && !s.endsWith(":"))
      s += ".";
    s = StringUtil.firstToUpper(s);
    addLine(s);
  }

  public void addLine(String line) {
    stringBuilder.append(line + "<br>");
  }

  public void addHTML(String html) {
    stringBuilder.append(html + "\n");
  }

  public String getHtml() {
    String html = stringBuilder.toString();
    /*if (centered)
      html = "<center>" + html + "</center>";*/
    return "<html>" + html + "</html>";
  }

  public static String quote(Object o) {
    String s = drjava.util.ObjectUtil.nice(o);
    if (StringUtil.isInteger(s)) return s;
    return "\"" + s + "\"";
  }

  public String getOneLiner() {
    return stringBuilder.toString().replace("<br>", " ");
  }

  public void setTimestamps(boolean timestamps, long startTime) {
    this.timestamps = timestamps;
    this.startTime = startTime;
  }

  public JComponent asComponent() {
    JComponent component = makeScrollableComponent();
    return new JScrollPane(component);
  }

  protected JComponent makeScrollableComponent() {
    JLabel label = new JLabel(getHtml());
    label.setBackground(Color.white);
    label.setOpaque(true);
    label.setVerticalAlignment(JLabel.TOP);
    label.setBorder(BorderFactory.createEmptyBorder(0, 10, 10, 10));
    if (font != null)
      label.setFont(font);
    return label;
  }

  public String cut(String s, int maxLength) {
    if (s.length() <= maxLength) return s;
    return s.substring(0, maxLength) + "...";
  }

  /*public void setCentered(boolean centered) {
    this.centered = centered;
  }*/

  public Report getReport() {
    return this;
  }

  public void setFont(Font font) {
    this.font = font;
  }

  public boolean resultEquals(Typewriter typewriter) {
    return getHtml().equals(typewriter.getHtml())
      && ObjectUtil.equal(font, typewriter.getFont());
  }

  public Font getFont() {
    return font;
  }
}