/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m02;

import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class WatcherTest extends TestCase {
  public void testInitial() {
    IntWatcher intWatcher = new IntWatcher();
    assertTrue(intWatcher.getModel() instanceof NoInput);
  }

  public void testOneValue() {
    IntWatcher intWatcher = new IntWatcher();
    intWatcher.feed(5);
    Model<Integer> model = intWatcher.getModel();
    assertStable(5, model);
  }

  private <A> void assertStable(A value, Model<A> model) {
    assertTrue(!(model instanceof NoInput));
    assertTrue(model instanceof Stable);
    assertEquals("value", value, ((Stable<A>) model).getValue());
    assertEquals("prediction", value, model.predict().getValue());
  }

  public void testTwoAndThreeValues() {
    IntWatcher intWatcher = new IntWatcher();
    intWatcher.feed(5);
    intWatcher.feed(6);
    Model<Integer> model = intWatcher.getModel();
    assertMulti(model, 5, 6);
    intWatcher.feed(4);
    model = intWatcher.getModel();
    assertMulti(model, 4, 5, 6);
  }

  private <A> void assertMulti(Model<A> model, A... examples) {
    assertTrue(!(model instanceof Stable) || !((Stable) model).isStable());
    assertTrue(model instanceof Multi);
    assertCollection(model.getExamples(), examples);
    A prediction = model.predict().getValue();
    assertTrue("prediction (" + prediction + ")", model.getExamples().contains(prediction));
  }

  private static <A> void assertCollection(Collection<A> collection, A... items) {
    assertEquals("Collection size", items.length, collection.size());
    List<A> list = new ArrayList<A>(collection);
    for (int i = 0; i < items.length; i++)
      assertEquals("Item #" + i, items[i], list.get(i));
  }

  public void testStringWatcher() {
    StringWatcher stringWatcher = new StringWatcher();
    stringWatcher.feed("hello");
    assertStable("hello", stringWatcher.getModel());
    assertStable(5, stringWatcher.getLengthModel());
    stringWatcher.feed("world");
    assertStable(5, stringWatcher.getLengthModel());
  }

  public void testCharsWatcher() {
    StringWatcher stringWatcher = new StringWatcher();
    stringWatcher.feed("hello");
    stringWatcher.feed("world");
    CharsWatcher charsWatcher = stringWatcher.makeCharsWatcher();
    assertStable("l", charsWatcher.getCharWatcher(3).getModel());
    assertMulti(charsWatcher.getCharWatcher(0).getModel(), "h", "w");
  }

  public void testRunlengthWatcher() {
    RunlengthWatcher runlengthWatcher = new RunlengthWatcher(false);
    assertEquals("length prediction", null, runlengthWatcher.getLengthWatcher().predict());
    runlengthWatcher.feed(true);
    runlengthWatcher.feed(true);
    runlengthWatcher.feed(false);
    assertEquals("length prediction", new Integer(2), runlengthWatcher.getLengthWatcher().predict().getValue());

    assertEquals("prediction", null, runlengthWatcher.predict());
    runlengthWatcher.feed(true);
    assertEquals("prediction", new Boolean(true), runlengthWatcher.predict().getValue());
    runlengthWatcher.feed(true);
    assertEquals("prediction", new Boolean(false), runlengthWatcher.predict().getValue());
  }
}