/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m03;

import drjava.util.Logger;
import drjava.util.StdOutLogger;
import drjava.util.Tree;
import drjava.util.TreePersistable;

public class DiskWatcher implements TreePersistable {
  String rememberedName;
  Long rememberedSize;
  Logger logger;

  public DiskWatcher(Logger logger) {
    this.logger = logger;
  }

  public void setDisk(Disk disk) {
    String name = disk.getName();
    if (name == null) name = "";
    if (rememberedName == null) {
      logger.log("Remembering disk name: " + name);
      rememberedName = name;
    } else if (rememberedName.equals(name))
      logger.logConfirmation("Disk name: " + name);
    else
      logger.logSurprise("Disk name used to be: " + rememberedName + ", now is: " + name);

    long size = disk.getSize();
    if (rememberedSize == null) {
      logger.log("Remembering disk size: " + size);
      rememberedSize = size;
    } else if (rememberedSize == size)
      logger.logConfirmation("Disk size: " + size);
    else
      logger.logSurprise("Disk size used to be: " + rememberedSize + ", now is: " + size);
  }

  public static void main(String[] args) {
    DiskWatcher diskWatcher = new DiskWatcher(new StdOutLogger());
    diskWatcher.setDisk(new RealDisk("C:\\"));
    Tree tree = diskWatcher.toTree();

    diskWatcher = new DiskWatcher(new StdOutLogger());
    diskWatcher.fromTree(tree);
    diskWatcher.setDisk(new RealDisk("C:\\"));
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("rememberedName", rememberedName)
      .add("rememberedSize", rememberedSize);
  }

  public void fromTree(Tree tree) {
    rememberedName = tree.getString("rememberedName");
    rememberedSize = tree.getLong("rememberedSize");
  }

  public String getRememberedName() {
    return rememberedName;
  }

  public Long getRememberedSize() {
    return rememberedSize;
  }
}