/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m03;

import drjava.util.StdOutLogger;
import drjava.util.Tree;
import drjava.util.TreePersistable;
import prophecy.common.BeanInspector;
import prophecy.common.InspectionUtil;
import prophecy.common.lifeform.Lifeform;
import prophecy.m.m02.Model;
import prophecy.m.m02.NoInput;

import java.util.ArrayList;
import java.util.List;

public class DiskWatcher2 extends Lifeform implements TreePersistable, IDiskWatcher {
  boolean initialized;
  String rememberedName;
  Long rememberedSize;
  Model<Long> freeSpace = new NoInput<Long>();
  List<IDiskWatcher> friends = new ArrayList<IDiskWatcher>();
  private Disk disk;

  public DiskWatcher2() {
  }

  public void setDisk(Disk disk) {
    if (initialized) {
      logger.logSurprise("Already initialized");
      return;
    }

    initialized = true;
    this.disk = disk;

    String name = disk.getName();
    if (name == null) name = "";
    logger.log("Remembering disk name: " + name);
    rememberedName = name;

    long size = disk.getSize();
    logger.log("Remembering disk size: " + size);
    rememberedSize = size;
  }

  public void introduce(Lifeform object) {
    super.introduce(object);
    if (object instanceof IDiskWatcher)
      introduce((IDiskWatcher) object);
  }

  public MatchResult introduce(IDiskWatcher diskWatcher) {
    if (friends.contains(diskWatcher))
      return null;
    else {
      MatchResult matchResult = matchStableFields(diskWatcher);
      if (matchResult.isMatch())
        friends.add(diskWatcher);
      return matchResult;
    }
  }

  private MatchResult matchStableFields(IDiskWatcher diskWatcher) {
    Checklist checklist = new Checklist();
    checklist.addCheck(new Comparison("disk name", rememberedName, diskWatcher.getRememberedName()));
    checklist.addCheck(new Comparison("disk size", rememberedSize, diskWatcher.getRememberedSize()));
    return checklist.fullMatch();
  }

  private Checklist matchVolatileFields(IDiskWatcher diskWatcher) {
    Checklist checklist = new Checklist();
    checklist.addCheck(new Comparison("free space", freeSpace, diskWatcher.getFreeSpaceModel()));
    return checklist;
  }

  public static void main(String[] args) {
    DiskWatcher2 diskWatcher1 = new DiskWatcher2();
    diskWatcher1.setLogger(new StdOutLogger());
    diskWatcher1.setDisk(new RealDisk("C:\\"));

    DiskWatcher2 diskWatcher2 = new DiskWatcher2();
    diskWatcher2.setLogger(new StdOutLogger());
    diskWatcher2.setDisk(new RealDisk("C:\\"));
    diskWatcher2.watch();

    //MatchResult bean = diskWatcher1.matchStableFields(diskWatcher2);
    Object bean = diskWatcher1.matchVolatileFields(diskWatcher2);
    InspectionUtil.inspect(BeanInspector.makeNode(bean));
  }

  public Tree toTree() {
    return new Tree(getClass())
      .add("rememberedName", rememberedName)
      .add("rememberedSize", rememberedSize);
  }

  public void fromTree(Tree tree) {
    rememberedName = tree.getString("rememberedName");
    rememberedSize = tree.getLong("rememberedSize");
  }

  public String getRememberedName() {
    return rememberedName;
  }

  public Long getRememberedSize() {
    return rememberedSize;
  }

  public boolean isInitialized() {
    return initialized;
  }

  public void watch() {
    if (disk != null) {
      freeSpace = freeSpace.plusExample(disk.getFreeSpace());
    }
  }

  public Model<Long> getFreeSpaceModel() {
    return freeSpace;
  }

  public String getStatus() {
    return isInitialized() ? "initialized" : "uninitialized";
  }

  public String toString() {
    return "DiskWatcher2 " + (rememberedName != null ? rememberedName : "");
  }
}