/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m03;

import drjava.util.*;

import static drjava.util.ObjectUtil.nice;

import prophecy.common.Prophecy;
import prophecy.common.*;
import prophecy.common.gui.Spacer;
import prophecy.common.lifeform.*;
import prophecy.m.m05.Screenshot;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;

public class HousePanel extends JPanel {
  private prophecy.common.lifeform.Environment environment;
  private PromiseBasedTableModel<Lifeform> tableModel;
  private JTable table;
  private JPanel detailPanel;
  private JButton btnInspect;
  private Lifeform detailLifeform;
  private boolean ignoreSelectionChanges;
  private JComponent detailComponent;

  public HousePanel(final Environment environment) {
    this.environment = environment;
    tableModel = new HouseTableModel();
    updateList();
    table = new JTable(tableModel);

    // table columns
    for (int i = 0; i < table.getColumnCount(); i++) {
      TableColumn col = table.getColumnModel().getColumn(i);
      if (HouseTableModel.OBJECT_ID.equals(col.getHeaderValue()))
        col.setMaxWidth(100);
      if (HouseTableModel.ICON.equals(col.getHeaderValue()))
        col.setMaxWidth(30);
      if (HouseTableModel.CPU_TIME.equals(col.getHeaderValue()))
        col.setMaxWidth(100);
    }

    TableColumn col = table.getColumnModel().getColumn(1); // icon
    col.setCellRenderer(new TableCellRenderer() {
      public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int column) {
        return new JLabel((Icon) value);
      }
    });

    // table popup menu
    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        int row = table.rowAtPoint(point);
        final Lifeform lifeform = tableModel.getItem(row);
        if (lifeform != null) {
          table.setRowSelectionInterval(row, row);
          LifeformUIUtil.objectMenuEntries(menu, lifeform);
        }
      }
    }.install(table);

    new SwingTimerHelper(new Runnable() {
      public void run() {
        update();

        // this is like the worst hack ever. it's for noticing that a
        // leader now returns a visualization instead of null
        if (detailLifeform == null && detailComponent instanceof LeaderView
          && ((LeaderView) detailComponent).getVisualization() == null) {
          setDetailLifeform(null);
        }
      }
    }, 1000).installOn(this);

    environment.addListener(new EnvironmentListener() {
      public void newInhabitant(final Environment environment, Lifeform object) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            update();
          }
        });
      }
    });

    JPanel buttonPanel = new JPanel(LetterLayout.leftAlignedRow(10));

    JButton btnRefresh = new JButton("Refresh", new ImageIcon("img/refresh16.gif"));
    btnRefresh.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        update();
        setDetailLifeform(detailLifeform);
      }
    });
    buttonPanel.add(btnRefresh);

    btnInspect = new JButton("Inspect", new ImageIcon("img/magnify16.gif"));
    btnInspect.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        final Lifeform item = tableModel.getItem(table.getSelectedRow());
        if (item != null)
          InspectionUtil.inspect(LifeformUIUtil.prepareForInspection(item));
      }
    });
    this.btnInspect.setEnabled(false);
    table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        selectionChanged();
      }
    });
    buttonPanel.add(btnInspect);

    JButton btnHappy = new JButton(Prophecy.happyIcon());
    btnHappy.setToolTipText("Applaud");
    btnHappy.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        happy();
      }
    });
    buttonPanel.add(btnHappy);

    buttonPanel.add(new Spacer(10, 0));

    final JComboBox cbClass = new JComboBox();
    cbClass.addItem("");
    cbClass.addItem(Introducer.class.getName());
    buttonPanel.add(cbClass);

    JButton btnNewLF = new JButton("Add");
    btnNewLF.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          String className = (String) cbClass.getSelectedItem();
          if (className.length() != 0) {
            Lifeform lf = (Lifeform) Class.forName(className).newInstance();
            environment.release(lf);
          }
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    });
    buttonPanel.add(btnNewLF);

    detailPanel = new JPanel(new BorderLayout());
    setDetailLifeform(null);

    ScalingSplitPane splitPane = new ScalingSplitPane(JSplitPane.VERTICAL_SPLIT);
    splitPane.setDividerLocationLater(0.5);
    splitPane.add(new JScrollPane(table));
    splitPane.add(detailPanel);

    setLayout(new LetterLayout("B", "T", "T").setSpacing(10).setBorder(10));
    add("T", splitPane);
    add("B", buttonPanel);
  }

  private void happy() {
    try {
      String jpegFileName = Prophecy.memoryDir() + "screenshot.jpg";
      JFrame frame = (JFrame) SwingUtilities.windowForComponent(this);
      new Screenshot().captureFrame(frame, jpegFileName);
      PraiseDialog dialog = new PraiseDialog(environment, jpegFileName, 0.5);
      dialog.asDialog(frame).setVisible(true);
    } catch (Throwable e) {
      Errors.add(e);
    }
  }

  private void selectionChanged() {
    if (ignoreSelectionChanges) return;
    final Lifeform item = tableModel.getItem(table.getSelectedRow());
    btnInspect.setEnabled(item != null);
    if (item != detailLifeform) {
      setDetailLifeform(item);
    }
  }

  private void setDetailLifeform(Lifeform item) {
    detailLifeform = item;
    detailComponent = item == null
      ? new LeaderView(environment)
      : new InspectionPanel(LifeformUIUtil.prepareForInspection(item)).getPanel();
    setDetailComponent(detailComponent);
  }

  private void setDetailComponent(JComponent component) {
    detailPanel.removeAll();
    if (component != null)
      detailPanel.add(component);
    detailPanel.revalidate();
  }

  private void update() {
    ignoreSelectionChanges = true;
    Collection<ObjectID> selection;
    try {
      selection = saveSelection();
      updateList();
    } finally {
      ignoreSelectionChanges = false;
    }
    restoreSelection(selection);
  }

  private void updateList() {
    List<Lifeform> list = new ArrayList<Lifeform>();
    list.add(this.environment);
    list.addAll(this.environment.getInhabitants());
    tableModel.setList(list);
  }

  private void restoreSelection(Collection<ObjectID> selection) {
    table.getSelectionModel().setValueIsAdjusting(true);
    List<Lifeform> objects = tableModel.getList();
    for (int i = 0; i < objects.size(); i++)
      if (selection.contains(objects.get(i).getObjectID()))
        table.getSelectionModel().addSelectionInterval(i, i);
    table.getSelectionModel().setValueIsAdjusting(false);
  }

  private Collection<ObjectID> saveSelection() {
    int[] rows = table.getSelectedRows();
    Collection<ObjectID> ids = new HashSet<ObjectID>();
    for (int row : rows)
      ids.add(tableModel.getList().get(row).getObjectID());
    return ids;
  }

  public static void main(String[] args) {
    Environment environment = new LiveEnvironment();
    Lifeform object = new Lifeform();
    environment.addInhabitant(object);
    SurfaceUtil.showAsMain("HousePanel", new HousePanel(environment));
  }

  public void showAsMainFrame(String title) {
    SurfaceUtil.showAsMain(title, this);
  }

  public void showFrame() {
    SurfaceUtil.show(nice(environment), this);
  }
}