/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m03;

import drjava.util.ObjectUtil;
import drjava.util.StringUtil;
import drjava.vats.FulfilledPromise;
import prophecy.common.PromiseBasedTableModel;
import prophecy.common.lifeform.Lifeform;
import prophecy.common.lifeform.LogEntry;
import prophecy.common.lifeform.MemoryLogger;

import java.util.List;

public class HouseTableModel extends PromiseBasedTableModel<Lifeform> {
  public static final String OBJECT_ID = "Object ID";
  public static final String ICON = "Icon";
  public static final String DESCRIPTION = "Description";
  public static final String STATUS = "Status";
  public static final String LAST_LOG_ENTRY = "Last log entry";
  public static final String CPU_TIME = "CPU Time";

  public HouseTableModel() {
    super(new String[]{OBJECT_ID, ICON, DESCRIPTION, STATUS, LAST_LOG_ENTRY, CPU_TIME});
  }

  protected int compareRows(Lifeform a, Lifeform b) {
    return 0;
  }

  protected Object getCell(Lifeform object, int row, int col) {
    String text = "";
    String colName = columnNames[col];
    if (colName.equals(OBJECT_ID)) {
      text = String.valueOf(object.getObjectID());
      if (object.isLeader())
        text = "[L] " + text;
    }
    if (colName.equals(ICON))
      return object.getIcon();
    if (colName.equals(DESCRIPTION))
      text = ObjectUtil.nice(object);
    if (colName.equals(STATUS))
      text = object.getStatus();
    if (colName.equals(LAST_LOG_ENTRY))
      if (object.getLogger() instanceof MemoryLogger) {
        List<LogEntry> log = ((MemoryLogger) object.getLogger()).getLog();
        if (!log.isEmpty())
          text = log.get(log.size()-1).getText();
      }
    if (colName.equals(CPU_TIME))
      text = StringUtil.formatDouble(object.getCpuTime(), 2) + " s";
    return new FulfilledPromise<String>(text);
  }
}