/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m03;

import static drjava.util.ObjectUtil.beautifyClassName;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;
import prophecy.common.BeanInspector;
import drjava.util.Errors;
import prophecy.common.InspectionNode;
import prophecy.common.InspectionUtil;
import prophecy.common.lifeform.Lifeform;
import prophecy.m.m05.BeanUtil;
import prophecy.m.m05.Biography;
import prophecy.m.m14.Function;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class LifeformUIUtil {
  public static void objectMenuEntries(JPopupMenu menu, final Object object) {
    final Lifeform lifeform = object instanceof Lifeform ? (Lifeform) object : null;
    if (lifeform != null) {
      JMenuItem miAbout = new JMenuItem("About");
      miAbout.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          Biography biography = new Biography(lifeform);
          biography.idle(); // write biography
          InspectionUtil.visualizeInFrame(biography, new Dimension(500, 600));
        }
      });
      menu.add(miAbout);

      JMenuItem miPraise = new JMenuItem("Applaud", Prophecy.happyIcon());
      miPraise.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          PraiseDialog dialog = new PraiseDialog(lifeform);
          dialog.asDialog(null).setVisible(true);
        }
      });
      menu.add(miPraise);
    }

    JMenuItem miInspect = new JMenuItem("Inspect instance of " + beautifyClassName(object.getClass().getName()));
    miInspect.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        InspectionUtil.inspect(prepareForInspection(object));
      }
    });
    menu.add(miInspect);

    JMenuItem miCopy = new JMenuItem("Copy object");
    miCopy.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        ObjectTransferable transferable = new ObjectTransferable(object, nice(object));
        Toolkit.getDefaultToolkit().getSystemClipboard().setContents(transferable, null);
      }
    });
    menu.add(miCopy);

    if (lifeform != null) {
      java.util.List<String> functions = lifeform.getCoreFunctions();
      if (!functions.isEmpty()) {
        menu.addSeparator();
        for (String functionName : functions) {
          try {
            final Function function = BeanUtil.getFunction(object, functionName);
            if (function.getMethod().getParameterTypes().length == 0) {
              JMenuItem menuItem = new JMenuItem(functionName);
              menuItem.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                  function.invoke(object);
                }
              });
              menu.add(menuItem);
            }
          } catch (Throwable e) {
            Errors.add(e);
          }
        }
      }
    }
  }

  static InspectionNode prepareForInspection(Object item) {
    return BeanInspector.makeNode(item);
  }
}