/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m03;

import drjava.util.Errors;
import drjava.util.GUIUtil;
import prophecy.common.gui.CancelButton;
import prophecy.common.gui.Sheet;
import prophecy.common.comments.ShoutboxDefaults;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.ImageSurface;
import prophecy.common.image.RGBImage;
import prophecy.common.lifeform.Lifeform;
import prophecy.video.v09.MinPixelMixer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class PraiseDialog extends Sheet {
  private JTextField tfAuthor;
  private JTextField tfText;

  public PraiseDialog(Lifeform beneficiary, String jpegFileName, double zoom) {
    if (jpegFileName != null) try {
      RGBImage image = RGBImage.load(jpegFileName);
      BWImage scaledImage = ImageProcessing.resize(image.toBW(),
        (int) (image.getWidth()*zoom), (int) (image.getHeight()*zoom),
        new MinPixelMixer());
      ImageSurface imageSurface = new ImageSurface(scaledImage);
      //imageSurface.setZoom(zoom);
      addBlueHeading("Screenshot");
      JPanel imagePanel = new JPanel(new BorderLayout());
      imagePanel.add(imageSurface);
      imagePanel.setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createLineBorder(Color.black),
        BorderFactory.createEmptyBorder(10, 10, 10, 10)
      ));
      imagePanel.setBackground(Color.white);
      addComponent(imagePanel);
    } catch (Throwable e) {
      Errors.add(e);
    }

    init(beneficiary);
  }

  protected void init(final Lifeform beneficiary) {
    addBlueHeading("Feedback");

    addComponent(new JLabel("Feedback is sent to: " + beneficiary));

    tfAuthor = new JTextField(ShoutboxDefaults.getUserName());
    addComponent(GUIUtil.withLabel("Your name:", tfAuthor));

    tfText = new JTextField();
    JLabel lHappy = new JLabel(new ImageIcon("img/happy.gif"));
    addComponent(GUIUtil.withLabel("Comment (optional): ", GUIUtil.withLabel(lHappy, tfText)));

    final JButton btnSave = new JButton("Save feedback");
    btnSave.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        beneficiary.release(new Praise(beneficiary, tfAuthor.getText(), tfText.getText()));
        SwingUtilities.windowForComponent(btnSave).dispose();
      }
    });
    addPair(btnSave, new CancelButton());
  }

  public PraiseDialog(Lifeform beneficiary) {
    init(beneficiary);
  }

  public static void main(String[] args) {
    GUIUtil.showMainFrame(new PraiseDialog(new Lifeform()).asFrame());
  }
}