/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m04;

import drjava.util.MultiSet;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.lifeform.Lifeform;

public class FrequentWords extends Lifeform {
  private Slot<ChatLog> chatLogSlot = new Slot<ChatLog>(ChatLog.class);
  private ChatLog chatLog;
  private MultiSet<String> multiSet = new MultiSet<String>();

  public FrequentWords() {
    setStatus("Waiting for log");
  }

  public void introduce(Lifeform object) {
    super.introduce(object);
    chatLogSlot.introduce(object);
    if (chatLogSlot.getOccupant() != chatLog)
      setChatLog(chatLogSlot.getOccupant());
  }

  private void setChatLog(ChatLog chatLog) {
    log("Chat log: " + nice(chatLog));
    this.chatLog = chatLog;
    scan();
  }

  private void scan() {
    setStatus("Scanning");
    multiSet = new MultiSet<String>();
    for (Msg msg : chatLog.getChatLog()) {
      String[] words = splitSentence(msg.getText());
      for (String word : words) {
        multiSet.add(word);
      }
    }
    setStatus("Log scan done");
  }

  public static String[] splitSentence(String s) {
    return s.trim().split(" +");
  }

  public String toString() {
    String s = "FrequentWords (" + multiSet.getNumberOfUniqueElements() + ")";
    if (chatLog != null) s += " for " + chatLog;
    return s;
  }

  public MultiSet<String> getMultiSet() {
    return multiSet;
  }
}