/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m04;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class LogReader {
  public Iterator<Msg> readLog(String logfile, String encoding) throws FileNotFoundException, UnsupportedEncodingException {
    if (!new File(logfile).exists())
      return new ArrayList<Msg>().iterator();

    System.out.println("Loading log...");
    InputStreamReader isr = encoding != null
      ? new InputStreamReader(new FileInputStream(logfile), encoding)
      : new InputStreamReader(new FileInputStream(logfile));
    final BufferedReader reader = new BufferedReader(isr);
    return new Iterator<Msg>() {
      Msg line;
      int lines = 0;

      {
        nextLine();
      }

      private void nextLine() {
        try {
          String s;
          while ((s = reader.readLine()) != null) {
            int idx = s.indexOf(':');
            String nick = s.substring(0, Math.max(idx, 0));
            String text = s.substring(idx + 1);
            if (!ignoreNick(nick)) {
              //if (lines >= 5000) { line = null; return; } //XXX
              line = new Msg(nick, text);
              if (++lines % 50000 == 0) {
                System.out.println(lines + " lines read");
              }
              return;
            }
          }
          line = null;
        } catch (IOException e) {
          throw new RuntimeException(e);
        }
      }

      public boolean hasNext() {
        return line != null;
      }

      public Msg next() {
        if (line != null) {
          Msg result = line;
          nextLine();
          return result;
        }
        return null;
      }

      public void remove() {
      }

      public void finalize() {
        try {
          reader.close();
        } catch (IOException e) {
        }
      }
    };
  }

  public boolean ignoreNick(String nick) {
    return false;
  }

  public List<Msg> logAsList(String fileName, String encoding, int maxLines) throws FileNotFoundException, UnsupportedEncodingException {
    ArrayList<Msg> log = new ArrayList<Msg>();
    Iterator<Msg> it = readLog(fileName, encoding);
    while (log.size() < maxLines & it.hasNext())
      log.add(it.next());
    return log;
  }
}