/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m05;

import static drjava.util.ObjectUtil.nice;
import drjava.util.StringUtil;
import drjava.util.TextView;
import prophecy.common.lifeform.Lifeform;
import prophecy.common.lifeform.LogEntry;
import prophecy.m.m02.Typewriter;

import javax.swing.*;
import java.util.List;

public class Biography extends Lifeform {
  private Lifeform lifeform;
  private String html;
  private boolean written;
  private Typewriter typewriter;

  public Biography(Lifeform lifeform) {
    this.lifeform = lifeform;
  }

  public void idleImpl() {
    if (!written) {
      written = true;
      typewriter = new Typewriter();
      String name = lifeform.toString();
      typewriter.addHeading("About " + name);
      typewriter.addLine("Name: " + name);
      if (lifeform.getBirthdate() != null)
        typewriter.addLine("Date of birth: " + lifeform.getBirthdate());
      if (StringUtil.notEmpty(lifeform.getStatus()))
        typewriter.addLine("Status: " + lifeform.getStatus());
      definingProperties();

      typewriter.addHeading("Log");
      if (lifeform.getLog().isEmpty())
        typewriter.addSentence("Log is empty");
      else
        for (LogEntry logEntry : lifeform.getLog()) {
          typewriter.addLine(logEntry.getDateAndText());
        }

      html = typewriter.getHtml();
    }
  }

  private void definingProperties() {
    try {
      List<String> properties = lifeform.getDefiningProperties();
      for (String property : properties) {
        Object value = BeanUtil.getProperty(lifeform, property);
        typewriter.addLine(StringUtil.firstToUpper(property) + ": " + nice(value));
      }
    } catch (Throwable e) {
      typewriter.addLine("Defining properties not reported due to exception");
    }
  }

  public String getHtml() {
    return html;
  }

  public JComponent visualize() {
    if (html == null) return null;
    TextView textView = new TextView(html);
    textView.setInset(10);
    return textView;
  }

  public String toString() {
    String s = "About " + lifeform;
    /*if (StringUtil.notEmpty(html))
      s += ". " + html;*/
    return s;
  }
}