/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m05;

import bsh.Interpreter;
import drjava.util.StringUtil;
import prophecy.common.lifeform.Lifeform;

public class Calculator extends Lifeform implements ICalculator {
  private int totalCalculations;
  private String lastQuestion, lastAnswer;

  public void introduce(Lifeform lifeform) {
    super.introduce(lifeform);
    if (lifeform instanceof IQuestion) {
      IQuestion iQuestion = (IQuestion) lifeform;
      String question = iQuestion.getQuestion();
      String answer = answer(question);
      if (answer != null && answer.length() > 0) {
        iQuestion.answer(answer);
      } else
        log("Not answering: " + question);
    }
  }

  private String answer(String question) {
    logActivity("Got question, processing: " + question);
    String answer = produceAnswer(question);
    if (answer != null && answer.length() > 0) {
      logActivity("Calculation done, answering: " + answer);
      lastQuestion = question;
      lastAnswer = answer;
      ++totalCalculations;
      updateStatus();
    }
    return answer;
  }

  private void updateStatus() {
    String text = StringUtil.n(totalCalculations, "calculation") + " done.";
    if (lastQuestion != null)
      text += " Last was: " + lastQuestion + " => " + lastAnswer;
    setStatus(text);
  }

  private String produceAnswer(String question) {
    if (question.matches("[0-9+\\-*/. ()]+")) {
      try {
        return new Interpreter().eval(question).toString();
      } catch (Exception e) {
        throw new RuntimeException(e);
      }
    }
    return null;
  }

  public int getTotalCalculations() {
    return totalCalculations;
  }

  public String getLastQuestion() {
    return lastQuestion;
  }

  public String getLastAnswer() {
    return lastAnswer;
  }

  public String calculate(String question) {
    return answer(question);
  }
}