/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m06;

import drjava.util.GUIUtil;
import drjava.util.ObjectUtil;
import prophecy.common.gui.Sheet;
import prophecy.common.lifeform.Lifeform;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;

public class FileArchivist extends Lifeform implements IArchivist {
  private File file;
  private PrintStream outStream;

  public FileArchivist() {
    setStatus("No file set");
  }

  public void setFile(File file) {
    if (file == null) return;
    this.file = file;
    setStatus("File set: " + file.getPath());
    try {
      file.getParentFile().mkdirs();
      String encoding = null;
      FileOutputStream fos = new FileOutputStream(file, true);
      outStream = encoding != null ? new PrintStream(fos, true, encoding) : new PrintStream(fos, true);
    } catch (IOException e) {
      error(e);
    }
  }

  public void store(Object object) {
    try {
      String text = ObjectUtil.nice(object);

      if (outStream != null) {
        outStream.println(text);
        log("Stored: " + text);
      } else {
        error("No output file set; holding data in memory");
      }
    } catch (Throwable e) {
      error(e);
    }
  }

  public JComponent visualize() {
    Sheet sheet = new Sheet();
    final JTextField textField = new JTextField();
    sheet.addComponent(GUIUtil.withTitle("Archive this: ", textField));
    ActionListener actionListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        store(textField.getText());
      }
    };
    sheet.addButton("Go", actionListener);
    textField.addActionListener(actionListener);
    return sheet.getPanel();
  }

  public File getFile() {
    return file;
  }
}