/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m07;

import prophecy.common.Prophecy;
import prophecy.common.lifeform.Environment;
import prophecy.common.lifeform.Lifeform;
import prophecy.common.lifeform.LiveEnvironment;
import prophecy.m.m02.BaseWatcher;
import prophecy.m.m02.Model;
import prophecy.m.m02.Typewriter;
import prophecy.m.m03.HousePanel;

public class PlayAround extends Lifeform {
  private ColoredToy toy;
  private BaseWatcher<Color> colorWatcher;
  private Typewriter reportTypewriter;
  private Model<Color> initialModel;

  public PlayAround(ColoredToy toy) {
    this.toy = toy;
    colorWatcher = new BaseWatcher<Color>();
    reportTypewriter = new Typewriter();
    reportTypewriter.setTimestamps(true, System.currentTimeMillis());
    reportTypewriter.addSentence("Got a new toy!");
  }

  public void idleImpl() {
    colorWatcher.feed(toy.getColor());
    //setStatus(getColorModel().toString());
    Typewriter typewriter = new Typewriter();
    getColorModel().describeYourself(typewriter, "toy color");
    setStatus(typewriter.getOneLiner());

    if (initialModel == null && getColorModel().isMature()) {
      initialModel = getColorModel();
      initialModel.describeYourself(reportTypewriter, "toy color");
    }
  }

  public Model<Color> getColorModel() {
    return colorWatcher.getModel();
  }

  public String getReport() {
    return reportTypewriter.getHtml();
  }

  public static void main(String[] args) {
    Prophecy.init();
    Environment environment = new LiveEnvironment();
    Coin coin = new Coin();
    environment.addInhabitant(coin);
    environment.addInhabitant(new PlayAround(coin));
    new HousePanel(environment).showAsMainFrame("M07");
  }
}