/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m08;

import drjava.util.SortedList;
import drjava.util.StringUtil;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Lifeform;

import java.io.File;
import java.util.Comparator;
import java.util.List;

public class AutonumDirsManager extends Lifeform {
  private File dir;
  private SortedList<AutonumDir> directories = new SortedList<AutonumDir>(new Comparator<AutonumDir>() {
    public int compare(AutonumDir o1, AutonumDir o2) {
      return o1.getID()-o2.getID();
    }
  });

  public AutonumDirsManager(String dir) {
    this.dir = new File(dir);
    setActivity(new Init());
  }

  public AutonumDirsManager(File dir) {
    this(dir.getPath());
  }

  public AutonumDir createNew() {
    return null;
  }

  public class Init extends Activity {
    protected void runOnce() {
      if (dir.isDirectory())
        logConfirmation(dir + " exists");
      else {
        if (dir.mkdirs())
          logConfirmation(dir + " created");
        else {
          logSurprise("Could not create " + dir);
          return;
        }
      }

      logActivity("Reading " + dir);
      File[] files = dir.listFiles();
      for (File file : files) {
        if (file.isDirectory() && StringUtil.isInteger(file.getName())) {
          Integer id = Integer.valueOf(file.getName());
          directories.add(new AutonumDir(id, file));
        }
      }
      log(StringUtil.n(directories.size(), "entry") + " found");
    }
  }

  public File getDir() {
    return dir;
  }

  public List<AutonumDir> getDirectories() {
    return directories;
  }

  public AutonumDir getLastDirectory() {
    return directories.isEmpty() ? null : directories.get(directories.size()-1);
  }
}