/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m09;

import static drjava.util.ObjectUtil.nice;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Environment;
import prophecy.common.lifeform.LiveEnvironment;
import prophecy.m.m02.Model;
import prophecy.m.m02.Stable;
import prophecy.m.m02.Typewriter;
import prophecy.m.m03.HousePanel;

import javax.swing.*;
import java.util.ArrayList;
import java.util.List;

public class ListAnalyzer extends Worker implements Reporter {
  private List<Object> list;
  private Environment environment;
  private ClassAnalyzer classAnalyzer;
  private Model<String> classModel;

  public ListAnalyzer(List list) {
    // let's make a copy so we don't follow a moving target
    this.list = new ArrayList<Object>(list);
    setJobDescription("Analyze list");

    environment = new LiveEnvironment("List Analyzer House");
    environment.addInhabitant(this);
    environment.setLeader(this);
    classAnalyzer = new ClassAnalyzer(this.list);
    environment.addInhabitant(classAnalyzer);
    setActivity(new CollectingResults());
  }

  public static void analyze(List list) {
    new HousePanel(new ListAnalyzer(list).getEnvironment()).showFrame();
  }

  public Environment getEnvironment() {
    return environment;
  }

  public boolean wantsToLead() {
    return true;
  }

  public JComponent visualize() {
    return new JReport(this);
  }

  public Typewriter writeReport() {
    Typewriter typewriter = new Typewriter();
    typewriter.addHeading("Task");
    typewriter.addLine("Analyzing list: " + typewriter.cut(drjava.util.ObjectUtil.nice(list), 100));
    typewriter.addHeading("Results");
    typewriter.addLine("List length: " + list.size());
    if (classModel != null)
      typewriter.addLine("Element class: " + renderModel(classModel));
    return typewriter;
  }

  private String renderModel(Model<String> model) {
    if (model instanceof Stable) {
      Stable<String> stable = (Stable<String>) model;
      if (stable.isStable())
        return stable.getValue();
    }
    return nice(model);
  }

  private class CollectingResults extends Activity {
    protected void idleImpl() {
      if (classAnalyzer != null && classAnalyzer.getModel() != null) {
        classModel = classAnalyzer.getModel();
        setJobDone();
        done();
      }
    }
  }
}