/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m09;

import drjava.util.StringUtil;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Lifeform;
import prophecy.common.lifeform.ProphecyEnv;

public class ThreadList extends Lifeform {
  private int count;

  public ThreadList() {
    setActivity(new Repeat(2, new Scan()));
  }

  private void scan() {
    ThreadGroup root = findRootGroup();
    count = 0;

    // Visit each thread group
    visit(root, 0);

    setStatus(StringUtil.n(count, "thread") + " in VM");
  }

  // This method recursively visits all thread groups under `group'.
  private void visit(ThreadGroup group, int level) {
    // Get threads in `group'
    int numThreads = group.activeCount();
    Thread[] threads = new Thread[numThreads * 2];
    numThreads = group.enumerate(threads, false);
    System.out.println("ThreadGroup: " + group + " (" + numThreads + ")");

    // Enumerate each thread in `group'
    /*for (int i = 0; i < numThreads; i++) {
      Thread thread = threads[i];
    }*/
    count += numThreads;

    // Get thread subgroups of `group'
    int numGroups = group.activeGroupCount();
    ThreadGroup[] groups = new ThreadGroup[numGroups * 2];
    numGroups = group.enumerate(groups, false);

    // Recursively visit each subgroup
    for (int i = 0; i < numGroups; i++) {
      visit(groups[i], level + 1);
    }
  }

  private ThreadGroup findRootGroup() {
    ThreadGroup root = Thread.currentThread().getThreadGroup().getParent();
    while (root.getParent() != null) {
      root = root.getParent();
    }
    return root;
  }

  public static void main(String[] args) {
    ProphecyEnv env = new ProphecyEnv("ThreadList");
    env.addInhabitant(new ThreadList());
    env.showMainFrame();
  }

  public class Scan extends Activity {
    protected void idleImpl() throws Exception {
      scan();
    }
  }
}