/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m12;

import drjava.util.Result;
import static drjava.util.StringUtil.n;
import drjava.util.Tree;
import drjava.util.TreePersistence;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Lifeform;
import prophecy.common.FrozenObject;
import prophecy.common.Freezable;
import prophecy.m.m09.Repeat;

import java.util.ArrayList;
import java.util.List;

public class AutoFreezer extends Lifeform {
  private TreePersistence persistence;
  //private int objectsFound, objectsFrozen;

  public AutoFreezer(TreePersistence persistence) {
    this.persistence = persistence;
    setActivity(new Repeat(5, new Scanning()));
  }

  public List<Object> unfreezeAll() {
    List<Object> list = new ArrayList<Object>();
    Tree tree = persistence.load();
    if (tree != null)
      for (Tree child : tree.get("objects").arguments()) {
        Result<Object> result = new FrozenObject(child).unfreeze();
        if (result.get() != null)
          list.add(result.get());
      }
    return list;
  }

  public class Scanning extends Activity {
    protected void idleImpl() throws Exception {
      super.idleImpl();
      int objectsFound = 0, objectsFrozen = 0;
      Tree tree = new Tree("AutoFrozen");
      Tree objectsTree = new Tree();
      tree.add("objects", objectsTree);
      for (Lifeform peer : getPeers()) {
        ++objectsFound;
        if (peer instanceof Freezable) {
          try {
            FrozenObject frozenObject = ((Freezable) peer).freeze();
            if (frozenObject != null) {
              ++objectsFrozen;
              objectsTree.add(frozenObject.toTree());
            }
          } catch (Throwable e) {
            error(e);
          }
        }
      }

      System.out.println(tree);
      if (persistence != null)
        persistence.store(tree);

      setStatus(objectsFrozen + "/" + n(objectsFound, "object") + " frozen"
        + (persistence == null ? "" : " and persisted to " + persistence));
    }
  }
}