/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m12;

import prophecy.common.lifeform.Lifeform;
import prophecy.m.m05.Calculator;

public class CalcStatsMaintainer extends Lifeform {
  private Calculator calculator;
  private int totalCalculations;
  private CalculatorStats stats;

  public void introduce(Lifeform lifeform) {
    super.introduce(lifeform);
    if (lifeform instanceof Calculator)
      setCalculator((Calculator) lifeform);
    if (lifeform instanceof CalculatorStats)
      stats = (CalculatorStats) lifeform;
  }

  public void setCalculator(Calculator calculator) {
    if (calculator != this.calculator) {
      this.calculator = calculator;
      totalCalculations = calculator.getTotalCalculations();
    }
  }

  protected void idleImpl() {
    if (calculator == null || stats == null) return;

    int diff = calculator.getTotalCalculations()-totalCalculations;
    if (diff > 0) {
      log("Updated " + totalCalculations + " => " + (totalCalculations + diff));
      totalCalculations += diff;
      stats.getCalculationCounter().increment(diff);
    }
  }

  public String getStatus() {
    return calculator != null && stats != null ? "ready" : "waiting for connections";
  }
}