/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m12;

import drjava.util.FileTreePersistence;
import prophecy.common.lifeform.Introducer;
import prophecy.common.lifeform.LiveEnvironment;
import prophecy.m.m05.Calculator;

import java.util.List;

public class CalculatorEnv extends LiveEnvironment {
  private Calculator calculator;
  private CalculatorStats stats;

  public CalculatorEnv(String memoryDir) {
    AutoFreezer autoFreezer = new AutoFreezer(new FileTreePersistence(memoryDir + "/autofreeze.tree"));
    List<Object> unfrozen = autoFreezer.unfreezeAll();
    add(autoFreezer);
    makeStats(unfrozen);
    calculator = new Calculator();
    add(calculator);
    add(stats);
    add(new CalcStatsMaintainer());
    add(new Introducer());
  }

  private void makeStats(List<Object> unfrozen) {
    if (unfrozen.isEmpty())
      stats = new CalculatorStats();
    else {
      System.out.println("Got stats from disk");
      stats = (CalculatorStats) unfrozen.get(0);
    }
  }

  public Calculator getCalculator() {
    return calculator;
  }

  public CalculatorStats getStats() {
    return stats;
  }
}