/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m12;

import drjava.util.GUIUtil;
import static drjava.util.GUIUtil.withLabel;
import drjava.util.LetterLayout;
import drjava.util.StringUtil;
import drjava.util.SwingTimerHelper;
import prophecy.common.Prophecy;
import prophecy.common.gui.Sheet;
import prophecy.common.ProphecyGUIUtil;
import prophecy.m.m05.Calculator;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class CalculatorGUI extends Sheet {
  private JTextField tfQuestion;
  private JLabel lResult;
  private String lastQuestion;
  private JLabel lStats;
  private CalculatorStats stats;

  public CalculatorGUI(final Calculator calculator, final CalculatorStats stats) {
    super("M12: Calculator");
    this.stats = stats;
    setSize(400, 280);
    tfQuestion = new JTextField("1+1");
    tfQuestion.selectAll();
    addComponent(withLabel("Enter formula here:", tfQuestion));
    addBlueHeading("Result");
    lResult = makeResultLabel();
    addComponent(lResult);
    addSpacer();
    JPanel buttonPanel = new JPanel(new LetterLayout("  B  "));
    JButton btnThanks = new JButton("Thank you");
    btnThanks.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        stats.getThanksCounter().increment();
      }
    });
    buttonPanel.add("B", btnThanks);
    addComponent(buttonPanel);

    addSpacer();
    addBlueHeading("Stats");
    lStats = addLabel(" ");

    new SwingTimerHelper(new Runnable() {
      public void run() {
        lStats.setText(StringUtil.n(stats.getCalculationCounter().get(), "calculation")
          + " done, " + StringUtil.n(stats.getThanksCounter().get(), "thank you") + " received");
        String question = tfQuestion.getText();
        if (!question.equals(lastQuestion)) {
          lastQuestion = question;
          String result = null;
          try {
            result = calculator.calculate(question);
          } catch (Throwable e) {
            e.printStackTrace();
          }
          /*if (result != null)
            stats.getCalculationCounter().increment();*/
          lResult.setText("<html><center>"
            + (result == null ? "?" : question + " = " + result)
            + "</center></html>");
        }
      }
    }, 500).installOn(lResult);

    ProphecyGUIUtil.registerF10(panel, calculator);
  }

  private JLabel makeResultLabel() {
    JLabel label = new JLabel(" ", JLabel.CENTER);
    label.setFont(new Font("Courier New", Font.PLAIN, 24));
    return label;
  }

  public static void main(String[] args) {
    Prophecy.init();
    CalculatorEnv calculatorEnv = new CalculatorEnv(Prophecy.memoryDir() + "m12");
    CalculatorGUI gui = new CalculatorGUI(calculatorEnv.getCalculator(),
      calculatorEnv.getStats());
    GUIUtil.showMainFrame(gui.asFrame());
  }

  public CalculatorStats getStats() {
    return stats;
  }
}