/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m12;

import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.common.Prophecy;
import prophecy.common.lifeform.Lifeform;
import prophecy.common.FrozenObject;
import prophecy.common.Freezable;
import prophecy.m.m13.Unfreezer;

import javax.swing.*;

public class CalculatorStats extends Lifeform implements Freezable {
  private Counter calculationCounter = new Counter(), thanksCounter = new Counter();
  private boolean happy;

  public Counter getCalculationCounter() {
    return calculationCounter;
  }

  public Counter getThanksCounter() {
    return thanksCounter;
  }

  public FrozenObject freeze() {
    Tree tree = toTree();
    return new FrozenObject(Unfreeze.class, tree.toString());
  }

  protected Tree toTree() {
    Tree tree = new Tree();
    tree.setInt("calculations", getCalculationCounter().get());
    tree.setInt("thanks", getThanksCounter().get());
    return tree;
  }

  public void setHappy(boolean happy) {
    if (this.happy != happy) {
      this.happy = happy;
      setIcon(null);
    }
  }

  public Icon makeIcon() {
    return happy ? Prophecy.happyIcon() : Prophecy.flatIcon();
  }

  public static class Unfreeze implements Unfreezer {
    public Object unfreeze(String data) {
      Tree tree = TreeUtil.textToTree(data);
      CalculatorStats stats = new CalculatorStats();
      stats.logActivity("unfreezing");
      stats.fromTree(tree);
      stats.setStatus("unfrozen");
      return stats;
    }
  }

  protected void fromTree(Tree tree) {
    getCalculationCounter().set(tree.getInt("calculations"));
    getThanksCounter().set(tree.getInt("thanks"));
  }

  protected void idleImpl() {
    super.idleImpl();
    setHappy(thanksCounter.get()*10 >= calculationCounter.get());
  }
}