/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m13;

import drjava.util.Tree;
import drjava.util.TreeUtil;
import drjava.util.XTestCase;
import prophecy.m.m12.CalculatorStats;
import prophecy.common.FrozenObject;

public class FrozenObjectTest extends XTestCase {
  private FrozenObject frozenString;

  protected void setUp() throws Exception {
    frozenString = new FrozenObject(StringUnfreezer.class.getName(), "it's cold out there");
  }

  public void testString() {
    checkUnfreeze("it's cold out there", frozenString);
  }

  public void testToFromTree() {
    Tree tree = TreeUtil.objectToTree(frozenString);
    frozenString = TreeUtil.treeToObject(tree);
    checkUnfreeze("it's cold out there", frozenString);
  }

  private void checkUnfreeze(Object expectedValue, FrozenObject frozenObject) {
    assertEquals(expectedValue, frozenObject.unfreeze().get());
  }

  public void testFreezeCalcStats() {
    CalculatorStats stats = new CalculatorStats();
    stats.getThanksCounter().set(5);
    FrozenObject frozenObject = stats.freeze();
    System.out.println(frozenObject);
    stats = (CalculatorStats) frozenObject.unfreeze().get();
    assertEquals(5, stats.getThanksCounter().get());
  }

  public void testEquality() {
    assertEquals(frozenString,
      new FrozenObject(StringUnfreezer.class.getName(), "it's cold out there"));
    assertTrue(!frozenString.equals(
      new FrozenObject(StringUnfreezer.class.getName(), "it's warm here")));
  }

  public void testNotes() {
    FrozenObject frozenObject = new FrozenObject(StringUnfreezer.class, "hello");
    frozenObject.addNote(new Tree("TestNote"));
    Tree tree = frozenObject.toTree();
    frozenObject = new FrozenObject(tree);
    assertListEquals(new Tree[] {new Tree("TestNote")}, frozenObject.getNotes());
  }
}