/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m13;

import drjava.util.*;
import prophecy.common.gui.Sheet;
import prophecy.common.gui.Form;
import prophecy.m.m12.CalculatorStats;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class SaveProperties extends Sheet implements Runnable {
  private Map<Property, JCheckBox> checkboxes;
  private Object object;
  private TextView tvResult;

  public SaveProperties(Object object) {
    super("Save properties of " + object);
    this.object = object;
    addComponent(new JLabel("Class: " + object.getClass().getSimpleName()));
    addBlueHeading("Properties");
    List<Property> properties = findProperties(object);
    Form form = new Form();
    checkboxes = new HashMap<Property, JCheckBox>();
    for (Property property : properties) {
      JCheckBox checkBox = new JCheckBox("persist", true);
      form.addRow(property.getName() + " (" + property.getType() + ")",
        checkBox);
      checkboxes.put(property, checkBox);
      checkBox.addChangeListener(new ChangeListener() {
        public void stateChanged(ChangeEvent e) {
          run();
        }
      });
    }
    addComponent(form);

    tvResult = new TextView();
    GUIUtil.setMinimumHeight(tvResult, 200);
    addComponent(tvResult);

    new SwingTimerHelper(this, 500).installOn(panel);
  }

  private List<Property> findProperties(Object object) {
    List<Property> properties = new ArrayList<Property>();
    if (object == null) return properties;
    Method[] methods = object.getClass().getDeclaredMethods();
    for (final Method method : methods) {
      String prefix = null;
      if (method.getName().startsWith("get")) prefix = "get";
      else if (method.getName().startsWith("is")) prefix = "is";
      if (prefix != null && method.getParameterTypes().length == 0
        /*&& !method.getName().equals("getClass")*/
        && !method.getName().equals("isEmpty")) {
        String propertyName = StringUtil.firstToLower(method.getName().substring(prefix.length()));
        properties.add(new Property(propertyName, method));
      }
    }
    return properties;
  }

  public static void main(String[] args) {
    GUIUtil.showMainFrame(new SaveProperties(new CalculatorStats()).asFrame());
  }

  public void run() {
    Tree tree = new Tree(BeanBuilder.class);
    tree.add("class", object.getClass().getName());
    for (Property property : checkboxes.keySet()) {
      if (checkboxes.get(property).isSelected())
        tree.add(property.getName(), String.valueOf(property.getValue(object)));
    }
    tvResult.setText(TextView.nlToBrWithIndent(tree.toMultiLineString()));
  }
}