/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m14;

import static drjava.util.ObjectUtil.nice;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Lifeform;

import java.util.ArrayList;
import java.util.List;

public class Tool extends Lifeform {
  private ToolStatus toolStatus = new Initializing();

  public Tool() {
    setActivity(new Init());
  }

  public boolean isReady() {
    return toolStatus instanceof Ready;
  }

  protected void setToolStatus(ToolStatus toolStatus) {
    this.toolStatus = toolStatus;
    setStatus(nice(toolStatus));
  }

  public ToolStatus getToolStatus() {
    return toolStatus;
  }

  public final List<Function> getFunctions() {
    if (!isReady())
      throw new RuntimeException("Tool not ready");
    List<Function> list = new ArrayList<Function>();
    collectFunctions(list);
    return list;
  }

  /** override me */
  protected void collectFunctions(List<Function> list) {
  }

  public class Init extends Activity {
    protected void runOnce() {
      try {
        init();
        setToolStatus(new Ready());
      } catch (Throwable e) {
        logSurprise(e);
        setToolStatus(new OutOfOrder(e));
      }
    }
  }

  /** override me */
  protected void init() throws Exception {
  }

  public void ready() {
    setToolStatus(new Ready());
  }
}