/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m15;

import drjava.util.FileUtil;
import drjava.util.GUIUtil;
import prophecy.common.gui.Sheet;
import prophecy.common.gui.SheetApp;
import drjava.util.Errors;
import prophecy.common.ProphecyGUIUtil;
import prophecy.common.lifeform.Environment;
import prophecy.common.lifeform.LiveEnvironment;
import static prophecy.m.m15.M15Util.*;

import javax.swing.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class M15FileAnalyzer extends SheetApp {
  private JButton btnName;
  private SourcePane sourcePane;
  private Associator associator = new Associator();
  private TripletBase tripletBase = new TripletBase();
  private String sourceText;
  private SourcePane analysisPane;
  private Environment environment = new LiveEnvironment();

  public M15FileAnalyzer() {
    super("M15: File Analyzer");

    ProphecyGUIUtil.registerF10(getPanel(), environment);
    environment.add(tripletBase);

    prefillDatabase();
    sourceText = "";
    try {
      sourceText = FileUtil.loadTextFile("src/prophecy/m/m13/StringUnfreezer.java");
    } catch (Throwable e) {
      Errors.add(e);
    }
    /*TextView textView = new TextView();
    textView.setTextFont(new Font("Courier New", Font.PLAIN, 12));
    textView.setText(TextView.nlToBrWithIndent(fileContents));
    addComponent(textView);*/
    sourcePane = new SourcePane();
    sourcePane.setText(sourceText);
    sourcePane.setMinimumSize(new Dimension(500, 250));
    addOutlines();
    addComponent(sourcePane);

    addLabel("Snippet info");
    analysisPane = new SourcePane();
    analysisPane.setMinimumSize(new Dimension(300, 150));
    addComponent(analysisPane);

    sourcePane.getChangeTrigger().addListener(new Runnable() {
      public void run() {
        analyze();
      }
    });

    btnName = new JButton("Describe selection...");
    btnName.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        describeSelection();
      }
    });
    addComponent(btnName);

    sourcePane.addCaretListener(new CaretListener() {
      public void caretUpdate(CaretEvent e) {
        update();
      }
    });
    update();
    showFrame();
  }

  private void analyze() {
    String analysis = "";
    SourcePane.Outline outline = sourcePane.getHighlightedOutline();
    if (outline != null) {
      String text = sourceText.substring(outline.getStart(), outline.getEnd());
      analysis = "\"" + text + "\"\n\n";
      String className = associator.get(text);
      if (className != null)
        analysis += "Class: " + className + "\n";
    }
    analysisPane.setText(analysis);
  }

  private void addOutlines() {
    WhitespaceSplitter splitter = new WhitespaceSplitter(sourceText);
    for (Range range : splitter.getRanges()) {
      sourcePane.addOutline(sourcePane.new Outline(sourceText, range.getStart(), range.getEnd()));
    }
  }

  private void prefillDatabase() {
    addTriplet(new Triplet(quote("package"), "isA", "keyword"));
    addTriplet(new Triplet(quote("public"), "isA", "keyword"));
  }

  private void addTriplet(Triplet triplet) {
    tripletBase.add(triplet);
    if (triplet.getPredicate().equals("isA") && isQuoted(triplet.getSubject()))
      associator.put(unquote(triplet.getSubject()), triplet.getObject());
  }

  private void describeSelection() {
    final Snippet snippet = makeSnippet();
    Sheet sheet = new Sheet("Describe selection");
    sheet.addLabel("Text:");
    JEditorPane editorPane = new SourcePane();
    editorPane.setText(snippet.getText());
    editorPane.setMinimumSize(new Dimension(300, 200));
    sheet.addComponent(editorPane);
    final JTextField tfClass = new JTextField();
    sheet.addComponent(GUIUtil.withLabel("This is a", tfClass));
    JButton btnOK = new JButton("OK");
    sheet.addComponent(btnOK);
    final JFrame frame = sheet.asFrame();
    tfClass.requestFocus();
    GUIUtil.showFrame(frame);
    btnOK.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        String className = tfClass.getText().trim();
        if (className.length() != 0) {
          addTriplet(new Triplet(quote(snippet.getText()), "isA", className));
          update();
        }
        frame.dispose();
      }
    });
  }

  private void update() {
    Snippet snippet = makeSnippet();
    btnName.setEnabled(snippet != null && isAcceptableSnippet(snippet));
  }

  private boolean isAcceptableSnippet(Snippet snippet) {
    return isAcceptableSeparator(snippet.getFrontSeparator())
      && isAcceptableSeparator(snippet.getBackSeparator());
  }

  private boolean isAcceptableSeparator(String separator) {
    return separator.length() == 0 || Character.isWhitespace(separator.charAt(0));
  }

  private Snippet makeSnippet() {
    if (sourcePane.getSelectedText() == null) return null;
    return new Snippet(sourcePane.getText(),
      sourcePane.getSelectionStart(), sourcePane.getSelectionEnd());
  }

  public static void main(String[] args) {
    new M15FileAnalyzer().asMainApplication();
  }
}