/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m15;

import drjava.util.Errors;
import prophecy.common.Trigger;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

public class SourcePane extends JEditorPane {
  private List<Outline> outlines = new ArrayList<Outline>();
  private Outline highlightedOutline;
  private Outline selectedOutline;
  private Trigger changeTrigger = new Trigger();

  public class Outline {
    private String sourceText;
    int start;
    int end;

    public Outline(String sourceText, int start, int end) {
      this.sourceText = sourceText;
      this.start = start;
      this.end = end;
    }

    public int getStart() {
      return start;
    }

    public int getEnd() {
      return end;
    }

    public Shape getShape() {
      try {
        Rectangle r1 = getUI().modelToView(SourcePane.this, getStart());
        Rectangle r2 = getUI().modelToView(SourcePane.this, getEnd());
        if (r1.y == r2.y)
          return new Rectangle(r1.x-1, r1.y, r2.x-r1.x+1, r1.height-2);
      } catch (BadLocationException e) {
        Errors.add(e);
      }
      return null;
    }

    public String getText() {
      return sourceText.substring(start, end);
    }
  }

  public SourcePane() {
    setFont(new Font("Courier New", Font.PLAIN, 12));
    setEditable(false);
    addMouseMotionListener(new MouseMotionListener() {
      public void mouseDragged(MouseEvent e) {
      }

      public void mouseMoved(MouseEvent e) {
        mouseMove(e.getPoint());
      }
    });

    addMouseListener(new MouseAdapter() {
      public void mouseEntered(MouseEvent e) {
        mouseMove(e.getPoint());
      }

      public void mouseExited(MouseEvent e) {
        setHighlightedOutline(null);
      }

      public void mousePressed(MouseEvent e) {
        setSelectedOutline(findOutlineAt(e.getPoint()));
      }
    });
  }

  private void mouseMove(Point point) {
    Outline outline = findOutlineAt(point);
    setHighlightedOutline(outline);
  }

  private Outline findOutlineAt(Point point) {
    for (Outline outline : outlines) {
      Shape shape = outline.getShape();
      if (shape != null && shape.contains(point))
        return outline;
    }
    return null;
  }

  public void addOutline(Outline outline) {
    outlines.add(outline);
    repaint();
  }

  public void paint(Graphics g) {
    super.paint(g);
    for (Outline outline : outlines) {
      paintOutline(g, outline);
    }
  }

  private void paintOutline(Graphics g, Outline outline) {
    Graphics2D g2 = (Graphics2D) g;
    Shape shape = outline.getShape();
    if (shape != null) {
      if (outline == selectedOutline) {
        g.setColor(Color.white);
        g.setXORMode(Color.black);
        g2.fill(shape);
        g.setPaintMode();
      }
      g.setColor(outline == highlightedOutline ? Color.blue : Color.LIGHT_GRAY);
      g2.draw(shape);
    }
  }

  public void setHighlightedOutline(Outline outline) {
    if (outline != this.highlightedOutline) {
      this.highlightedOutline = outline;
      zing();
    }
  }

  private void zing() {
    repaint();
    changeTrigger.trigger();
  }

  public void setSelectedOutline(Outline outline) {
    if (outline != this.selectedOutline) {
      this.selectedOutline = outline;
      zing();
    }
  }

  public Trigger getChangeTrigger() {
    return changeTrigger;
  }

  public List<Outline> getOutlines() {
    return outlines;
  }

  public Outline getHighlightedOutline() {
    return highlightedOutline;
  }

  public Outline getSelectedOutline() {
    return selectedOutline;
  }
}