/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m17;

import static drjava.util.StringUtil.n;
import prophecy.common.Trigger;
import prophecy.common.lifeform.*;
import prophecy.m.m02.Report;
import prophecy.m.m16.StringLF;

import javax.swing.*;
import java.util.Arrays;
import java.util.List;

public class LabAssistant extends Lifeform {
  private Class<? extends Lifeform> subjectClass;
  private UlfReport ulfReport;
  private Report report;
  private Trigger reportTrigger = new Trigger();

  public void test(Class<? extends Lifeform> subjectClass) {
    this.subjectClass = subjectClass;
    setActivity(new Testing());
  }

  public class Testing extends Activity {
    protected void runOnce() {
      test();
      report = writeReport();
      setActivity(null);
      setStatus("done, report written");
      reportTrigger.trigger();
    }
  }

  private void test() {
    ulfReport = new UlfReport();
    ulfReport.setSubjectClass(subjectClass.getName());
    Lifeform ulf;
    try {
      ulf = makeULF();
    } catch (Throwable e) {
      error(e);
      ulfReport.setException(e);
      return;
    }

    ulfReport.setDefaultConstructorOK(true);
    ulfReport.setLifeform(true);

    Environment testEnv = new TestEnvironment();
    Introducer introducer = new Introducer(false);
    testEnv.add(introducer);
    testEnv.add(ulf);
    int icycles = idle(testEnv);
    UlfState standardState = new UlfState(ulf, testEnv, Arrays.asList(introducer, ulf));
    registerProducts(ulf, standardState, icycles);

    testTrigger(new StringLF(""));
    testTrigger(new StringLF("hello"));
    testTrigger(new StringLF("hello\nworld"));
  }

  private int idle(Environment testEnv) {
    int lastCount = testEnv.getInhabitants().size();
    int icycles = 0;
    for (int i = 1; i <= 100; i++) {
      testEnv.idle();
      int count = testEnv.getInhabitants().size();
      if (count != lastCount) {
        icycles = i;
        lastCount = count;
      }
    }
    return icycles;
  }

  private void registerProducts(Lifeform ulf, UlfState state, int icycles) {
    if (!state.getProducts().isEmpty()) {
      for (Lifeform lifeform : state.getProducts()) {
        ulfReport.addProductClass(lifeform.getClass().getName());
      }
      Timing timing = new Timing(icycles, ulf.getCpuTime());
      ulfReport.setOutputTiming(timing);  // UNFINISHED: merge if existant
    }
  }

  private void testTrigger(Lifeform trigger) {
    Lifeform ulf;
    try {
      ulf = makeULF();
    } catch (Throwable e) {
      error(e);
      ulfReport.setException(e);
      return;
    }

    Environment testEnv = new TestEnvironment();
    Introducer introducer = new Introducer(false);
    testEnv.add(introducer);
    testEnv.add(ulf);
    testEnv.add(trigger);
    int icycles = idle(testEnv);
    UlfState state = new UlfState(ulf, testEnv, Arrays.asList(introducer, ulf, trigger));
    registerProducts(ulf, state, icycles);
    if (!state.getProducts().isEmpty())
      ulfReport.addTriggerClass(trigger.getClass().getName());
    else
      ulfReport.addNonTriggerClass(trigger.getClass().getName());
  }

  private Report writeReport() {
    return UlfReportRenderer.render(ulfReport);
  }

  private String describeProducts(UlfState state) {
    List<Lifeform> products = state.getProducts();
    if (products.isEmpty())
      return "nothing";
    return n(products.size(), " lifeform");
  }

  private Lifeform makeULF() throws Exception {
    return subjectClass.newInstance();
  }

  public JComponent visualize() {
    return report == null ? null : report.asComponent();
  }

  public boolean wantsToLead() {
    return true;
  }

  public Trigger getReportTrigger() {
    return reportTrigger;
  }
}