/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m17;

import drjava.util.ObjectUtil;
import drjava.util.StringUtil;
import static drjava.util.StringUtil.formatDouble;
import prophecy.m.m02.Report;
import prophecy.m.m02.Typewriter;

import java.util.ArrayList;
import java.util.List;
import java.util.TreeSet;

public class UlfReportRenderer {
  public static Report render(UlfReport ulfReport) {
    Typewriter typewriter = new Typewriter();
    typewriter.addHeading("ULF Report " + ObjectUtil.shortenClassName(ulfReport.getSubjectClass()));
    typewriter.addLine("Java class: " + ulfReport.getSubjectClass());
    typewriter.addLine("Can create instances: "
      + (ulfReport.isDefaultConstructorOK() ? "yes" : "no"));
    if (ulfReport.isDefaultConstructorOK()) {
      typewriter.addLine("Is lifeform: " + (ulfReport.isLifeform() ? "yes" : "no"));
      typewriter.addLine("Product classes: " + joinClasses(ulfReport.getProductClasses()));
      typewriter.addLine("Reacts to: " + joinClasses(ulfReport.getTriggerClasses()));
      typewriter.addLine("Does not react to: " + joinClasses(ulfReport.getNonTriggerClasses()));
      if (ulfReport.getOutputTiming() != null) {
        typewriter.addHeading("Timing");
        typewriter.addLine("icycles to product stability: " + ulfReport.getOutputTiming().getIdleCycles());
        typewriter.addLine("CPU time: " + formatDouble(ulfReport.getOutputTiming().getCpuTime(), 1) + " s");
      }
    }
    //typewriter.addLine("Typical status: " + state.getStatus());
    return typewriter.getReport();
  }

  private static String joinClasses(TreeSet<String> strings) {
    List<String> list = new ArrayList<String>();
    for (String s : strings) {
      list.add(ObjectUtil.shortenClassName(s));
    }
    return list.isEmpty() ? "-" : StringUtil.join(", ", list);
  }
}