/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m19;

import drjava.util.Lizt;
import drjava.util.ObjectUtil;
import drjava.util.Updatable;
import prophecy.m.m14.Tool;

import java.io.File;
import java.util.Arrays;
import java.util.List;
import java.util.TreeSet;

public class DirMonitor extends Tool implements Updatable {
  protected File dir;
  private boolean changeDetected;

  /** includes "." for the directory itself */
  private TreeSet<String> files;

  public DirMonitor(String dir) {
    this(new File(dir));
  }

  public DirMonitor(File dir) {
    this.dir = dir;
  }

  protected void init() throws Exception {
    super.init();
    update();
  }

  public File getDir() {
    return dir;
  }

  public List<String> getDefiningProperties() {
    return Lizt.of("dir");
  }

  public void setChangeDetected(boolean changeDetected) {
    this.changeDetected = changeDetected;
  }

  public boolean getChangeDetected() {
    return changeDetected;
  }

  public void update() {
    TreeSet<String> files = new TreeSet<String>();
    if (dir.isDirectory()) {
      files.add(".");
      files.addAll(Arrays.asList(dir.list()));
    }
    if (this.files != null)
      if (!this.files.equals(files))
        setChangeDetected(true);
    setFiles(files);
  }

  private void setFiles(TreeSet<String> files) {
    this.files = files;
  }

  public TreeSet<String> getFileNames() {
    return files;
  }

  public String toString() {
    return "DirMonitor" + ObjectUtil.showSize(files) + " " + dir.getPath();
  }
}