/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m20;

import drjava.util.FileUtil;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Lifeform;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class SourcesAnalyzer extends Lifeform implements SourcesList {
  private List<PairMatcher> pairMatchers = new ArrayList<PairMatcher>();
  private List<SourceFile> sources = new ArrayList<SourceFile>();
  
  public SourcesAnalyzer() {
    pairMatchers.add(new HeadMatcher());
    setActivity(new Loading());
  }

  public class Loading extends Activity {
    protected void runOnce() throws Exception {
      JavaSrcDir srcDir = new JavaSrcDir(new File("src/prophecy/m/m19"));
      srcDir.idle();
      List<File> files = srcDir.getFiles();
      if (files == null) {
        error("No file list");
        return;
      }
      for (File file : files) {
        sources.add(new SourceFile(file.getName(), FileUtil.loadTextFile(file)));
      }
      setActivity(new Thinking());
    }
  }

  public List<SourceFile> getSources() {
    return sources;
  }

  public class Thinking extends Activity {
    protected void idleImpl() throws Exception {
      SourceFile source1 = getRandomizer().oneOf(sources);
      SourceFile source2 = getRandomizer().oneOf(sources);
      if (source1 == source2) return;
      PairMatcher matcher = getRandomizer().oneOf(pairMatchers);
      Assertion assertion = matcher.match(source1, source2);
      if (assertion != null) {
        releaseTheory(source1, source2, assertion);
      }
    }
  }

  public void releaseTheory(SourceFile source1, SourceFile source2, Assertion assertion) {
    TheoryLF theory = new TheoryLF(assertion);
    theory.addExample(source1);
    theory.addExample(source2);
    releaseTheory(theory);
  }

  public void releaseTheory(TheoryLF theory) {
    for (Lifeform peer : getPeers()) {
      if (peer instanceof TheoryLF
        && ((TheoryLF) peer).getAssertion().equals(theory.getAssertion()))
        return;
    }
    release(theory);
  }
}