/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m20;

import static drjava.util.ObjectUtil.nice;
import prophecy.common.lifeform.Activity;
import prophecy.common.lifeform.Lifeform;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class TheoryLF extends Lifeform {
  private Assertion assertion;
  private List<SourceFile> examples = new ArrayList<SourceFile>();
  private SourcesList allSources;
  private boolean growthComplete;

  public TheoryLF(Assertion assertion) {
    this.assertion = assertion;
  }

  public void introduce(Lifeform lifeform) {
    super.introduce(lifeform);
    if (lifeform instanceof SourcesList && allSources == null)
      setAllSources((SourcesList) lifeform);
  }

  public void addExample(SourceFile source) {
    if (!examples.contains(source) && assertion.holdsFor(source.getText()))
      examples.add(source);
  }

  public Assertion getAssertion() {
    return assertion;
  }

  public List<SourceFile> getExamples() {
    return examples;
  }

  public String toString() {
    return "Theory [" + examples.size() + "]: " + nice(assertion);
  }

  public void setAllSources(SourcesList allSources) {
    this.allSources = allSources;
    //setActivity(new RandomGrowth());
    setActivity(new ExhaustingGrowth());
  }

  public class RandomGrowth extends Activity {
    protected void idleImpl() throws Exception {
      if (allSources != null) {
        SourceFile sourceFile = getRandomizer().oneOf(allSources.getSources());
        if (sourceFile != null)
          addExample(sourceFile);
      }
    }
  }

  public class ExhaustingGrowth extends Activity {
    private Iterator<SourceFile> iterator;

    public ExhaustingGrowth() {
      iterator = allSources.getSources().iterator();
    }

    protected void idleImpl() throws Exception {
      if (iterator.hasNext()) {
        SourceFile sourceFile = iterator.next();
        addExample(sourceFile);
      } else {
        growthComplete = true;
        done();
      }
    }
  }

  public boolean isGrowthComplete() {
    return growthComplete;
  }
}