/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m21;

import drjava.util.ObjectUtil;
import drjava.util.StringUtil;
import prophecy.m.m02.Typewriter;
import prophecy.m.m20.SourceFile;
import prophecy.m.m20.SourcesList;
import prophecy.m.m20.TheoryLF;

public class TheoryReport extends ReportLF {
  private SourcesList sources;
  private TheoryLF theory;
  private DataSheet dataSheet = new DataSheet();

  public TheoryReport(SourcesList sources, TheoryLF theory) {
    this.sources = sources;
    this.theory = theory;
    collectData();
    writeReport();
  }

  private void collectData() {
    int positives = countPositiveExamples();
    int negatives = sources.getSources().size() - positives;
    dataSheet.addItem("Claim", ObjectUtil.nice(theory.getAssertion()));
    dataSheet.addItem("Data set", /*ObjectUtil.nice(sources)*/
      StringUtil.n(sources.getSources().size(), " text file"));
    dataSheet.addItem("Positive examples", String.valueOf(positives));
    dataSheet.addItem("Negative examples", String.valueOf(negatives));
    String incidence = new IncidenceClassifier().getIncidence(positives, negatives);
    dataSheet.addItem("True?", incidence);
  }

  private int countPositiveExamples() {
    int count = 0;
    for (SourceFile source : sources.getSources()) {
      if (theory.getAssertion().holdsFor(source.getText()))
        ++count;
    }
    return count;
  }

  private void writeReport() {
    Typewriter typewriter = new Typewriter();
    typewriter.addHeading("Theory report");
    dataSheet.render(typewriter);
    setReport(typewriter);
  }

  public TheoryLF getTheory() {
    return theory;
  }

  public String toString() {
    return "Report about theory " + theory.getObjectID();
  }
}