/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m22;

import drjava.util.AnalogSet;
import drjava.util.Tree;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class StandardBotGUI extends JPanel {
  private Bot bot;
  private double botTime;
  private ChatPanel chatPanel;
  private int timestep = 2000;
  private boolean realtime = true;
  private String botName;

  public StandardBotGUI() {
  }

  public void init(final Bot bot, final String botName) {
    this.botName = botName;

    this.bot = bot;
    botTime = System.currentTimeMillis()*0.001;

    chatPanel = new ChatPanel() {
      public void userInput(String text) {
        super.userInput(text);
        bot.userSays(text);
        if (!realtime) {
          bot.timePasses(AnalogSet.<Tree> emptySet(), timestep);
          showBotOutput();
        }
        //chatPanel.logAppend("[" + bot.state() + "]");
      }
    };

    setLayout(new BorderLayout());
    add(BorderLayout.CENTER, chatPanel);

    if (realtime)
      startTimer();
  }

  private void startTimer() {
    new Timer(timestep, new ActionListener() {
      public void actionPerformed(ActionEvent actionEvent) {
        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            double time = System.currentTimeMillis()*0.001;
            bot.timePasses(AnalogSet.<Tree> emptySet(), time-botTime);
            botTime = time;
            showBotOutput();
          }
        });
      }
    }).start();
  }

  protected void showBotOutput() {
    for (String s : bot.getAndClearOutput()) {
      chatPanel.logAppend(botName, s);
    }
  }

  public JFrame makeFrame() {
    JFrame frame = new JFrame(botName);
    frame.setSize(800, 500);
    frame.getContentPane().setLayout(new BorderLayout());
    frame.getContentPane().add(this);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    return frame;
  }

  public void setSouthernComponent(JComponent component) {
    add(BorderLayout.SOUTH, component);
    validate();
  }

  public Bot getBot() {
    return bot;
  }

  public int getTimestep() {
    return timestep;
  }

  public void setTimestep(int timestep) {
    this.timestep = timestep;
  }

  public boolean isRealtime() {
    return realtime;
  }

  public void setRealtime(boolean realtime) {
    this.realtime = realtime;
  }

  public void setChatFont(Font font) {
    chatPanel.setChatFont(font);
  }

  public ChatPanel getChatPanel() {
    return chatPanel;
  }

  public void focus() {
    chatPanel.focus();
  }
}