/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m23;

import drjava.util.Errors;
import drjava.util.LetterLayout;
import static drjava.util.ObjectUtil.nice;
import drjava.util.SwingTimerHelper;
import drjava.util.Updatable;
import prophecy.common.LogView;
import prophecy.common.ProphecyGUIUtil;

import javax.swing.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

public class ObjectCollectorPanel extends JPanel implements Updatable {
  private JLabel lStatus;
  private ObjectCollector collector;

  public ObjectCollectorPanel(ObjectCollector collector) {
    init(collector);
  }

  protected ObjectCollectorPanel() {
  }

  protected void init(ObjectCollector collector) {
    this.collector = collector;
    setLayout(new LetterLayout("S", "L", "L").setBorder(10));
    lStatus = new JLabel(" ");
    add("S", lStatus);
    lStatus.setTransferHandler(new TransferHandler("text"));
    lStatus.addMouseListener(new MouseAdapter() {
      public void mousePressed(MouseEvent e) {
        JComponent c = (JComponent) e.getSource();
        TransferHandler th = c.getTransferHandler();
        th.exportAsDrag(c, e, TransferHandler.COPY);
      }
    });


    JComponent dropTarget = makeDropComponent();
    add("L", dropTarget);

    new DropTarget(dropTarget, new DropTargetAdapter() {
      public void drop(DropTargetDropEvent evt) {
        handleDrop(evt);
      }
    });

    new SwingTimerHelper(this, 1000).installOn(this);
    update();
    ProphecyGUIUtil.registerF10(this, collector);
  }

  protected JComponent makeDropComponent() {
    LogView logView = new LogView();
    logView.logText("Drop things here!");
    return logView;
  }

  private void handleDrop(DropTargetDropEvent evt) {
    try {
      Transferable tr = evt.getTransferable();
      DataFlavor[] flavors = tr.getTransferDataFlavors();
      for (DataFlavor flavor : flavors) {
        System.out.println("flavor: " + flavor);
        if (flavor.isFlavorTextType()
          || flavor.isMimeTypeEqual(DataFlavor.javaJVMLocalObjectMimeType)) {
          evt.acceptDrop(evt.getDropAction());
          Object object = tr.getTransferData(flavor);
          evt.dropComplete(true);
          collectObject(object);
          return;
        }
      }
      System.out.println("Unknown data flavors (" + flavors.length + ")");
    } catch (Throwable t) {
      Errors.add(t);
    }
    evt.rejectDrop();
  }

  private void collectObject(Object object) {
    try {
      System.out.println("Collecting: " + nice(object));
      getCollector().addObject(object);
    } catch (Throwable e) {
      Errors.add(e);
    }
  }

  public void update() {
    lStatus.setText("Status: " + collector.getStatus());
  }

  public ObjectCollector getCollector() {
    return collector;
  }
}