/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m24;

import drjava.util.GUIUtil;
import drjava.util.StringUtil;
import drjava.util.TextView;
import prophecy.common.gui.Sheet;
import drjava.util.Errors;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.List;

public class LogPanel {
  private Sheet sheet;
  private LogManager logManager;
  private List<String> paragraphs;
  private JTextArea taText;
  private JButton btnShout;

  public LogPanel(LogManager logManager) {
    this.logManager = logManager;
    sheet = new Sheet("Daily log");
    sheet.setSize(300, 700);

    loadAndRender();
  }

  private void loadAndRender() {
    new Thread() {
      public void run() {
        try {
          loadData();
          SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              render();
            }
          });
        } catch (Throwable e) {
          Errors.add(e);
          sheet.addLabel("[Could not connect to server]");
        }
      }
    }.start();
  }

  private void render() {
    final Sheet.Snapshot snapshot = sheet.snapshot();

    if (paragraphs.isEmpty()) {
      sheet.addLabel("Log is empty. Add entry:");
    } else {
      for (String comment : paragraphs) {
        sheet.addComponent(new JLabel("<html>"
          + TextView.nlToBr(StringUtil.escapeHtml(comment)) + "</html>"));
      }

      sheet.addBlueHeading("Shout");
    }

    taText = new JTextArea();
    JScrollPane scrollText = new JScrollPane(taText);
    GUIUtil.setMinimumHeight(scrollText, 200);
    sheet.addComponent(scrollText);

    btnShout = sheet.addButton("Save", new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          logManager.addParagraph(taText.getText());
          snapshot.restore();
          loadAndRender();
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    });

    DocumentListener documentListener = new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        update();
      }

      public void removeUpdate(DocumentEvent e) {
        update();
      }

      public void changedUpdate(DocumentEvent e) {
        update();
      }
    };
    taText.getDocument().addDocumentListener(documentListener);
    update();
  }

  private void update() {
    btnShout.setEnabled(taText.getText().length() > 0);
  }

  private void loadData() throws IOException {
    paragraphs = logManager.getParagraphs();
  }

  public Sheet getSheet() {
    return sheet;
  }
}