/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m25;

import static drjava.util.ObjectUtil.nice;

public class Step {
  private String description;
  private boolean done, success;
  private String comment;

  public Step(String description) {
    setDescription(description);
  }

  public Step() {
  }

  public String toString() {
    String s = description != null ? description : super.toString();
    if (done) s += success ? " (OK)" : " (FAILED)";
    return s;
  }

  public boolean isDone() {
    return done;
  }

  public void setDone(boolean done) {
    this.done = done;
  }

  public void success(String comment) {
    done = true;
    success = true;
    setComment(comment);
    //setStatus("OK - " + comment);
  }

  public void setComment(String comment) {
    this.comment = comment;
  }

  public void failure(String comment) {
    done = true;
    success = false;
    setComment(comment);
    //setStatus("FAILED - " + comment);
  }

  public String getDescription() {
    return description;
  }

  public void setDescription(String description) {
    this.description = description;
  }

  public void failure(Throwable throwable) {
    failure(nice(throwable));
  }

  public void success() {
    success(null);
  }

  public void run(Runnable runnable) {
    try {
      runnable.run();
      success();
    } catch (RuntimeException e) {
      failure(e);
      throw e;
    }
  }
}