/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m26;

import java.io.*;
import java.util.Enumeration;
import java.util.Set;
import java.util.HashSet;
import java.util.jar.*;
import java.util.zip.ZipEntry;

public class JarMaker {
  private JarOutputStream destJar;
  private String fileName;
  private Manifest manifest;
  private Set<String> entries;

  public JarMaker(String fileName, String mainClass) {
    this(fileName, makeManifest(mainClass));
  }

  public JarMaker(String fileName, Manifest manifest) {
    this.fileName = fileName;
    this.manifest = manifest;
  }

  public static void main(String[] args) throws IOException {
    String fileName = "temp/hello.jar";
    String mainClass = Hello.class.getName();
    new JarMaker(fileName, mainClass).run();
  }

  private static Manifest makeManifest(String mainClass) {
    Manifest manifest = new Manifest();
    Attributes mainAttributes = manifest.getMainAttributes();
    mainAttributes.put(Attributes.Name.MANIFEST_VERSION, "1.0");
    if (mainClass != null)
      mainAttributes.put(Attributes.Name.MAIN_CLASS, mainClass);
    return manifest;
  }

  public void run() throws IOException {
    begin();
    //jarToJar(destJar, "lib/gi-1.2.jar");
    dirToJar("out/production/Prophecy");
    dirToJar("out/production/Util");
    end();
  }

  public void begin() throws IOException {
    System.out.println("Writing " + fileName);
    File parentFile = new File(fileName).getParentFile();
    if (parentFile != null) parentFile.mkdirs();
    destJar = new JarOutputStream(new FileOutputStream(fileName), manifest);
    entries = new HashSet<String>();
  }

  public void end() throws IOException {
    destJar.close();
    System.out.println(fileName + " written (" + new File(fileName).length()/1024 + " K)");
  }

  public void dirToJar(String srcDir) throws IOException {
    dirToJar(srcDir, "");
  }

  public void dirToJar(String srcDir, String prefix) throws IOException {
    if (!new File(srcDir).exists()) return;
    System.out.println("Copying " + srcDir);
    dirToJarImpl(srcDir, prefix.length() == 0 ? "" :
      (prefix.endsWith("/") ? prefix : prefix + "/"));
  }

  private void dirToJarImpl(String srcDir, String prefix) throws IOException {
    for (File file : new File(srcDir).listFiles()) {
      if (file.isFile()) {
        String dstName = prefix + file.getName();
        //System.out.println("Copying " + dstName);
        if (addEntry(dstName))
          copy(new FileInputStream(file), destJar);
      }
      if (file.isDirectory()) {
        dirToJarImpl(file.getPath(), prefix + file.getName() + "/");
      }
    }
  }

  private boolean addEntry(String name) throws IOException {
    if (entries == null)
      throw new IOException("Call begin() first!");
    if (entries.contains(name)) return false;
    entries.add(name);
    destJar.putNextEntry(new ZipEntry(name));
    return true;
  }

  public void jarToJar(String srcJar) throws IOException {
    System.out.println("Copying " + srcJar);
    JarFile jar = new JarFile(srcJar);
    Enumeration<JarEntry> e = jar.entries();
    while (e.hasMoreElements()) {
      JarEntry entry = e.nextElement();
      if (entry.getName().equals("META-INF/MANIFEST.MF"))
        continue;
      //System.out.println("Copying " + entry.getName());
      copyJarEntry(jar, entry);
    }
  }

  public void copyJarEntry(JarFile jar, JarEntry entry) throws IOException {
    if (addEntry(entry.getName()))
      copy(jar.getInputStream(entry), destJar);
  }

  private static void copy(InputStream inputStream, OutputStream outputStream) throws IOException {
    byte[] buf = new byte[1024];
    int l;
    while ((l = inputStream.read(buf)) > 0) {
      outputStream.write(buf, 0, l);
    }
    inputStream.close();
  }
}