/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m26;

import drjava.util.FileUtil;
import drjava.util.StringUtil;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Mixin {
  private String packageName, className;
  private List<String> imports = new ArrayList<String>();
  private List<String> lines;

  public Mixin() {
  }

  public Mixin(String filePath) throws IOException {
    String src = FileUtil.loadTextFile(filePath);
    List<String> lines = StringUtil.toLines(src);
    int idx1 = -1, idx2 = -1;
    for (int i = 0; i < lines.size(); i++) {
      String line = lines.get(i);
      Matcher matcher = Pattern.compile("class\\s+(\\w+)\\s+extends\\s+Lifeform\\s*\\{")
        .matcher(line);
      if (matcher.find()) {
        idx1 = i;
        className = matcher.group(1);
      }

      if (line.trim().equals("}")) {
        idx2 = i;
      }

      if (line.startsWith("import "))
        imports.add(line);

      matcher = Pattern.compile("package (.+);").matcher(line);
      if (matcher.find())
        packageName = matcher.group(1);
    }

    if (idx1 >= 0 && idx2 >= 0) {
      System.out.println("class " + className + ": " + (idx2-idx1-1) + " lines");
      this.lines = lines.subList(idx1+1, idx2);
    }
  }

  public boolean isValid() {
    return lines.size() != 0;
  }

  public String getPackageName() {
    return packageName;
  }

  public void setPackageName(String packageName) {
    this.packageName = packageName;
  }

  public String getClassName() {
    return className;
  }

  public void setClassName(String className) {
    this.className = className;
  }

  public List<String> getLines() {
    return lines;
  }

  public void setLines(List<String> lines) {
    this.lines = lines;
  }

  public List<String> getImports() {
    return imports;
  }

  public void setImports(List<String> imports) {
    this.imports = imports;
  }
}