/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m33;

import drjava.util.*;
import static drjava.util.GUIUtil.setMinimumHeight;
import prophecy.common.gui.Sheet;
import drjava.util.Errors;
import prophecy.m.m26.ProphecyClassPath;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

public class Restart2 {
  private TextView tvStatus;
  private Sheet sheet;
  private int fatherPort;
  private JFrame frame;

  public Restart2() {
    sheet = new Sheet("M33: Restart v2");
    sheet.setSize(800, 400);
    sheet.addLabel("This application can start clones of itself in a new Virtual Machine.");
    sheet.addButton("Clone", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        startClone();
      }
    });

    sheet.addButton("Restart (clone and terminate if successful)", new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         restart();
       }
     });

    tvStatus = new TextView();
    setMinimumHeight(tvStatus, 200);
    sheet.addComponent(tvStatus);
  }

  private void startClone() {
    startClone(new Runnable() {
      public void run() {
        appendStatus("Received notice from son");
      }
    });
  }

  private void restart() {
    setAppState("restarting");
    startClone(new Runnable() {
      public void run() {
        appendStatus("Received notice from son, exiting");
        setAppState("terminating");
        MultiCoreUtil.sleep(1000);
        System.exit(0);
      }
    });
  }

  private void setAppState(String state) {
    frame.setTitle("M01: Restart [" + state + "]");
  }

  public void setStatus(final String status) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        tvStatus.setText(status, true);
      }
    });
  }

  public static void main(String[] args) {
    Errors.setPopup(true);
    Restart2 restart = new Restart2();
    for (String arg : args) {
      if (arg.startsWith("port[")) {
        restart.setFatherPort(Integer.parseInt(arg.substring("port[".length(), arg.length() - 1)));
      }
    }
    restart.run();
  }

  private void run() {
    frame = GUIUtil.showMainFrame(sheet.asFrame());
    Randomizer randomizer = new RealRandomizer();
    frame.setLocation(randomizer.random(500), randomizer.random(300));
    contactFather();
  }

  private void contactFather() {
    if (fatherPort != 0) {
      appendStatus("Contacting father at port " + fatherPort);
      new Son(fatherPort).contactFather();
    }
  }

  private void startClone(final Runnable onConnection) {
    new Thread() {
      public void run() {
        try {
          setStatus("Starting server");
          Father father = new Father();
          father.listen(onConnection);

          setStatus("Starting clone");
          VMStarter vmStarter = new VMStarter(ProphecyClassPath.get());
          //Log.info("Running: " + StringUtil.join(" ", commandLine));
          List<String> args = new ArrayList<String>();
          args.add("port[" + father.getPort() + "]");
          final VM vm = vmStarter.newVM(Restart2.class, args);

          new Thread() {
            public void run() {
              int exitValue = vm.waitForExit();
              appendStatus("child VM exited (" + exitValue + ")");
            }
          }.start();
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    }.start();
  }

  private void appendStatus(String text) {
    setStatus(getStatus() + "<br>" + text);
  }

  private String getStatus() {
    return tvStatus.getText();
  }

  public void setFatherPort(int fatherPort) {
    this.fatherPort = fatherPort;
  }
}