/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m33;

import drjava.util.StringUtil;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class VMStarter {
  private String javaExe;
  private List<String> classPath = new ArrayList<String>();
  private List<String> args = new ArrayList<String>();

  public VMStarter() {
    String javaHome = System.getProperty("java.home");
    System.out.println("Java Home: " + javaHome);
    javaExe = javaHome + "/bin/java.exe";
  }

  public VMStarter(ClassPath classPath) {
    this();
    appendToClassPath(classPath);
  }

  public VM newVM(Class mainClass) {
    return newVM(mainClass, new ArrayList<String>());
  }

  public VM newVM(Class mainClass, List<String> args) {
    try {
      List<String> cmd = new ArrayList<String>();
      cmd.add(javaExe);
      if (!classPath.isEmpty()) {
        cmd.add("-cp");
        cmd.add(StringUtil.join(File.pathSeparator, classPath));
      }
      cmd.add(mainClass.getName());
      cmd.addAll(this.args);
      cmd.addAll(args);
      System.out.println(cmdToString(cmd));
      return new VM(Runtime.getRuntime().exec(cmd.toArray(new String[cmd.size()])));
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  private String cmdToString(List<String> cmd) {
    List<String> list = new ArrayList<String>();
    for (String s : cmd) {
      if (s.indexOf(' ') >= 0 || s.indexOf('"') >= 0)
        s = "\"" + s + "\"";
      list.add(s);
    }
    return StringUtil.join(" ", list);
  }

  public void appendToClassPath(String path) {
    classPath.add(path);
  }

  public void addArg(String arg) {
    args.add(arg);
  }

  public void appendToClassPath(ClassPath path) {
    classPath.addAll(path.getPaths());
  }
}