/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m34;

import drjava.util.Errors;
import prophecy.common.Flag;
import prophecy.m.m25.Step;
import prophecy.m.m25.Steps;
import prophecy.m.m26.ProphecyClassPath;
import prophecy.m.m33.Father;
import prophecy.m.m33.VM;
import prophecy.m.m33.VMStarter;

import java.util.ArrayList;
import java.util.List;

public class Cloner {
  private Steps steps = new Steps();
  private Step stepFather = new Step("Set up father socket");
  private Step stepEnv = new Step("Construct VM environment");
  private Step stepRun = new Step("Start VM");
  private Step stepWait = new Step("Wait for contact");
  private Class mainClass;

  public Cloner(Class mainClass) {
    this.mainClass = mainClass;
    steps.add(stepFather);
    steps.add(stepEnv);
    steps.add(stepRun);
    steps.add(stepWait);
  }

  public Steps getSteps() {
    return steps;
  }

  public void start() {
    new Thread() {
      public void run() {
        Father father;
        Flag contactFlag = new Flag();
        try {
          father = new Father();
          father.listen(contactFlag);
          stepFather.success("Listening on port " + father.getPort());
        } catch (Throwable e) {
          Errors.add(e);
          stepFather.failure(e);
          return;
        }

        VMStarter vmStarter;
        try {
          vmStarter = new VMStarter(ProphecyClassPath.get());
          stepEnv.success();
        } catch (Throwable e) {
          Errors.add(e);
          stepEnv.failure(e);
          return;
        }

        VM vm;
        try {
          List<String> args = new ArrayList<String>();
          args.add("fatherport[" + father.getPort() + "]");
          vm = vmStarter.newVM(mainClass, args);
          stepRun.success();
        } catch (Throwable e) {
          Errors.add(e);
          stepRun.failure(e);
          return;
        }

        try {
          contactFlag.waitUntilUp();
          stepWait.success();
        } catch (Throwable e) {
          Errors.add(e);
          stepWait.failure(e);
          return;
        }
      }
    }.start();
  }

  public boolean isDone() {
    return steps.isDone();
  }
}