/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m34;

import drjava.util.*;
import prophecy.common.Prophecy;
import prophecy.common.gui.Sheet;
import drjava.util.Errors;
import prophecy.m.m33.Son;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class Restart3 {
  private Sheet sheet;
  private int fatherPort;
  private JFrame frame;

  public Restart3() {
    sheet = new Sheet("M34: Restart v3");
    sheet.setSize(300, 250);
    sheet.addLabel("This application can start clones of itself in a new Virtual Machine.");
    sheet.addSpacer();
    sheet.addButton("Clone", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        startClone(null);
      }
    });

    sheet.addButton("Restart (clone and terminate if successful)", new ActionListener() {
       public void actionPerformed(ActionEvent e) {
         restart();
       }
     });
  }

  private void setAppState(String state) {
    frame.setTitle("M34: Restart v3 [" + state + "]");
  }

  private void restart() {
    setAppState("restarting");
    startClone(new Runnable() {
      public void run() {
        setAppState("terminating");
        new Thread() {
          public void run() {
            MultiCoreUtil.sleep(2000);
            System.exit(0);
          }
        }.start();
      }
    });
  }

  public static void main(String[] args) {
    Prophecy.init();
    Errors.setPopup(true);
    Restart3 restart = new Restart3();
    for (String arg : args) {
      if (arg.startsWith("fatherport[")) {
        restart.setFatherPort(Integer.parseInt(arg.substring("fatherport[".length(), arg.length() - 1)));
      }
    }
    restart.run();
  }

  private void run() {
    frame = GUIUtil.showMainFrame(sheet.asFrame());
    Randomizer randomizer = new RealRandomizer();
    frame.setLocation(randomizer.random(500), randomizer.random(300));
    contactFather();
  }

  private void contactFather() {
    if (fatherPort != 0) {
      new Son(fatherPort).contactFather();
    }
  }

  private void startClone(final Runnable onConnection) {
    final Cloner cloner = new Cloner(getClass());
    ClonerSheet clonerSheet = new ClonerSheet(cloner);
    new SwingTimerHelper(new Runnable() {
      public void run() {
        if (cloner.isDone())
          onConnection.run();
      }
    }, 1000).installOn(clonerSheet.getPanel());
    GUIUtil.showFrame(clonerSheet.asFrame());
  }

  public void setFatherPort(int fatherPort) {
    this.fatherPort = fatherPort;
  }}