/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m35;

import prophecy.m.m15.Associator;
import prophecy.m.m15.SourcePane;

import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

public class LivingInput {
  private Outlines outlines;
  private Associator associator = new Associator();
  private Script script = new Script();
  private SourcePane analysisPane;
  private Config config;

  public LivingInput(final Outlines outlines, SourcePane analysisPane,
                     Config config) {
    this.outlines = outlines;
    this.analysisPane = analysisPane;
    this.config = config;

    outlines.getTextComponent().getDocument().addDocumentListener(new DocumentListener() {
      public void insertUpdate(DocumentEvent e) {
        textChanged();
      }

      public void removeUpdate(DocumentEvent e) {
        textChanged();
      }

      public void changedUpdate(DocumentEvent e) {
        textChanged();
      }
    });

    outlines.getTextComponent().addCaretListener(new CaretListener() {
      public void caretUpdate(CaretEvent e) {
        analyze();
      }
    });

    outlines.getChangeTrigger().addListener(new Runnable() {
      public void run() {
        analyze();
      }
    });
  }

  private void textChanged() {
    runScript();
  }

  public void runScript() {
    outlines.clear();
    for (int i = 0; i < script.getCommands().size(); i++) {
      Cmd cmd = script.getCommands().get(i);
      System.out.println("Running cmd " + (i+1) + "/" + script.getCommands().size()
        + ": " + cmd.getName());
      cmd.run();
    }
  }

  public Script getScript() {
    return script;
  }

  public Outlines getOutlines() {
    return outlines;
  }

  public String getText() {
    return outlines.getText();
  }

  public void analyze() {
    IOutlineAnalyzer analyzer = config.makeOutlineAnalyzer();
    analyzer.setAssociator(associator);
    String analysis = analyzer.analyze(outlines);
    analysisPane.setText(analysis);
  }

  public Associator getAssociator() {
    return associator;
  }
}