/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m35;

import prophecy.common.Trigger;

import javax.swing.text.JTextComponent;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.util.ArrayList;
import java.util.List;

public class Outlines implements Painter, IOutlines {
  private JTextComponent textComponent;
  private List<Outline> outlines = new ArrayList<Outline>();
  private Outline highlightedOutline;
  private Outline selectedOutline;
  private Trigger changeTrigger = new Trigger();

  public Outlines(JTextComponent textComponent, PaintHook paintHook) {
    this.textComponent = textComponent;
    textComponent.setFont(new Font("Courier New", Font.PLAIN, 12));
    textComponent.addMouseMotionListener(new MouseMotionListener() {
      public void mouseDragged(MouseEvent e) {
      }

      public void mouseMoved(MouseEvent e) {
        mouseMove(e.getPoint());
      }
    });

    textComponent.addMouseListener(new MouseAdapter() {
      public void mouseEntered(MouseEvent e) {
        mouseMove(e.getPoint());
      }

      public void mouseExited(MouseEvent e) {
        setHighlightedOutline(null);
      }

      public void mousePressed(MouseEvent e) {
        setSelectedOutline(findOutlineAt(e.getPoint()));
      }
    });

    paintHook.setPainter(this);
  }

  private void mouseMove(Point point) {
    Outline outline = findOutlineAt(point);
    setHighlightedOutline(outline);
  }

  private Outline findOutlineAt(Point point) {
    for (Outline outline : outlines) {
      Shape shape = outline.getShape();
      if (shape != null && shape.contains(point))
        return outline;
    }
    return null;
  }

  public void addOutline(Outline outline) {
    outlines.add(outline);
  }

  public void paint(Graphics g) {
    for (Outline outline : outlines) {
      paintOutline(g, outline);
    }
  }

  private void paintOutline(Graphics g, Outline outline) {
    Graphics2D g2 = (Graphics2D) g;
    Shape shape = outline.getShape();
    if (shape != null) {
      if (outline == selectedOutline) {
        g.setColor(Color.white);
        g.setXORMode(Color.black);
        g2.fill(shape);
        g.setPaintMode();
      }
      g.setColor(outline == highlightedOutline ? Color.blue : Color.LIGHT_GRAY);
      g2.draw(shape);
    }
  }

  public void setHighlightedOutline(Outline outline) {
    if (outline != this.highlightedOutline) {
      this.highlightedOutline = outline;
      zing();
    }
  }

  private void zing() {
    textComponent.repaint();
    changeTrigger.trigger();
  }

  public void setSelectedOutline(Outline outline) {
    if (outline != this.selectedOutline) {
      this.selectedOutline = outline;
      zing();
    }
  }

  public Trigger getChangeTrigger() {
    return changeTrigger;
  }

  public List<Outline> getOutlines() {
    return outlines;
  }

  public Outline getHighlightedOutline() {
    return highlightedOutline;
  }

  public Outline getSelectedOutline() {
    return selectedOutline;
  }

  public void clear() {
    outlines = new ArrayList<Outline>();
    setHighlightedOutline(null);
    setSelectedOutline(null);
    textComponent.repaint();
  }

  public String getText() {
    return textComponent.getText();
  }

  public JTextComponent getTextComponent() {
    return textComponent;
  }

  public Outline selectionAsOutline() {
    int start = textComponent.getSelectionStart();
    int end = textComponent.getSelectionEnd();
    if (end > start)
      return new Outline(textComponent, getText(), start, end);
    return null;
  }

  public Outline getActiveOutline() {
    Outline outline = getHighlightedOutline();
    if (outline == null)
      outline = selectionAsOutline();
    return outline;
  }
}