/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m37;

import drjava.util.Tree;
import drjava.util.TreeToObject;
import drjava.util.TreeUtil;
import prophecy.m.m13.Property;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

public class AutoUnpersist implements TreeToObject {
  public Object treeToObject(Tree tree) {
    try {
      Class theClass = Class.forName(tree.get("class").getName());
      Tree attributes = tree.get("attributes");
      Constructor[] constructors = theClass.getDeclaredConstructors();
      if (constructors.length == 1) {
        Constructor constructor = constructors[0];
        if (attributes == null)
          throw new RuntimeException("Attributes section not found: " + tree);

        if (constructor.getParameterTypes().length == 0) {
          // default constructor, set fields directly
          Object object = useDefaultConstructor(theClass, attributes);
          return object;
        } else if (attributes.size() == 1) {
          Object attrValue = TreeUtil.treeToObject(attributes.get(0).get(1));
          constructor.setAccessible(true);
          Object object = constructor.newInstance(attrValue);
          return object;
        } else {
          Property[] properties = new DetectConstructorArguments()
            .detectConstructorArguments(constructor);
          Object[] arguments = new Object[properties.length];
          Map<String, Object> attrValues = new HashMap<String, Object>();
          TreeUtil.treeToMap(attributes, attrValues);
          for (int i = 0; i < arguments.length; i++)
            arguments[i] = attrValues.get(properties[i].getName());
          Object object = constructor.newInstance(arguments);
          return object;
          //throw new RuntimeException("more than one constructor argument not supported");
        }
      } else {
        // more than one constructor

        Constructor defaultConstructor = theClass.getDeclaredConstructor();
        if (defaultConstructor != null) {
          return useDefaultConstructor(theClass, attributes);
        }

        throw new RuntimeException("Doesn't know which constructor to use (" + theClass.getClass().getName() + ")");
      }
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  private Object useDefaultConstructor(Class theClass, Tree attributes) throws InstantiationException, IllegalAccessException, InvocationTargetException, NoSuchFieldException, NoSuchMethodException {
    Constructor constructor = theClass.getDeclaredConstructor();
    constructor.setAccessible(true);
    Object object = constructor.newInstance();
    for (int i = 0; i < attributes.namelessChildrenCount(); i++) {
      String name = attributes.get(i).getString(0);
      Object value = attributes.get(i).getObject(1);
      Field field = findField(theClass, name);
      field.setAccessible(true);
      field.set(object, value);
    }
    return object;
  }

  private Field findField(Class theClass, String name) throws NoSuchFieldException {
    return theClass.getDeclaredField(name);
    /*Field[] fields = theClass.getDeclaredFields();
    for (Field field : fields) {
      if (field.getName().equals(name))
        return field;
    }
    return null;*/
  }
}