/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m37;

import prophecy.m.m13.Property;

import java.lang.reflect.Constructor;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DetectConstructorArguments {
  public Property[] detectConstructorArguments(Constructor constructor) {
    try {
      Class[] parameterTypes = constructor.getParameterTypes();
      Object[] argValues = new Object[parameterTypes.length];
      for (int i = 0; i < parameterTypes.length; i++) {
        Object value = makeDummyValue(parameterTypes[i], i);
        argValues[i] = value;
      }
      constructor.setAccessible(true);
      Object instance = constructor.newInstance(argValues);
      List<Property> properties = PropertyFinder.getProperties(instance);
      Map<Object, Property> propertiesByValue = new HashMap<Object, Property>();
      for (Property property : properties) {
        propertiesByValue.put(property.getValue(instance), property);
      }
      Property[] result = new Property[parameterTypes.length];
      for (int i = 0; i < result.length; i++) {
        result[i] = propertiesByValue.get(argValues[i]);
      }
      return result;
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  private Object makeDummyValue(Class type, int index) {
    if (type == String.class)
      return String.valueOf(index);
    if (type == Integer.TYPE)
      return index;
    if (type == Boolean.TYPE)
      return true; // let's hope it's only one boolean
    return null;
  }
}