/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m37;

import drjava.util.GUIUtil;
import drjava.util.ObjectUtil;
import static drjava.util.ObjectUtil.nice;
import drjava.util.PopupMenuHelper;
import drjava.util.SwingTimerHelper;
import prophecy.m.m03.ObjectTransferable;
import prophecy.t.t08.Bag;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;
import java.util.Vector;

/** a JList with these features:
 * -can update itself from a Bag in regular intervals
 * -shows a popup menu with a "delete" command
 * -is a drag source */
public class JBag extends JList {
  private Bag<Object> objectBag;

  public JBag(final Bag<Object> objectBag) {
    this.objectBag = objectBag;

    super.setTransferHandler(new TransferHandlerImpl());
    setDragEnabled(true);

    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        int idx = locationToIndex(point);
        final Object item = getItem(idx);
        if (item != null) {
          JMenuItem miDelete = new JMenuItem("Delete");
          miDelete.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
              delete(item);
            }
          });
          menu.add(miDelete);
        }
      }
    }.install(this);
  }

  private void delete(Object item) {
    objectBag.remove(item);
  }

  private Object getItem(int idx) {
    return idx >= 0 && idx < getModel().getSize() ? getModel().getElementAt(idx) : null;
  }

  public void setTransferHandler(TransferHandler newHandler) {
  }

  public static JComponent make(final Bag<Object> bag, int updateInterval) {
    final JBag jBag = new JBag(bag);
    jBag.updateEvery(updateInterval);

    JScrollPane scrollPane = new JScrollPane(jBag);
    GUIUtil.setMinimumWidth(scrollPane, 150);
    return scrollPane;
  }

  private void setData(List<Object> newData) {
    if (anyChange(getModel(), newData))
      setListData(new Vector<Object>(newData));
  }

  private static boolean anyChange(ListModel listModel, List<?> entries) {
    if (listModel == null) return true;
    if (listModel.getSize() != entries.size()) return true;
    for (int i = 0; i < entries.size(); i++)
      if (!ObjectUtil.equal(entries.get(i), listModel.getElementAt(i)))
        return true;
    return false;
  }

  private class TransferHandlerImpl extends TransferHandler {
    public int getSourceActions(JComponent c) {
      return COPY;
    }

    protected Transferable createTransferable(JComponent c) {
      final Object selectedItem = ((JBag) c).getSelectedValue();
      return new ObjectTransferable(selectedItem, extractText(selectedItem));
    }
  }

  public void updateEvery(int updateInterval) {
    new SwingTimerHelper(new Runnable() {
      public void run() {
        setData(objectBag.getSnapshot());
      }
    }, updateInterval).installOn(this);
  }

  private String extractText(Object object) {
    if (object instanceof Text)
      return ((Text) object).getText();
    return nice(object);
  }
}