/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m37;

import drjava.util.*;
import prophecy.common.Prophecy;
import drjava.util.Line;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.m.m35.XEditorPane;
import prophecy.t.t08.Bag;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;

/**
 * Outlines are disabled because they are not properly updated when
 * text is edited.
 */
public class M37Editor extends JFrame {
  protected XEditorPane editorPane;
  protected Bag<Object> objects = new Bag<Object>();
  protected ObjectBag objectBag = new ObjectBag(objects);
  //private Outlines outlines;

  public M37Editor() {
    setTitle("M37: Editor");
    setSize(800, 600);

    editorPane = new XEditorPane();
    //outlines = new Outlines(editorPane, editorPane.getPaintHook());
    JScrollPane scrollPane = new JScrollPane(editorPane);
    scrollPane.setMinimumSize(new Dimension(200, 250));

    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        String text = editorPane.getText();
        Selection selection = new Selection(text,
          editorPane.getSelectionStart(), editorPane.getSelectionEnd());
        if (!selection.isEmpty())
          addSelectionMenuItems(menu, selection);
      }
    }.install(editorPane);

    JComponent objectList = JBag.make(objects, 500);

    /*ScalingSplitPane splitPane = new ScalingSplitPane(0.7f);
    splitPane.setLeftComponent(GUIUtil.withInset(GUIUtil.withTitle(GUIUtil.centeredLabel("Text"), scrollPane)));
    splitPane.setRightComponent(GUIUtil.withInset(GUIUtil.withTitle(GUIUtil.centeredLabel("Objects"), objectList)));*/

    JPanel splitPane = new JPanel(new LetterLayout("EEO"));
    splitPane.add("E", GUIUtil.withTitle(GUIUtil.centeredLabel("Text"), scrollPane));
    GUIUtil.setMinimumWidth(objectList, 200);
    splitPane.add("O", GUIUtil.withTitle(GUIUtil.centeredLabel(objectListTitle()), objectList));

    Line toolBar = new Line().setBorder(10);
    JButton btnRememberText = new JButton("Remember text");
    btnRememberText.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rememberText();
      }
    });
    toolBar.add(btnRememberText);

    JButton btnLoadObjects = new JButton("Load objects...");
    btnLoadObjects.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        objectBag.loadObjects(M37Editor.this);
      }
    });
    toolBar.add(btnLoadObjects);

    JButton btnSaveObjects = new JButton("Save objects...");
    btnSaveObjects.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        objectBag.saveObjects(M37Editor.this);
      }
    });
    toolBar.add(btnSaveObjects);

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(BorderLayout.CENTER, GUIUtil.withInset(splitPane));
    getContentPane().add(BorderLayout.NORTH, toolBar);

    SwingUtil.focusOnOpen(editorPane);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {
        try {
          autoSave();
        } catch (Throwable e) {
          e.printStackTrace();
        }
      }
    });

    autoLoad();
  }

  protected String objectListTitle() {
    return "Objects";
  }

  protected void addSelectionMenuItems(JPopupMenu menu, final Selection selection) {
    JMenuItem miRemember = new JMenuItem("Remember");
    miRemember.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addObject(new Text(selection.getText()));
      }
    });
    menu.add(miRemember);
  }

  public void addObject(Object object) {
    objects.add(object);
  }

  public void autoLoad() {
    Tree tree = getAutoSavePersistence().load();
    if (tree != null)
      objectBag.objectsFromTree(tree);
  }

  private void autoSave() {
    Tree tree = objectBag.objectsToTree();
    getAutoSavePersistence().store(tree);
  }

  private FileTreePersistence getAutoSavePersistence() {
    return new FileTreePersistence(memoryDir() + "/autosaved_objects.tree");
  }

  protected String memoryDir() {
    return "memory/m37";
  }

  private void rememberText() {
    String text = StringUtil.stripCR(editorPane.getText());
    objects.add(new Text(text));
  }

  public static void main(String[] args) {
    Prophecy.init();
    M37Editor editor = new M37Editor();
    GUIUtil.showMainFrame(editor);
  }

  public static class MakeJar {
    public static void main(String[] args) throws IOException {
      ProphecyJarMaker.makeJar("standalone/m37editor.jar", M37Editor.class);
    }
  }
}