/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m37;

import drjava.util.FileTreePersistence;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.t.t08.Bag;

import javax.swing.*;
import java.io.File;
import java.util.List;

public class ObjectBag {
  private Bag<Object> objects;

  public ObjectBag(Bag<Object> objects) {
    this.objects = objects;
  }

  public void saveObjects(JFrame parentFrame) {
    Tree tree = objectsToTree_reportErrors(parentFrame);

    if (tree != null) {
      JFileChooser fileChooser = new JFileChooser("memory");
      if (fileChooser.showSaveDialog(parentFrame) == JFileChooser.APPROVE_OPTION) {
        File file = fileChooser.getSelectedFile();
        saveObjects(tree, file);
      }
    }
  }

  private void saveObjects(Tree tree, File file) {
    new FileTreePersistence(file).store(tree);
  }

  private Tree objectsToTree_reportErrors(JFrame parentFrame) {
    Tree tree = null;
    try {
      tree = objectsToTree();
    } catch (Throwable e) {
      e.printStackTrace();
      JOptionPane.showMessageDialog(parentFrame, e);
    }
    return tree;
  }

  public Tree objectsToTree() {
    Tree tree = new Tree();
    for (Object o : objects.getSnapshot()) {
      tree.add(new AutoPersister().objectToTree(o, new StandardTreeifier()));
    }
    return tree;
  }

  public void loadObjects(JFrame parentFrame) {
    JFileChooser fileChooser = new JFileChooser("memory");
    if (fileChooser.showOpenDialog(parentFrame) != JFileChooser.APPROVE_OPTION)
      return;

    try {
      File file = fileChooser.getSelectedFile();
      Tree tree = new FileTreePersistence(file).load();
      objectsFromTree(tree);
    } catch (Throwable e) {
      e.printStackTrace();
      JOptionPane.showMessageDialog(parentFrame, e);
    }
  }

  public void objectsFromTree(Tree tree) {
    objects.clear();
    for (Tree child : tree.arguments()) {
      objects.add(TreeUtil.treeToObject(child));
    }
  }

  public List<Object> getObjects() {
    return objects.getSnapshot();
  }
}