/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m38;

import drjava.util.*;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;
import drjava.util.Line;
import prophecy.m.m35.Outline;
import prophecy.m.m35.Outlines;
import prophecy.m.m35.XEditorPane;
import prophecy.m.m37.*;
import prophecy.t.t08.Bag;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class M38Editor extends JFrame {
  protected XEditorPane editorPane;
  protected Bag<Object> objects = new Bag<Object>();
  protected ObjectBag objectBag = new ObjectBag(objects);
  private Outlines outlines;
  private History history;

  public static void main(String[] args) {
    Prophecy.init();
    GUIUtil.showMainFrame(new M38Editor());
  }

  public M38Editor() {
    setTitle("M38: Editor");

    setSize(800, 600);

    editorPane = new XEditorPane();
    //outlines = new Outlines(editorPane, editorPane.getPaintHook());
    JScrollPane scrollPane = new JScrollPane(editorPane);
    scrollPane.setMinimumSize(new Dimension(200, 250));

    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        String text = editorPane.getText();
        Selection selection = new Selection(text,
          editorPane.getSelectionStart(), editorPane.getSelectionEnd());
        if (!selection.isEmpty())
          addSelectionMenuItems(menu, selection);
      }
    }.install(editorPane);

    JComponent objectList = JBag.make(objects, 500);

    /*ScalingSplitPane splitPane = new ScalingSplitPane(0.7f);
    splitPane.setLeftComponent(GUIUtil.withInset(GUIUtil.withTitle(GUIUtil.centeredLabel("Text"), scrollPane)));
    splitPane.setRightComponent(GUIUtil.withInset(GUIUtil.withTitle(GUIUtil.centeredLabel("Objects"), objectList)));*/

    JPanel splitPane = new JPanel(new LetterLayout("EEO"));
    splitPane.add("E", GUIUtil.withTitle(GUIUtil.centeredLabel("Text"), scrollPane));
    GUIUtil.setMinimumWidth(objectList, 200);
    splitPane.add("O", GUIUtil.withTitle(GUIUtil.centeredLabel(objectListTitle()), objectList));

    Line toolBar = new Line().setBorder(10);
    JButton btnRememberText = new JButton("Remember text");
    btnRememberText.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        rememberText();
      }
    });
    toolBar.add(btnRememberText);

    JButton btnLoadObjects = new JButton("Load objects...");
    btnLoadObjects.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        objectBag.loadObjects(M38Editor.this);
      }
    });
    toolBar.add(btnLoadObjects);

    JButton btnSaveObjects = new JButton("Save objects...");
    btnSaveObjects.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        objectBag.saveObjects(M38Editor.this);
      }
    });
    toolBar.add(btnSaveObjects);

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(BorderLayout.CENTER, GUIUtil.withInset(splitPane));
    getContentPane().add(BorderLayout.NORTH, toolBar);

    SwingUtil.focusOnOpen(editorPane);

    addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent evt) {
        try {
          autoSave();
        } catch (Throwable e) {
          e.printStackTrace();
        }
      }
    });

    outlines = new Outlines(editorPane, editorPane.getPaintHook());

    new EditorObjectsBridge(new RealEditor(editorPane), objects);

    //updateObjectsFromEditor();

    history = new History();
    /*history.addEventReceiver(new SoupListener() {
      public void ingredientAdded(Soup soup, Object ingredient) {
        System.out.println("+" + nice(ingredient));
      }
    });*/

    //autoLoad();
  }

  protected String memoryDir() {
    return "memory/m38";
  }

  protected String objectListTitle() {
    return "Visible objects";
  }

  protected void addSelectionMenuItems(JPopupMenu menu, final Selection selection) {
    JMenuItem miRemember = new JMenuItem("Remember");
    miRemember.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addObject(new Text(selection.getText()));
      }
    });
    menu.add(miRemember);

    JMenuItem miOutline = new JMenuItem("Outline");
    miOutline.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        addObject(new TextOutline(selection));
          /*new Outline(editorPane,
          selection.getText(), selection.getStart(), selection.getEnd()));*/
        editorPane.select(selection.getEnd(), selection.getEnd());
      }
    });
    menu.add(miOutline);
  }

  public void addObject(Object object) {
    objects.add(object);
    //addObjectToEditor(object);
  }

  private void addObjectToEditor(Object object) {
    if (object instanceof Text) {
      Text text = (Text) object;
      editorPane.setText(text.getText());
    }

    if (object instanceof TextOutline) {
      TextOutline textOutline = (TextOutline) object;
      Selection sel = textOutline.getSelection();
      if (sel.getFullText().equals(editorPane.getText())) {
        outlines.addOutline(new Outline(editorPane,
          sel.getFullText(), sel.getStart(), sel.getEnd()));
      }
    }
  }

  class Done {
    public Object action;

    Done(Object action) {
      this.action = action;
    }

    public String toString() {
      return "Done " + nice(action);
    }
  }

  class AutoLoad {}

  public void autoLoad() {
    AutoLoad action = new AutoLoad();
    //history.add(action);
    Tree tree = getAutoSavePersistence().load();
    if (tree != null)
      objectBag.objectsFromTree(tree);
    objectsChanged();
    history.addEvent(new Done(action));
  }

  class ObjectsChanged {}

  private void objectsChanged() {
    history.addEvent(new ObjectsChanged());
    updateEditorFromObjects();
  }

  private void updateEditorFromObjects() {
    for (Text text : BagUtil.getObjects(objects, Text.class)) {
      addObjectToEditor(text);
    }

    for (Object object : objectBag.getObjects()) {
      if (!(object instanceof Text))
        addObjectToEditor(object);
    }
  }

  private void autoSave() {
    Tree tree = objectBag.objectsToTree();
    getAutoSavePersistence().store(tree);
  }

  private FileTreePersistence getAutoSavePersistence() {
    return new FileTreePersistence(memoryDir() + "/autosaved_objects.tree");
  }

  private void rememberText() {
    String text = StringUtil.stripCR(editorPane.getText());
    objects.add(new Text(text));
  }
}