package prophecy.m.m40;

import drjava.util.Function;
import drjava.util.Lizt;
import static drjava.util.StringUtil.n;

import java.util.Timer;
import java.util.TimerTask;
import java.lang.management.ThreadMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.ThreadInfo;

public class LiveProfiler {
  private long delay = 5000;
  private int stackDepth = 4;
  private boolean runnableOnly = true;
  private Function<StackTraceElement, Boolean> methodFilter = new StandardMethodFilter();

  public static void main(String[] args) {
    LiveProfiler profiler = new LiveProfiler();
    profiler.start();
  }

  public void start() {
    Timer timer = new Timer("LiveProfiler");
    timer.scheduleAtFixedRate(new TimerTask() {
      public void run() {
        dumpThreads();
      }
    }, 0, delay);
  }

  public void dumpThreads() {
    ThreadMXBean threadMXBean = ManagementFactory.getThreadMXBean();
    ThreadInfo[] threadInfos = threadMXBean.dumpAllThreads(false, false);
    int threadsDumped = 0;
    for (ThreadInfo threadInfo : threadInfos) {
      if (runnableOnly && threadInfo.getThreadState() != Thread.State.RUNNABLE)
        continue;

      StackTraceElement[] stackTrace = threadInfo.getStackTrace();
      if (stackTrace != null && stackTrace.length != 0) {
        if (methodFilter != null && !methodFilter.get(stackTrace[0]))
          continue;

        System.out.println(threadInfo.getThreadState() + " "
          + threadInfo.getThreadId() + " ("
          + threadInfo.getThreadName() + ")");
        for (int i = 0; i < Math.min(stackTrace.length, stackDepth); i++)
          System.out.println("  " + stackTrace[i]);
        ++threadsDumped;
      }
    }
    if (threadsDumped != 0)
      System.out.println("LiveProfiler: " + n(threadsDumped, "thread") + " dumped");
  }

  public void setMethodFilter(Function<StackTraceElement, Boolean> filter) {
    this.methodFilter = filter;
  }

  public static class StandardMethodFilter implements Function<StackTraceElement, Boolean> {
    public Boolean get(StackTraceElement stackTraceElement) {
      String method = stackTraceElement.getClassName() + "." + stackTraceElement.getMethodName();
      return !Lizt.of(
        "java.net.PlainSocketImpl.socketListen",
        "java.net.PlainSocketImpl.socketAccept",
        "sun.management.ThreadImpl.dumpThreads0",
        "sun.awt.windows.WToolkit.eventLoop")
        .contains(method);
    }
  }
}
