package prophecy.marathon.ma01;

import drjava.superversion.tools.FileUtil;
import org.mortbay.jetty.Request;
import org.mortbay.jetty.handler.AbstractHandler;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.util.regex.Pattern;

class FileServer extends AbstractHandler {
  private String htmlBaseDir = "html";

  public void handle(String target, HttpServletRequest request, HttpServletResponse response, int dispatch)
    throws IOException, ServletException {
    System.out.println(request.getRemoteHost() + " - " + request.getRequestURI());

    new Serving(request, response).serve();
  }

  private class Serving {
    private HttpServletRequest request;
    private HttpServletResponse response;

    public Serving(HttpServletRequest request, HttpServletResponse response) {
      this.request = request;
      this.response = response;
    }

    public void serve() throws IOException {
      String path = request.getRequestURI();
      if (request.getRequestURI().equals("/"))
        path = "/index.html";

      try {
        serveFile(path);
      } catch (Served served) {
        // ok
      }

      /*
      response.setContentType("text/html");
      response.setStatus(HttpServletResponse.SC_OK);
      response.getWriter().println("<h1>Hello God</h1>");
      */
      ((Request) request).setHandled(true);
    }

    private void serveFile(CharSequence path) throws IOException, Served {
      if (!Pattern.compile("/[/a-zA-Z0-9_\\-. ]+").matcher(path).matches())
        serveError(404);
      String file = new File("html" + path).getAbsolutePath();
      String htmlBase = new File(htmlBaseDir).getAbsolutePath();
      if (!file.startsWith(htmlBase))
        serveError(404);

      if (!new File(file).isFile())
        serveError(404);

      response.setContentType(guessContentType(new File(file)));
      response.setStatus(HttpServletResponse.SC_OK);
      byte[] data = FileUtil.loadFile(new File(file));
      response.getOutputStream().write(data);
    }

    private void serveError(int errorCode) throws IOException, Served {
      response.setStatus(errorCode);
      response.setContentType("text/html");
      response.getWriter().println("<h1>" + errorCode + "</h1>");
      throw new Served();
    }
  }

  private String guessContentType(File file) {
    String name = file.getName();
    int idx = name.lastIndexOf(".");
    if (idx >= 0) {
      String ext = name.substring(idx);
      if (ext.equals(".css")) return "text/css";
      if (ext.equals(".txt")) return "text/plain";
      if (ext.equals(".jpg") || ext.equals(".jpeg")) return "image/jpeg";
      if (ext.equals(".gif")) return "image/gif";
      if (ext.equals(".png")) return "image/png";
    }
    return "text/html";
  }

  private class Served extends Exception {
  }
}
