package prophecy.mcos.mcos01;

import drjava.util.Errors;
import drjava.util.MultiCoreUtil;
import prophecy.common.Flag;

public class OneProcessEnv implements Env {
  Process process;
  private Flag terminated = new Flag();
  private double defaultIdleTime = 1.0;
  private boolean idleVerbosely;
  private Thread thread;

  public OneProcessEnv() {
  }

  public OneProcessEnv(Process process) {
    init(process);
  }

  public void init(Process process) {
    this.process = process;
    process.init(this);
  }

  public void run(Process process) {
    init(process);
    runUntilTerminated();
  }

  public void runUntilTerminated() {
    process.setRunning(true);
    while (!terminated.isUp())
      process.doStep();
    process.setRunning(false);
  }

  public void step() {
    if (!terminated.isUp())
      process.doStep();
  }

  public void print(String text) {
    System.out.println("<process> " + text);
  }

  public void idle() {
    idleFor(defaultIdleTime);
  }

  public void idleFor(double seconds) {
    if (idleVerbosely) System.out.println("<idle>");
    MultiCoreUtil.sleep((long) (seconds*1000));
  }

  public void requestTermination() {
    System.out.println("<terminating process>");
    terminated.raise();
  }

  public void runInSeparateThread() {
    thread = new Thread() {
      public void run() {
        try {
          runUntilTerminated();
        } catch (Throwable t) {
          Errors.report(t);
        }
      }
    };
    thread.start();
  }

  public void terminateSeparateThread() {
    requestTermination();
  }

  public double getDefaultIdleTime() {
    return defaultIdleTime;
  }

  public void setDefaultIdleTime(double defaultIdleTime) {
    this.defaultIdleTime = defaultIdleTime;
  }

  public void setIdleVerbosely(boolean idleVerbosely) {
    this.idleVerbosely = idleVerbosely;
  }

  public Thread getThread() {
    return thread;
  }
}
