package prophecy.mcos.mcos02;

import drjava.util.Lizt;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import drjava.util.Result;
import prophecy.common.CodeQueue;
import prophecy.common.FrozenObject;
import prophecy.m.m13.Unfreezer;
import prophecy.mcos.mcos01.Env;
import prophecy.mcos.mcos01.Process;
import prophecy.mcos.mcos01.ProcessImpl;

import java.util.ArrayList;
import java.util.List;

public class ProcessArray extends ProcessImpl {
  List<Process> processes = new ArrayList<Process>();
  List<SubprocessEnv> envs = new ArrayList<SubprocessEnv>();
  private boolean idleRequested;
  CodeQueue codeQueue = new CodeQueue();

  public Lizt<Process> getProcesses() {
    return new Lizt<Process>(processes);
  }

  public void start_threadSafe(final Process process) {
    codeQueue.add(new Runnable() {
      public void run() {
        add(process);
      }
    });
  }

  class SubprocessEnv implements Env {
    private int originalProcessIdx;

    SubprocessEnv(int processIdx) {
      this.originalProcessIdx = processIdx;
    }

    public void print(String text) {
      env.print("<p" + originalProcessIdx + "> " + text);
    }

    public void idle() {
      idleRequested = true;
    }

    public void idleFor(double seconds) {
      idleRequested = true;
    }

    public void requestTermination() {
      int idx = envs.indexOf(this);
      Process process = processes.get(idx);
      print("terminating subprocess " + idx + ": " + process);
      process.setRunning(false);
      processes.remove(idx);
      envs.remove(idx);
    }
  }

  public void add(Process process) {
    int idx = processes.size();
    processes.add(process);
    SubprocessEnv subenv = new SubprocessEnv(idx);
    envs.add(subenv);
    process.init(subenv);
    process.setRunning(running.get());
  }

  public void step() {
    codeQueue.runAll();

    idleRequested = false;
    for (Process process : new Lizt<Process>(processes)) {
      process.doStep();
    }
    if (idleRequested)
      env.idle();
  }

  @Override
  public void setRunning(boolean running) {
    super.setRunning(running);
    for (Process process : processes) {
      process.setRunning(running);
    }
  }

  // persistence methods follow

  private Tree toTree() {
    Tree list = new Tree();
    for (Process process : processes) {
      FrozenObject frozenProcess = process.freeze();
      if (frozenProcess == null) {
        //throw new RuntimeException("Can't freeze " + process);
        env.print("Note - can't freeze process, skipping: " + process);
      } else
        list.add(frozenProcess);
    }
    return new Tree().add("processes", list);
  }

  protected void fromTree(Tree tree) {
    Tree tProcesses = tree.get("processes");
    System.out.println("Loading processes: " + tProcesses);
    for (Tree tProcess : tProcesses.arguments()) {
      FrozenObject frozenObject = new FrozenObject(tProcess);
      Result<Object> unfrozen = frozenObject.unfreeze();
      add((Process) unfrozen.get());
    }
  }

  public FrozenObject freeze() {
    return new FrozenObject(Unfreeze.class, toTree());
  }

  public static class Unfreeze implements Unfreezer {
    public Object unfreeze(String data) {
      Tree tree = TreeUtil.textToTree(data);
      ProcessArray array = new ProcessArray();
      array.fromTree(tree);
      return array;
    }
  }
}
