package prophecy.mcos.mcos02;

import prophecy.common.Var;
import prophecy.mcos.mcos01.ProcessImpl;

import javax.swing.*;
import java.awt.*;

public class SingleFrameGui extends ProcessImpl implements GuiProcess {
  protected boolean guiVisible;
  protected GuiEnv guiEnv;
  protected Container frame;
  protected Var<Boolean> closing = new Var<Boolean>(false);
  private boolean terminateOnWindowClose = false;
  protected boolean guiVisibleOnStartup = false;

  public void step() {
    if (closing.get()) {
      closing.set(false);
      hideGui();
      if (terminateOnWindowClose)
        env.requestTermination();
    } else
      env.idle();
  }

  public boolean isTerminateOnWindowClose() {
    return terminateOnWindowClose;
  }

  public void setTerminateOnWindowClose(boolean terminateOnWindowClose) {
    this.terminateOnWindowClose = terminateOnWindowClose;
  }

  public void showGui(final GuiEnv guiEnv) {
    if (!guiVisible) {
      guiVisible = true;
      this.guiEnv = guiEnv;
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          makeFrame();
          guiEnv.onClosing(frame, new Runnable() {
            public void run() {
              closing.set(true);
            }
          });
          guiEnv.showFrame(frame);
        }
      });
    }
  }

  protected void makeFrame() {
    frame = guiEnv.makeFrame("SingleFrameGui dummy (someone didn't override makeFrame)");
    frame.setSize(300, 200);
    guiEnv.centerOnScreen(frame);
  }

  public void hideGui() {
    if (guiVisible) {
      guiVisible = false;
      SwingUtilities.invokeLater(new Runnable() {
        public void run() {
          if (frame != null)
            guiEnv.disposeFrame(frame);
          frame = null;
        }
      });
    }
  }

  public boolean isGuiVisible() {
    return guiVisible;
  }

  public boolean isGuiVisibleOnStartup() {
    return guiVisibleOnStartup;
  }
}