package prophecy.mcos.mcos03;

import prophecy.mcos.mcos02.GuiEnv;

import javax.swing.*;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import java.awt.*;

public class InternalGuiEnv implements GuiEnv {
  private JDesktopPane desktopPane;

  public InternalGuiEnv(JDesktopPane desktopPane) {
    this.desktopPane = desktopPane;
  }

  public Container makeFrame(String title) {
    JInternalFrame frame = new JInternalFrame(title);
    frame.setResizable(true);
    frame.setClosable(true);
    return frame;
  }

  public void showFrame(Container frame) {
    // make frame smaller if it extends beyond border

    Rectangle bounds = frame.getBounds();
    bounds.width = Math.min(bounds.width, desktopPane.getWidth()-bounds.x);
    bounds.height = Math.min(bounds.height, desktopPane.getHeight()-bounds.y);
    frame.setBounds(bounds);

    // add and make visible

    desktopPane.add(frame);
    frame.setVisible(true);
  }

  public void disposeFrame(Container frame) {
    ((JInternalFrame) frame).dispose();
  }

  public void centerOnScreen(Container frame) {
    frame.setLocation(
      Math.max(0, (desktopPane.getWidth()-frame.getWidth())/2),
      Math.max(0, (desktopPane.getHeight()-frame.getHeight())/2));
  }

  public void onClosing(Container frame, final Runnable runnable) {
    ((JInternalFrame) frame).addInternalFrameListener(new InternalFrameAdapter() {
      @Override
      public void internalFrameClosing(InternalFrameEvent e) {
        runnable.run();
      }
    });
  }

  public Container getContentPane(Container frame) {
    return ((JInternalFrame) frame).getContentPane();
  }

  public void setClosable(Container frame, boolean closable) {
    ((JInternalFrame) frame).setClosable(closable);
  }

  public void pack(Container frame) {
    ((JInternalFrame) frame).pack();
  }
}
