package prophecy.mcos.mcos03;

import drjava.util.*;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.mcos.mcos01.OneProcessEnv;
import prophecy.mcos.mcos02.GuiProcess;
import prophecy.mcos.mcos02.ProcessArray;
import prophecy.mcos.mcos02.ExampleGuiProcess;
import prophecy.common.FrozenObject;
import prophecy.common.ClassData;
import drjava.util.Errors;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

public class Main implements SystemActions {
  private OneProcessEnv env;
  private ProcessArray processes;
  private InternalGuiEnv guiEnv;
  private JFrame frame;
  private TaskBar taskBar;
  private int restartDelay = 500, shutdownDelay = 1000;

  Main() {
    frame = new JFrame("Mobile Code OS 0.1");
    frame.setSize(500, 400);

    showBootScreen();
    //init(false);

    GUIUtil.centerOnScreen(frame);
    GUIUtil.showMainFrame(frame);
  }

  private void showBootScreen() {
    init1();
    init2();
    taskBar.setMinimalMenu(true);
    boolean loadEnabled = hasSavedSystemState();
    startGuiProcess(new GPBootDialog(new BootDialogListener() {
      public void selectionMade(int selection) {
        try {
          softShutdown(); // shutdown boot screen

          if (selection == 0) {
            init1();
            startDummyProcesses();
            init2();
          } else {
            init1();
            int dataLength = loadSystemState();
            init2();
            //startGuiProcess(new GPMessageBox("System state loaded OK (" + dataLength + " bytes)"));
          }
        } catch (Throwable t) {
          Errors.report(t);
        }
      }
    }, loadEnabled));
  }

  private boolean hasSavedSystemState() {
    return getSavedSystemState() != null;
  }

  private int loadSystemState() {
    String data = getSavedSystemState();
    FrozenObject frozenObject = new FrozenObject(data);
    Result<Object> result = frozenObject.unfreeze();
    processes = (ProcessArray) result.get();
    return data.length();
  }

  private void init1() {
    processes = new ProcessArray();
  }

  private void init2() {
    env = new OneProcessEnv(processes);
    env.setDefaultIdleTime(0.1);
    env.runInSeparateThread();

    final JDesktopPane desktopPane = new JDesktopPane();
    installWindowPositionFixer(desktopPane);

    guiEnv = new InternalGuiEnv(desktopPane);
    taskBar = new TaskBar(guiEnv, processes, this);

    taskBar.setAdditionalSystemMenuItems(new MenuFiller() {
      public void fillMenu(JPopupMenu menu) {
        fillStartMenu(menu);
      }
    });

    // make process guis visible (after system state was loaded)
    for (prophecy.mcos.mcos01.Process p : processes.getProcesses()) {
      if (p instanceof GuiProcess) {
        GuiProcess guiProcess = (GuiProcess) p;
        if (guiProcess.isGuiVisibleOnStartup())
          guiProcess.showGui(guiEnv);
      }
    }

    frame.getContentPane().removeAll();
    frame.getContentPane().setLayout(
      new LetterLayout("D", "D", "T").setBorder(1).setSpacing(0));
    frame.getContentPane().add("D", desktopPane);
    frame.getContentPane().add("T", taskBar);
    frame.getContentPane().validate();
  }

  private void startDummyProcesses() {
    for (int i = 1; i <= 2; i++) {
      GuiProcess process = new ExampleGuiProcess();
      process.setName("app" + i);
      processes.add(process);
    }
  }

  private void fillStartMenu(JPopupMenu menu) {
    /*
    JMenuItem miStartApp = new JMenuItem("Start an application");
    miStartApp.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          startApp();
        } catch (Throwable t) {
          Errors.report(t);
        }
      }
    });

    // add the stuff backwards, on top of the menu
    menu.add(new JPopupMenu.Separator(), 0);
    menu.add(miStartApp, 0);
    */
  }

  private void startApp() {
    // TODO
  }

  public static void main(String[] args) {
    Errors.setPopup(true);
    new Main();
  }

  private static void installWindowPositionFixer(final JDesktopPane desktopPane) {
    new SwingTimerHelper(new Runnable() {
      public void run() {
        for (JInternalFrame frame : desktopPane.getAllFrames()) {
          int x = frame.getX(), y = frame.getY();
          x = Math.min(x, Math.max(desktopPane.getWidth() - 40, 0));
          y = Math.min(y, Math.max(desktopPane.getHeight() - 10, 0));
          frame.setLocation(x, y);
        }
      }
    }, 1000).installOn(desktopPane);
  }

  public void saveSystemState() {
    HibernaterProcess hibernater = new HibernaterProcess(processes);
    processes.start_threadSafe(hibernater);
    while (!hibernater.done())
      MultiCoreUtil.sleep(10);
    Result<FrozenObject> result = hibernater.getResult();
    if (result.isSuccess()) {
      System.out.println("<hibernate> " + result);
      String data = result.get().toTree().toString();
      int dataLength = data.length();
      saveSystemState(data);
      startGuiProcess(new GPMessageBox("Session saved! (" + dataLength + " bytes)"));
    } else {
      System.out.println("<hibernate failed>");
      result.getException().printStackTrace();
      startGuiProcess(new GPMessageBox("Session could not be saved"));
    }
  }

  private String getSavedSystemState() {
    ClassData classData = ClassData.get(this);
    return classData.getString("system state");
  }

  private void saveSystemState(String data) {
    ClassData classData = ClassData.get(this);
    classData.set("system state", data);
    classData.save();
  }

  private void startGuiProcess(GuiProcess guiProcess) {
    processes.start_threadSafe(guiProcess);
    guiProcess.showGui(guiEnv);
  }

  public void shutdown() {
    Container frame = guiEnv.makeFrame("MCOS is shutting down...");
    guiEnv.setClosable(frame, false);
    frame.setSize(400, 100);
    guiEnv.centerOnScreen(frame);
    guiEnv.showFrame(frame);
    Timer timer = new Timer(shutdownDelay, new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.exit(0);
      }
    });
    timer.setRepeats(false);
    timer.start();
  }

  public void restart() {
    softShutdown();
    Timer timer = new Timer(restartDelay, new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showBootScreen();
      }
    });
    timer.setRepeats(false);
    timer.start();
  }

  private void softShutdown() {
    env.terminateSeparateThread();
    frame.getContentPane().removeAll();
    frame.getContentPane().repaint();
  }

  public static class MakeJar {
    public static void main(String[] args) throws IOException {
      ProphecyJarMaker.makeJar("standalone/MobileCodeOS_01.jar", Main.class);
    }
  }
}
