package prophecy.mcos.mcos03;

import drjava.util.LetterLayout;
import drjava.util.SwingTimerHelper;
import drjava.util.Updatable;
import drjava.util.Lizt;
import drjava.util.Errors;
import prophecy.common.Var;
import prophecy.mcos.mcos02.GuiEnv;
import prophecy.mcos.mcos02.GuiProcess;
import prophecy.mcos.mcos02.ProcessArray;
import prophecy.mcos.mcos05.TaskInfo;
import prophecy.mcos.mcos05.PTaskReporter;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Collection;

import prophecy.mcos.mcos01.Process;

public class TaskBar extends JPanel implements Updatable {
  java.util.List<TaskButton> taskButtons = new ArrayList<TaskButton>();
  private GuiEnv guiEnv;
  private ProcessArray processes;
  private SystemActions systemActions;
  private Font normalFont, boldFont;
  private MenuFiller additionalSystemMenuItems;
  private boolean minimalMenu;
  private JButton btnSystem;
  private Var<Lizt<TaskInfo>> reportInput = new Var<Lizt<TaskInfo>>();

  class TaskButton extends JButton {
    GuiProcess guiProcess;

    TaskButton(String procName, final GuiProcess guiProcess) {
      super(procName);
      this.guiProcess = guiProcess;
      addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          guiProcess.showGui(guiEnv);
        }
      });
    }
  }

  public TaskBar(GuiEnv guiEnv, ProcessArray processes, SystemActions systemActions) {
    this.guiEnv = guiEnv;
    this.processes = processes;
    this.systemActions = systemActions;
    setLayout(LetterLayout.leftAlignedRow().setBorder(2));
    setBackground(new Color(0xdd, 0xdd, 0xdd));
    setOpaque(true);

    btnSystem = new JButton("MCOS");
    add(btnSystem);
    btnSystem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showSystemMenu();
      }
    });

    // start task reporter

    PTaskReporter taskReporter = new PTaskReporter();
    taskReporter.setProcessArray(processes);
    taskReporter.setReportTarget(reportInput);
    processes.start_threadSafe(taskReporter);

    normalFont = new Font("Dialog", Font.PLAIN, 12);
    boldFont = new Font("Dialog", Font.BOLD, 12);
    new SwingTimerHelper(this, 100).installOn(this);
  }

  public void showSystemMenu() {
    try {
      final JPopupMenu menu = new JPopupMenu();
      menu.setLightWeightPopupEnabled(false);
      fillSystemMenu(menu);
      menu.show(btnSystem, 0, 0);

      // make it pop up above the button
      Point loc = btnSystem.getLocationOnScreen();
      menu.setLocation(loc.x, loc.y-menu.getHeight()-2);
    } catch (Throwable t) {
      Errors.report(t);
    }
  }

  private void addTaskButtons(ProcessArray processes) {
    for (prophecy.mcos.mcos01.Process p : processes.getProcesses()) {
      addProcess(p);
    }
  }

  private void fillSystemMenu(JPopupMenu menu) {
    if (!minimalMenu) {
      JMenuItem miHibernate = new JMenuItem("Save session");
      miHibernate.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          try {
            systemActions.saveSystemState();
          } catch (Throwable t) {
            Errors.report(t);
          }
        }
      });
      menu.add(miHibernate);
    }

    JMenuItem miRestart = new JMenuItem("Restart");
    miRestart.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          systemActions.restart();
        } catch (Throwable t) {
          Errors.report(t);
        }
      }
    });
    menu.add(miRestart);

    JMenuItem miShutdown = new JMenuItem("Shutdown");
    miShutdown.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try {
          systemActions.shutdown();
        } catch (Throwable t) {
          Errors.report(t);
        }
      }
    });
    menu.add(miShutdown);

    if (additionalSystemMenuItems != null)
      additionalSystemMenuItems.fillMenu(menu);
  }

  public void update() {
    Lizt<TaskInfo> tasks = reportInput.get();

    Collection<Long> ids = new HashSet<Long>();
    if (tasks != null)
      for (TaskInfo task : tasks)
        ids.add(task.id);

    for (TaskButton button : new Lizt<TaskButton>(taskButtons)) {
      if (!ids.contains(button.guiProcess.getId())) {
        System.out.println("removing process " + button.guiProcess.getId());
        taskButtons.remove(button);
        remove(button);
        continue;
      }

      // update button

      ids.remove(button.guiProcess.getId());
      button.setEnabled(button.guiProcess.isRunning());
      boolean guiVisible = button.guiProcess.isGuiVisible();
      Font font = guiVisible ? boldFont : normalFont;
      if (font != button.getFont())
        button.setFont(font);
    }

    //System.out.println("reportInput: " + reportInput);
    if (tasks != null) {
      for (TaskInfo task : tasks) {
        if (ids.contains(task.id)) {
          Process p = task.getProcess();
          if (p != null)
            addProcess(p);
        }
      }
    }
  }

  void addProcess(prophecy.mcos.mcos01.Process p) {
    if (p instanceof GuiProcess) {
      GuiProcess guiProcess = (GuiProcess) p;
      TaskButton button = new TaskButton(guiProcess.getName(), guiProcess);
      taskButtons.add(button);
      add(button);
    }
  }

  public MenuFiller getAdditionalSystemMenuItems() {
    return additionalSystemMenuItems;
  }

  public void setAdditionalSystemMenuItems(MenuFiller additionalSystemMenuItems) {
    this.additionalSystemMenuItems = additionalSystemMenuItems;
  }

  public void setMinimalMenu(boolean minimalMenu) {
    this.minimalMenu = minimalMenu;
  }
}