/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.n.n01.subject3;

import drjava.util.Port;
import prophecy.t.t12.Event;
import prophecy.t.t12.EventCatcher;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

/** an add-on to Subject3 */
public class S3SequenceFinder implements Port<Event> {
  public void receive(Event event) {
    if (event instanceof Subject3.ExplorationCancelled) {
      Subject3.ExplorationCancelled explorationCancelled = (Subject3.ExplorationCancelled) event;
      findSequence(explorationCancelled.getModel(), explorationCancelled.getNode());
    }
  }

  private void findSequence(S3Model model, String node) {
    Map<Class,String> links = model.getWeb().getReverseLinkMap(node);
    //System.out.println("node: " + node + ", links: " + links.size());
    if (links.size() == 1) {
      Class direction = links.keySet().iterator().next();
      //System.out.println("direction: " + nice(direction));
      findSequence(model, node, direction);
    }
  }

  private void findSequence(final S3Model model, String node, final Class direction) {
    final List<String> sequence = new ArrayList<String>();
    sequence.add(node);
    while (node != null) {
      node = model.getWeb().followReverseLink(node, direction);
      if (node != null) {
        sequence.add(node);
        //System.out.println(sequence.size() + " - " + node);
      }
    }
    Collections.reverse(sequence);
    final SequenceAnalysisRequest analysisRequest = new SequenceAnalysisRequest(sequence);
    analysisRequest.getNewModelTrigger().addListener(new Runnable() {
      public void run() {
        if (analysisRequest.getModels().size() == 1) {
          final SequenceModel sequenceModel = analysisRequest.getModels().get(0);
          for (final String s : sequence) {
            addDynamicState(sequenceModel, s, model, direction);
          }
        }
      }
    });
    releaseEvent(analysisRequest);
  }

  /** recursively adds dynamic states - a class III hack */
  private void addDynamicState(final SequenceModel sequenceModel, final String s, final S3Model model, final Class direction) {
    model.dynamicStates.put(s, new DynamicState() {
      public Map<Class, String> getLinkMap() {
        String successor = sequenceModel.getSuccessor(s);
        addDynamicState(sequenceModel, successor, model, direction);
        return Collections.singletonMap(direction, successor);
      }
    });
  }

  public void installOn(Subject3 subject3) {
    EventCatcher.get(subject3).addReceiver(this);
  }

  private void releaseEvent(Event event) {
    EventCatcher.receive(this, event);
  }
}