/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.n.n01.subject3;

import drjava.util.Lizt;
import drjava.util.StringUtil;
import prophecy.common.PortUtil;
import prophecy.common.StdOutRecorder;
import prophecy.n.n01.setup.Model;
import prophecy.n.n01.setup.Operation;
import prophecy.n.n01.setup.N01Tests;
import prophecy.n.n01.setup.Workspace;
import prophecy.t.t12.Event;
import prophecy.t.t12.EventCatcher;

import java.util.Set;
import java.util.TreeSet;

public class Subject3 extends AbstractOperationModellingSubject {
  private S3Model model;
  private Set<String> statesSeen;
  private static int maxExplorationDepth = 5;

  public Model createModel() {
    statesSeen = new TreeSet<String>();
    model = new S3Model();
    Workspace workspace = workspaceFactory.make();
    model.emptyWorkspaceValue = workspace.getValue();

    explore(workspace, 0);
    return model;
  }

  private void explore(Workspace workspace, int depth) {
    String value = workspace.getValue();
    //System.out.println("explore " + value + " (" + depth + ")");
    if (statesSeen.contains(value))
      return;
    statesSeen.add(value);

    if (depth >= maxExplorationDepth) {
      releaseEvent(new ExplorationCancelled(value, depth));
      return;
    }

    for (Operation operation : allOperations) {
      Workspace workspace2 = workspace.clone();
      operation.run(workspace2);
      String newValue = workspace2.getValue();
      model.web.link(value, S3Model.representOperation(operation), newValue);
      explore(workspace2, depth+1);
    }
  }

  public class ExplorationCancelled extends Event {
    private String node;
    private int depth;

    public ExplorationCancelled(String node, int depth) {
      this.node = node;
      this.depth = depth;
    }

    public String getNode() {
      return node;
    }

    public int getDepth() {
      return depth;
    }

    public String toString() {
      return "ExplorationCancelled at " + StringUtil.quote(node) + " (depth=" + depth + ")";
    }

    public S3Model getModel() {
      return model;
    }
  }

  private void releaseEvent(Event event) {
    EventCatcher.receive(this, event);
  }

  public static void main(String[] args) {
    StdOutRecorder.recordAndFileUnder(Subject3.class);
    Subject3 subject3 = getInstanceWithSequenceAnalyzer();
    N01Tests.runTests(subject3);
  }

  public static Subject3 getInstanceWithSequenceAnalyzer() {
    Subject3 subject3 = new Subject3();
    S3SequenceFinder sequenceFinder = new S3SequenceFinder();
    sequenceFinder.installOn(subject3);
    EventCatcher.get(sequenceFinder).addReceiver(
      PortUtil.<SequenceAnalysisRequest, Event> cast(new SequenceAnalyzer(),
        SequenceAnalysisRequest.class));
    return subject3;
  }
}