package prophecy.n.n04;

import drjava.util.*;
import prophecy.common.Prophecy;
import prophecy.common.ClassData;
import drjava.util.Errors;
import prophecy.common.gui.JObject;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;

public class JScript extends JPanel {
  private ScriptLanguage language;
  private JEditorPane editorPane = new JEditorPane();
  private Line control;
  private LoadSaveAgent loadSaveAgent;
  private Bus<Object> resultBus = new Bus<Object>();

  public JScript(ScriptLanguage language) {
    this.language = language;
    setLayout(new LetterLayout("S", "S", "C"));
    setBorder(BorderFactory.createTitledBorder(language.getPublicName()));
    add("S", new JScrollPane(editorPane));

    loadSaveAgent = new LoadSaveAgent(ClassData.get(JScript.class).subTree("LoadSaveAgent"),
      language.getFileExtension());

    JButton btnRun = new JButton("Run");
    btnRun.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        run();
      }
    });

    JButton btnLoad = new JButton(Prophecy.getIcon("open16.gif"));
    btnLoad.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        load();
      }
    });

    JButton btnSave = new JButton(Prophecy.getIcon("save16.gif"));
    btnSave.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        save();
      }
    });

    control = new Line(btnLoad, btnSave, btnRun);
    add("C", control);

    control.setFiller(new JObject(language.selfTest()));
  }

  private void run() {
    run(editorPane.getText());
  }

  private void load() {
    try {
      File file = loadSaveAgent.showOpenDialog(this);
      if (file != null) {
        String text = FileUtil.loadTextFile(file);
        editorPane.setText(text);
      }
    } catch (Throwable e) {
      Errors.add(e);
    }
  }

  private void save() {
    try {
      File file = loadSaveAgent.showSaveDialog(this);
      if (file != null) {
        FileUtil.saveTextFile(file, editorPane.getText());
      }
    } catch (Throwable e) {
      Errors.add(e);
    }
  }

  private void run(String script) {
    Object result;
    try {
      result = runScript(script);
      resultBus.receive(result);
    } catch (Throwable e) {
      Errors.add(e);
      result = e;
    }
    control.setFiller(new JObject(result));
  }

  public Object runScript(String script) {
    return new Script(language, script).run();
  }

  public Sender<Object> getResultBus() {
    return resultBus;
  }
}