package prophecy.n.n04;

import drjava.util.*;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.common.ClassData;
import prophecy.common.PersistenceUtil;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

public class N04TextMapper extends JFrame {
  private EditableInOutTable table;
  private LoadSaveAgent loadSaveAgent;
  private UneditableInOutTable table2;

  public N04TextMapper() {
    super("N04 Text Mapper");
    setSize(600, 600);

    table = new EditableInOutTable();

    // table popup menu
    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        final int row = table.rowAtPoint(point);
        final InOutPair pair = table.getModel().getItem(row);
        table.setRowSelectionInterval(row, row);

        JMenuItem miInsertRow = new JMenuItem("Insert row");
        miInsertRow.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
            table.getModel().insertItem(row, new InOutPair("", ""));
          }
        });
        menu.add(miInsertRow);

        if (pair != null) {
          JMenuItem miRemoveRow = new JMenuItem("Remove row");
          miRemoveRow.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
              table.getModel().removeItem(row);
            }
          });
          menu.add(miRemoveRow);
        }
      }
    }.install(table);

    Line toolBar = new Line();

    JButton btnLoad = new JButton("Load pairs...");
    toolBar.add(btnLoad);
    btnLoad.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        loadPairs();
      }
    });

    JButton btnSave = new JButton("Save pairs...");
    toolBar.add(btnSave);
    btnSave.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        savePairs();
      }
    });

    JScript jScript = new JScript(ScriptLanguage.getBeanShell()) {
      public Object runScript(String script) {
        return new BshTextMapperRunner(script).getTextMapper();
      }
    };

    jScript.getResultBus().addReceiver(new Port<Object>() {
      public void receive(Object o) {
        if (o instanceof TextMapper)
          setTextMapper((TextMapper) o);
      }
    });

    table2 = new UneditableInOutTable();

    ScalingSplitPane splitPane2 = new ScalingSplitPane(ScalingSplitPane.VERTICAL_SPLIT, 0.5f);
    splitPane2.add(jScript);
    splitPane2.add(new JScrollPane(table2));

    ScalingSplitPane splitPane = new ScalingSplitPane(ScalingSplitPane.VERTICAL_SPLIT, 0.3f);
    splitPane.add(new JScrollPane(table));
    splitPane.add(splitPane2);

    getContentPane().setLayout(new LetterLayout("C", "T", "T").setBorder(10));
    getContentPane().add("T", splitPane);
    getContentPane().add("C", toolBar);
    loadSaveAgent = new LoadSaveAgent(ClassData.get(this).subTree("LoadSaveAgent"), "tree");
  }

  private void setTextMapper(TextMapper textMapper) {
    List<InOutPair> pairs = new ArrayList<InOutPair>();
    for (InOutPair pair : table.getModel().getList()) {
      String output;
      try {
        output = textMapper.get(pair.getInput());
      } catch (Throwable e) {
        output = "<" + nice(e) + ">";
      }
      pairs.add(new InOutPair(pair.getInput(), output));
    }
    table2.getModel().setList(pairs);
  }

  private void loadPairs() {
    File file = loadSaveAgent.showOpenDialog(this);
    if (file != null) {
      Tree tree = new FileTreePersistence(file).load();
      Collection collection = (Collection) TreeUtil.treeToObject(tree);
      table.getModel().setList(collection);
    }
  }

  private void savePairs() {
    File file = loadSaveAgent.showSaveDialog(this);
    if (file != null) {
      Tree tree = PersistenceUtil.getTreeifier().objectToTree(table.getModel().getList());
      new FileTreePersistence(file).store(tree);
    }
  }

  public static void main(String[] args) {
    Prophecy.init();
    GUIUtil.showMainFrame(new N04TextMapper());
  }

  public static class MakeJar {
    public static void main(String[] args) throws IOException {
      ProphecyJarMaker.makeJar(N04TextMapper.class);
    }
  }
}
