/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.omnivore.o.o01;

import drjava.util.Errors;
import drjava.util.LetterLayout;
import drjava.util.PopupMenuHelper;
import prophecy.common.Trigger;
import prophecy.common.gui.JObject;
import prophecy.common.gui.PopDownButton;
import prophecy.common.gui.SingleComponentPanel;
import prophecy.m.m03.ObjectTransferable;

import javax.swing.*;
import java.awt.*;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class JReplaceableObject extends JPanel {
  private SingleComponentPanel objectPanel = new SingleComponentPanel();
  private Object object;
  private Trigger newObjectTrigger = new Trigger();

  public JReplaceableObject() {
    setBorder(BorderFactory.createEtchedBorder());
    setLayout(new LetterLayout("OOB"));
    add("O", objectPanel);

    new PopupMenuHelper() {
      protected void fillMenu(JPopupMenu menu, Point point) {
        JReplaceableObject.this.fillMenu(menu);
      }
    }.install(this);

    add("B", new PopDownButton() {
      protected void fillMenu(JPopupMenu menu) {
        JReplaceableObject.this.fillMenu(menu);
      }
    });

    setObject(null);
  }

  private void fillMenu(JPopupMenu menu) {
    JMenuItem miCopy = new JMenuItem("Paste object");
    miCopy.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          Transferable transferable = Toolkit.getDefaultToolkit().getSystemClipboard().getContents(null);
          if (transferable == null) return;
          if (transferable.isDataFlavorSupported(ObjectTransferable.localObjectDataFlavor())) {
            Object object = transferable.getTransferData(ObjectTransferable.localObjectDataFlavor());
            if (object != null) {
              setObject(object);
              return;
            }
          }

          if (transferable.isDataFlavorSupported(DataFlavor.stringFlavor)) {
            Object object = transferable.getTransferData(DataFlavor.stringFlavor);
            if (object != null) {
              setObject(object);
              return;
            }
          }
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    });
    menu.add(miCopy);

    JMenuItem miClear = new JMenuItem("Clear");
    miClear.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setObject(null);
      }
    });
    menu.add(miClear);
  }

  public void setObject(Object object) {
    this.object = object;
    if (object == null)
      objectPanel.setComponent(new JLabel("<nothing>"));
    else
      objectPanel.setComponent(new JObject(object));
    newObjectTrigger.trigger();
  }

  public Trigger getNewObjectTrigger() {
    return newObjectTrigger;
  }

  public Object getObject() {
    return object;
  }
}