package prophecy.omnivore.o.o02;

import drjava.util.Port;

public class LiveRef<O> {
  private Class<O> objectClass;
  private O object;

  private Port<ReplacementEvent> eventReceiver = new Port<ReplacementEvent>() {
    public void receive(ReplacementEvent event) {
      handle(event);
    }
  };

  public LiveRef(Class<O> objectClass, O object) {
    this.objectClass = objectClass;
    setObject(object);
  }

  private void setObject(O object) {
    if (object == this.object) return;

    if (this.object instanceof Replaceable) {
      Replaceable replaceable = (Replaceable) this.object;
      replaceable.getReplacementEventSender().removeReceiver(eventReceiver);
    }

    this.object = object;

    if (object instanceof Replaceable) {
      Replaceable replaceable = (Replaceable) object;
      if (replaceable.getReplacement() != null)
        setArbitraryObject(replaceable.getReplacement());
      else
        replaceable.getReplacementEventSender().addReceiver(eventReceiver);
    }
  }

  private void handle(ReplacementEvent event) {
    if (event.getOriginal() == object) {
      Object o = event.getReplacement();
      setArbitraryObject(o);
    }
  }

  private void setArbitraryObject(Object o) {
    if (objectClass.isInstance(o))
      setObject(objectClass.cast(o));
    else
      setObject(null);
  }

  public O get() {
    return object;
  }

  public boolean isValid() {
    return object != null;
  }
}
