package prophecy.omnivore.o.o02;

import drjava.util.ToTree;
import drjava.util.XTestCase;
import drjava.util.Tree;

public class TestReplaceableObjects extends XTestCase {
  public void testRefToPlainObject() {
    Object original = new Object();
    LiveRef<Object> ref = new LiveRef<Object>(Object.class, original);
    assertEquals(original, ref.get());
  }

  public void testRefUpdate() {
    ReplaceableObject original = new ReplaceableObject();
    LiveRef<Object> ref = new LiveRef<Object>(Object.class, original);
    assertEquals(original, ref.get());
    Object replacement = new Object();
    original.setReplacement(replacement);
    assertEquals(replacement, ref.get());
  }

  public void testBrokenRef() {
    ReplaceableTree original = new ReplaceableTree();
    LiveRef<ToTree> ref = new LiveRef<ToTree>(ToTree.class, original);
    assertTrue(ref.isValid());
    original.setReplacement(new Object());
    assertFalse(ref.isValid());
    assertEquals(null, ref.get());
  }

  private class ReplaceableTree extends ReplaceableObject implements ToTree {
    public Tree toTree() {
      check();
      return new Tree();
    }
  }

  public void testDoubleUpdate() {
    ReplaceableObject original = new ReplaceableObject();
    LiveRef<Object> ref = new LiveRef<Object>(Object.class, original);
    assertEquals(original, ref.get());
    ReplaceableObject replacement = new ReplaceableObject();
    original.setReplacement(replacement);
    Object replacement2 = new Object();
    replacement.setReplacement(replacement2);
    assertEquals(replacement2, ref.get());
  }

  public void testReverseDoubleUpdate() {
    ReplaceableObject original = new ReplaceableObject();
    LiveRef<Object> ref = new LiveRef<Object>(Object.class, original);
    assertEquals(original, ref.get());
    ReplaceableObject replacement = new ReplaceableObject();
    Object replacement2 = new Object();
    replacement.setReplacement(replacement2);
    original.setReplacement(replacement);
    assertEquals(replacement2, ref.get());
  }

  public void testInvalidObject() {
    ReplaceableTree object = new ReplaceableTree();
    assertNotNull(object.toTree());
    assertFalse(object.isInvalid());
    object.invalidate();
    assertTrue(object.isInvalid());
    try {
      object.toTree();
      fail("no exception");
    } catch (InvalidObjectException e) {
      // ok
    }
  }
}
