package prophecy.p.p01;

import static drjava.util.ObjectUtil.nice;

import drjava.util.Errors;

import java.io.File;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;
import java.util.jar.Attributes;

public class Jar {
  private File file;
  private Throwable error;
  private int entries;
  private String mainClass;
  private JarFile jarFile;

  public Jar(File file) {
    this.file = file;

    try {
      jarFile = new JarFile(file);
      Enumeration<JarEntry> e = jarFile.entries();
      while (e.hasMoreElements()) {
        e.nextElement();
        ++entries;
      }
      Manifest manifest = jarFile.getManifest();
      Attributes attributes = manifest.getMainAttributes();
      mainClass = attributes.getValue("Main-Class");
    } catch (Throwable error) {
      Errors.add(error);
      this.error = error;
    }
  }

  public String getInfo() {
    String text = "File: " + file.getPath();
    text += "\nEntries: " + entries;
    text += "\nMain class: " + mainClass;
    if (error != null)
      text += "\nError: " + nice(error);
    return text;
  }

  public boolean isRunnable() {
    return mainClass != null;
  }

  public File getFile() {
    return file;
  }

  public String getMainClass() {
    return mainClass;
  }

  public JarFile getJarFile() {
    return jarFile;
  }
}
