package prophecy.p.p01;

import drjava.superversion.filesystem.RealFileSystem;
import drjava.util.Errors;
import drjava.util.GUIUtil;
import static drjava.util.GUIUtil.setMinimumHeight;
import static drjava.util.GUIUtil.withTitle;
import drjava.util.TextView;
import prophecy.common.Prophecy;
import prophecy.common.gui.Sheet;
import prophecy.common.Source;
import prophecy.common.ClassData;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;

public class JarBox extends Sheet {
  private TextView infoView = new TextView();
  private ClassData classData = ClassData.get(this);
  private JButton btnRun;
  private Jar jar;
  private JarDatabase jarDatabase;
  private JList jarList;

  JarBox() {
    super("P01 JarBox");

    jarDatabase = new JarDatabase(new RealFileSystem()
      .getDirectory(new File(Prophecy.memoryDir(), "JarBox")));

    addButton("Import jar...", new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        JFileChooser fileChooser = new JFileChooser(classData.getString("lastDir", ""));
        if (fileChooser.showOpenDialog(getPanel()) == JFileChooser.APPROVE_OPTION) {
          File file = fileChooser.getSelectedFile();
          classData.setString("lastDir", file.getParent());
          classData.save();
          try {
            jarDatabase.addJar(file);
            selectJar(file.getName());
          } catch (Exception e) {
            Errors.add(e);
          }
        }
      }
    });

    jarList = new JList();
    jarList.addMouseListener(new MouseAdapter() {
      public void mouseClicked(MouseEvent e) {
        if (e.getClickCount() == 2) {
          String jarName = (String) jarList.getSelectedValue();
          if (jarName != null)
            selectJar(jarName);
        }
      }
    });
    new UpdateListFromSource(jarList, (Source) jarDatabase.getJarNamesProperty()).install();

    addComponent(withTitle("Jars in database",
      setMinimumHeight(new JScrollPane(jarList), 150)));

    infoView.setMinimumSize(new Dimension(250, 75));
    addComponent(withTitle("Jar Info", infoView));
    btnRun = addButton("Run", new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          JarRunner.run(jar);
        } catch (Throwable e) {
          Errors.add(e);
        }
      }
    });
    btnRun.setEnabled(false);
  }

  private void selectJar(String name) {
    jar = jarDatabase.getJar(name);
    infoView.setText(TextView.nlToBr(jar.getInfo()));
    if (jar.isRunnable()) {
      btnRun.setEnabled(true);
      btnRun.setText("Run " + jar.getFile().getName());
      btnRun.requestFocusInWindow();
    }
  }

  public static void main(String[] args) {
    Prophecy.init();
    Errors.setPopup(true);
    final Sheet sheet = new JarBox();
    GUIUtil.showMainFrame(sheet.asFrame());
  }
}
