package prophecy.p.p01;

import java.io.DataInputStream;
import java.io.File;
import java.io.IOException;
import java.util.Set;
import java.util.TreeSet;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.net.*;

public class JarClassLoader extends ClassLoader {
  private Jar jar;
  private Set<String> classesLoadedFromJar = new TreeSet<String>();
  private Set<String> resourcesLoadedFromJar = new TreeSet<String>();

  public JarClassLoader(Jar jar) {
    super(null);
    this.jar = jar;
  }

  public JarClassLoader(String jarFile) {
    this(new Jar(new File(jarFile)));
  }

  protected synchronized Class<?> loadClass(String name, boolean resolve) throws ClassNotFoundException {
    //System.out.println("loadClass " + name);
    Class result = findLoadedClass(name);
    if (result != null)
      return result;

    JarEntry jarEntry = jar.getJarFile().getJarEntry(name.replace(".", "/") + ".class");
    if (jarEntry != null) {
      byte[] buf = new byte[(int) jarEntry.getSize()];
      try {
        new DataInputStream(jar.getJarFile().getInputStream(jarEntry)).readFully(buf);
      } catch (IOException e) {
        throw new RuntimeException(e);
      }
      result = defineClass(name,  buf, 0, buf.length);
      classesLoadedFromJar.add(name);
      System.out.println("Loaded from " + jar.getFile().getName() + ": " + name);
      return result;
    }

    if (isSystemClass(name)) {
      result = super.loadClass(name, resolve);
      System.out.println("Loaded from boot environment: " + name);
      return result;
    }

    System.out.println("Class not loaded: " + name);
    throw new ClassNotFoundException(name);
  }

  public static boolean isSystemClass(String name) {
    return name.startsWith("java.") || name.startsWith("javax.") || name.startsWith("sun.");
  }

  public Set<String> getClassesLoadedFromJar() {
    return classesLoadedFromJar;
  }

  public Set<String> getResourcesLoadedFromJar() {
    return resourcesLoadedFromJar;
  }

  protected URL findResource(String name) {
    JarEntry jarEntry = jar.getJarFile().getJarEntry(name);
    if (jarEntry != null) {
      try {
        URL url = new URL("jar:file:" + jar.getFile().getPath().replace(File.separator, "/") + "!/" 
          + jarEntry.getName());
        System.out.println("Resource found: " + url);
        resourcesLoadedFromJar.add(jarEntry.getName());
        return url;
      } catch (MalformedURLException e) {
        throw new RuntimeException(e);
      }
    }
    
    return super.findResource(name);
  }
}
