package prophecy.p.p01;

import drjava.superversion.filesystem.DirectoryContents;
import drjava.superversion.filesystem.File;
import drjava.superversion.filesystem.MutableDirectory;
import drjava.superversion.filesystem.RealFile;
import drjava.superversion.tools.FileUtil;
import drjava.util.Logger;
import drjava.util.StdOutLogger;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class JarDatabase {
  protected MutableDirectory dir;
  protected Logger logger = new StdOutLogger();
  private ReadProperty<List<String>> jarNamesProperty
    = new ReadProperty<List<String>>(this, "getJarNames");

  public JarDatabase(MutableDirectory dir) {
    this.dir = dir;
  }

  public List<String> getJarNames() {
    DirectoryContents contents = dir.getContents(logger);
    List<String> jars = new ArrayList<String>();
    for (File file : contents.getFiles()) {
      if (file.getName().endsWith(".jar")) {
        jars.add(file.getName());
      }
    }
    return jars;
  }

  public String addJar(java.io.File jarFile) throws IOException {
    String name = jarFile.getName();
    byte[] contents = FileUtil.loadFile(jarFile);
    addJar(name, contents);
    return name;
  }

  public void addJar(String name, byte[] contents) {
    dir.saveFile(name, contents);
    jarNamesProperty.trigger();
  }

  public ReadProperty<List<String>> getJarNamesProperty() {
    return jarNamesProperty;
  }

  public Jar getJar(String name) {
    return new Jar(getJarFile(name));
  }

  private java.io.File getJarFile(String name) {
    return ((RealFile) dir.getFile(name)).getNativeFile();
  }

  public boolean containsJar(String name) {
    return getJarFile(name).exists();
  }
}
