/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.p.p01;

import drjava.util.StringUtil;
import prophecy.common.Source;
import prophecy.common.Trigger;

import java.lang.reflect.Method;

public class ReadProperty<T> extends Trigger implements Source<T> {
  private Method method;
  private Object object;

  public ReadProperty(Object object, String methodName) {
    this.object = object;
    method = findGetter(object, methodName);
  }

  private static Method findGetter(Object object, String methodName) {
    try {
      return object.getClass().getMethod(methodName);
    } catch (NoSuchMethodException e) {
      throw new RuntimeException(e);
    }
  }

  private static String methodToProperty(String methodName) {
    return StringUtil.firstToLower(methodName.replaceFirst("^get", ""));
  }

  public String getName() {
    return methodToProperty(method.getName());
  }

  public Method getMethod() {
    return method;
  }

  public String getType() {
    return method.getReturnType().toString();
  }

  public T get() {
    try {
      return (T) method.invoke(object);
    } catch (Throwable e) {
      return null;
    }
  }
}