package prophecy.p.p02;

import drjava.util.FromTree;
import static drjava.util.ObjectUtil.nice;
import drjava.util.ToTree2;
import drjava.util.Tree;
import drjava.util.Treeifier;

public class Hash implements SelfTestable, ToTree2, FromTree {
  private Object object;
  private long storedHashCode;

  public Hash(Object object) {
    this.object = object;
    storedHashCode = calculateHashCode();
  }

  public Hash() {
    this(null);
  }

  public SelfTestResult selfTest() {
    long calculatedHashCode = calculateHashCode();
    if (calculatedHashCode == storedHashCode)
      return SelfTestResult.passed(this);
    else
      return SelfTestResult.failed(this, "Bad hash code for " + nice(object)
        + " (expected " + storedHashCode + ", was " + calculatedHashCode + ")");
  }

  private long calculateHashCode() {
    return (object == null ? 0 : object.hashCode()) + 123;
  }

  public Object getObject() {
    return object;
  }

  public long getStoredHashCode() {
    return storedHashCode;
  }

  public Tree toTree(Treeifier mainTreeifier) {
    return new Tree(getClass())
      .setObject("object", object, mainTreeifier)
      .setLong("storedHashCode", storedHashCode);
  }

  public void fromTree(Tree tree) {
    object = tree.getObject("object");
    storedHashCode = tree.getLong("storedHashCode");
  }

  public String toString() {
    return "Hash<" + nice(object) + ">";
  }
}
