package prophecy.p.p02;

import drjava.superversion.filesystem.MutableDirectory;
import drjava.superversion.filesystem.RealDirectory;
import drjava.superversion.filesystem.DirectoryContents;
import drjava.superversion.filesystem.File;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.p.p01.Jar;
import prophecy.p.p01.JarClassLoader;
import prophecy.p.p01.JarDatabase;

import java.lang.reflect.Method;
import java.util.List;
import java.util.ArrayList;

public class ObjectDatabase extends JarDatabase {
  public ObjectDatabase(MutableDirectory dir) {
    super(dir);
  }

  public ObjectDatabase(String dir) {
    this(new RealDirectory(dir));
  }

  public String addObject(String objectName, String jarName, Tree objectTree) {
    String fileName = objectNameToFileName(objectName);
    if (!containsJar(jarName))
      throw new RuntimeException("Jar not in database: " + jarName);
    Tree infoTree = new Tree("ObjectInfo")
      .add("object", objectTree)
      .add("jar", jarName)
      .add("objectName", objectName);
    dir.saveFile(fileName, infoTree.toMultiLineString().getBytes());
    return fileName;
  }

  private String objectNameToFileName(String objectName) {
    String fileName = objectName;
    if (!fileName.endsWith(".object")) fileName += ".object";
    return fileName;
  }

  /** Quite some magic takes place here. A jar class loader is made and the object is
   *  born into the virgin space defined by that class loader.
   *
   *  @throws RuntimeException if anything goes wrong
   * */
  public Object loadObject(String objectName) {
    try {
      String fileName = objectNameToFileName(objectName);
      Tree infoTree = Tree.parse(new String(dir.getFile(fileName).getContents().getBytes()));
      Jar jar = getJar(infoTree.getString("jar"));
      JarClassLoader classLoader = makeClassLoader(jar);
      Class treeUtilClass = classLoader.loadClass(TreeUtil.class.getName());
      Method stringToObject = treeUtilClass.getMethod("stringToObject", String.class);
      Object object = stringToObject.invoke(null, infoTree.get("object").toString());
      return object;
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  protected JarClassLoader makeClassLoader(Jar jar) {
    return new JarClassLoader(jar);
  }

  public List<String> getObjectNames() {
    DirectoryContents contents = dir.getContents(logger);
    List<String> names = new ArrayList<String>();
    for (File file : contents.getFiles()) {
      if (file.getName().endsWith(".object")) {
        names.add(file.getName().substring(0, file.getName().length()-".object".length()));
      }
    }
    return names;
  }
}
