package prophecy.p.p02;

import drjava.util.StringUtil;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.m.m26.JarMaker;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.p.p01.Jar;
import prophecy.p.p01.JarClassLoader;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

public class TestClassHull {
  public static void main(String[] args) throws IOException, ClassNotFoundException, NoSuchMethodException, InvocationTargetException, IllegalAccessException {
    Hash hash = new Hash("Hello hull");
    Tree tree = TreeUtil.objectToTree(hash);

    String jarFile = "temp/p02.jar";
    ProphecyJarMaker.makeJar(jarFile);
    JarClassLoader classLoader = new JarClassLoader(new Jar(new File(jarFile)));
    Class treeUtilClass = classLoader.loadClass(TreeUtil.class.getName());
    Method stringToObject = treeUtilClass.getMethod("stringToObject", String.class);
    Object object = stringToObject.invoke(null, tree.toString());
    System.out.println("Object: " + object + " loaded by " + object.getClass().getClassLoader());
    System.out.println("Self test: " + selfTest(object));
    System.out.println("Hull size: "
      + StringUtil.n(classLoader.getClassesLoadedFromJar().size(), "class", "classes"));

    String hullFile = "temp/p02-hull.jar";
    makeHullJar(jarFile, classLoader, hullFile);

    ObjectDatabase objectDatabase = new ObjectDatabase("memory/p02/ObjectDatabase");
    String jarName = objectDatabase.addJar(new File(hullFile));
    String objFile = objectDatabase.addObject("hash", jarName, tree);
    System.out.println("Object saved: " + objFile);

    System.out.println("Loading " + objFile);
    Object object2 = objectDatabase.loadObject(objFile);
    System.out.println("Object 2: " + object2 + " loaded by " + object2.getClass().getClassLoader());
    System.out.println("Self test: " + selfTest(object2));
  }

  private static void makeHullJar(String jarFile, JarClassLoader classLoader, String hullFile) throws IOException {
    JarMaker jarMaker = new JarMaker(hullFile, new Manifest());
    jarMaker.begin();
    JarFile jar = new JarFile(jarFile);
    Enumeration<JarEntry> e = jar.entries();
    while (e.hasMoreElements()) {
      JarEntry entry = e.nextElement();
      String name = entry.getName();
      if (name.endsWith(".class") && classLoader.getClassesLoadedFromJar().contains(
        name.substring(0, name.length()-6).replace("/", ".")))
        jarMaker.copyJarEntry(jar, entry);
    }
    jarMaker.end();
  }

  private static Object selfTest(Object object) throws IllegalAccessException, InvocationTargetException, NoSuchMethodException {
    return object.getClass().getMethod("selfTest").invoke(object);
  }
}
