package prophecy.spirits2d;

import drjava.util.StringUtil;

import java.util.ArrayList;
import java.util.List;

public class Spirits2DTest {
  Spirit sun;
  List<Spirit> spirits = new ArrayList<Spirit>();
  private double initialSpace = 2.0;
  private double minDistance = 0.1;

  class Point {
    double x, y;

    Point(double x, double y) {
      this.x = x;
      this.y = y;
    }

    Point minus(Point p) {
      return new Point(x-p.x, y-p.y);
    }

    public double len() {
      return Math.sqrt(x*x+y*y);
    }

    public Point times(double d) {
      return new Point(x*d, y*d);
    }

    public Point plus(Point p) {
      return new Point(x+p.x, y+p.y);
    }

    Point direction(Point p) {
      Point distance = p.minus(this);
      double len = distance.len();
      if (len < minDistance)
        return new Point(0, 0);
      Point direction = distance.times(1 / len);
      return direction;
    }

    // result vector starts at "this" and points towards "p"
    Point affection(Point p) {
      Point distance = p.minus(this);
      double len = distance.len();
      if (len < minDistance)
        return new Point(0, 0);
      Point direction = distance.times(1/len);
      double strength = 1/len;
      return direction.times(strength);
    }

    public String toString() {
      return formatCoord(x)+"/"+formatCoord(y);
    }

    private String formatCoord(double coord) {
      return StringUtil.formatDouble(coord, 3);
    }
  }

  class Spirit {
    Point location = new Point(0, 0);
    private double storedEnergy = 0;

    // override me
    void step() {
    }

    // dir > 0: attract
    // dir < 0: repel
    // or is it the other way around...?
    void attract(Spirit spirit, double dir) {
      spirit.move(spirit.affection(this).times(dir));
    }

    // speed > 0: move towards
    // speed < 0: move away from
    void moveTowards_by_affection(Spirit spirit, double speed) {
      spirit.attract(this, speed);
    }

    // speed > 0: move towards
    // speed < 0: move away from
    void moveTowards_with_constant_speed(Spirit spirit, double speed) {
      move(direction(spirit).times(speed));
    }

    private Point affection(Spirit spirit) {
      return location.affection(spirit.location);
    }

    private Point direction(Spirit spirit) {
      return location.direction(spirit.location);
    }

    private void move(Point point) {
      location = location.plus(point);
    }

    void radiate(double energy) {
      for (Spirit spirit : spirits) {
        spirit.receiveEnergy(affection(spirit).len()*energy);
      }
    }

    private void receiveEnergy(double energy) {
      storedEnergy += energy;
    }

    // convenience method for setting location of a new spirit
    public Spirit at(Point location) {
      this.location = location;
      return this;
    }
  }

  class Sun extends Spirit {
    void step() {
      radiate(1);
    }
  }

  class MovesToSun extends Spirit {
    void step() {
      //moveTowards_by_affection(sun, 0.1);
      moveTowards_with_constant_speed(sun, 0.1);
    }
  }

  Spirits2DTest() {
    sun = new Sun();
    spirits.add(new MovesToSun().at(randomLocation()));
    run();
  }

  private Point randomLocation() {
    return new Point(randomCoord(), randomCoord());
  }

  private double randomCoord() {
    return (Math.random()-0.5)*initialSpace;
  }

  private void run() {
    for (int i = 0; i < 10; i++)
      step();
  }

  private void step() {
    printLocations();
    for (Spirit spirit : new ArrayList<Spirit>(spirits)) {
      spirit.step();
    }
  }

  private void printLocations() {
    StringBuffer buf = new StringBuffer();
    for (Spirit spirit : spirits) {
      buf.append(spirit.location).append(" ");
    }
    System.out.println(buf);
  }

  public static void main(String[] args) {
    new Spirits2DTest();
  }
}
