/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t01;

import static drjava.util.ObjectUtil.nice;
import drjava.util.TextView;
import prophecy.common.gui.Sheet;
import prophecy.t.t04.ExUtil2;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class RunnerSheet extends Sheet {
  private Object object;
  private Snapshot marker;
  private Runnable collapser;

  public RunnerSheet() {
    super("Runner");
    setSize(300, 500);
  }

  public RunnerSheet(String title) {
    this();
    setTitle(title);
  }

  public void setObject(Object object) {
    object = ExUtil2.runIfRunnable(object);
    this.object = object;
    renderObject();
  }

  public void renderObject() {
    if (collapser != null) {
      marker.restore();
      collapser.run();
      collapser = null;
    }

    marker = snapshot();
    renderObject(this.object);
  }

  private void renderObject(Object object) {
    if (object == null)
      addLeftAlignedLabel("<null>");
    else if (object instanceof AbstractAction) {
      final AbstractAction action = (AbstractAction) object;
      renderAction(action);
    } else if (object instanceof Choice) {
      Choice choice = (Choice) object;
      for (AbstractAction action : choice.getActions()) {
        renderAction(action);
      }
    } else if (object instanceof Union) {
      Union union = (Union) object;
      for (Object o : union.getParts())
        renderObject(o);
    } else
      addNote(nice(object));
  }

  private void renderAction(final AbstractAction action) {
    addButton(action.getDescription(), new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        setCollapser(new Runnable() {
          public void run() {
            addLeftAlignedLabel("Action taken: " + action.getDescription());
          }
        });

        Object result;
        try {
          result = action.run();
        } catch (Throwable t) {
          result = t;
        }
        setObject(result);
      }
    });
  }

  private void setCollapser(Runnable collapser) {
    this.collapser = collapser;
  }

  public void addNote(String text) {
    String html = "<html>" + TextView.nlToBrWithIndent(text) + "</html>";
    addLeftAlignedLabel(html);
  }
}