/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t02;

import com.sun.org.apache.xml.internal.security.utils.Base64;
import drjava.util.ObjectUtil;
import static drjava.util.ObjectUtil.nice;
import drjava.util.StreamUtil;

import javax.swing.*;
import java.io.IOException;
import java.net.InetAddress;
import java.net.URL;
import java.net.URLConnection;
import java.net.UnknownHostException;

public class DynDNSUpdater {
  
  public String update(String user, String pass, String hostname, String ip) throws IOException {
    String urlString = "http://members.dyndns.org/nic/update"
      + "?hostname=" + hostname + "&myip=" + ip
      + "&wildcard=NOCHG&mx=NOCHG&backmx=NOCHG";
    System.out.println("URL: " + urlString);
    URLConnection connection = new URL(urlString).openConnection();
    authorize(connection, user, pass);
    String response = StreamUtil.readFully(connection.getInputStream());
    System.out.println("Server response: " + response);
    return response;
  }

  public static void main(final String[] args) throws IOException {
    String user = args[0], pass = args[1], hostname = args[2];
    String ip = ObjectUtil.cast(DynDNSCheckIP.getIP(), String.class);
    System.out.println("Our IP (from checkip.dyndns.com): " + ip);

    String address = dnsLookup(hostname);
    System.out.println("Current IP (from DNS lookup): " + address);
    if (!address.equals(ip)) {
      System.out.println("Update needed");
      if (JOptionPane.showConfirmDialog(null, "Update " + hostname + " to " + ip + "?", "DynDNS", JOptionPane.OK_CANCEL_OPTION)
        == JOptionPane.OK_OPTION) {
        new DynDNSUpdater().update(user, pass, hostname, ip);
        System.out.println("IP updated");
      }
    } else
      System.out.println("No update needed");
  }

  private static String dnsLookup(String hostname) throws UnknownHostException {
    InetAddress host = InetAddress.getByName(hostname);
    String address = host.getHostAddress();
    return address;
  }

  private static String toString(Object o) {
    if (o instanceof String)
      return (String) o;
    throw new RuntimeException("Not a string: " + nice(o));
  }

  private static void authorize(URLConnection connection, String user, String pass) {
    String password = user + ":" + pass;
    String encodedPassword = Base64.encode(password.getBytes());
    connection.setRequestProperty("Authorization", "Basic " + encodedPassword);
  }
}