package prophecy.t.t02;

import drjava.util.ObjectUtil;
import prophecy.common.Timestamp;

import java.io.IOException;

public class StatefulDNSUpdater {
  private String hostname;
  private String user, pass;
  private String lastIP;
  private Timestamp lastUpdate;
  private Throwable lastException;

  public StatefulDNSUpdater(String hostname, String user, String pass) {
    this.hostname = hostname;
    this.user = user;
    this.pass = pass;
    if (hostname == null || user == null || pass == null)
      throw new NullPointerException();
  }

  public boolean hasSeries() {
    return lastUpdate != null && lastIP != null;
  }

  public void startSeries() {
    lastIP = "0.0.0.0";
    lastUpdate = new Timestamp();
    update();
  }

  public void continueSeries() {
    update();
  }

  private void update() {
    if (!hasSeries())
      throw new RuntimeException("No series started");
    if (lastException != null)
      throw new RuntimeException("Series paused after error: " + lastException);
    try {
      String ip = ObjectUtil.cast(DynDNSCheckIP.getIP(), String.class);
      if (ip.equals(lastIP)) {
        System.out.println("IP unchanged (" + ip + ")");
      } else {
        System.out.println("IP changed: " + ip);
        setIP(ip);
        lastIP = ip;
      }
    } catch (Throwable e) {
      e.printStackTrace();
      lastException = e;
    }
    lastUpdate = new Timestamp();
  }

  private void setIP(String ip) {
    try {
      System.out.println("Updating " + hostname);
      new DynDNSUpdater().update(user, pass, hostname, ip);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public Throwable getLastException() {
    return lastException;
  }

  public void clearException() {
    lastException = null;
  }
}
