/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t03;

import drjava.util.Lizt;
import drjava.util.Attachments;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Rand;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.SocketCallable;
import prophecy.common.socket.ObjectURL;
import prophecy.t.t04.ExUtil2;

import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;
import java.util.regex.Pattern;

public class Receptionist implements SocketCallable {
  int idCounter = Rand.fourDigits();
  private Map<String, Object> map = new TreeMap<String, Object>(
    new CaseInsensitiveComparator());
  protected String serverName = anonymous();

  private String anonymous() {
    return "anon" + Rand.fourDigits();
  }

  public void takeCall(SocketHandler handler) {
    handler.println("This is object server " + serverName + ". Please enter an address.");
    String address = handler.readLine();
    System.out.println("address: " + address);
    if (address == null) return;
    Object object = lookup(address);
    if (object == null)
      handler.println("Sorry, this address is unknown.");
    else {
      SocketCallable dialoglet = makeDialoglet(object);
      if (dialoglet == null) {
        handler.println("Sorry, " + address + " does not seem to take calls.");
        System.out.println("Failed: makeDialoglet " + nice(object));
      } else {
        handler.println("OK, patching you through.");
        dialoglet.takeCall(handler);
      }
    }
  }

  protected Object lookup(String address) {
    Object object;
    synchronized(this) {
      object = map.get(address);
    }
    return object;
  }

  protected SocketCallable makeDialoglet(Object object) {
    return ExUtil2.convert(object, SocketCallable.class);
  }

  public synchronized String addObject(Object o) {
    String address = newAddress();
    addObject(address, o);
    return address;
  }

  public synchronized ObjectURL addObject(String address, Object o) {
    if (!isAcceptableAddress(address))
      throw new RuntimeException("Address not acceptable: " + address);
    map.put(address, o);

    ObjectURL myURL = ObjectURL.get(this);
    if (myURL != null) {
      ObjectURL url = myURL.append(address);
      Attachments.add(o, url);
      return url;
    }
    return null;
  }

  private boolean isAcceptableAddress(String address) {
    return Pattern.compile("[a-zA-Z0-9#_. ]+").matcher(address).matches();
  }

  private String newAddress() {
    return "#" + (idCounter++);
  }

  public void setServerName(String serverName) {
    this.serverName = serverName;
  }

  public Collection<String> getAddresses() {
    return new Lizt<String>(map.keySet());
  }
}