/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t04;

import drjava.util.IRunnable;
import drjava.util.NoResult;
import static drjava.util.ObjectUtil.nice;
import drjava.util.Result;
import prophecy.t.t01.IRunner;
import prophecy.t.t01.RunnableC;

public class Runner implements IRunner {
  /** execution mode a (Standard Java) */
  public Object run(Object input) {
    Result<Object> result = runB(input);
    if (result.isSuccess())
      return result.getValue();
    else
      throw asRuntimeException(((NoResult) result).getException());
  }

  private RuntimeException asRuntimeException(Throwable throwable) {
    if (throwable instanceof RuntimeException)
      return (RuntimeException) throwable;
    if (throwable == null)
      return new RuntimeException();
    return new RuntimeException(throwable);
  }

  // execution mode b
  public Result<Object> runB(Object input) {
    input = ExUtil2.expandFully(input);
    if (input instanceof IRunnable) {
      try {
        return new Result<Object>(((IRunnable) input).run());
      } catch (Throwable e) {
        return new /*No*/Result<Object>(e);
      }
    }

    /*if (input instanceof AbstractAction) {
      AbstractAction action = (AbstractAction) input;
      if ("step".equals(action.getDescription()))
        return new Result<Object>(action.run());
    }*/

    /*if (input instanceof Steppable) {
      Steppable steppable = (Steppable) input;
      steppable.step();
    }*/

    if (input instanceof RunnableC) {
      RunnableC runnableC = (RunnableC) input;
      return new Result<Object>(runnableC.run(this));
    }

    return new NoResult<Object>("Don't know how to run " + nice(input));
  }

  public Object runUntil(Object input, Predicate predicate) {
    Object o = input;
    while (!predicate.check(input))
      o = run(o);
    return o;
  }

  public Object runWhileRunnable(Object input) {
    Object o = input;
    while (true) {
      Result<Object> result = runB(o);
      if (!result.isSuccess())
        return o;
      o = result.get();
    }
  }
}