/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t05;

import drjava.util.NoResult;
import drjava.util.Result;
import drjava.util.Tree;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.ObjectURL;
import prophecy.m.m14.Failure;
import prophecy.t.t04.Contacter;
import prophecy.t.t04.ExUtil2;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class AddressVerificationClient {
  private Mailbox mailbox;

  public AddressVerificationClient(Mailbox mailbox) {
    this.mailbox = mailbox;
  }

  public Object verifyWith(ObjectURL url) {
    Object myURL = ObjectURL.get(mailbox);
    if (myURL == null)
      return new Failure("Mailbox has no URL");
    Object result = ExUtil2.runIfRunnable(new Contacter(url).call());
    final SocketHandler socketHandler = ExUtil2.convert(result, SocketHandler.class);
    socketHandler.println(new Tree("verify address")
      .add("url", myURL).toString());
    String line = socketHandler.readLine();
    checkFragment(line, "enter key");
    socketHandler.setKeepOpen(true);
    return new WaitState() {
      public Object check() {
        Result<String> key = checkMail();
        if (key.isSuccess()) {
          socketHandler.println(key.get());
          String line = socketHandler.readLine();
          if (!line.startsWith("OK"))
            throw new Failure("??? " + line);
          socketHandler.close();
          return line;
        } else
          return key;
      }
    };
  }

  private void checkFragment(String line, String fragment) {
    if (!line.contains(fragment))
      throw new Failure("\"" + fragment + "\" expected: " + line);
  }

  public Result<String> checkMail() {
    String key = findKeyInMails();
    if (key == null)
      return new NoResult<String>("No key received yet");
    return new Result<String>(key);
  }

  private String findKeyInMails() {
    for (Message message : mailbox.getMessages()) {
      Matcher matcher = Pattern.compile("key is: (\\w+)").matcher(message.getText());
      if (matcher.find())
        return matcher.group(1);
    }
    return null;
  }
}