/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t06;

import drjava.util.Tree;
import drjava.util.Errors;
import prophecy.common.socket.SocketHandler;
import prophecy.t.t03.CaseInsensitiveComparator;

import java.util.Map;
import java.util.TreeMap;

public class TreeCLI {
  Map<String, Cmd> cmds = new TreeMap<String, Cmd>(new CaseInsensitiveComparator());

  public void addCmd(Cmd cmd) {
    cmds.put(cmd.getName(), cmd);
  }

  public void loop(SocketHandler handler) {
    Tree input;
    while (true) {
      // read line

      String line = handler.readLine();

      // leave loop if conn closed

      if (line == null) break;

      // parse line as tree

      try {
        input = Tree.parse(line);
      } catch (Throwable e) {
        handler.println(new Tree("Syntax error"));
        continue;
      }

      // lookup and handle cmd

      Cmd cmd = lookupCmd(input);
      if (cmd == null)
        unknownCommand(handler, input);
      else
        try {
          int charactersSent = handler.getCharactersSent();
          cmd.run(input, handler);
          if (charactersSent == handler.getCharactersSent())
            handler.println("OK");
        } catch (ExitLoop e) {
          break;
        } catch (Throwable e) {
          Errors.add(e);
          handler.println("Sorry, an internal error occured.");
        }
    }
  }

  /** overridable */
  protected void unknownCommand(SocketHandler handler, Tree input) {
    handler.println("??? " + input.getName());
  }

  public Cmd lookupCmd(Tree input) {
    return cmds.get(input.getName());
  }

  public void addStandardCmds() {
    addCmd(new CmdHelp("?"));
    addCmd(new CmdHelp("Help"));
    addCmd(new CmdBye("Bye"));
  }

  public static TreeCLI withStandardCmds() {
    TreeCLI cli = new TreeCLI();
    cli.addStandardCmds();
    return cli;
  }

  public class CmdHelp extends Cmd {
    public CmdHelp(String cmd) {
      super(cmd);
    }

    public void run(Tree input, SocketHandler handler) {
      Tree tCmds = new Tree();
      for (String cmd : cmds.keySet()) {
        tCmds.add(new Tree(cmd));
      }
      Tree tree = new Tree().add("commands", tCmds);
      handler.println(tree.toString());
    }
  }

  private class CmdBye extends Cmd {
    public CmdBye(String name) {
      super(name);
    }

    public void run(Tree input, SocketHandler handler) {
      handler.ok(new Tree("Bye"));
      exitLoop();
    }
  }

  private static class ExitLoop extends RuntimeException {
  }

  public static void exitLoop() {
    throw new ExitLoop();
  }
}