/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t07;

import prophecy.common.socket.SocketHandler;

import java.util.ArrayList;
import java.util.List;

public class InputBuffer {
  private SocketHandler socketHandler;
  private List<String> lines = new ArrayList<String>();
  private boolean closed;
  private boolean closedCleanly;

  public InputBuffer(SocketHandler socketHandler) {
    this.socketHandler = socketHandler;
  }

  public void startThread() {
    new Thread() {
      public void run() {
        try {
          while (true) {
            String line = socketHandler.readLine();
            if (line == null) {
              setClosedCleanly(true);
              return;
            }
            addLine(line);
          }
        } catch (RuntimeException e) {
          if (!socketHandler.isClosed())
            throw e;
        } finally {
          if (socketHandler.isClosed())
            setClosedCleanly(true);
          setClosed(true);
        }
      }
    }.start();
  }

  private synchronized void addLine(String line) {
    lines.add(line);
  }

  public synchronized List<String> flush() {
    List<String> result = lines;
    lines = new ArrayList<String>();
    return result;
  }

  public synchronized boolean isClosed() {
    return closed;
  }

  private synchronized void setClosed(boolean closed) {
    this.closed = closed;
  }

  public synchronized boolean isClosedCleanly() {
    return closedCleanly;
  }

  private synchronized void setClosedCleanly(boolean closedCleanly) {
    this.closedCleanly = closedCleanly;
  }
}