/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t07;

import drjava.util.AnalogSet;
import drjava.util.Tree;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.ObjectURL;
import prophecy.m.m22.Bot;
import prophecy.m.m22.ChatRenderer;
import prophecy.m.m22.StandardBotGUI;
import prophecy.t.t04.Contacter;

import javax.swing.event.AncestorListener;
import javax.swing.event.AncestorEvent;
import java.awt.*;
import java.util.List;

public class SocketConsole {
  public static StandardBotGUI make(ObjectURL url) {
    final SocketHandler socketHandler = new Contacter(url).callJ();

    final InputBuffer inputBuffer = new InputBuffer(socketHandler);
    inputBuffer.startThread();

    Bot bot = new Bot() {
      boolean closed;

      public void timePasses(AnalogSet<Tree> event, double seconds) {
      }

      public void userSays(String text) {
        socketHandler.println(text);
      }

      public List<String> getAndClearOutput() {
        List<String> lines = inputBuffer.flush();
        if (inputBuffer.isClosed() && !closed) {
          closed = true;
          lines.add("&lt;&lt; connection "
            + (inputBuffer.isClosedCleanly() ? "closed" : "lost")
            + " &gt;&gt;");
        }
        return lines;
      }
    };

    StandardBotGUI gui = new StandardBotGUI();
    gui.setTimestep(250);
    gui.init(bot, url.toString());
    gui.setSize(750, 600);
    //gui.setChatFont(new Font("Curlz MT", Font.PLAIN, 24));
    gui.setChatFont(new Font("Book Antiqua", Font.PLAIN, 18));
    gui.getChatPanel().setRenderer(new ChatRenderer() {
      public String renderMessage(String nick, String text) {
        String color;
        if (!nick.equals("User"))
          color = "#FF0000";
        else
          color = "#000000";
        return "<font color=" + color + ">" + text + "</font> ";
        //return "<b>" + nick + "</b> " + text;
      }
    });
    gui.getChatPanel().setInset(10);

    gui.addAncestorListener(new AncestorListener() {
      public void ancestorAdded(AncestorEvent event) {
      }

      public void ancestorRemoved(AncestorEvent event) {
        System.out.println("SocketConsole: Closing connection");
        socketHandler.close();
      }

      public void ancestorMoved(AncestorEvent event) {
      }
    });

    return gui;
  }
}