/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t07;

import drjava.util.Errors;
import drjava.util.MultiCoreUtil;
import prophecy.common.socket.ObjectServer;
import prophecy.m.m25.Step;
import prophecy.m.m25.Steps;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.t.t05.ServerT05;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;

public class T07 {
  private VMTrayIcon trayIcon;
  private int stepDelay = 1500;
  private StepsProgressSheet progressSheet;
  private JFrame frame;

  public static void main(String[] args) throws AWTException {
    T07 t07 = new T07();
    t07.run();
    t07.autoClose();
  }

  public void run() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Throwable e) {
      //
    }

    Step installTray = new Step("Install tray icon");
    Step startServer = new Step("Start object server");
    Steps steps = new Steps(installTray, startServer);
    progressSheet = new StepsProgressSheet("T07 Startup", steps, true);
    progressSheet.setProgressBarWidth(550);
    Font font = new Font(
      //"Arial"
      "Rockwell"              // looks good
      //"Plantagenet Cherokee"  // poor spacing
      //"Palatino Linotype"     // Y pos slightly off
      //"Bookman Old Style"     // ok
      //"Book Antiqua"          // ok
      //"Curlz MT"              // funny!!
      , Font.PLAIN, 18);
    progressSheet.setLogFont(font);
    progressSheet.setProgressBarFont(font);
    frame = progressSheet.showFrame();

    installTray.run(slowdown(new Runnable() {
      public void run() {
        try {
          trayIcon = new VMTrayIcon();
          trayIcon.setTooltip("Java process (T07)");
          trayIcon.install();
          progressSheet.logText("Tray icon installed in task bar :)");
        } catch (Throwable e) {
          e.printStackTrace();
          JOptionPane.showMessageDialog(null, e.toString());
          System.exit(1);
        }
      }
    }));

    startServer.run(slowdown(new Runnable() {
      public void run() {
        try {
          ObjectServer objectServer = ServerT05.start();
          progressSheet.logText("Object server started :)");
          trayIcon.addSubMenu(new ObjectServerMenu(objectServer));
          progressSheet.logText("Server control added to tray menu :)");
        } catch (IOException e) {
          Errors.add(e);
          JOptionPane.showMessageDialog(null, e.toString());
        }
      }
    }));
  }

  private void autoClose() {
    progressSheet.logText("");
    progressSheet.logText("Closing this window in 5 seconds...");
    new Thread() {
      public void run() {
        MultiCoreUtil.sleep(5000);
        frame.dispose();
      }
    }.start();
  }

  private Runnable slowdown(final Runnable runnable) {
    return new Runnable() {
      public void run() {
        long time = System.currentTimeMillis() + stepDelay;
        runnable.run();
        int ms = (int) Math.max(0, time - System.currentTimeMillis());
        MultiCoreUtil.sleep(ms);
      }
    };
  }

  public JFrame getFrame() {
    return frame;
  }

  public static class MakeJar {
    public static void main(String[] args) throws IOException {
      ProphecyJarMaker.makeJar("standalone/t07.jar", T07.class);
    }
  }

  public static class TestUI {
    public static void main(String[] args) throws IOException {
      T07 t07 = new T07();
      t07.run();
      t07.getFrame().setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
  }
}