/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t07;

import static drjava.util.ObjectUtil.nice;

import drjava.util.Errors;
import prophecy.common.Prophecy;
import prophecy.t.t13.ConsoleConnectDialog;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class VMTrayIcon {
  private PopupMenu menu;
  private String vmName;

  public static boolean isSupported() {
    return SystemTray.isSupported();
  }

  public VMTrayIcon() {
    menu = new PopupMenu();
    addStandardTools();
  }

  private void addStandardTools() {
    addTool(new Tool("Console") {
      public void run() {
        new ConsoleConnectDialog().showFrame();
      }
    });
  }

  /*public void installIfPossible() {
    try {
      install();
    } catch (Exception e) {
      Errors.add(e);
    }
  }*/

  public void install() throws AWTException {
    SystemTray tray = SystemTray.getSystemTray();

    MenuItem miExit = new MenuItem("Exit");
    miExit.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.exit(0);
      }
    });
    menu.add(miExit);

/*
    MenuItem miErrors = new MenuItem("Errors");
    miErrors.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // TODO
      }
    });
    menu.add(miErrors);
*/

    TrayIcon trayIcon = new TrayIcon(Prophecy.cupIcon().getImage(),
      vmName == null ? "Java process" : vmName, menu);
    trayIcon.setImageAutoSize(true);
    tray.add(trayIcon);
    trayIcon.displayMessage("Prophecy icon", "installed here", TrayIcon.MessageType.INFO);
  }

  public void addSubMenu(Menu submenu) {
    menu.insert(submenu, 0);
  }

  public void setTooltip(String vmName) {
    this.vmName = vmName;
  }

  public void addTool(final Tool tool) {
    MenuItem menuItem = new MenuItem(tool.getName());
    menuItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        try {
          tool.run();
        } catch (Throwable e) {
          Errors.add(e);
          JOptionPane.showMessageDialog(null, nice(e));
        }
      }
    });
    menu.add(menuItem);
  }
}