/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t08;

import drjava.util.Lizt;
import drjava.util.RealRandomizer;
import drjava.util.Tree;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.SocketCallable;
import prophecy.t.t06.Cmd;
import prophecy.t.t06.TreeCLI;

public class ConversationDB implements SocketCallable {
  private Bag<RecordedConversation> conversations;

  public ConversationDB(Bag<RecordedConversation> conversations) {
    this.conversations = conversations;
  }

  public void takeCall(SocketHandler handler) {
    new SocketHelper(handler).thisIs(this);
    TreeCLI cli = new TreeCLI();
    cli.addStandardCmds();
    cli.addCmd(new Random());
    cli.loop(handler);
  }

  private class Random extends Cmd {
    public Random() {
      super("random");
    }

    public void run(Tree input, SocketHandler handler) {
      Lizt<RecordedConversation> snapshot = conversations.getSnapshot();
      handler.println("Total conversations recorded: " + snapshot.size());
      RecordedConversation conversation = new RealRandomizer().oneOf(snapshot);
      if (conversation != null) {
        handler.println("Random conversation:");
        print(handler, conversation);
      }
    }
  }

  private void print(SocketHandler handler, RecordedConversation conversation) {
    handler.println(String.valueOf(conversation.getInfo()));
    for (Packet packet : conversation.getPackets()) {
      String prefix = packet.getDirection() == InOut.in ? "< " : "> ";
      handler.println(prepend(prefix, packet.getContents()));
    }
  }

  private String prepend(String prefix, String contents) {
    return prefix + contents.replace("\n", "\n" + prefix);
  }
}