/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t08;

import drjava.util.Port;
import drjava.util.Tree;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.SocketCallable;
import prophecy.t.t06.Cmd;
import prophecy.t.t06.TreeCLI;

import java.util.ArrayList;
import java.util.List;

public class ConversationLogger implements SocketCallable {
  private Tree info = new Tree();
  private List<Packet> packets = new ArrayList<Packet>();
  private boolean closed;
  private TreeCLI cli;
  private Port<RecordedConversation> outputPort;

  public ConversationLogger(Port<RecordedConversation> outputPort) {
    this.outputPort = outputPort;
  }

  public void takeCall(SocketHandler handler) {
    handler.println("ConversationLogger ready");
    cli = new TreeCLI();
    cli.addStandardCmds();
    cli.addCmd(new CmdAddInfo());
    cli.addCmd(new CmdAddInput());
    cli.addCmd(new CmdAddOutput());
    cli.addCmd(new CmdClose());
    try {
      cli.loop(handler);
    } finally {
      save(handler);
    }
  }

  private void save(SocketHandler handler) {
    RecordedConversation conversation = new RecordedConversation();
    conversation.setInfo(info);
    conversation.setPackets(packets);
    conversation.setClosed(closed);
    outputPort.receive(conversation);
    handler.println("Conversation saved");
  }

  public class CmdAddInfo extends Cmd {
    public CmdAddInfo() {
      super("addInfo");
    }

    public void run(Tree input, SocketHandler handler) {
      info.add(input.get(0));
    }
  }

  public class CmdAddInput extends Cmd {
    public CmdAddInput() {
      super("addInput");
    }

    public void run(Tree input, SocketHandler handler) {
      packets.add(new Packet(InOut.in, input.getString(0)));
    }
  }

  public class CmdAddOutput extends Cmd {
    public CmdAddOutput() {
      super("addOutput");
    }

    public void run(Tree input, SocketHandler handler) {
      packets.add(new Packet(InOut.out, input.getString(0)));
    }
  }

  public class CmdClose extends Cmd {
    public CmdClose() {
      super("close");
    }

    public void run(Tree input, SocketHandler handler) {
      closed = true;
      cli.exitLoop();
    }
  }
}