/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t08;

import static drjava.util.ObjectUtil.nice;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.SocketCallable;
import prophecy.t.t03.Receptionist;
import prophecy.t.t06.Cmd;
import prophecy.t.t06.TreeCLI;

public class Receptionist2 extends Receptionist {
  public void takeCall(SocketHandler handler) {
    welcomeMessage(handler);
    TreeCLI cli = new TreeCLI() {
      public Cmd lookupCmd(Tree input) {
        Cmd cmd = super.lookupCmd(input);
        if (cmd == null)
          return new CmdQuickGo();
        return cmd;
      }
    };
    cli.addStandardCmds();
    cli.addCmd(new CmdGo());
    cli.addCmd(new CmdList());
    cli.loop(handler);
  }

  private void welcomeMessage(SocketHandler handler) {
    handler.println("This is object server " + Tree.quoteString(serverName) + ". Please enter ? for help at any time.");
  }

  public void go(SocketHandler handler, String address) {
    if (address == null) return;
    Object object = lookup(address);
    if (object == null)
      handler.println("Sorry, this address is unknown.");
    else {
      SocketCallable dialoglet = makeDialoglet(object);
      if (dialoglet == null) {
        handler.println("Sorry, " + address + " does not seem to take calls.");
        System.out.println("Failed: makeDialoglet " + nice(object));
      } else {
        handler.println("OK, patching you through.");
        dialoglet.takeCall(handler);

        // call ended
        handler.println("");
        welcomeMessage(handler);
      }
    }
  }

  private class CmdList extends Cmd {
    public CmdList() {
      super("list");
    }

    public void run(Tree input, SocketHandler handler) {
      handler.println(TreeUtil.objectToTree(getAddresses()).toString());
    }
  }

  private class CmdGo extends Cmd {
    public CmdGo() {
      super("go");
    }

    public void run(Tree input, SocketHandler handler) {
      String address = input.get(0).unquote();
      go(handler, address);
    }
  }

  private class CmdQuickGo extends Cmd {
    public CmdQuickGo() {
      super("");
    }

    public void run(Tree input, SocketHandler handler) {
      String address = input.toString();
      go(handler, address);
    }
  }
}