/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t08;

import drjava.util.Errors;
import drjava.util.GUIUtil;
import drjava.util.MultiCoreUtil;
import prophecy.common.socket.ObjectServer;
import prophecy.m.m25.Step;
import prophecy.m.m25.Steps;
import prophecy.t.t07.ObjectServerMenu;
import prophecy.t.t07.SocketConsole;
import prophecy.t.t07.StepsProgressSheet;
import prophecy.t.t07.VMTrayIcon;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

public abstract class ServerTrayIcon {
  private VMTrayIcon trayIcon;
  private int stepDelay = 1500;
  private StepsProgressSheet progressSheet;
  private JFrame frame;
  private String serverName;
  private boolean openConsole;
  private JButton btnConsole;
  private ObjectServer objectServer;
  private volatile boolean ready;

  public ServerTrayIcon(String serverName) {
    this.serverName = serverName;
  }

  public void install() {
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Throwable e) {
      //
    }

    Step installTray = new Step("Install tray icon");
    Step startServer = new Step("Start object server");
    Steps steps = new Steps(installTray, startServer);
    progressSheet = new StepsProgressSheet(serverName + " Startup", steps, true);
    progressSheet.setProgressBarWidth(550);
    Font font = new Font(
      //"Arial"
      "Rockwell"              // looks good
      //"Plantagenet Cherokee"  // poor spacing
      //"Palatino Linotype"     // Y pos slightly off
      //"Bookman Old Style"     // ok
      //"Book Antiqua"          // ok
      //"Curlz MT"              // funny!!
      , Font.PLAIN, 18);
    progressSheet.setLogFont(font);
    progressSheet.setProgressBarFont(font);

    btnConsole = progressSheet.addButton("Open console", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (ready)
          openConsole();
        else {
          openConsole = true;
          btnConsole.setEnabled(false);
        }
      }
    });

    frame = progressSheet.showFrame();

    installTray.run(slowdown(new Runnable() {
      public void run() {
        try {
          trayIcon = new VMTrayIcon();
          trayIcon.setTooltip("Java process (" + serverName + ")");
          trayIcon.install();
          progressSheet.logText("Tray icon installed in task bar :)");
        } catch (Throwable e) {
          e.printStackTrace();
          JOptionPane.showMessageDialog(null, e.toString());
          System.exit(1);
        }
      }
    }));

    startServer.run(slowdown(new Runnable() {
      public void run() {
        try {
          objectServer = makeObjectServer();
          objectServer.setName(serverName);
          objectServer.start();
          progressSheet.logText("Object server started :)");
          trayIcon.addSubMenu(new ObjectServerMenu(objectServer));
          progressSheet.logText("Server control added to tray menu :)");
        } catch (IOException e) {
          Errors.add(e);
          JOptionPane.showMessageDialog(null, e.toString());
        }
      }
    }));

    ready = true;
    if (openConsole && objectServer != null) {
      openConsole();
    }
    btnConsole.setEnabled(true);
  }

  private void openConsole() {
    GUIUtil.showFrame(SocketConsole.make(objectServer.getURL()).makeFrame());
  }

  public abstract ObjectServer makeObjectServer();

  public void autoClose() {
    progressSheet.logText("");
    progressSheet.logText("Closing this window in 5 seconds...");
    new Thread() {
      public void run() {
        MultiCoreUtil.sleep(5000);
        frame.dispose();
      }
    }.start();
  }

  private Runnable slowdown(final Runnable runnable) {
    return new Runnable() {
      public void run() {
        long time = System.currentTimeMillis() + stepDelay;
        runnable.run();
        int ms = (int) Math.max(0, time - System.currentTimeMillis());
        MultiCoreUtil.sleep(ms);
      }
    };
  }

  public JFrame getFrame() {
    return frame;
  }

  public void installAndAutoClose() {
    install();
    autoClose();
  }
}