/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t09;

import drjava.util.GUIUtil;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;
import prophecy.common.LogView;
import prophecy.m.m26.ProphecyJarMaker;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;

public class ServerDiscoveryFrame extends JFrame {
  private LogView hostsLog = new LogView();
  private LogView portsLog = new LogView();
  private LogView activityLog = new LogView();

  public ServerDiscoveryFrame() {
    super("T09: Server Discovery");
    setSize(600, 500);

    JPanel panel = new JPanel(new GridLayout(1, 2));
    hostsLog.setInset(6);
    portsLog.setInset(6);
    panel.add(GUIUtil.withTitle("Hosts found", hostsLog));
    panel.add(GUIUtil.withTitle("Open ports found", portsLog));

    getContentPane().setLayout(new GridLayout(2, 1));
    getContentPane().add(panel);
    getContentPane().add(GUIUtil.withTitle("Log", activityLog));

    new Thread() {
      public void run() {
        startDiscovery();
      }
    }.start();
  }

  private void startDiscovery() {
    ServerDiscovery serverDiscovery = new ServerDiscovery();
    serverDiscovery.getSoup().addSoupListener(new SoupListener() {
      public void ingredientAdded(Soup soup, Object ingredient) {
        process(ingredient);
      }
    });
    serverDiscovery.seed();
    done();
  }

  private void done() {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        activityLog.logText("\nDone!\n\n");
        activityLog.scrollDown();
      }
    });
  }

  void process(final Object o) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        activityLog.logText(nice(o));
        activityLog.scrollDown();
        if (o instanceof HostFound) {
          hostsLog.logText(((HostFound) o).getHost());
        }
        if (o instanceof OpenPortFound) {
          OpenPortFound openPort = (OpenPortFound) o;
          portsLog.logText(openPort.getHost() + ":" + openPort.getPort());
          //explorePort(openPort.getHost(), openPort.getPort());
        }
      }
    });
  }

  /*
  private void explorePort(final String host, final int port) {
    new Thread() {
      public void run() {
        System.out.println("Detecting server type: " + host + ":" + port);
        new ServerTypeDetector(host, port).go();
      }
    }.start();
  }
  */

  public static void main(String[] args) {
    Prophecy.initUI();
    GUIUtil.showMainFrame(new ServerDiscoveryFrame());
  }

  public static class MakeJar {
    public static void main(String[] args) throws IOException {
      ProphecyJarMaker.makeJar("standalone/ServerDiscovery.jar", ServerDiscoveryFrame.class);
    }
  }
}