/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t09;

import java.util.ArrayList;
import java.util.List;

public class Soup {
  private List<Object> ingredients = new ArrayList<Object>();
  private List<SoupListener> listeners = new ArrayList<SoupListener>();

  public void add(Object object) {
    ActiveIngredient ingredient = null;
    if (object instanceof ActiveIngredient)
      ingredient = (ActiveIngredient) object;

    if (ingredient != null) {
      ingredient.setSoup(this);
      if (ingredient.isDuplicate()) {
        System.out.println("Dropping duplicate: " + ingredient);
        return;
      }
    }

    ingredients.add(object);

    for (SoupListener listener : listeners) {
      listener.ingredientAdded(this, object);
    }

    if (ingredient != null) {
      ingredient.run();
    }
  }

  public void addSoupListener(SoupListener listener) {
    listeners.add(listener);
  }

  public <A> List<A> getIngredients(Class<A> aClass) {
    List<A> result = new ArrayList<A>();
    for (Object ingredient : ingredients) {
      if (aClass.isInstance(ingredient))
        result.add((A) ingredient);
    }
    return result;
  }
}