/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t10;

import drjava.util.GUIUtil;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;
import prophecy.common.LogView;
import prophecy.common.gui.SexyColumn;
import prophecy.common.gui.SexyTableModel;
import prophecy.t.t09.*;

import javax.swing.*;
import java.awt.*;
import java.net.InetAddress;

public class ServerDiscoveryFrame2 extends JFrame {
  private JTable hostsTable;
  private JTable portsTable;
  private LogView activityLog = new LogView();
  private SexyTableModel<Host> hostsTableModel;
  private String localhostIP;
  private SexyTableModel<Port> portsTableModel;

  public ServerDiscoveryFrame2() {
    super("T10: Server Discovery");
    setSize(600, 600);

    hostsTableModel = new SexyTableModel<Host>(new ColHostAddress(), new ColHostName());
    hostsTable = new JTable(hostsTableModel);

    portsTableModel = new SexyTableModel<Port>(new ColPortAddress(), new ColPortServerType());
    portsTable = new JTable(portsTableModel);

    JPanel panel = new JPanel(new GridLayout(3, 1));
    panel.add(GUIUtil.withTitle("Hosts found", new JScrollPane(hostsTable)));
    panel.add(GUIUtil.withTitle("Open ports found", new JScrollPane(portsTable)));
    panel.add(GUIUtil.withTitle("Log", activityLog));

    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(panel);

    new Thread() {
      public void run() {
        startDiscovery();
      }
    }.start();
  }

  private void startDiscovery() {
    try {
      localhostIP = InetAddress.getLocalHost().getHostAddress();
    } catch (Throwable e) {
      e.printStackTrace();
    }
    ServerDiscovery serverDiscovery = new ServerDiscovery();
    serverDiscovery.getSoup().addSoupListener(new SoupListener() {
      public void ingredientAdded(Soup soup, Object ingredient) {
        process(ingredient);
      }
    });
    serverDiscovery.seed();
    done();
  }

  private void done() {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        activityLog.logText("\nDone!\n\n");
        activityLog.scrollDown();
      }
    });
  }

  void process(final Object o) {
    SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        activityLog.logText(nice(o));
        activityLog.scrollDown();
        if (o instanceof HostFound) {
          String host = ((HostFound) o).getHost();
          hostsTableModel.addItem(new Host(host));
        }
        if (o instanceof OpenPortFound) {
          OpenPortFound openPort = (OpenPortFound) o;
          Port port = new Port(openPort.getHost() + ":" + openPort.getPort());
          portsTableModel.addItem(port);
          explorePort(openPort.getHost(), openPort.getPort(), port);
        }
      }
    });
  }

  private void explorePort(final String host, final int port, final ServerDiscoveryFrame2.Port portObject) {
    new Thread() {
      public void run() {
        System.out.println("Detecting server type: " + host + ":" + port);
        final ServerTypeDetector typeDetector = new ServerTypeDetector(host, port);
        typeDetector.go();

        SwingUtilities.invokeLater(new Runnable() {
          public void run() {
            portObject.serverType = typeDetector.getServerType();
            portsTableModel.fireTableDataChanged();
          }
        });
      }
    }.start();
  }

  public static void main(String[] args) {
    Prophecy.initUI();
    GUIUtil.showMainFrame(new ServerDiscoveryFrame2());
  }

  private class Host {
    String address, name;
    boolean localhost;

    public Host(String host) {
      if (ServerDiscovery.isIP(host)) {
        address = host;
        localhost = address.equals(localhostIP);
      } else
        name = host;
    }
  }

  static class ColHostAddress extends SexyColumn<Host> {
    public String getName() {
      return "Address";
    }

    public Object getCell(int row, Host host) {
      return host.address;
    }
  }

  static class ColHostName extends SexyColumn<Host> {
    public String getName() {
      return "Name";
    }

    public Object getCell(int row, Host host) {
      return host.name;
    }
  }

  private class Port {
    String address, serverType;

    public Port(String address) {
      this.address = address;
    }
  }

  static class ColPortAddress extends SexyColumn<Port> {
    public String getName() {
      return "Address";
    }

    public Object getCell(int row, Port port) {
      return port.address;
    }
  }

  static class ColPortServerType extends SexyColumn<Port> {
    public String getName() {
      return "Server type";
    }

    public Object getCell(int row, Port port) {
      return port.serverType;
    }
  }
}