/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t11;

import drjava.util.Lizt;
import drjava.util.Errors;
import prophecy.common.socket.SocketCallable;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.ObjectServer;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

public class LocalCallsOnly implements SocketGuard {
  private List<String> allowedAddresses = new ArrayList<String>(
    Lizt.of("0:0:0:0:0:0:0:1", "127.0.0.1"));
  private SocketCallable followUp;
  private String localAddress;

  public LocalCallsOnly() {
    try {
      localAddress = InetAddress.getLocalHost().getHostAddress();
      allowedAddresses.add(localAddress);
    } catch (UnknownHostException e) {
      Errors.add(e);
    }
  }

  public void setFollowUp(SocketCallable followUp) {
    this.followUp = followUp;
  }

  public void takeCall(SocketHandler handler) {
    InetSocketAddress remoteAddress = handler.getRemoteAddress();
    String address = remoteAddress.getAddress().getHostAddress();
    System.out.println("Caller address: " + address);
    if (allowedAddresses.contains(address)) {
      // ok
      followUp.takeCall(handler);
    } else {
      // close without notice
      System.out.println("Address rejected, not local");
    }
  }

  public static void main(String[] args) throws IOException {
    ObjectServer objectServer = new ObjectServer();
    objectServer.addGuard(new LocalCallsOnly());
    objectServer.start();
  }
}