/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t12;

import drjava.util.Lizt;
import static drjava.util.ObjectUtil.nice;
import drjava.util.Port;
import drjava.util.Tree;
import drjava.util.Treeifier;
import prophecy.ce.ce01.*;
import prophecy.ce.ce02.DumpSoupChanges;
import prophecy.common.CodeQueue;

import java.util.List;

public class EventTreeifierSoup extends Soup {
  private Treeifier treeifier;

  public EventTreeifierSoup(CodeQueue codeQueue, Treeifier treeifier) {
    this.treeifier = treeifier;
    DumpSoupChanges.install(this);
    codeQueue.attachTo(this);
    setAutoStir(true);
    add(new EventTreeifier());
  }

  public void newEvent(Object event) {
    add(new NewEvent(event));
  }

  public void sendTreesTo(final Port<Tree> treePort) {
    addSoupListener(new SoupAdapter() {
      public void ingredientAdded(Soup soup, Object ingredient) {
        if (ingredient instanceof TreeifiedEvent)
          treePort.receive(((TreeifiedEvent) ingredient).tree);
      }
    });
  }

  class NewEvent {
    private Object event;

    public NewEvent(Object event) {
      this.event = event;
    }

    public Object getEvent() {
      return event;
    }

    public String toString() {
      return "NewEvent " + nice(event);
    }
  }

  class EventTreeifier extends ActiveIngredient {
    public EventTreeifier() {
    }

    public List<PossibleReaction> getPossibleReactions() {
      return Lizt.<PossibleReaction> of(new PossibleReaction<NewEvent>(this, NewEvent.class) {
        public boolean runWith(NewEvent event) {
          add(new TreeifyEvent(event));
          return true;
        }
      });
    }
  }

  class TreeifyEvent extends ActiveIngredient {
    private Object event;

    public TreeifyEvent(Object event) {
      this.event = event;
    }

    public Object getEvent() {
      return event;
    }

    public boolean isDuplicate() {
      return soup.getIngredients(TreeifyEvent.class).contains(this);
    }

    public boolean equals(Object o) {
      if (this == o) return true;
      if (o == null || getClass() != o.getClass()) return false;

      TreeifyEvent event1 = (TreeifyEvent) o;

      if (event != null ? !event.equals(event1.event) : event1.event != null) return false;

      return true;
    }

    public int hashCode() {
      return (event != null ? event.hashCode() : 0);
    }

    public String toString() {
      return "TreeifyEvent " + nice(event);
    }

    public void run() {
      try {
        Tree tree = treeifier.objectToTree(event);
        add(new TreeifiedEvent(event, tree));
      } catch (Throwable e) {
        e.printStackTrace();
        add(new TreeificationProblem(event, e));
      }
    }
  }

  class TreeifiedEvent {
    private Object event;
    private Tree tree;

    public TreeifiedEvent(Object event, Tree tree) {
      this.event = event;
      this.tree = tree;
    }

    public String toString() {
      return "TreeifiedEvent " + nice(event) + " => " + nice(tree);
    }
  }

  class TreeificationProblem {
    private Object event;
    private Throwable problem;

    public TreeificationProblem(Object event, Throwable problem) {
      this.event = event;
      this.problem = problem;
    }

    public String toString() {
      return "TreeificationProblem " + nice(event) + ": " + nice(problem);
    }
  }
}