/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t12;

import drjava.util.Tree;
import drjava.util.TreeUtil;
import drjava.util.XTestCase;
import prophecy.m.m37.TryStandard;

public class MultiTreeifierTest extends XTestCase {
  public void testPrivateFieldScan() {
    MultiTreeifier multiTreeifier = new MultiTreeifier(
      new TryStandard(), new PrivateFieldScan());
    A a = new A("test");
    System.out.println("Original: " + a);
    Tree tree = multiTreeifier.objectToTree(a);
    System.out.println(tree);
    A a2 = TreeUtil.treeToObject(tree);
    System.out.println("Restored: " + a);
    assertEquals(a, a2);
  }

  static class A {
    private String string;

    A(String string) {
      this.string = string;
    }

    public boolean equals(Object o) {
      if (this == o) return true;
      if (o == null || getClass() != o.getClass()) return false;

      A a = (A) o;

      if (string != null ? !string.equals(a.string) : a.string != null) return false;

      return true;
    }

    public int hashCode() {
      return (string != null ? string.hashCode() : 0);
    }

    public String toString() {
      return "A " + Tree.quoteString(string);
    }
  }

  static class B extends A {
    transient String bla;

    B(String string) {
      super(string);
    }
  }

  public void testTransientField() {
    MultiTreeifier multiTreeifier = new MultiTreeifier(new TryStandard(), new PrivateFieldScan());
    B b = new B("test");
    Tree tree = multiTreeifier.objectToTree(b);
    System.out.println(tree);
    B b2 = TreeUtil.treeToObject(tree);
    assertEquals(b, b2);
    assertNull(b2.bla);
  }

  static class C extends A {
    static String bla;

    C(String string) {
      super(string);
    }
  }

  public void testStaticField() {
    MultiTreeifier multiTreeifier = new MultiTreeifier(new TryStandard(), new PrivateFieldScan());
    C c = new C("test");
    Tree tree = multiTreeifier.objectToTree(c);
    System.out.println(tree);
    C c2 = TreeUtil.treeToObject(tree);
    assertEquals(c, c2);
    assertNull(c2.bla);
  }

  static class D {
    boolean flag;
  }

  public void testBooleanField() {
    MultiTreeifier multiTreeifier = new MultiTreeifier(new TryStandard(), new PrivateFieldScan());
    D d = new D();
    d.flag = true;
    Tree tree = multiTreeifier.objectToTree(d);
    D d2 = TreeUtil.treeToObject(tree);
    assertTrue(d2.flag);
  }
}