/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t12;

import drjava.util.NoResult;
import drjava.util.Result;
import drjava.util.Tree;
import drjava.util.Treeifier;
import prophecy.m.m37.AutoUnpersist;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

/** converts objects to trees by scanning their private fields.
 *  <p>
 *  Transient fields are skipped.
 */
public class PrivateFieldScan implements SpecialTreeifier {
  public Result<Tree> objectToTree(Object object, Treeifier mainTreeifier) {
    try {
      Class c = object.getClass();
      if (c.getName().startsWith("java.lang."))
        return new NoResult<Tree>();
      
      //System.out.println("PrivateFieldScan " + object);
      Tree tree = new Tree(AutoUnpersist.class);
      tree.add("class", new Tree(object.getClass()));
      Tree attributes = new Tree();
      tree.add("attributes", attributes);
      while (c != Object.class) {
        scanFields(object, mainTreeifier, attributes, c);
        c = c.getSuperclass();
      }
      return new Result<Tree>(tree);
    } catch (IllegalAccessException e) {
      e.printStackTrace();
      return new NoResult<Tree>(e);
    }
  }

  private void scanFields(Object object, Treeifier mainTreeifier, Tree attributes, Class c)
    throws IllegalAccessException {
    Field[] fields = c.getDeclaredFields();
    for (Field field : fields) {
      if ((field.getModifiers() & (Modifier.TRANSIENT | Modifier.STATIC)) != 0) continue;
      field.setAccessible(true);
      Object fieldValue = field.get(object);
      //System.out.println("field " + field.getName() + " value: " + nice(fieldValue));
      Tree fieldTree = mainTreeifier.objectToTree(fieldValue);
      attributes.add(new Tree().add(field.getName()).add(fieldTree));
    }
  }
}