/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t13;

import drjava.util.Port;
import prophecy.common.socket.SocketCallable;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.ObjectServer;
import prophecy.t.t08.Packet;
import prophecy.t.t11.LocalCallsOnly;
import prophecy.t.t11.SocketGuard;

import java.io.IOException;

public class CallMonitor implements SocketGuard {
  private SocketCallable followUp;
  private CallLogger callLogger;

  public CallMonitor(CallLogger callLogger) {
    this.callLogger = callLogger;
  }

  public void setFollowUp(SocketCallable followUp) {
    this.followUp = followUp;
  }

  public void takeCall(SocketHandler handler) {
    final PacketLogger logger = callLogger.makePacketLogger();
    handler.getPacketBus().addReceiver(new Port<Packet>() {
      public void receive(Packet packet) {
        logger.logPacket(packet);
      }
    });
    try {
      followUp.takeCall(handler);
    } finally {
      logger.closeLog();
    }
  }

  public static void main(String[] args) throws IOException {
    ObjectServer objectServer = new ObjectServer();
    objectServer.addGuard(new LocalCallsOnly());
    objectServer.start();
  }
}