/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t13;

import drjava.util.GUIUtil;
import static drjava.util.ObjectUtil.nice;
import prophecy.common.Prophecy;
import prophecy.common.gui.Sheet;
import prophecy.common.ClassData;
import prophecy.common.socket.ObjectURL;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.t.t07.SocketConsole;

import javax.swing.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

public class ConsoleConnectDialog extends Sheet {
  private ClassData classData = ClassData.get(this);
  private JTextField tfURL;

  public ConsoleConnectDialog() {
    setTitle("Connect to remote object");
    tfURL = new JTextField(getMostRecentURL());
    GUIUtil.setMinimumWidth(tfURL, 300);
    addComponent(GUIUtil.withTitle("URL", tfURL));
    ActionListener connectAction = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        connect();
      }
    };

    addButton("Connect", connectAction);
    tfURL.addActionListener(connectAction);
  }

  private String getMostRecentURL() {
    return classData.getString("mostRecentURL", "obj:socket[400]");
  }

  private void setMostRecentURL(String url) {
    classData.setString("mostRecentURL", url);
    classData.save();
  }

  private void connect() {
    try {
      setMostRecentURL(tfURL.getText());
      ObjectURL url = new ObjectURL(ObjectURL.fixURL(tfURL.getText()));
      GUIUtil.showFrame(SocketConsole.make(url).makeFrame());
      disposeFrame();
    } catch (Throwable e) {
      e.printStackTrace();
      JOptionPane.showMessageDialog(null, nice(e));
    }
  }

  public static void main(String[] args) {
    Prophecy.init();
    new ConsoleConnectDialog().showFrame();
  }

  public static class MakeJar {
    public static void main(String[] args) throws IOException {
      ProphecyJarMaker.makeJar("standalone/console.jar", ConsoleConnectDialog.class);
    }
  }
}