/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t13;

import static drjava.util.ObjectUtil.nice;
import drjava.util.Tree;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.SocketCallable;
import prophecy.common.SafetyUtil;
import prophecy.t.t06.Cmd;
import prophecy.t.t06.TreeCLI;

import java.io.File;

public class LogOperator implements SocketCallable {
  private String logDir;
  private FileAppender appender;
  private String logName;

  public LogOperator(String logDir) {
    this.logDir = logDir;
  }

  public void takeCall(SocketHandler handler) {
    handler.ok(new Tree(nice(this) + " ready"));
    TreeCLI cli = new TreeCLI();
    cli.addStandardCmds();
    cli.addCmd(new Open());
    cli.addCmd(new Log());
    cli.addCmd(new Close());
    try {
      cli.loop(handler);
    } finally {
      closeLog();
    }
  }

  private class Open extends Cmd {
    public void run(Tree input, SocketHandler handler) {
      closeLog();
      String name = input.getString("name");
      if (name == null) name = input.getString(0);
      if (name == null) name = "log";
      name = cleanLogName(name);
      File file = new File(logDir, name + ".txt");
      if (file.exists()) {
        int i = 2;
        while ((file = new File(logDir, name + "-" + i + ".txt")).exists())
          ++i;
      }
      appender = new FileAppender(file.getPath());
      logName = file.getName().substring(0, file.getName().lastIndexOf('.'));
      appender.start();
      handler.ok(new Tree("Log opened").addString(logName));
    }
  }

  private class Log extends Cmd {
    public void run(Tree input, SocketHandler handler) {
      if (appender != null) {
        Tree tree = input.get(0);
        String text = tree.toString();
        appender.println(text);
        handler.ok(new Tree("Line logged").add("characters", text.length()));
      } else
        handler.error(new Tree("Please open a log first"));
    }
  }

  private class Close extends Cmd {
    public void run(Tree input, SocketHandler handler) {
      if (appender != null) {
        String logName = LogOperator.this.logName;
        closeLog();
        handler.ok(new Tree("Log closed").addString(logName));
      }
    }
  }

  private void closeLog() {
    if (appender != null) {
      appender.close();
      appender = null;
      logName = null;
    }
  }

  private String cleanLogName(String name) {
    return SafetyUtil.cleanFileName(name);
  }
}