package prophecy.t.t16;

import static drjava.util.ObjectUtil.nice;

import drjava.util.Errors;
import drjava.util.Tree;
import prophecy.common.SafetyUtil;
import prophecy.common.socket.SocketCallable;
import prophecy.common.socket.SocketHandler;
import prophecy.m.m10.IntIdMaker;
import prophecy.p.p02.ObjectDatabase;
import prophecy.t.t06.Cmd;
import prophecy.t.t06.TreeCLI;

public class AgentEntryPoint implements SocketCallable {
  private ObjectDatabase objectDatabase;
  private int counter;
  private IntIdMaker idMaker;

  public AgentEntryPoint(ObjectDatabase objectDatabase) {
    this.objectDatabase = objectDatabase;
  }

  public void takeCall(SocketHandler handler) {
    handler.ok(new Tree("ready").add("name", nice(this)));
    TreeCLI cli = new TreeCLI();
    cli.addStandardCmds();
    cli.addCmd(new UploadAgent());
    cli.addCmd(new RunAgent());
    cli.loop(handler);
  }

  private class UploadAgent extends Cmd {
    public void run(Tree input, SocketHandler handler) {
      String jarName = input.getString("jarName");
      objectDatabase.addJar(jarName, input.getBytes("jarContents"));
      String objectName = objectDatabase.addObject(input.getString("objectName"), jarName, input.get("objectInfo"));
      handler.ok(new Tree("Agent uploaded").add("objectName", objectName));
    }
  }

  private class RunAgent extends Cmd {
    public void run(Tree input, SocketHandler handler) {
      // save agent info + jar in db

      String number = newAgentNumber();
      String agentName = number + "-" + SafetyUtil.cleanFileName(input.getString("agentName"));
      String jarName = agentName + ".jar";
      objectDatabase.addJar(jarName, input.getBytes("jarContents"));
      String objectName = objectDatabase.addObject(agentName, jarName, input.get("objectInfo"));

      // create agent and run
      Object agent = objectDatabase.loadObject(objectName);
      if (!(agent instanceof Runnable)) {
        handler.error(new Tree("Agent not runnable").add("class", agent.getClass().getName()));
        return;
      }

      try {
        ((Runnable) agent).run();
        handler.ok(new Tree("Agent run").add("agentName", agentName));
      } catch (Throwable e) {
        handler.error(new Tree("Agent run with exception")
          .add("agentName", agentName)
          .add("exception", Errors.getStackTrace(e)));
      }
    }
  }

  private String newAgentNumber() {
    if (idMaker == null) {
      idMaker = new IntIdMaker();
      for (String name : objectDatabase.getObjectNames())
        idMaker.fileFound(name);
    }
    return idMaker.makeID();
  }
}
