package prophecy.t.t16;

import drjava.util.StringUtil;
import drjava.util.Tree;
import drjava.util.TreeUtil;
import static drjava.util.ObjectUtil.nice;
import prophecy.m.m26.JarMaker;
import prophecy.m.m26.ProphecyJarMaker;
import prophecy.p.p01.Jar;
import prophecy.p.p01.JarClassLoader;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.util.Enumeration;
import java.util.Set;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

public class HullMaker {
  private String tempJar = "temp/hullmaker.jar";

  public HullMaker() {
    try {
      ProphecyJarMaker.makeJar(tempJar);
    } catch (IOException e) {
      throw new RuntimeException(e);
    }
  }

  public HullMaker(String tempJar) {
    this.tempJar = tempJar;
  }

  public Hull makeHull(Object treeifiableObject) {
    try {
      final Tree objectInfo = TreeUtil.objectToTree(treeifiableObject);

      JarClassLoader classLoader = new JarClassLoader(new Jar(new File(tempJar)));
      Class treeUtilClass = classLoader.loadClass(TreeUtil.class.getName());
      Method stringToObject = treeUtilClass.getMethod("stringToObject", String.class);
      Object object = stringToObject.invoke(null, objectInfo.toString());
      System.out.println("Object: " + object + " loaded by " + object.getClass().getClassLoader());
      testObject(object);
      final Set<String> classNames = classLoader.getClassesLoadedFromJar();
      final Set<String> resources = classLoader.getResourcesLoadedFromJar();
      System.out.println("Hull size: "
        + StringUtil.n(classNames.size(), "class", "classes")
        + ", " + StringUtil.n(resources.size(), "resource"));

      return new Hull() {
        public Tree getObjectInfo() {
          return objectInfo;
        }

        public void makeJar(String hullFile) {
          try {
            makeHullJar(tempJar, classNames, resources, hullFile);
          } catch (IOException e) {
            throw new RuntimeException(e);
          }
        }
      };
    } catch (Exception e) {
      throw new RuntimeException(e);
    }
  }

  public void testObject(Object object) {
    selfTest(object);
  }

  public static void selfTest(Object object) {
    try {
      Object result = object.getClass().getMethod("selfTest").invoke(object);
      System.out.println("Self test result: " + nice(result));
      // TODO: check test result
    } catch (NoSuchMethodException e) {
      // can't self-test
    } catch (Throwable e) {
      throw new RuntimeException("Self test failed", e);
    }
  }

  private static void makeHullJar(String jarFile, Set<String> classNames, 
                                  Set<String> resources, String hullFile) throws IOException {
    JarMaker jarMaker = new JarMaker(hullFile, new Manifest());
    jarMaker.begin();
    JarFile jar = new JarFile(jarFile);
    Enumeration<JarEntry> e = jar.entries();
    while (e.hasMoreElements()) {
      JarEntry entry = e.nextElement();
      String name = entry.getName();
      boolean copy = false;
      if (name.endsWith(".class")) {
        String className = name.substring(0, name.length() - 6).replace("/", ".");
        if (classNames.contains(className))
          copy = true;
      }
      if (resources.contains(name))
        copy = true;
      if (copy)
        jarMaker.copyJarEntry(jar, entry);
    }
    jarMaker.end();
  }

}
