/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.t.t17;

import drjava.util.MultiCoreUtil;
import drjava.util.Tree;
import prophecy.common.socket.SocketCallable;
import prophecy.common.socket.SocketHandler;
import prophecy.common.socket.ObjectServer;
import prophecy.t.t06.Cmd;
import prophecy.t.t06.TreeCLI;
import prophecy.t.t11.SocketGuard;

import java.io.IOException;

public class PasswordCheck implements SocketGuard {
  private SocketCallable followUp;
  private String password;

  public PasswordCheck(String password) {
    this.password = password;
  }

  public void setFollowUp(SocketCallable followUp) {
    this.followUp = followUp;
  }

  public void takeCall(SocketHandler handler) {
    new Session().takeCall(handler);
  }

  private class Session {
    private boolean passOK;

    public void takeCall(SocketHandler handler) {
      handler.request(new Tree("Please enter password"));

      TreeCLI cli = new StandardCLI();
      cli.addCmd(new Try());
      cli.loop(handler);
      if (passOK) {
        // ok
        followUp.takeCall(handler);
      } else {
        // close without notice
        System.out.println("PasswordCheck: Call rejected");
      }
    }

    private class Try extends Cmd {
      public void run(Tree input, SocketHandler handler) {
        if (password.equals(input.getString(0))) {
          passOK = true;
          handler.ok(new Tree("Password correct"));
          TreeCLI.exitLoop();
        } else {
          handler.error(new Tree("Bad password"));
          MultiCoreUtil.sleep(1000);
        }
      }
    }
  }

  public static void main(String[] args) throws IOException {
    ObjectServer objectServer = new ObjectServer();
    objectServer.addGuard(new PasswordCheck("geheim"));
    objectServer.start();
  }

  public void setPassword(String password) {
    this.password = password;
  }
}