/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v01;

import drjava.util.Errors;
import drjava.util.GUIUtil;
import prophecy.common.Prophecy;
import prophecy.common.image.ImageSet;
import prophecy.video.v06.SaveButton;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

public class ImageBrowser extends JFrame {
  public ImageBrowser(String imageDir, final String defaultObjectName) {
    super("Image Browser [Project Prophecy]");
    setSize(800, 600);
    ImagesPanel imagesPanel = new ImagesPanel() {
      protected JComponent visualize(final ImageSet.Image image) {
        JPanel topPanel = new JPanel(new BorderLayout());
        final JTextArea textArea = new JTextArea(3, 20);
        textArea.setText(image.comment);
        topPanel.add(BorderLayout.CENTER, new JScrollPane(textArea));

        final JButton btnSave = new SaveButton("Save");
        btnSave.setEnabled(false);
        btnSave.addActionListener(new ActionListener() {
          public void actionPerformed(ActionEvent e) {
            try {
              File file = image.image.getFile();
              image.comment = textArea.getText();
              JpegUtil.changeComment(file, image.comment);
            } catch (IOException e1) {
              Errors.add(e1);
            }
            btnSave.setEnabled(false);
          }
        });
        topPanel.add(BorderLayout.EAST, btnSave);

        JPanel panel = new JPanel(new BorderLayout());
        panel.add(BorderLayout.NORTH, topPanel);

        final ImageMetaSurface surface = new ImageMetaSurface(image) {
          protected void mouseStroke(Point startingPoint, Point endPoint) {
            String comment = ImageMeta.append(textArea.getText(), new ImageMeta.Object(defaultObjectName,
              new Rectangle(startingPoint.x, startingPoint.y,
                endPoint.x-startingPoint.x, endPoint.y-startingPoint.y)));
            textArea.setText(comment);
          }
        };

        panel.add(BorderLayout.CENTER, surface);
        textArea.getDocument().addDocumentListener(new DocumentListener() {
          public void insertUpdate(DocumentEvent e) {
            update();
          }

          public void removeUpdate(DocumentEvent e) {
            update();
          }

          public void changedUpdate(DocumentEvent e) {
            update();
          }

          private void update() {
            surface.setComment(textArea.getText());
            btnSave.setEnabled(true);
          }
        });

        return panel;
      }
    };
    getContentPane().add(imagesPanel.makePanel(ImageSet.allImages(imageDir), imageDir));
  }

  public static void main(String[] args) {
    GUIUtil.showMainFrame(new ImageBrowser(Prophecy.dataDir() + "ball", "ball"));
  }
}