/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v01;

import drjava.util.Errors;
import prophecy.common.image.ImageSet;
import prophecy.video.v10.Bounded;

import java.awt.*;
import static java.lang.Integer.parseInt;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ImageMeta {
  private List<Object> objects = new ArrayList<Object>();

  public ImageMeta() {
  }

  public ImageMeta(ImageSet.Image image) {
    this(image.comment);
  }

  public ImageMeta(String comment) {
    Matcher matcher = Pattern.compile("(\\S+)=(\\d+),(\\d+)([,/])(\\d+),(\\d+)").matcher(comment);
    while (matcher.find()) {
      try {
        int x1 = parseInt(matcher.group(2)), y1 = parseInt(matcher.group(3));
        int x2 = parseInt(matcher.group(5)), y2 = parseInt(matcher.group(6));
        if (matcher.group(4).equals(",")) {
          x2 -= x1;
          y2 -= y1;
        }
        Rectangle r = new Rectangle(x1, y1, x2, y2);
        objects.add(new Object(matcher.group(1), r));
      } catch (Throwable e) {
        Errors.add(e);
      }
    }
  }

  public ImageMeta(String objectName, List<Rectangle> rects) {
    add(objectName, rects);
  }

  public List<Object> getObjects() {
    return objects;
  }

  public void add(String objectName, List<Rectangle> rects) {
    for (Rectangle rect : rects)
      addObject(new ImageMeta.Object(objectName, rect));
  }

  public ImageMeta(List<? extends Bounded> list) {
    for (Bounded bounded : list) {
      addObject(new Object("", bounded.getBounds()));
    }
  }

  public static class Object {
    public final String name;
    public final Rectangle rect;

    public Object(String name, Rectangle rect) {
      this.name = name;
      this.rect = rect;
    }
  }

  public static String append(String comment, Object object) {
    if (comment.length() != 0) comment += "\n";
    comment += object.name + "=" + object.rect.x + "," + object.rect.y
      + "/" + object.rect.width + "," + object.rect.height;
    return comment;
  }

  public void addObject(Object object) {
    objects.add(object);
  }
}