/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v01;

import prophecy.common.image.ImageSet;
import prophecy.common.image.ImageSurface;
import prophecy.common.image.RGBImage;

import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;

public class ImageMetaSurface extends ImageSurface {
  private ImageMeta meta;

  public ImageMetaSurface() {
    this(RGBImage.dummyImage());
  }

  public ImageMetaSurface(final ImageSet.Image image) {
    this(image.image, new ImageMeta(image));
  }

  public ImageMetaSurface(final RGBImage image, ImageMeta meta) {
    super(image);
    this.meta = meta;
    addMouseListener(new MouseAdapter() {
      public Point startingPoint;

      public void mousePressed(MouseEvent e) {
        startingPoint = e.getPoint();
      }

      public void mouseReleased(MouseEvent e) {
        if (startingPoint != null) {
          mouseStroke(startingPoint, e.getPoint());
          startingPoint = null;
        }
      }
    });
  }

  public ImageMetaSurface(RGBImage rgbImage) {
    this(rgbImage, new ImageMeta());
  }

  protected void mouseStroke(Point startingPoint, Point endPoint) {
  }

  public void render(int w, int h, Graphics2D g) {
    super.render(w, h, g);
    g.setTransform(AffineTransform.getScaleInstance(getZoomX(), getZoomY()));
    FontMetrics fontMetrics = g.getFontMetrics();
    for (ImageMeta.Object object : meta.getObjects()) {
      g.setColor(Color.yellow);
      setAlpha(g, 0.25f);
      int objW = Math.max(2, object.rect.width);
      int objH = Math.max(2, object.rect.height);
      g.fillRect(object.rect.x, object.rect.y, objW, objH);
      setAlpha(g, 0.5f);
      g.setColor(Color.blue);
      drawCorners(g, object.rect.x, object.rect.y, objW, objH);
      //int y = object.rect.y + fontMetrics.getMaxAscent() + 2;
      int y = Math.max(fontMetrics.getMaxAscent(), object.rect.y - fontMetrics.getMaxDescent() - 2);
      int x = object.rect.x + (objW - fontMetrics.stringWidth(object.name))/2;
      g.drawString(object.name, x, y);
    }
  }

  private void drawCorners(Graphics2D g, int x, int y, int w, int h) {
    int inset = 2, size = 3;
    drawCorner(g, x-inset, y-inset, size, size);
    drawCorner(g, x+w+inset, y-inset, -size, size);
    drawCorner(g, x+w+inset, y+h+inset, -size, -size);
    drawCorner(g, x-inset, y+h+inset, size, -size);
  }

  private void drawCorner(Graphics2D g, int x, int y, int dx, int dy) {
    g.setStroke(new BasicStroke(2));
    g.drawLine(x, y, x+dx, y);
    g.drawLine(x, y, x, y+dy);
  }

  public void setComment(String text) {
    meta = new ImageMeta(text);
    repaint();
  }

  public String toString() {
    return "Annotated image";
  }

  public void setMeta(ImageMeta meta) {
    this.meta = meta;
  }

  public ImageMeta getImageMeta() {
    return meta;
  }
}