/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v02;

import prophecy.common.Contest;
import prophecy.common.image.ImageSet;

import java.awt.*;
import static java.lang.Math.*;
import java.util.ArrayList;
import java.util.List;

public class EyeDetectionContest extends Contest<EyeDetector> {
  private List<EyesImage> images = new ArrayList<EyesImage>();

  public EyeDetectionContest(ImageSet imageSet) {
    super(EyeDetector.class);
    for (ImageSet.Image image : imageSet.getImages()) {
      images.add(new EyesImage(image));
    }
  }

  public double evaluate(EyeDetector contestant) {
    double score = 0;
    for (EyesImage image : images) {
      List<Point> points = contestant.findEyes(image.getImage());
      double real = image.getEyes().size();
      score -= min(1, abs(points.size()-real)/max(2, real));
    }
    return score;
  }

  public Double getBestPossibleScore() {
    return 0.0;
  }

  public Double getWorstPossibleScore() {
    return (double) -images.size();
  }

  public List<EyesImage> getImages() {
    return images;
  }
}