/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v03;

import drjava.util.Function;
import drjava.util.LetterLayout;
import prophecy.common.image.ImageProcessing;
import static prophecy.common.image.ImageProcessing.*;
import prophecy.common.image.RGBImage;

import javax.swing.*;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

public class EyesPanel extends JPanel {
  private JComponent eyesComponent;

  public EyesPanel(final EyeExamples examples) {
    setLayout(new LetterLayout("C", "E", "E").setSpacing(10).setBorder(10));
    final double zoom = 1.5;
    eyesComponent = examples.displayAll(zoom);
    add("E", eyesComponent);

    List<ImageProcessor> processors = new ArrayList<ImageProcessor>();

    processors.add(new ImageProcessor("") {
      public RGBImage process(RGBImage image) {
        return image;
      }
    });

    processors.add(new ImageProcessor("b/w 50") {
      public RGBImage process(RGBImage image) {
        return threshold(image.toBW(), 0.5).toRGB();
      }
    });

    processors.add(new UpperLower(1, 99));
    processors.add(new UpperLower(5, 99));
    processors.add(new UpperLower(10, 90));
    processors.add(new UpperLower(20, 80));
    processors.add(new UpperLower(50, 50));
    processors.add(new UpperLower(30, 90));
    processors.add(new UpperLower(10, 70));

    processors.add(new ImageProcessor("hdiff+vdiff") {
      public RGBImage process(RGBImage image) {
        return ImageProcessing.add(hdiff(image), vdiff(image)).toRGB();
      }
    });

    processors.add(new ImageProcessor("square") {
      public RGBImage process(RGBImage image) {
        return ImageProcessing.apply(image.toBW(), new Function<Float, Float>() {
          public Float get(Float pixel) {
            return pixel*pixel;
          }
        }).toRGB();
      }
    });

    processors.add(new ImageProcessor("squareroot") {
      public RGBImage process(RGBImage image) {
        return ImageProcessing.apply(image.toBW(), new Function<Float, Float>() {
          public Float get(Float pixel) {
            return (float) Math.sqrt(pixel);
          }
        }).toRGB();
      }
    });

    final JComboBox[] comboBoxes = new JComboBox[3];
    JPanel comboBoxPanel = new JPanel(LetterLayout.stalactite().setSpacing(10));
    ItemListener itemListener = new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        EyeExamples images = examples;
        for (JComboBox comboBox : comboBoxes) {
          ImageProcessor processor = (ImageProcessor) comboBox.getSelectedItem();
          images = process(images, processor);
        }

        remove(eyesComponent);
        eyesComponent = images.displayAll(zoom);
        add("E", eyesComponent);
        revalidate();
      }
    };
    for (int i = 0; i < comboBoxes.length; i++) {
      comboBoxes[i] = new JComboBox(new Vector<ImageProcessor>(processors));
      comboBoxes[i].addItemListener(itemListener);
      comboBoxPanel.add(comboBoxes[i]);
    }

    add("C", comboBoxPanel);
  }

  private EyeExamples process(EyeExamples examples, ImageProcessor processor) {
    EyeExamples result = new EyeExamples();
    for (RGBImage image : examples.getEyeImages()) {
      result.add(processor.process(image));
    }
    return result;
  }

}