/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v03;

import drjava.util.Function;
import prophecy.common.image.BWImage;

public class Histogram {
  double[] data = new double[256];
  private int sum;

  public Histogram(BWImage image) {
    for (int y = 0; y < image.getHeight(); y++)
      for (int x = 0; x < image.getWidth(); x++)
        ++data[((int) (image.getPixel(x, y) * 255))];
    sum = image.getHeight()*image.getWidth();
  }

  public Histogram(double[] data) {
    this.data = data;
    sum = 0;
    for (int i = 0; i < data.length; i++)
      sum += data[i];
  }

  public double getThresholdBrightness(double pixelFraction) {
    int i = 0, sum = 0, target = (int) (pixelFraction* this.sum);
    while (sum < target && i < 255) {
      sum += data[i];
      ++i;
    }
    return i/255.0;
  }

  public int getSections() {
    return data.length;
  }

  public double[] getData() {
    return data;
  }

  public int getSum() {
    return sum;
  }

  public double getMaxValue() {
    double max = 0;
    for (double i : data)
      max = Math.max(max, i);
    return max;
  }

  public double getData(int section) {
    return data[section];
  }

  public Histogram integrate() {
    double[] newData = new double[data.length];
    double sum = 0;
    for (int i = 0; i < data.length; i++)
      newData[i] = (sum += data[i]);
    return new Histogram(newData);
  }

  public Histogram map(Function<Double, Double> function) {
    double[] newData = new double[data.length];
    for (int i = 0; i < data.length; i++)
      newData[i] = function.get(data[i]);
    return new Histogram(newData);
  }
}