/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v03;

import drjava.util.Function;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGBImage;

class UpperLower extends ImageProcessor {
  private int pLower;
  private int pUpper;

  public UpperLower(int pLower, int pUpper) {
    super("normalize " + pLower + "%-" + pUpper + "%");
    this.pLower = pLower;
    this.pUpper = pUpper;
  }

  public RGBImage process(RGBImage image) {
    BWImage bw = image.toBW();
    Histogram histogram = new Histogram(bw);
    final double lower = histogram.getThresholdBrightness(pLower*0.01);
    final double upper = Math.max(histogram.getThresholdBrightness(pUpper*0.01), lower+0.01);
    /*double threshold = (lower+upper)/2;
    System.out.println("lower=" + formatProbability(lower) + " upper=" + formatProbability(upper)
      + " threshold=" + formatProbability(threshold));
    return ImageProcessing.threshold(bw, threshold).toRGB();*/
    return ImageProcessing.apply(bw, new Function<Float, Float>() {
      public Float get(Float pixel) {
        return (pixel-(float) lower)/((float) upper-(float) lower);
      }
    }).toRGB();
  }
}