/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v04;

import drjava.util.Errors;
import drjava.util.LetterLayout;
import prophecy.common.Prophecy;
import prophecy.common.SurfaceUtil;
import prophecy.common.image.RGBImage;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetAdapter;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.List;

public class V04 {
  public static void main(String[] args) {
    JPanel panel = new JPanel(new LetterLayout("C", "S", "S").setBorder(10).setSpacing(10));
    JPanel controlPanel = new JPanel(new LetterLayout("IDsSS").setSpacing(10));
    final NeighborsSurface surface = new NeighborsSurface(RGBImage.load(Prophecy.dataDir() + "jazztracking/jazzfeuer1.jpg"));
    surface.setMinSimilarity(-0.15);
    panel.add("S", surface);
    final JSlider slider = new JSlider(-50, 0, JSlider.HORIZONTAL);
    slider.setValue((int) (surface.getMinSimilarity()/0.01));
    slider.setMajorTickSpacing(5);
    slider.setPaintTicks(true);
    slider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        if (!slider.getValueIsAdjusting())
          surface.setMinSimilarity(slider.getValue()*0.01);
      }
    });
    controlPanel.add("s", new JLabel("Brightness:"));
    controlPanel.add("S", slider);

    final JCheckBox diagonal = new JCheckBox("diagonal", surface.getDiagonal());
    diagonal.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        surface.setDiagonal(diagonal.isSelected());
      }
    });
    controlPanel.add("D", diagonal);

    final JCheckBox showImage = new JCheckBox("image", surface.getShowImage());
    showImage.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        surface.setShowImage(showImage.isSelected());
      }
    });
    controlPanel.add("I", showImage);

    new DropTarget(surface, new DropTargetAdapter() {
      public void drop(DropTargetDropEvent evt) {
        handleDrop(evt, surface);
      }
    });

    panel.add("C", controlPanel);
    SurfaceUtil.showAsMain("V04", panel);
  }

  private static void handleDrop(DropTargetDropEvent evt, NeighborsSurface surface) {
    try {
      Transferable tr = evt.getTransferable();
      DataFlavor[] flavors = tr.getTransferDataFlavors();
      for (DataFlavor flavor : flavors) {
        System.out.println("flavor: " + flavor);
        if (flavor.isFlavorJavaFileListType()) {
          evt.acceptDrop(evt.getDropAction());
          List list = (List) tr.getTransferData(flavor);
          evt.dropComplete(true);
          for (Object file : list) {
            if (file instanceof File) {
              surface.setImage(RGBImage.load((File) file));
              return;
            }
          }
          return;
        }
        /*if (flavor.isFlavorTextType()
          || flavor.isMimeTypeEqual(DataFlavor.javaJVMLocalObjectMimeType)) {
          evt.acceptDrop(evt.getDropAction());
          Object object = tr.getTransferData(flavor);
          evt.dropComplete(true);
          collectObject(object);
          return;
        }*/
      }
      System.out.println("Unknown data flavor");
    } catch (Throwable t) {
      Errors.add(t);
    }
    evt.rejectDrop();
  }
}