/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v05;

import drjava.util.Randomizer;
import drjava.util.RealRandomizer;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class Worm {
  List<Rectangle> segments = new ArrayList<Rectangle>();
  private Randomizer randomizer = new RealRandomizer();

  public Worm(Neighbors neighbors) {
    Neighbors.Clip clip;
    double maxDiff;
    int attempts = 0, x, y;
    Neighbors.Clip[][] clips = neighbors.getClips();
    int clipSize = neighbors.getClipSize();
    do {
      y = randomizer.random(clips.length);
      x = randomizer.random(clips[y].length);
      clip = clips[y][x];
      if (clip == null) return;
      maxDiff = maxDiff(clip.getNeighborDiffs());
    } while (maxDiff < 0.5 && ++attempts < 5);

    segments.add(clip.getRect());

    int lastDirection = -1;
    for (int i = 0; i < 10; i++) {
      double[] diffs = new double[8];
      System.arraycopy(clip.getNeighborDiffs(), 0, diffs, 0, 8);
      if (lastDirection != -1)
        diffs[(lastDirection+4) / 8] = 1.0; // don't go straight back
      int direction = findBestDirection(diffs);
      Rectangle r = clip.getNeighbor(direction);
      if (r == null)
        break;
      segments.add(r);
      y = r.y/clipSize;
      x = r.x/clipSize;
      clip = clips[y][x];
      lastDirection = direction;
    }
  }

  private double maxDiff(double[] diffs) {
    double max = 0;
    for (double diff : diffs) {
      if (diff > max) max = diff;
    }
    return max;
  }

  private int findBestDirection(double[] diffs) {
    int best = -1;
    double bestVal = 0;
    for (int i = 0; i < 8; i++) {
      double val = evalDirection(diffs, i);
      if (best == -1 || val > bestVal) {
        best = i;
        bestVal = val;
      }
    }
    return best;
  }

  private double evalDirection(double[] diffs, int i) {
    double ahead = diffs[i];
    double left = diffs[(i-2) & 7];
    double right = diffs[(i+2) & 7];
    return (left+right)/2-ahead;
  }

  public List<Rectangle> getSegments() {
    return segments;
  }
}