/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v05;

import prophecy.common.Prophecy;
import prophecy.common.AnimatingSurface;
import prophecy.common.SurfaceUtil;
import prophecy.common.image.RGBImage;

import java.awt.*;
import java.util.ArrayList;

public class Worms extends AnimatingSurface {
  private Neighbors neighbors;
  private ArrayList<Worm> worms = new ArrayList<Worm>();

  public Worms(Neighbors neighbors) {
    this.neighbors = neighbors;
  }

  public void step(int w, int h) {
    makeWorms();
  }

  private void makeWorms() {
    worms = new ArrayList<Worm>();
    for (int i = 0; i < 50; i++)
      worms.add(new Worm(neighbors));
  }

  public void reset(int newwidth, int newheight) {
  }

  public void render(int w, int h, Graphics2D g) {
    /*g.setColor(Color.white);
    g.fillRect(0, 0, w, h);*/
    g.drawImage(neighbors.getImage().getBufferedImage(), 0, 0, null);
    g.setColor(Color.black);
    g.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0.5f));
    for (Worm worm : worms) {
      for (Rectangle segment : worm.getSegments()) {
        g.fillRect(segment.x+2, segment.y+2, segment.width-4, segment.height-4);
      }
    }
  }

  public static void main(String[] args) {
    Neighbors neighbors = new Neighbors(RGBImage.load(Prophecy.dataDir() + "jazztracking/1.jpg"));
    Worms worms = new Worms(neighbors);
    SurfaceUtil.showAsMain("V05", worms);
    worms.start();
  }
}