/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v06;

import static drjava.util.GUIUtil.showFrame;
import drjava.util.LetterLayout;
import drjava.util.Tree;
import prophecy.common.ContestOnDisk;
import prophecy.common.ContestRun;
import drjava.util.Errors;
import prophecy.common.ContestInfo;

import javax.swing.*;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.List;

public class ContestPanel extends JPanel {
  private ContestOnDisk contestOnDisk;
  private OngoingContest ongoingContest;

  public ContestPanel(final ContestOnDisk contestOnDisk, boolean editable) {
    this.contestOnDisk = contestOnDisk;
    ongoingContest = new OngoingContest(contestOnDisk);
    final ContestInfo contestInfo = contestOnDisk.getContestInfo();

    setLayout(new LetterLayout("aAA", "bBB", "cCC", "dDD", "eEE", "fFF",
      "HHH", "HHH", "ZZZ").setSpacing(5).setBorder(10));
    JPanel buttonPanel = new JPanel(LetterLayout.leftAlignedRow().setSpacing(10));
    add("a", new JLabel("Contest ID"));
    add("A", new JLabel(contestOnDisk.getFullName()));
    add("b", new JLabel("Short description"));
    final JTextField tfShortDescription = new JTextField(contestInfo.getShortDescription());
    add("B", tfShortDescription);
    add("c", new JLabel("Long description"));
    final JTextArea taLongDescription = new JTextArea(contestInfo.getLongDescription());
    add("C", new JScrollPane(taLongDescription));
    add("d", new JLabel("Author"));
    final JTextField tfAuthor = new JTextField(contestInfo.getAuthor());
    add("D", tfAuthor);
    add("e", new JLabel("Contest class"));
    add("E", new JLabel(ongoingContest.getContest() == null ? "<load error, see log>"
      : ongoingContest.getContest().getClass().getName()));
    add("f", new JLabel("Current champion"));

    if (ongoingContest.getHallOfFame() == null) {
      add("F", new JLabel("<load error, see log>"));
    } else {
      Tree champion = ongoingContest.getHallOfFame().getWinner();
      add("F", new JLabel(champion == null ? "<none>" : HallOfFameTableModel.getContestantName(champion)));

      final HallOfFameTableModel tableModel = new HallOfFameTableModel(ongoingContest.getHallOfFame());
      final JTable table = new JTable(tableModel);
      table.getColumnModel().getColumn(0).setMaxWidth(78);
      table.getColumnModel().getColumn(1).setMaxWidth(78);

      JPanel hallOfFamePanel = new JPanel(new LetterLayout("h", "H", "H").setSpacing(5).setTopBorder(10));
      hallOfFamePanel.add("h", new JLabel("Hall of fame", JLabel.CENTER));
      hallOfFamePanel.add("H", new JScrollPane(table));
      add("H", hallOfFamePanel);

      JButton btnVerifyAll = new JButton("Verify scores");
      btnVerifyAll.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
          /*LogFrame logFrame = new LogFrame("Contest run " + contestOnDisk.getFullName());
          GUIUtil.showFrame(logFrame);*/
          //logFrame.catchStdOut();
          try {
            ContestRun contestRun = new ContestRun(ongoingContest.getContest());
            List objects = ongoingContest.getHallOfFameObjects();
            for (Object o : objects) {
              if (o != null) contestRun.addContestant(o);
            }
            contestRun.run();
            //logFrame.logDisplayable(contestRun);

            // write back verified scores - looks a bit hacky, but works fine

            List<Tree> contestants = ongoingContest.getHallOfFame().getList();
            for (int i = 0; i < objects.size(); i++)
              if (objects.get(i) != null) {
                tableModel.getVerifiedScores().add(contestants.get(i),
                  contestRun.getScores().getScore(objects.get(i)));
              }
            tableModel.fireTableDataChanged();
          } catch (Throwable e) {
            Errors.add(e);
          } finally {
            //logFrame.stopCatching();
          }
        }
      });
      buttonPanel.add(btnVerifyAll);

      final JButton btnShowContestant = new JButton("Show contestant");
      btnShowContestant.setEnabled(false);
      table.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
        public void valueChanged(ListSelectionEvent e) {
          btnShowContestant.setEnabled(table.getSelectedRowCount() != 0);
        }
      });
      btnShowContestant.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          LogFrame logFrame = new LogFrame(
            tableModel.getContestantName(table.getSelectedRow()));
          logFrame.logDisplayable(tableModel.getItem(table.getSelectedRow()));
          showFrame(logFrame);
        }
      });
      buttonPanel.add(btnShowContestant);
    }

    if (editable) {
      final JButton btnSaveInfo = new SaveButton("Save changes");
      btnSaveInfo.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent e) {
          contestInfo.setShortDescription(tfShortDescription.getText().trim());
          contestInfo.setLongDescription(taLongDescription.getText().trim());
          contestInfo.setAuthor(tfAuthor.getText().trim());
          contestOnDisk.saveContestInfo(contestInfo);
          btnSaveInfo.setEnabled(false);
        }
      });
      btnSaveInfo.setEnabled(false);
      DocumentListener documentListener = new DocumentListener() {
        public void insertUpdate(DocumentEvent e) {
          btnSaveInfo.setEnabled(true);
        }

        public void removeUpdate(DocumentEvent e) {
          btnSaveInfo.setEnabled(true);
        }

        public void changedUpdate(DocumentEvent e) {
          btnSaveInfo.setEnabled(true);
        }
      };
      tfShortDescription.getDocument().addDocumentListener(documentListener);
      taLongDescription.getDocument().addDocumentListener(documentListener);
      tfAuthor.getDocument().addDocumentListener(documentListener);
      buttonPanel.add(btnSaveInfo);
    }

    tfShortDescription.setEnabled(editable);
    taLongDescription.setEnabled(editable);
    tfAuthor.setEnabled(editable);

    add("Z", buttonPanel);
  }
}