/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v06;

import static drjava.util.StringUtil.formatDouble;

public class ExpectedNumber implements EvaluationItem {
  private String description;
  private double correctAnswer;
  private double contestantAnswer;
  private int typicalValue;
  private double tolerance;
  private double maxScore;

  public ExpectedNumber(String description, double correctAnswer, double contestantAnswer,
                        int typicalValue, double tolerance, double maxScore) {
    this.description = description;
    this.correctAnswer = correctAnswer;
    this.contestantAnswer = contestantAnswer;
    this.typicalValue = typicalValue;
    this.tolerance = tolerance;
    this.maxScore = maxScore;
  }

  public boolean isCorrect() {
    return Math.abs(correctAnswer-contestantAnswer) <= tolerance;
  }

  public double getScore() {
    double range = Math.max(typicalValue, correctAnswer);
    double diff = Math.abs(contestantAnswer - correctAnswer);
    double diffT = Math.max(0, diff - tolerance);
    return Math.max(0, range-diffT)/range*maxScore;
  }

  public String toString() {
    return "Score " + Evaluation.formatScore(getScore()) + ": " + description + (isCorrect()
      ? " = " + formatDouble(correctAnswer)
      : " (expected " + formatDouble(correctAnswer) + " but was: " + formatDouble(contestantAnswer) + ")");
  }
}