/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v06;

import drjava.util.Errors;
import drjava.util.ObjectUtil;
import drjava.util.Tree;
import static drjava.util.TreeUtil.treeToObject;
import drjava.vats.FulfilledPromise;
import drjava.vats.Promise;
import prophecy.common.HallOfFame;
import prophecy.common.PromiseBasedTableModel;
import prophecy.common.Scores;

public class HallOfFameTableModel extends PromiseBasedTableModel<Tree> {
  private HallOfFame<Tree> hallOfFame;
  private Scores<Tree> verifiedScores = new Scores<Tree>();

  public HallOfFameTableModel(HallOfFame<Tree> hallOfFame) {
    super(new String[] { "Score", "Verified score", "Contestant" }, hallOfFame.getList());
    this.hallOfFame = hallOfFame;
  }

  protected int compareRows(Tree tree, Tree b) {
    return 0;
  }

  protected Promise<String> getCell(Tree tree, int row, int col) {
    if (col == 0)
      return new FulfilledPromise<String>(hallOfFame.formatScore(tree));
    if (col == 1) {
      if (!verifiedScores.contains(tree))
        return null;
      else
        return new FulfilledPromise<String>(verifiedScores.formatScore(tree));
    }
    if (col == 2)
      return new FulfilledPromise<String>(getContestantName(row));
    return null;
  }

  public Scores<Tree> getVerifiedScores() {
    return verifiedScores;
  }

  public String getContestantName(int row) {
    Tree tree = getItem(row);
    return getContestantName(tree);
  }

  public static String getContestantName(Tree tree) {
    try {
      Object o = treeToObject(tree);
      return ObjectUtil.toNiceString(o);
    } catch (Throwable e) {
      Errors.add(e);
    }
    return "<" + tree.getName() + ">";
  }
}