/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v06;

import drjava.util.Tree;
import prophecy.common.Contest;
import prophecy.common.InspectableParent;
import prophecy.common.InspectionNode;
import prophecy.common.InspectionUtil;
import prophecy.common.image.ImageSet;
import prophecy.video.v01.ImageMeta;
import prophecy.video.v01.ImageMetaSurface;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class ObjectDetectionContest extends Contest<ObjectDetector> implements InspectableParent {
  private ImageSet imageSet;
  private String objectName;
  private List<ObjectsImage> images = new ArrayList<ObjectsImage>();
  private Evaluation evaluation;
  private int tolerance = 2;

  public ObjectDetectionContest(ImageSet imageSet, String objectName) {
    super(ObjectDetector.class);
    init(imageSet, objectName);
  }

  private void init(ImageSet imageSet, String objectName) {
    this.imageSet = imageSet;
    this.objectName = objectName;
    for (ImageSet.Image image : imageSet.getImages()) {
      images.add(new ObjectsImage(image, objectName));
    }
  }

  public double evaluate(ObjectDetector contestant) {
    evaluation = new Evaluation();
    for (ObjectsImage image : images) {
      List<Rectangle> rects = contestant.findObjects(image.getImage());
      if (rects == null) rects = new ArrayList<Rectangle>();
      evaluate(image, rects);
    }
    return evaluation.getScore();
  }

  private void evaluate(ObjectsImage image, List<Rectangle> rects) {
    List<Rectangle> rects2 = new ArrayList<Rectangle>(rects);
    double real = image.getObjectPositions().size();
    evaluation.addItem(new ExpectedNumber("Number of objects", real, rects.size(), 2, 0, 10));
    for (Rectangle rectangle : image.getObjectPositions()) {
      Rectangle r = findIntersectingRect(rects2, rectangle);
      if (r == null) continue;
      rects2.remove(r);
      /*evaluation.addItem(new GeneralItem(
        "rect: expected " + toNiceString(rectangle) + ", was " + toNiceString(r),
          rectangle.getCenterX()-));*/
      evaluation.addItem(new ExpectedNumber("center x", rectangle.getCenterX(), r.getCenterX(),
        image.getImage().getWidth()/2, tolerance, 2));
      evaluation.addItem(new ExpectedNumber("center y", rectangle.getCenterY(), r.getCenterY(),
        image.getImage().getWidth()/2, tolerance, 2));
    }
    evaluation.addItem(new Evidence(new ImageMetaSurface(image.getImage(),
      new ImageMeta(objectName, rects))));
  }

  private Rectangle findIntersectingRect(List<Rectangle> rects, Rectangle rectangle) {
    for (Rectangle r : rects) {
      if (rectangle.intersects(r))
        return r;
    }
    return null;
  }

  public Double getBestPossibleScore() {
    return 0.0;
  }

  public Double getWorstPossibleScore() {
    return (double) -images.size();
  }

  public List<ObjectsImage> getImages() {
    return images;
  }

  public Tree toTree() {
    return super.toTree()
      .add("imageSet", imageSet)
      .add("objectName", objectName);
  }

  public ObjectDetectionContest(Tree tree) {
    super(tree);
    init(tree.<ImageSet> getObject("imageSet"), tree.getString("objectName"));
  }

  public String toString() {
    return getClass().getName() + ", " + images.size() + " image(s)";
  }

  public List<InspectionNode> getInspectionChildren() {
    return InspectionUtil.getNodeList(images);
  }

  public Evaluation getEvaluation() {
    return evaluation;
  }
}