/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v06;

import prophecy.common.Visualizable;
import prophecy.common.image.ImageSet;
import prophecy.common.image.ImageSurface;
import prophecy.common.image.RGBImage;
import prophecy.video.v01.ImageMeta;

import javax.swing.*;
import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class ObjectsImage implements Visualizable {
  private RGBImage image;
  private List<Rectangle> objectPositions = new ArrayList<Rectangle>();

  public ObjectsImage(ImageSet.Image image, String objectName) {
    this.image = image.image;
    for (ImageMeta.Object object : new ImageMeta(image).getObjects()) {
      if (object.name.equals(objectName))
        objectPositions.add(object.rect);
    }
  }

  public RGBImage getImage() {
    return image;
  }

  public List<Rectangle> getObjectPositions() {
    return objectPositions;
  }

  public void grow(double factor) {
    for (int i = 0; i < objectPositions.size(); i++)
      objectPositions.set(i, grow(objectPositions.get(i), factor));
  }

  private Rectangle grow(Rectangle r, double factor) {
    return new Rectangle((int) (r.x-r.width*factor), (int) (r.y-r.height*factor),
      (int) (r.width*(1+2*factor)), (int) (r.height*(1+2*factor)));
  }

  public JComponent visualize() {
    return new ImageSurface(image);
  }
}