/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v07;

import drjava.util.LetterLayout;
import prophecy.common.image.AllImages;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.ImageSet;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;
import static java.util.regex.Pattern.CASE_INSENSITIVE;

public class NewContestPanel extends JPanel {
  private List<JCheckBox> checkBoxes;
  private JTextField tfName;
  private AllImages allImages;

  public NewContestPanel(AllImages allImages) {
    this.allImages = allImages;
    JPanel imagesPanel = new JPanel(LetterLayout.stalactite().setSpacing(10));
    checkBoxes = new ArrayList<JCheckBox>();
    for (ImageSet.Image image : allImages.getImages()) {
      JPanel p = new JPanel(new LetterLayout("ci  ").setSpacing(10));
      JCheckBox checkBox = new JCheckBox();
      checkBoxes.add(checkBox);
      p.add("c", checkBox);
      int height = 100;
      int width = image.image.getWidth()*height/image.image.getHeight();
      ImageIcon imageIcon = new ImageIcon(ImageProcessing.resize(image.image, width, height).getBufferedImage());
      p.add("i", new JLabel(imageIcon));
      imagesPanel.add(p);
    }

    JPanel buttonPanel = new JPanel(LetterLayout.leftAlignedRow().setSpacing(10));
    buttonPanel.add(new JLabel("Contest name:"));
    /*final JFormattedTextField tfName;
    try {
      //tfName = new JFormattedTextField(new MaskFormatter("LLLLLLLLLL"));
      DefaultFormatter format = new DefaultFormatter();
      tfName = new JFormattedTextField(format);
    } catch (ParseException e) {
      throw new RuntimeException(e);
    }*/
    tfName = new JTextField();
    tfName.setMinimumSize(new Dimension(75, tfName.getMinimumSize().height));
    buttonPanel.add(tfName);

    JButton btnDefine = new JButton("Define contest");
    btnDefine.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        doIt();
      }
    });
    buttonPanel.add(btnDefine);

    Enabler enabler = new Enabler(btnDefine) {
      public boolean isEnabled() {
        return Pattern.compile("[a-z0-9]+", CASE_INSENSITIVE).matcher(tfName.getText()).matches()
           && anythingChecked();
      }
    };
    enabler.addTrigger(tfName);
    for (JCheckBox checkBox : checkBoxes)
      enabler.addTrigger(checkBox);

    setLayout(new LetterLayout("i", "I", "I", "B").setBorder(10).setSpacing(10));
    add("i", new JLabel("Images to include in contest:"));
    add("I", new JScrollPane(imagesPanel));
    add("B", buttonPanel);
  }

  private void doIt() {
    List<String> images = new ArrayList<String>();
    for (int i = 0; i < checkBoxes.size(); i++) {
      JCheckBox checkBox = checkBoxes.get(i);
      if (checkBox.isSelected())
        images.add(allImages.getImages().get(i).name);
    }
    V07Data data = new V07Data(images);
    //V07.contestGroup.getContest(tfName.getText()).saveData(data);
  }

  private boolean anythingChecked() {
    for (JCheckBox checkBox : checkBoxes)
      if (checkBox.isSelected())
        return true;
    return false;
  }

  public static JFrame makeFrame(AllImages allImages) {
    JFrame frame = new JFrame("New contest");
    frame.setSize(350, 400);
    frame.getContentPane().add(new NewContestPanel(allImages));
    return frame;
  }
}