/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v07;

import static drjava.util.GUIUtil.showMainFrame;
import prophecy.common.Prophecy;
import prophecy.common.gui.Sheet;
import static prophecy.common.InspectionUtil.inspect;
import prophecy.common.SurfaceUtil;
import prophecy.common.image.AllImages;
import prophecy.video.v01.ImageBrowser;
import prophecy.video.v03.Help;
import prophecy.video.v06.ContestPanel;
import prophecy.video.v06.V06;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

public class V07Menu extends Sheet {
  private final String imageDir = Prophecy.dataDir() + "v07";
  private AllImages allImages;

  public static void main(String[] args) {
    showMainFrame(new V07Menu().asFrame());
  }

  public V07Menu() {
    super("V07: Squares [Project Prophecy]");
    setSize(300, 500);

    allImages = new AllImages(imageDir);
    addComponent(Help.makeHelpComponent("Experiment V07 contains "
      + allImages.size() + " image(s) and "
      + V07.contestGroup.getContests().size() + " subcontest(s).", 0));

    addHeading("[management]");

    addButton("Prepare images", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showMainFrame(new ImageBrowser(imageDir, "square"));
      }
    });

    addButton("Define new subcontest", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showMainFrame(NewContestPanel.makeFrame(allImages));
      }
    });

    addHeading("[nerd stuff]");
    addButton("Inspect contest group", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        SurfaceUtil.show("V07 Contest Group", ContestGroupPanel.makePanel(V07.contestGroup));
      }
    });

    addButton("Show contest", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        SurfaceUtil.show("V07", new ContestPanel(V06.contestOnDisk, false));
      }
    });

    addButton("Inspect contest", new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        inspect("V07", V06.contestOnDisk.load());
      }
    });
  }
}