/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v08;

import prophecy.common.image.RGB;
import prophecy.common.image.RGBImage;
import prophecy.video.v06.ObjectDetector;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class Naive2 extends ObjectDetector {
  private RGBImage image;

  public List<Rectangle> findObjects(RGBImage image) {
    this.image = image;
    try {
      List<Rectangle> result = new ArrayList<Rectangle>();
      for (int y = 0; y < image.getHeight(); y++)
        for (int x = 0; x < image.getWidth(); x++)
          if (isLeftUpperCorner(x, y)) {
            int x2 = x, y2 = y;
            while (!isBackground(x2+1, y2))
              ++x2;
            while (!isBackground(x2, y2+1))
              ++y2;
            result.add(new Rectangle(x, y, x2-x, y2-y));
          }

      return result;
    } finally {
      this.image = null;
    }
  }

  private boolean isLeftUpperCorner(int x, int y) {
    return isBackground(x-1, y) && isBackground(x, y-1) && isBackground(x-1, y-1)
      && !isBackground(x, y);
  }

  private boolean isBackground(int x, int y) {
    return x < 0 || y < 0 || x >= image.getWidth() || y >= image.getHeight()
      || isBackground(image.getPixel(x, y));
  }

  public boolean isBackground(RGB pixel) {
    return pixel.r+pixel.g+pixel.b >= 2.8f;
  }
}