/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v08;

import drjava.util.FileTreePersistence;
import drjava.util.Tree;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

public class TestHistory {
  private List<TestRecord> records = new ArrayList<TestRecord>();
  private String file;

  public TestHistory() {
  }

  public static void main(String[] args) {
    TestHistory history = new TestHistory();
    TestRecord testRecord = new TestRecord("simple square detection", "jazz", new Date(), "aContestant", 125.0);
    history.addTestRecord(testRecord);
  }

  public void addTestRecord(TestRecord record) {
    records.add(record);
  }

  public List<TestRecord> getRecords() {
    return records;
  }

  public void save(String file) throws IOException {
    this.file = file;
    save();
  }

  public void save() throws IOException {
    new FileTreePersistence(file).store(toTree());
  }

  private Tree toTree() {
    Tree tree = new Tree(getClass());
    tree.add("records", records);
    return tree;
  }

  public void load(String file) throws IOException {
    this.file = file;
    fromTree(new FileTreePersistence(file).load());
  }

  public double getHighscore() {
    double highscore = Double.NEGATIVE_INFINITY;
    for (TestRecord record : records) {
      if (Double.isInfinite(highscore) || record.getScore() > highscore)
        highscore = record.getScore();
    }
    return highscore;
  }

  private void fromTree(Tree tree) {
    if (tree == null) return;
    records = tree.getObject("records");
  }

  public boolean scoreSeen(double score) {
    for (TestRecord record : records) {
      if (record.getScore() == score)
        return true;
    }
    return false;
  }
}