/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v08;

import static drjava.util.ObjectUtil.toNiceString;
import prophecy.common.ContestUtil;
import static prophecy.common.InspectionUtil.inspect;
import prophecy.common.image.ImageSet;
import prophecy.video.v06.ObjectDetectionContest;
import prophecy.video.v06.ObjectDetector;

import java.io.IOException;
import java.util.Date;

public class TestNaive2 {
  public static void main(String[] args) throws IOException {
    //simpleContest(new Naive2());
    photoContest(new Naive2());
  }

  public static void photoContest(ObjectDetector contestant) throws IOException {
    runContest(photoImageSet(),
      "square detection in photos", "output/v08/photos.history.tree", contestant);
  }

  public static ImageSet photoImageSet() {
    return new ImageSet("data/v07", "wall", "typewriter");
  }

  private static void simpleContest(ObjectDetector contestant) throws IOException {
    runContest(new ImageSet("data/v07", "blacksquare", "twosquares"),
      "simple square detection", "output/v08/simple.history.tree", contestant);
  }

  private static void runContest(ImageSet imageSet, String contestDescription, String historyFile,
                                 ObjectDetector contestant) throws IOException {
    ObjectDetectionContest contest = new ObjectDetectionContest(imageSet, "square");
    double score = ContestUtil.getScore(contest, contestant);
    TestRecord record = new TestRecord(contestDescription + " (" + imageSet.getImages().size() + " images)",
      "jazz", new Date(), drjava.util.ObjectUtil.toNiceString(contestant), score);
    //StdOutDisplay.get().putnl(record.toTree());

    TestHistory history = new TestHistory();
    history.load(historyFile);

    double highscore = history.getHighscore();
    System.out.println("Score: " + score);
    System.out.println("Highscore: " + highscore);

    if (record.getScore() > highscore) {
      System.out.println("New highscore!!");
      history.addTestRecord(record);
      history.save();
    } else if (record.getScore() == highscore) {
      System.out.println("Score is equal to highscore");
    } else {
      System.out.println("Score (" + record.getScore() + ") is lower than highscore (" + highscore + ")");
      if (!history.scoreSeen(score)) {
        System.out.println("Score not seen before, saving");
        history.addTestRecord(record);
        history.save();
      }
    }

    inspect(toNiceString(contestant) + " / " + contestDescription, contest.getEvaluation());
  }
}