/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v09;

import prophecy.common.image.BWImage;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class BlobFinder {
  private BWImage image;
  private Rectangle bounds;
  private List<Point> toScan;

  public List<Rectangle> findBlobs(BWImage image) {
    image = new BWImage(image);
    this.image = image;
    List<Rectangle> blobs = new ArrayList<Rectangle>();
    for (int y = 0; y < image.getHeight(); y++)
      for (int x = 0; x < image.getWidth(); x++) {
        if (isBlack(x, y)) {
          toScan = new ArrayList<Point>();
          bounds = new Rectangle(x, y, 1, 1);
          scanBlob(x, y);
          while (!toScan.isEmpty()) {
            Point p = toScan.get(toScan.size()-1);
            toScan.remove(toScan.size()-1);
            scanBlob(p.x, p.y);
          }
          blobs.add(bounds);
        }
      }
    return blobs;
  }

  private boolean isBlack(int x, int y) {
    return x >= 0 && x < image.getWidth() && y >= 0 && y < image.getHeight()
      && image.getPixel(x, y) < 0.5;
  }

  private void scanBlob(int x, int y) {
    if (!markPixel(x, y)) return;
    toScan(x+1, y);
    toScan(x, y-1);
    toScan(x-1, y);
    toScan(x, y+1);
  }

  private void toScan(int x, int y) {
    toScan.add(new Point(x, y));
  }

  private boolean markPixel(int x, int y) {
    if (!isBlack(x, y)) return false;
    image.setPixel(x, y, 1);
    bounds = bounds.union(new Rectangle(x, y, 1, 1));
    return true;
  }
}