/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v09;

import prophecy.common.Surface;
import prophecy.video.v03.Histogram;

import java.awt.*;

public class HistogramSurface extends Surface {
  private Histogram histogram;
  private boolean cumulative;

  public HistogramSurface(Histogram histogram) {
    this.histogram = histogram;
    setMinimumSize(new Dimension(150, 150));
  }

  public void render(int w, int h, Graphics2D g) {
    /*g.setColor(Color.white);
    g.fillRect(0, 0, w, h);*/
    g.setColor(Color.black);
    int sections = histogram.getSections();
    double scale = cumulative ? histogram.getSum() : histogram.getMaxValue();
    double sum = 0;
    for (int i = 0; i < sections; i++) {
      double value = histogram.getData(i);
      sum += value;
      if (cumulative) value += sum;
      int y = (int) (h-h*value/scale);
      int x1 = i*w/sections;
      int x2 = (i+1)*w/sections;
      /*g.setColor(Color.white);
      g.fillRect(x1, 0, x2-x1, y);*/
      g.setColor(Color.black);
      g.fillRect(x1, y, x2-x1, h-y);
    }
  }

  public void setHistogram(Histogram histogram) {
    this.histogram = histogram;
    repaint();
  }

  public boolean isCumulative() {
    return cumulative;
  }

  public void setCumulative(boolean cumulative) {
    this.cumulative = cumulative;
  }
}