/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v10;

import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class CornerFinder {
  private int clipSize = 16;
  private double threshold = 0.9;

  public CornerFinder(int clipSize) {
    this.clipSize = clipSize;
  }

  public List<Corner> findCorners(BWImage originalImage) {
    BWImage image = ImageProcessing.addBorder(originalImage, clipSize);

    List<Corner> corners = new ArrayList<Corner>();
    int m = clipSize / 2;
    List<Rectangle> miniClips = ImageProcessing.getMiniclipRects(image.getWidth(), image.getHeight(),
      clipSize, clipSize, m, m);
    for (Rectangle miniClip : miniClips) {
      BWImage clip = image.clip(miniClip);
      Quad quad = new Quad(clip, threshold);
      if (quad.getSum(threshold) == 1) {
        Rectangle r = new Rectangle(miniClip);
        r.translate(-clipSize, -clipSize);
        Corner.Type type = quad.get(0) < threshold ? Corner.Type.northWest
          : quad.get(1) < threshold ? Corner.Type.northEast
          : quad.get(2) < threshold ? Corner.Type.southEast
          : Corner.Type.southWest;
        corners.add(new Corner(r, type));
      }
    }
    return corners;
  }

}