/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v10;

import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class EdgeFinder {
  private int clipSize = 8;
  private double threshold = 0.9;

  public List<Edge> findEdges(BWImage originalImage) {
    BWImage image = ImageProcessing.addBorder(originalImage, clipSize);

    List<Edge> edges = new ArrayList<Edge>();
    int m = clipSize / 2;
    List<Rectangle> miniClips = ImageProcessing.getMiniclipRects(image.getWidth(), image.getHeight(),
      clipSize, clipSize, m, m);
    for (Rectangle miniClip : miniClips) {
      BWImage clip = image.clip(miniClip);
      Quad quad = new Quad(clip, threshold);
      Edge.Type type = isEdge(quad);
      if (type != null) {
        Rectangle r = new Rectangle(miniClip);
        r.translate(-clipSize, -clipSize);
        edges.add(new Edge(r, type));
      }
    }
    return edges;
  }

  public static Edge.Type isEdge(Quad quad) {
    //System.out.println(quad);
    for (int i = 0; i < 4; i++) {
      //System.out.println("  " + quad.isBlack(i) + " " + quad.isBlack(i+1) + " " + !quad.isBlack(i+2) + " " + !quad.isBlack(i+3));
      if (quad.isBlack(i) && quad.isBlack(i+1) && !quad.isBlack(i+2) && !quad.isBlack(i+3)) {
        //System.out.println("!");
        return Edge.Type.values()[i];
      }
    }
    return null;
  }

}