/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v10;

import drjava.util.Randomizer;
import drjava.util.RealRandomizer;
import static drjava.util.StringUtil.formatDouble;
import drjava.util.TextView;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGBImage;
import prophecy.common.gui.SheetApp;
import prophecy.video.v01.ImageMeta;
import prophecy.video.v01.ImageMetaSurface;
import prophecy.video.v09.BlobFinder;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

public class V10NervousAlphabet extends SheetApp {
  private JSlider slider;
  protected TextView analysisView;
  protected StringBuffer stringBuffer;
  private ImageMetaSurface imageSurface;
  private Timer timer;
  private Randomizer randomizer = new RealRandomizer();
  private boolean showBlobs = false;
  protected List<Corner> corners;

  public V10NervousAlphabet() {
    this("V10: Nervous Alphabet");
  }

  protected V10NervousAlphabet(String title) {
    super(title);
    init();
  }

  public void init() {
    addGroups();
    slider.setValue((int) 'i'-(int) 'a');

    update();
    startTimer();

    showFrame();
  }

  protected void addGroups() {
    addImageGroup();
    addAnalysisGroup();
  }

  private void startTimer() {
    timer = new Timer(150, new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        update();
      }
    });
    System.out.println("timer start");
    timer.start();
  }

  private void addAnalysisGroup() {
    addBlueHeading("Analysis");
    analysisView = new TextView();
    analysisView.setMinimumSize(new Dimension(100, 60));
    analysisView.setInset(3);
    addComponent(analysisView);
  }

  private void addImageGroup() {
    addBlueHeading("Image");

    slider = new JSlider(JSlider.HORIZONTAL, 0, 25, 0);
    slider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        update();
      }
    });
    addComponent(slider);
    imageSurface = new ImageMetaSurface();
    addComponent(imageSurface);
  }

  public void retire() {
    System.out.println("timer stop");
    if (timer != null)
      timer.stop();
    super.retire();
  }

  protected void update() {
    BWImage image = makeImage(getLetter());
    imageSurface.setImage(image.toRGB());

    stringBuffer = new StringBuffer();
    analyze(image);

    analysisView.setText(stringBuffer.toString());
  }

  protected void analyze(BWImage image) {
    BlobFinder blobFinder = new BlobFinder();
    List<Rectangle> blobs = blobFinder.findBlobs(image);
    //stringBuffer.append("things: " + blobs.size() + "<br>");
    ImageMeta imageMeta = new ImageMeta();
    if (showBlobs)
      imageMeta.add("", blobs);
    imageSurface.setMeta(imageMeta);

    corners = new ArrayList<Corner>();

    if (blobs.size() == 1) {
      Rectangle blob = blobs.get(0);
      BWImage clip = image.clip(blob);
      addPair("width", clip.getWidth(), "height", clip.getHeight());
      addPair("top width", realSize(upperPart(clip, 0.25)).width,
        "bottom width", realSize(lowerPart(clip, 0.25)).width);

      /*List<Rectangle> ends = new EndFinder().findEnds(clip);
      stringBuffer.append("ends: " + ends.size());
      imageMeta.add("", ends);*/

      findCorners(blob, clip);
      //findEdges(blob, clip);
    }
  }

  protected String getLetter() {
    return "" + (char) ('A' + slider.getValue());
  }

  private void findEdges(Rectangle blob, BWImage clip) {
    List<Edge> edges = new EdgeFinder().findEdges(clip);
    stringBuffer.append("edges: " + edges.size() + "<br>");
    for (Edge edge : edges) {
      Rectangle r = new Rectangle(edge.getBounds());
      r.translate(blob.x, blob.y);
      imageSurface.getImageMeta().addObject(new ImageMeta.Object("", r));
    }
  }

  private void findCorners(Rectangle clipRect, BWImage clip) {
    int clipSize = 8, border = clipSize;
    clip = ImageProcessing.addBorder(clip, border);

    corners = new CornerFinder(clipSize).findCorners(clip);
    corners = hardCorners(clip, corners);
    stringBuffer.append((corners.size() == 1 ? "1 corner found"
      : corners.size() + " corners found") + "<br>");
    List<Rectangle> cornerRects = new ArrayList<Rectangle>();
    for (Corner corner : corners) {
      Rectangle r = new Rectangle(corner.getBounds());
      r.translate(clipRect.x-border, clipRect.y-border);
      cornerRects.add(r);
    }
    imageSurface.getImageMeta().add("", cornerRects);

    /*int northCorners = northCorners(corners);
    addPair("north c", northCorners, "south c", corners.size()-northCorners);
    int westCorners = westCorners(corners);
    addPair("west c", westCorners, "east c", corners.size()-westCorners);*/
  }

  private List<Corner> hardCorners(BWImage image, List<Corner> corners) {
    List<Corner> hardCorners = new ArrayList<Corner>();
    for (Corner corner : corners) {
      Rectangle r = corner.getBounds();
      int size = r.width, step = size/2;
      Rectangle edge1 = null, edge2 = null;
      switch (corner.getType()) {
        case northWest: {
          edge1 = new Rectangle(r.x-step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y-step, size, size);
          break;
        }
        case northEast: {
          edge1 = new Rectangle(r.x+step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y-step, size, size);
          break;
        }
        case southWest: {
          edge1 = new Rectangle(r.x-step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y+step, size, size);
          break;
        }
        case southEast: {
          edge1 = new Rectangle(r.x+step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y+step, size, size);
          break;
        }
      }
      if (isEdge(image.clip(edge1), size) && isEdge(image.clip(edge2), size))
        hardCorners.add(corner);
    }
    return hardCorners;
  }

  private boolean isEdge(BWImage image, int size) {
    if (image.getWidth() != size || image.getHeight() != size) return false;
    Quad quad = new Quad(image, 0.9);
    return EdgeFinder.isEdge(quad) != null;
  }

  private int northCorners(List<Corner> corners) {
    int sum = 0;
    for (Corner corner : corners) {
      if (corner.getType() == Corner.Type.northEast
        || corner.getType() == Corner.Type.northWest)
        ++sum;
    }
    return sum;
  }

  private int westCorners(List<Corner> corners) {
    int sum = 0;
    for (Corner corner : corners) {
      if (corner.getType() == Corner.Type.northWest
        || corner.getType() == Corner.Type.southWest)
        ++sum;
    }
    return sum;
  }

  private Dimension realSize(BWImage image) {
    java.util.List<Rectangle> blobs = new BlobFinder().findBlobs(image);
    return union(blobs).getSize();
  }

  private Rectangle union(List<Rectangle> blobs) {
    Rectangle r = null;
    for (Rectangle blob : blobs) {
      r = r == null ? blob : r.union(blob);
    }
    return r;
  }

  private BWImage upperPart(BWImage image, double ratio) {
    return image.clip(0, 0, image.getWidth(), (int) (image.getHeight()*ratio));
  }

  private BWImage lowerPart(BWImage image, double ratio) {
    int h = (int) (image.getHeight()*ratio);
    return image.clip(0, image.getHeight()-h, image.getWidth(), h);
  }

  private void addPair(String name1, double value1, String name2, double value2) {
    String ratio;
    if (value1*10 < value2)
      ratio = "1 : >10";
    else if (value1 > value2*10)
      ratio = ">10 : 1";
    else if (value1 > value2)
      ratio = formatDouble(value1/value2, 1) + " : 1";
    else
      ratio = "1 : " + formatDouble(value2/value1, 1);
    stringBuffer.append(name1 + " : " + name2 + " = " + ratio + "<br>");
  }

  private BWImage makeImage(String text) {
    BufferedImage image = new BufferedImage(200, 150, BufferedImage.TYPE_BYTE_BINARY);
    Graphics g = image.getGraphics();
    g.setColor(Color.white);
    g.fillRect(0, 0, image.getWidth(), image.getHeight());
    g.setColor(Color.black);
    g.setFont(new Font("Arial", Font.PLAIN, randomizer.random(90, 100)));
    int x = (image.getWidth()-g.getFontMetrics().stringWidth(text))/2;
    int y = 5 + g.getFontMetrics().getAscent();
    x += randomizer.random(8)-4;
    y += randomizer.random(8);
    g.drawString(text, x, y);

    return new RGBImage(image).toBW();
  }

  public static void main(String[] args) {
    new V10NervousAlphabet().asMainApplication();
  }
}