/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v11;

import prophecy.common.image.BWImage;
import prophecy.video.v10.Corner;
import prophecy.video.v10.CornerFinder;
import prophecy.video.v10.EdgeFinder;
import prophecy.video.v10.Quad;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class HardCornerFinder {
  private CornerFinder cornerFinder;

  public HardCornerFinder(int clipSize) {
    cornerFinder = new CornerFinder(clipSize);
  }

  private List<Corner> hardCorners(BWImage image, List<Corner> corners) {
    List<Corner> hardCorners = new ArrayList<Corner>();
    for (Corner corner : corners) {
      Rectangle r = corner.getBounds();
      int size = r.width, step = size/2;
      Rectangle edge1 = null, edge2 = null;
      switch (corner.getType()) {
        case northWest: {
          edge1 = new Rectangle(r.x-step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y-step, size, size);
          break;
        }
        case northEast: {
          edge1 = new Rectangle(r.x+step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y-step, size, size);
          break;
        }
        case southWest: {
          edge1 = new Rectangle(r.x-step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y+step, size, size);
          break;
        }
        case southEast: {
          edge1 = new Rectangle(r.x+step, r.y, size, size);
          edge2 = new Rectangle(r.x, r.y+step, size, size);
          break;
        }
      }
      boolean isEdge1 = isEdge(image.clip(edge1), size);
      boolean isEdge2 = isEdge(image.clip(edge2), size);
      //System.out.println(r.x + "/" + r.y + ": " + isEdge1 + "/" + isEdge2);
      if (isEdge1 && isEdge2)
        hardCorners.add(corner);
    }
    return hardCorners;
  }

  private boolean isEdge(BWImage image, int size) {
    if (image.getWidth() != size || image.getHeight() != size) return false;
    Quad quad = new Quad(image, 0.9);
    return EdgeFinder.isEdge(quad) != null;
  }

  public List<Corner> findCorners(BWImage image) {
    List<Corner> corners = cornerFinder.findCorners(image);
    return hardCorners(image, corners);
  }
}