/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v11;

import drjava.util.Randomizer;
import drjava.util.RepeatableRandomizer;
import drjava.util.TextView;
import prophecy.common.image.BWImage;
import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGBImage;
import prophecy.common.gui.SheetApp;
import prophecy.video.v01.ImageMeta;
import prophecy.video.v01.ImageMetaSurface;
import prophecy.video.v09.BlobFinder;
import prophecy.video.v10.Corner;

import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.List;

public class V10NervousAlphabet extends SheetApp {
  private JSlider slider;
  protected TextView textView;
  protected StringBuffer stringBuffer;
  private ImageMetaSurface imageSurface;
  private Timer timer;
  private Randomizer randomizer = new RepeatableRandomizer();
  private boolean showBlobs = false;
  protected List<Corner> corners;

  public V10NervousAlphabet() {
    this("V10: Nervous Alphabet");
  }

  protected V10NervousAlphabet(String title) {
    super(title);
    init();
  }

  public void init() {
    addGroups();
    slider.setValue((int) 'i'-(int) 'a');

    update();
    startTimer();

    showFrame();
  }

  protected void addGroups() {
    addImageGroup();
    addAnalysisGroup();
  }

  private void startTimer() {
    timer = new Timer(150, new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        update();
      }
    });
    System.out.println("timer start");
    timer.start();
  }

  private void addAnalysisGroup() {
    addBlueHeading("Analysis");
    textView = new TextView();
    textView.setMinimumSize(new Dimension(100, 60));
    textView.setInset(3);
    addComponent(textView);
  }

  private void addImageGroup() {
    addBlueHeading("Image");

    slider = new JSlider(JSlider.HORIZONTAL, 0, 25, 0);
    slider.addChangeListener(new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        update();
      }
    });
    addComponent(slider);
    imageSurface = new ImageMetaSurface();
    addComponent(imageSurface);
  }

  public void retire() {
    System.out.println("timer stop");
    if (timer != null)
      timer.stop();
    super.retire();
  }

  protected void update() {
    BWImage image = makeImage(getLetter());
    imageSurface.setImage(image.toRGB());

    stringBuffer = new StringBuffer();
    analyze(image);

    textView.setText(stringBuffer.toString());
  }

  protected void analyze(BWImage image) {
    BlobFinder blobFinder = new BlobFinder();
    List<Rectangle> blobs = blobFinder.findBlobs(image);
    //stringBuffer.append("things: " + blobs.size() + "<br>");
    ImageMeta imageMeta = new ImageMeta();
    if (showBlobs)
      imageMeta.add("", blobs);
    imageSurface.setMeta(imageMeta);

    corners = new ArrayList<Corner>();

    if (blobs.size() == 1) {
      Rectangle blob = blobs.get(0);
      BWImage clip = image.clip(blob);

      /*List<Rectangle> ends = new EndFinder().findEnds(clip);
      stringBuffer.append("ends: " + ends.size());
      imageMeta.add("", ends);*/

      findCorners(blob, clip);
      //findEdges(blob, clip);
    }
  }

  protected String getLetter() {
    return "" + (char) ('A' + slider.getValue());
  }

  private void findCorners(Rectangle clipRect, BWImage clip) {
    int clipSize = 6, border = clipSize;
    clip = ImageProcessing.addBorder(clip, border);

    corners = new HardCornerFinder(clipSize).findCorners(clip);
    stringBuffer.append((corners.size() == 1 ? "1 corner found"
      : corners.size() + " corners found") + "<br>");
    List<Rectangle> cornerRects = new ArrayList<Rectangle>();
    for (Corner corner : corners) {
      Rectangle r = new Rectangle(corner.getBounds());
      r.translate(clipRect.x-border, clipRect.y-border);
      cornerRects.add(r);
    }
    imageSurface.getImageMeta().add("", cornerRects);
  }

  private BWImage makeImage(String text) {
    BufferedImage image = new BufferedImage(200, 150, BufferedImage.TYPE_BYTE_BINARY);
    Graphics g = image.getGraphics();
    g.setColor(Color.white);
    g.fillRect(0, 0, image.getWidth(), image.getHeight());
    g.setColor(Color.black);
    g.setFont(new Font("Arial", Font.PLAIN, randomizer.random(90, 100)));
    int x = (image.getWidth()-g.getFontMetrics().stringWidth(text))/2;
    int y = 5 + g.getFontMetrics().getAscent();
    x += randomizer.random(8)-4;
    y += randomizer.random(8);
    g.drawString(text, x, y);

    return new RGBImage(image).toBW();
  }

  public static void main(String[] args) {
    new prophecy.video.v10.V10NervousAlphabet().asMainApplication();
  }
}