/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v11;

import drjava.util.FileTreePersistence;
import prophecy.common.Prophecy;
import prophecy.common.image.BWImage;
import prophecy.common.RevolvingSuccessMeter;

import javax.swing.*;
import java.awt.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class V11 extends V10NervousAlphabet {
  private JLabel commentsHeading;
  private String commentsLetter;
  private Comments comments;
  private JTextField tfComments;
  private FileTreePersistence commentsPersistence;
  private RevolvingSuccessMeter successMeter;
  private int failures;

  public V11() {
    super("V11");
  }

  public void init() {
    successMeter = new RevolvingSuccessMeter();
    commentsPersistence = new FileTreePersistence(Prophecy.memoryDir() + "v11/comments.tree");
    comments = (Comments) commentsPersistence.loadObject(new Comments());
    super.init();
  }

  protected void addGroups() {
    super.addGroups();
    textView.setMinimumSize(new Dimension(100, 100));
    commentsHeading = addBlueHeading("");

    tfComments = new JTextField();
    addComponent(tfComments);
  }

  protected void update() {
    super.update();

    if (!getLetter().equals(commentsLetter)) {
      if (commentsLetter != null) {
        if (comments.put(commentsLetter, tfComments.getText()))
          commentsPersistence.store(comments);
      }
      commentsLetter = getLetter();
      commentsHeading.setText("Comments for " + commentsLetter);
      String text = comments.get(commentsLetter);
      tfComments.setText(text != null ? text : "");
      successMeter = new RevolvingSuccessMeter();
    }
  }

  protected void analyze(BWImage image) {
    super.analyze(image);
    Integer target = getCornersTargetFromComment();
    if (target != null) {
      boolean success = corners.size() == target;
      /*if (!success && failures++ == 0) {
        SurfaceUtil.show(new ImageSurface(image.toRGB()));
      }*/
      successMeter.add(success);
    }
    if (successMeter.getTotal() != 0)
      stringBuffer.append("Success rate: " + successMeter.getPercent() +"%<br>");
  }

  private Integer getCornersTargetFromComment() {
    Matcher matcher = Pattern.compile("corners=(\\d+)").matcher(tfComments.getText());
    if (matcher.find())
      return new Integer(matcher.group(1));
    else
      return null;
  }

  public void retire() {
    commentsPersistence.store(comments);
    super.retire();
  }

  public static void main(String[] args) {
    new V11().asMainApplication();
  }
}