/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.video.v12;

import drjava.util.StringUtil;
import prophecy.video.v05.Neighbors;

import java.util.ArrayList;
import java.util.List;

public class Rows {
  private double threshold = 11.0;
  private Neighbors neighbors;
  private List<Row> rows = new ArrayList<Row>();

  public void process(Neighbors neighbors) {
    this.neighbors = neighbors;
    for (int y = 0; y < neighbors.getRows(); y++)
      processRow(y);
  }

  private void processRow(int y) {
    RowElement[] elements = new RowElement[neighbors.getCols()];
    boolean[] flags = new boolean[neighbors.getCols()];
    for (int x = 0; x < neighbors.getCols(); x++) {
      elements[x] = new RowElement(neighbors, x, y);
      flags[x] = elements[x].getQuality() >= threshold;
    }
    dump(elements);
    for (int x = 0; x < flags.length; x++) {
      if (flags[x]) {
        int x1 = x;
        double qualitySum = 0;
        do {
          qualitySum += elements[x].getQuality();
          ++x;
        } while (x < flags.length && flags[x]);
        rows.add(new Row(x1, y, x-x1, 1, qualitySum/(x-x1)));
      }
    }
  }

  private void dump(RowElement[] elements) {
    for (RowElement element : elements) {
      System.out.print(StringUtil.formatDouble(element.getQuality(), 1) + " ");
    }
    System.out.println();
  }

  public List<Row> getRows() {
    return rows;
  }

  public Neighbors getNeighbors() {
    return neighbors;
  }

  public double getThreshold() {
    return threshold;
  }

  public void setThreshold(double threshold) {
    this.threshold = threshold;
  }
}