package prophecy.video.v13;

import prophecy.common.image.RGB;
import prophecy.common.image.RGBImage;

public class ImageAnalysis {
  private static int defaultDominantColorResolution = 8;

  public static double getDifferencePercentage(RGBImage oldImage, RGBImage newImage) {
    //long startTime = System.currentTimeMillis();
    int differences = 0;
    int w = oldImage.getWidth();
    int h = oldImage.getHeight();
    for (int y = 0; y < h; y++)
      for (int x = 0; x < w; x++)
        if (oldImage.getIntPixel(x, y) != newImage.getIntPixel(x, y))
          ++differences;
    double percentage = (double) differences*100/(w*h);
    //long endTime = System.currentTimeMillis();
    //System.out.println(w*h + " pixels diffed in " + (endTime-startTime) + " ms");
    return percentage;
  }

  public static RGB getDominantColor(RGBImage image) {
    return getDominantColor(image, defaultDominantColorResolution);
  }

  public static RGB getDominantColor(RGBImage image, int resolution) {
    int[] histogram = new int[resolution*resolution*resolution];
    for (int y = 0; y < image.getHeight(); y++)
      for (int x = 0; x < image.getWidth(); x++) {
        RGB pixel = image.getPixel(x, y);
        int r = (int) (pixel.r*(resolution-1));
        int g = (int) (pixel.g*(resolution-1));
        int b = (int) (pixel.b*(resolution-1));
        histogram[b+resolution*(g+resolution*r)]++;
      }
    int max = 0;
    for (int i = 1; i < histogram.length; i++)
      if (histogram[i] > histogram[max])
        max = i;
    float b = max % resolution;
    max /= resolution;
    float g = max % resolution;
    max /= resolution;
    float r = max;
    return new RGB(r/(resolution-1), g/(resolution-1), b/(resolution-1));
  }
}
