package prophecy.video.v13;

import prophecy.common.image.ImageProcessing;
import prophecy.common.image.RGB;
import prophecy.common.image.RGBImage;

public class ScreenshotAnalysis {
  private RGBImage image, lastImage;
  private RGB dominantColor;
  private TilesAnalysis tilesAnalysis;
  private int tileSize = 8;

  public void newImage(RGBImage image) {
    image = resizeImage(image);
    this.lastImage = this.image;
    this.image = image;

    // clear cache vars
    dominantColor = null;
    tilesAnalysis = null;
  }

  private RGBImage resizeImage(RGBImage image) {
    int newWidth = 256;
    return ImageProcessing.resizeToWidth(image, newWidth, new AvgPixelMixer());
  }

  public double getDifferencePercentage() {
    return lastImage == null ? 0 : ImageAnalysis.getDifferencePercentage(lastImage, image);
  }

  public RGB getDominantColor() {
    if (dominantColor == null)
      dominantColor = ImageAnalysis.getDominantColor(image, 8);
    return dominantColor;
  }

  public TilesAnalysis getTilesAnalysis() {
    if (tilesAnalysis == null && image != null)
      tilesAnalysis = new TilesAnalysis(new Tiles(image, tileSize));
    return tilesAnalysis;
  }
}
