package prophecy.video.v13;

import prophecy.common.image.RGB;

import java.util.ArrayList;
import java.util.List;

public class TilesAnalysis {
  private Tiles tiles;
  private RGB[][] colors;
  private List<TileShape> shapes = new ArrayList<TileShape>();
  private TileShape[][] shapesByTile;

  public TilesAnalysis(Tiles tiles) {
    this.tiles = tiles;
    colors = new RGB[tiles.getCols()][tiles.getRows()];
    shapesByTile = new TileShape[tiles.getCols()][tiles.getRows()];
    for (int y = 0; y < tiles.getRows(); y++)
      for (int x = 0; x < tiles.getCols(); x++)
        colors[x][y] = ImageAnalysis.getDominantColor(tiles.getTile(x, y));

    for (int y = 0; y < tiles.getRows(); y++)
      for (int x = 0; x < tiles.getCols(); x++)
        if (colors[x][y] != null)
          createShape(x, y);
  }

  private void createShape(int x, int y) {
    TileShape shape = new TileShape();
    trace(shape, x, y, colors[x][y]);
  }

  private void trace(TileShape shape, int x, int y, RGB color) {
    if (!color.equals(colors[x][y]))
      return;
    shape.addTile(x, y);
    colors[x][y] = null;
    shapesByTile[x][y] = shape;
    if (x > 0)
      trace(shape, x-1, y, color);
    if (x < tiles.getCols()-1)
      trace(shape, x+1, y, color);
    if (y > 0)
      trace(shape, x, y-1, color);
    if (y < tiles.getRows()-1)
      trace(shape, x, y+1, color);
  }

  public TileShape getShape(int x, int y) {
    return shapesByTile[x][y];
  }

  public Tiles getTiles() {
    return tiles;
  }
}
