package prophecy.video.v14;

import drjava.util.Tree;

import java.util.ArrayList;
import java.util.List;

/** A class for constructing simple symbolic structures.
 *  Structures are mutable.
 *  Each node has a head (a string) and 0 or more arguments (nodes) */
public class S {
  public String head;
  private List<S> args = new ArrayList<S>();

  public static S empty() {
    return new S();
  }

  public S() {
    this("");
  }

  public S(String head, Object... args) {
    if (head == null)
      throw new NullPointerException("head is null");
    this.head = head;
    for (Object arg : args) {
      add(arg);
    }
  }

  public void add(Object arg) {
    if (arg instanceof S)
      add((S) arg);
    else if (arg instanceof String)
      add(new S((String) arg));
    else
      throw new RuntimeException("Not String or S: " + arg);
  }

  public String getHead() {
    return head;
  }

  public void setHead(String head) {
    this.head = head;
  }

  public S a(int idx) {
    return args.get(idx);
  }

  public S a0() { return a(0); }
  public S a1() { return a(1); }
  public S a2() { return a(2); }
  public S a3() { return a(3); }

  public void set(int idx, S arg) {
    while (idx >= args.size())
      args.add(empty());
    args.set(idx, arg);
  }

  public void add(S arg) {
    args.add(arg);
  }

  public String toString() {
    return toTree().toString();
  }

  public Tree toTree() {
    Tree tree = new Tree(head);
    for (S arg : args)
      tree.add(arg.toTree());
    return tree;
  }
}
