/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.web;

import drjava.util.Errors;
import org.webmacro.Context;
import org.webmacro.Template;
import org.webmacro.WM;
import prophecy.common.ContestOnDisk;
import prophecy.common.*;

import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.*;

public class MakeContestsPage {
  public static void main(String[] args) throws Exception {
    Locale.setDefault(Locale.ENGLISH);

    Properties properties = new Properties();
    properties.setProperty("TemplatePath", "webtemplates");
    WM wm = new WM(properties);
    Context context = new Context(wm.getBroker());

    List<Map> contests = new ArrayList<Map>();
    for (ContestOnDisk contestOnDisk : ContestUtil.getAllContestsOnDisk()) {
      Map<String, Object> map = new HashMap<String, Object>();
      map.put("name", contestOnDisk.getFullName());
      try {
        Contest contest = contestOnDisk.load();
        HallOfFame hallOfFame = contestOnDisk.getHallOfFame();
        if (hallOfFame != null && hallOfFame.getWinner() != null) {
          map.put("champion", hallOfFame.getWinner());
          map.put("highscore", hallOfFame.formatScore(hallOfFame.getWinner()));
        }
      } catch (Exception e) {
        Errors.add(e);
      }
      contests.add(map);
    }
    context.put("contests", contests);

    Template template = wm.getTemplate("contests.wm");
    String html = template.evaluateAsString(context);
    PrintStream ps = new PrintStream(new FileOutputStream("web/contests.html"));
    ps.print(html);
    ps.close();
  }
}