package smartcmd;

import drjava.util.Errors;
import drjava.util.GUIUtil;

import javax.swing.*;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;

public class SmartCmd {
  private JFrame frame;
  private JTextArea textArea;
  private Document document;

  public SmartCmd() {
    textArea = new JTextArea();
    document = textArea.getDocument();

    textArea.addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent evt) {
          if (evt.getKeyCode() == KeyEvent.VK_ENTER)
          SwingUtilities.invokeLater(new Runnable() {
            public void run() {
              try {
                userPressedEnter();
              } catch (Throwable e) {
                Errors.report(e);
              }
            }
          });
      }
    });
  }

  public static void main(String[] args) {
    new SmartCmd().makeFrame();
  }

  private void makeFrame() {
    frame = new JFrame("Smart Command");
    frame.setSize(500, 300);
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.getContentPane().add(new JScrollPane(textArea));

    println("Hi! This is Smart Command. Type a command if you want me to do something.");
    printPrompt();

    GUIUtil.showFrame(frame);
  }

  private void printPrompt() {
    print(">");
  }

  private void println(String s) {
    print(s + "\n");
  }

  private void print(String s) {
    int start = document.getLength();
    try {
      if (start > 0 && !document.getText(start-1, 1).equals("\n")) {
        System.out.println("Breaking before " + s);
        s = "\n" + s;
      }
    } catch (BadLocationException e) {
      Errors.report(e);
      // don't see this happening anyway
    }
    textArea.insert(s, start);
    int pos = textArea.getDocument().getLength();
    textArea.select(pos, pos);
  }

  private void userPressedEnter() throws BadLocationException {
    int len = document.getLength();
    int caret = textArea.getCaretPosition();
    if (caret != len) {
      // cursor not at end of document
      return;
    }
    if (caret == 0) return;
    int lineNr = textArea.getLineOfOffset(caret-1);
    int startOfLine = textArea.getLineStartOffset(lineNr);
    int endOfLine = textArea.getLineEndOffset(lineNr)-1;
    String lineText = textArea.getText(startOfLine, endOfLine - startOfLine);
    System.out.println("line: '" + lineText + "'");
    if (lineText.startsWith(">")) lineText = lineText.substring(1);
    userTypedLine(lineText);
  }

  private void userTypedLine(String line) {
    println("You typed: " + line);
    printPrompt();
  }
}
