/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package sorttreetable;

import javax.swing.*;
import javax.swing.event.*;

/***
 * Abstracts the eventListener-management code so cell-
 * editors which extend from this class dont have to.<br/>
 * <br/>
 * Original code by Philip Milne and Scott Violet for Sun Microsystems<br/>
 * TreeTable Example 2 can be found here:<br/>
 * <a href="http://java.sun.com/products/jfc/tsc/articles/treetable2/">
 * http://java.sun.com/products/jfc/tsc/articles/treetable2/
 * </a><br/>
 * <br/>
 * - Removed dummy methods and made the class abstract
 *   because I think it's nicer that way.<br/>
 * <br/>
 * @author Philip Milne
 * @author Scott Violet
 * @author Vincent Vollers
 */
public abstract class AbstractCellEditor implements CellEditor {
	/***
	 * A list of eventlisteners to call when an event is fired
	 */
	protected EventListenerList listenerList = new EventListenerList();

	/***
	 * Adds a new cellEditorListener to this cellEditor
	 */
	public void addCellEditorListener(CellEditorListener l) {
		listenerList.add(CellEditorListener.class, l);
	}

	/***
	 * Removes a cellEditorListener from this cellEditor
	 */
	public void removeCellEditorListener(CellEditorListener l) {
		listenerList.remove(CellEditorListener.class, l);
	}

	/***
	 * Notify all listeners that have registered interest for notification on
	 * this event type.
	 * 
	 * @see EventListenerList
	 */
	protected void fireEditingStopped() {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == CellEditorListener.class) {
				((CellEditorListener) listeners[i + 1]).editingStopped(
					new ChangeEvent(this));
			}
		}
	}

	/***
	 * Notify all listeners that have registered interest for notification on
	 * this event type.
	 * 
	 * @see EventListenerList
	 */
	protected void fireEditingCanceled() {
		// Guaranteed to return a non-null array
		Object[] listeners = listenerList.getListenerList();
		// Process the listeners last to first, notifying
		// those that are interested in this event
		for (int i = listeners.length - 2; i >= 0; i -= 2) {
			if (listeners[i] == CellEditorListener.class) {
				((CellEditorListener) listeners[i + 1]).editingCanceled(
					new ChangeEvent(this));
			}
		}
	}
}