/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package sorttreetable;

/***
 * Compares two elements in a tree<br/>
 * <br/>
 * It assumes the elements in the tree are 'Comparable'<br/>
 * <br/>
 * Because this is the 'default' comparator for the treeTable,
 * it doesn't pay attention to columns. it compares the
 * userObject found in one treeNode with one found in another.<br/>
 * 
 * @author Vincent Vollers
 */
public class DefaultSortTableComparator implements SortTableComparator {
	private boolean ascending;
	
	/***
	 * Creates a new DefaultSortTableComparator.
	 */
	public DefaultSortTableComparator() {
		this.ascending = true;
	}

	/***
	 * Compares the userObjects of two TreeTableNodes.
	 */
	public int compare(Object one, Object two) {
		TreeTableNode nodeOne = (TreeTableNode) one;
		TreeTableNode nodeTwo = (TreeTableNode) two;
		Comparable first = (Comparable) nodeOne.getUserObject();
		Comparable second = (Comparable) nodeTwo.getUserObject();

		if (ascending) {
			return first.compareTo(second);
		} else {
			return second.compareTo(first);
		}
	}

	/***
	 * Sets the column-number to reflect the new situation.
	 */
	public void setColumn(int column) { }
	
	/***
	 * Sets the sorting order.
	 */
	public void setAscending(boolean ascending) {
		this.ascending = ascending;
	}
}