/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m10;

import drjava.util.FileUtil;
import drjava.util.StringUtil;
import prophecy.common.lifeform.Activity;
import prophecy.m.m09.StringVar;
import prophecy.m.m14.Tool;
import prophecy.m.m23.FileDesc;
import prophecy.m.m23.FileSet;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class AutonumFilesManager extends Tool {
  private File dir;
  private StringVar extension = new StringVar();
  private List<String> files = new ArrayList<String>();
  private IdMaker idMaker = new IntIdMaker();

  public AutonumFilesManager(String dir) {
    this.dir = new File(dir);
    setActivity(new Init());
  }

  public void setExtension(String extension) {
    this.extension.set(extension);
  }

  public File makeFile() {
    return new File(dir, makeFileName());
  }

  public void addFileSet(FileSet fileSet) {
    String id = makeFileName();
    for (FileDesc fileDesc : fileSet.getFiles()) try {
      File file = new File(dir, id + fileDesc.getName());
      FileUtil.saveFile(file, fileDesc.getContents());
      log("File saved: " + file.getName());
    } catch (Throwable e) {
      error(e);
    }
  }

  public class Init extends Activity {
    protected void runOnce() {
      if (dir.isDirectory())
        logConfirmation(dir + " exists");
      else {
        if (dir.mkdirs())
          logConfirmation(dir + " created");
        else {
          logSurprise("Could not create " + dir);
          return;
        }
      }

      try {
        check();
        ready();
      } catch (Throwable e) {
        error(e);
      }
    }
  }

  public void check() {
    logActivity("Reading " + dir);
    this.files = new ArrayList<String>();
    File[] files = dir.listFiles();
    for (File file : files) {
      if (file.isFile() && isManagedFile(file.getName())) {
        this.files.add(file.getName());
        idMaker.fileFound(file.getName());
      }
    }
    log(StringUtil.n(this.files.size(), "file") + " found");
  }

  public boolean isManagedFile(String name) {
    return name.endsWith(extension.get());
  }

  public File getDir() {
    return dir;
  }

  public StringVar getExtension() {
    return extension;
  }

  public List<String> getFiles() {
    return files;
  }

  public String makeFileName() {
    return idMaker.makeID() + extension;
  }
}