/*
(C) 2007 Stefan Reich (jazz@drjava.de)
This source file is part of Project Prophecy.
For up-to-date information, see http://www.drjava.de/prophecy

This source file is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation, version 2.1.
*/

package prophecy.m.m19;

import static drjava.util.StringUtil.n;

import drjava.util.Errors;
import prophecy.common.image.RGBImage;
import prophecy.common.lifeform.Lifeform;
import prophecy.m.m08.CompositeLifeform;
import prophecy.m.m10.AutonumFilesManager;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;

public class ImageCollector extends CompositeLifeform {
  private AutonumFilesManager filesManager;
  private ImageQC imageQC;
  private ImageIDSet imageIDSet = new ImageIDSet();

  public ImageCollector(String dir) {
    filesManager = new AutonumFilesManager(dir);
    filesManager.setExtension(".jpg");
    addPart(filesManager);
  }

  public void introduce(Lifeform lifeform) {
    super.introduce(lifeform);
    if (lifeform instanceof ImageLF)
      introduce((ImageLF) lifeform);
  }

  public void introduce(ImageLF image) {
    if (!containsImage(image.getImage())) {
      if (!qualityCheck(image.getImage()))
        log("Quality check failed");
      else
        addImage(image.getImage());
    }
  }

  private boolean containsImage(RGBImage image) {
    return imageIDSet.contains(image);
  }

  private boolean qualityCheck(RGBImage image) {
    return imageQC == null || imageQC.qualityCheck(image);
  }

  public void addImage(RGBImage image) {
    File file = filesManager.makeFile();
    logActivity("Saving " + file.getPath());
    try {
      ImageIO.write(image.getBufferedImage(), "jpeg", file);
      imageIDSet.add(image);
      filesManager.check();
      log("Saved OK");
    } catch (IOException e) {
      Errors.add(e);
    }
  }

  public void setImageQC(ImageQC imageQC) {
    this.imageQC = imageQC;
  }

  public int getNumberOfImages() {
    return filesManager.getFiles().size();
  }

  public String toString() {
    return filesManager.getDir() + " [" + n(getNumberOfImages(), "image") + "]";
  }
}